"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const boltz_core_1 = require("boltz-core");
const liquid_1 = require("boltz-core/dist/lib/liquid");
const Config_1 = __importDefault(require("./Config"));
const Core_1 = require("./Core");
const ExitHandler_1 = require("./ExitHandler");
const Logger_1 = __importDefault(require("./Logger"));
const Prometheus_1 = __importDefault(require("./Prometheus"));
const Utils_1 = require("./Utils");
const VersionCheck_1 = __importDefault(require("./VersionCheck"));
const Api_1 = __importDefault(require("./api/Api"));
const BackupScheduler_1 = __importDefault(require("./backup/BackupScheduler"));
const ChainClient_1 = __importDefault(require("./chain/ChainClient"));
const ElementsWrapper_1 = __importDefault(require("./chain/ElementsWrapper"));
const Enums_1 = require("./consts/Enums");
const Database_1 = __importDefault(require("./db/Database"));
const ChainTipRepository_1 = __importDefault(require("./db/repositories/ChainTipRepository"));
const GrpcServer_1 = __importDefault(require("./grpc/GrpcServer"));
const GrpcService_1 = __importDefault(require("./grpc/GrpcService"));
const LndClient_1 = __importDefault(require("./lightning/LndClient"));
const ClnClient_1 = __importDefault(require("./lightning/cln/ClnClient"));
const MpayClient_1 = __importDefault(require("./lightning/cln/MpayClient"));
const NotificationProvider_1 = __importDefault(require("./notifications/NotificationProvider"));
const Blocks_1 = __importDefault(require("./service/Blocks"));
const CountryCodes_1 = __importDefault(require("./service/CountryCodes"));
const Service_1 = __importDefault(require("./service/Service"));
const NodeSwitch_1 = __importDefault(require("./swap/NodeSwitch"));
const WalletManager_1 = __importDefault(require("./wallet/WalletManager"));
const EthereumManager_1 = __importDefault(require("./wallet/ethereum/EthereumManager"));
const EvmNetworks_1 = require("./wallet/ethereum/EvmNetworks");
class Boltz {
    constructor(config) {
        this.start = async () => {
            await (0, Core_1.setup)();
            try {
                await this.db.migrate(this.currencies);
                await this.db.init();
                await this.backup.init();
                await this.prometheus.start();
                // Query the chain tips now to avoid them being updated after the chain clients are initialized
                const chainTips = await ChainTipRepository_1.default.getChainTips();
                await Promise.all(Array.from(this.currencies.values()).flatMap((currency) => {
                    const prms = [];
                    if (currency.chainClient) {
                        prms.push(this.connectChainClient(currency.chainClient));
                    }
                    prms.concat([currency.lndClient, currency.clnClient]
                        .filter((client) => client !== undefined)
                        .map((client) => this.connectLightningClient(client)));
                    return prms;
                }));
                await this.walletManager.init(this.config.currencies);
                await this.service.init(this.config.pairs);
                await this.service.swapManager.init(Array.from(this.currencies.values()), this.config.pairs);
                await this.notifications.init();
                await this.grpcServer.listen();
                await Promise.all([
                    this.countryCodes.downloadRanges(),
                    this.blocks.updateBlocks(),
                ]);
                await this.api.init();
                // Rescan chains after everything else was initialized to avoid race conditions
                if (chainTips.length === 0) {
                    return;
                }
                this.logger.verbose(`Starting rescan of chains: ${chainTips
                    .map((chainTip) => chainTip.symbol)
                    .join(', ')}`);
                const logRescan = (chainTip) => {
                    this.logger.debug(`Rescanning ${chainTip.symbol} from height: ${chainTip.height}`);
                };
                const rescanPromises = [];
                for (const chainTip of chainTips) {
                    const ethereumManager = this.ethereumManagers.find((manager) => manager.networkDetails.symbol === chainTip.symbol);
                    if (ethereumManager !== undefined) {
                        logRescan(chainTip);
                        rescanPromises.push(ethereumManager.contractEventHandler.rescan(chainTip.height));
                    }
                    else {
                        if (!this.currencies.has(chainTip.symbol)) {
                            this.logger.warn(`Not rescanning ${chainTip.symbol} because no chain client was configured`);
                            continue;
                        }
                        const { chainClient } = this.currencies.get(chainTip.symbol);
                        if (chainClient) {
                            logRescan(chainTip);
                            rescanPromises.push(chainClient.rescanChain(chainTip.height));
                        }
                    }
                }
                await Promise.all(rescanPromises);
                this.logger.verbose('Finished rescanning');
            }
            catch (error) {
                this.logger.error(`Could not initialize Boltz: ${(0, Utils_1.formatError)(error)}`);
                console.log(error);
                // eslint-disable-next-line no-process-exit
                process.exit(1);
            }
        };
        this.connectChainClient = async (client) => {
            const formatChainInfo = (networkInfo, blockchainInfo) => {
                if (networkInfo === undefined || blockchainInfo === undefined) {
                    return undefined;
                }
                const res = {
                    version: networkInfo.version,
                    protocolversion: networkInfo.protocolversion,
                    connections: networkInfo.connections,
                    blocks: blockchainInfo.blocks,
                    bestblockhash: blockchainInfo.bestblockhash,
                    verificationprogress: blockchainInfo.verificationprogress,
                };
                if (networkInfo.lowball && blockchainInfo.lowball) {
                    res.lowball = formatChainInfo(networkInfo.lowball, blockchainInfo.lowball);
                }
                return res;
            };
            const service = `${client.symbol} chain`;
            try {
                await client.connect();
                const blockchainInfo = await client.getBlockchainInfo();
                const networkInfo = await client.getNetworkInfo();
                VersionCheck_1.default.checkChainClientVersion(client.symbol, networkInfo.version);
                this.logStatus(service, formatChainInfo(networkInfo, blockchainInfo));
            }
            catch (error) {
                this.logCouldNotConnect(service, error);
            }
        };
        this.connectLightningClient = async (client) => {
            const service = `${client.symbol} ${client.serviceName()}`;
            try {
                await client.connect();
                const info = await client.getInfo();
                VersionCheck_1.default.checkLightningVersion(client.serviceName(), client.symbol, info.version);
                if (client instanceof ClnClient_1.default) {
                    const holdInfo = await client.getHoldInfo();
                    this.logger.verbose(`${client.symbol} ${ClnClient_1.default.serviceNameHold} version: ${holdInfo.version}`);
                    VersionCheck_1.default.checkLightningVersion(ClnClient_1.default.serviceNameHold, client.symbol, holdInfo.version);
                    if (client.useMpay()) {
                        const mpayInfo = await client.mpay.getInfo();
                        this.logger.verbose(`${client.symbol} ${MpayClient_1.default.serviceName} version: ${mpayInfo.version}`);
                        VersionCheck_1.default.checkLightningVersion(MpayClient_1.default.serviceName, client.symbol, mpayInfo.version);
                    }
                }
                this.logStatus(service, info);
            }
            catch (error) {
                this.logCouldNotConnect(service, error);
            }
        };
        this.parseCurrencies = () => {
            const result = new Map();
            this.config.currencies.forEach((currency) => {
                try {
                    const chainClient = new ChainClient_1.default(this.logger, currency.chain, currency.symbol);
                    result.set(currency.symbol, {
                        chainClient,
                        symbol: currency.symbol,
                        type: Enums_1.CurrencyType.BitcoinLike,
                        network: boltz_core_1.Networks[currency.network],
                        lndClient: currency.lnd !== undefined
                            ? new LndClient_1.default(this.logger, currency.symbol, currency.lnd)
                            : undefined,
                        clnClient: currency.cln !== undefined
                            ? new ClnClient_1.default(this.logger, currency.symbol, currency.cln)
                            : undefined,
                        limits: {
                            ...currency,
                        },
                    });
                }
                catch (error) {
                    this.logger.error(`Could not initialize currency ${currency.symbol}: ${error.message}`);
                }
            });
            [
                { network: EvmNetworks_1.Ethereum, config: this.config.ethereum.tokens },
                { network: EvmNetworks_1.Rsk, config: this.config.rsk?.tokens },
            ]
                .map((tokens) => {
                const manager = this.ethereumManagers.find((manager) => manager.networkDetails === tokens.network);
                if (manager === undefined) {
                    return undefined;
                }
                return {
                    ...tokens,
                    manager,
                };
            })
                .filter((tokens) => tokens !== undefined && tokens.config !== undefined)
                .forEach(({ config, manager, network }) => {
                config.forEach((token) => {
                    result.set(token.symbol, {
                        symbol: token.symbol,
                        provider: manager.provider,
                        type: token.symbol === network.symbol
                            ? Enums_1.CurrencyType.Ether
                            : Enums_1.CurrencyType.ERC20,
                        limits: {
                            ...token,
                        },
                    });
                });
            });
            if (this.config.liquid) {
                const { symbol, chain, network } = this.config.liquid;
                result.set(symbol, {
                    type: Enums_1.CurrencyType.Liquid,
                    symbol: symbol,
                    network: liquid_1.Networks[network],
                    chainClient: new ElementsWrapper_1.default(this.logger, chain),
                    limits: {
                        ...this.config.liquid,
                    },
                });
            }
            return result;
        };
        this.logStatus = (service, status) => {
            this.logger.verbose(`${service} status: ${JSON.stringify(status, undefined, 2)}`);
        };
        this.logCouldNotConnect = (service, error) => {
            this.logger.error(`Could not connect to ${service}: ${(0, Utils_1.formatError)(error)}`);
        };
        this.config = new Config_1.default().load(config);
        this.logger = new Logger_1.default(this.config.loglevel, this.config.logpath, this.config.lokiHost, this.config.lokiNetwork);
        this.logger.info(`Starting Boltz ${(0, Utils_1.getVersion)()}`);
        (0, ExitHandler_1.registerExitHandler)(async () => {
            await this.db.close();
            await this.logger.close();
        });
        process.on('unhandledRejection', (reason, promise) => {
            console.log(promise);
            this.logger.error(`Unhandled rejection: ${(0, Utils_1.formatError)(reason)}`);
        });
        process.on('uncaughtException', (error) => {
            this.logger.error(`Uncaught exception: ${(0, Utils_1.formatError)(error)}`);
        });
        process.on('exit', (code) => {
            (code === 0 ? this.logger.debug : this.logger.error)(`Application shutting down with code: ${code}`);
        });
        this.db = new Database_1.default(this.logger, this.config.dbpath, this.config.postgres);
        this.ethereumManagers = [
            { name: EvmNetworks_1.Ethereum.name, isRsk: false, config: this.config.ethereum },
            { name: EvmNetworks_1.Rsk.name, isRsk: true, config: this.config.rsk },
        ]
            .map(({ name, isRsk, config }) => {
            try {
                return new EthereumManager_1.default(this.logger, isRsk, config);
            }
            catch (error) {
                this.logger.warn(`Disabled ${name} integration because: ${(0, Utils_1.formatError)(error)}`);
            }
            return undefined;
        })
            .filter((manager) => manager !== undefined);
        this.currencies = this.parseCurrencies();
        const walletCurrencies = Array.from(this.currencies.values());
        this.walletManager = new WalletManager_1.default(this.logger, this.config.mnemonicpath, walletCurrencies, this.ethereumManagers);
        this.blocks = new Blocks_1.default(this.logger, this.config.blocks);
        try {
            this.service = new Service_1.default(this.logger, this.config, this.walletManager, new NodeSwitch_1.default(this.logger, this.config.nodeSwitch), this.currencies, this.blocks);
            this.backup = new BackupScheduler_1.default(this.logger, this.config.dbpath, this.config.postgres, this.config.backup, this.service.eventHandler);
            this.notifications = new NotificationProvider_1.default(this.logger, this.service, this.walletManager, this.backup, this.config.notification, [this.config.liquid].concat(this.config.currencies), this.config.ethereum.tokens);
            this.grpcServer = new GrpcServer_1.default(this.logger, this.config.grpc, new GrpcService_1.default(this.logger, this.service));
            this.prometheus = new Prometheus_1.default(this.logger, this.config.prometheus, this.config.pairs);
            this.countryCodes = new CountryCodes_1.default(this.logger, this.config.marking);
            this.api = new Api_1.default(this.logger, this.config.api, this.service, this.countryCodes);
        }
        catch (error) {
            this.logger.error(`Could not start Boltz: ${(0, Utils_1.formatError)(error)}`);
            // eslint-disable-next-line no-process-exit
            process.exit(1);
        }
    }
}
exports.default = Boltz;
//# sourceMappingURL=Boltz.js.map