import { Arguments } from 'yargs';
import { PrometheusConfig } from './Prometheus';
import { GoogleCloudConfig } from './backup/providers/GoogleCloud';
import { WebdavConfig } from './backup/providers/Webdav';
import { Network } from './consts/Enums';
import { PairConfig } from './consts/Types';
import { LndConfig } from './lightning/LndClient';
import { ClnConfig } from './lightning/cln/Types';
import { BlocksConfig } from './service/Blocks';
import { MarkingsConfig } from './service/CountryCodes';
import { SwapConfig } from './service/cooperative/DeferredClaimer';
import { NodeSwitchConfig } from './swap/NodeSwitch';
type PostgresConfig = {
    host: string;
    port: number;
    database: string;
    username: string;
    password: string;
};
type ChainConfig = {
    host: string;
    port: number;
    cookie?: string;
    user?: string;
    password?: string;
    zmqpubrawtx?: string;
    zmqpubrawblock?: string;
    zmqpubhashblock?: string;
    mempoolSpace?: string;
};
type LiquidChainConfig = ChainConfig & {
    lowball?: ChainConfig;
};
type PreferredWallet = 'lnd' | 'core' | undefined;
type BaseCurrencyConfig<T = ChainConfig> = {
    symbol: string;
    network: Network;
    preferredWallet: PreferredWallet;
    minWalletBalance: number;
    maxWalletBalance?: number;
    maxUnusedWalletBalance?: number;
    maxZeroConfAmount: number;
    chain: T;
};
type RoutingOffsetException = {
    nodeId: string;
    offset: number;
};
type CurrencyConfig = BaseCurrencyConfig & {
    lnd?: LndConfig;
    cln?: ClnConfig;
    routingOffsetExceptions?: RoutingOffsetException[];
    maxPaymentFeeRatio?: number;
    minLocalBalance: number;
    minRemoteBalance: number;
};
type TokenConfig = {
    symbol: string;
    decimals?: number;
    contractAddress?: string;
    minWalletBalance: number;
    maxWalletBalance?: number;
};
type EthProviderServiceConfig = {
    network: string;
    apiKey: string;
};
type RskConfig = {
    providerEndpoint: string;
    etherSwapAddress: string;
    erc20SwapAddress: string;
    tokens: TokenConfig[];
};
type EthereumConfig = RskConfig & {
    infura: EthProviderServiceConfig;
    alchemy: EthProviderServiceConfig;
};
type ApiConfig = {
    host: string;
    port: number;
    cors?: string | string[];
};
type GrpcConfig = {
    host: string;
    port: number;
};
type RatesConfig = {
    interval: number;
};
type BackupConfig = {
    interval: string;
    webdav?: WebdavConfig;
    gcloud?: GoogleCloudConfig;
};
type NotificationConfig = {
    mattermostUrl?: string;
    token: string;
    channel: string;
    channelAlerts?: string;
    prefix: string;
    interval: number;
    otpsecretpath: string;
};
type ConfigType = {
    datadir: string;
    configpath: string;
    mnemonicpath: string;
    dbpath: string;
    postgres?: PostgresConfig;
    logpath: string;
    loglevel: string;
    lokiHost?: string;
    lokiNetwork?: string;
    prometheus?: PrometheusConfig;
    retryInterval: number;
    prepayminerfee: boolean;
    swapwitnessaddress: boolean;
    swap: SwapConfig;
    marking: MarkingsConfig;
    blocks: BlocksConfig;
    api: ApiConfig;
    grpc: GrpcConfig;
    rates: RatesConfig;
    backup: BackupConfig;
    notification: NotificationConfig;
    nodeSwitch?: NodeSwitchConfig;
    pairs: PairConfig[];
    currencies: CurrencyConfig[];
    liquid?: BaseCurrencyConfig<LiquidChainConfig>;
    rsk?: RskConfig;
    ethereum: EthereumConfig;
};
declare class Config {
    static defaultDataDir: string;
    static defaultConfigPath: string;
    static defaultMnemonicPath: string;
    static defaultLogPath: string;
    static defaultDbPath: string;
    static defaultPrivatekeyPath: string;
    static defaultOtpSecretPath: string;
    private readonly config;
    private readonly dataDir;
    /**
     * The constructor sets the default values
     */
    constructor();
    /**
     * This loads arguments specified by the user either with a TOML config file or via command line arguments
     */
    load: (args: Arguments<any>) => ConfigType;
    private parseCurrency;
    private parseTomlConfig;
    private getDataDirPaths;
    private resolveConfigPath;
    private getDefaultLogLevel;
}
export default Config;
export { ApiConfig, RskConfig, ConfigType, GrpcConfig, ChainConfig, TokenConfig, BackupConfig, EthereumConfig, PostgresConfig, CurrencyConfig, PreferredWallet, LiquidChainConfig, BaseCurrencyConfig, NotificationConfig, EthProviderServiceConfig, };
