"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const toml_1 = __importDefault(require("@iarna/toml"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const Utils_1 = require("./Utils");
const Enums_1 = require("./consts/Enums");
const Errors_1 = __importDefault(require("./consts/Errors"));
class Config {
    /**
     * The constructor sets the default values
     */
    constructor() {
        this.dataDir = Config.defaultDataDir;
        /**
         * This loads arguments specified by the user either with a TOML config file or via command line arguments
         */
        this.load = (args) => {
            const boltzConfigFile = this.resolveConfigPath(args.configpath, this.config.configpath);
            if (fs_1.default.existsSync(boltzConfigFile)) {
                const tomlConfig = this.parseTomlConfig(boltzConfigFile);
                (0, Utils_1.deepMerge)(this.config, tomlConfig);
            }
            if (args.datadir) {
                this.config.datadir = (0, Utils_1.resolveHome)(args.datadir);
                (0, Utils_1.deepMerge)(this.config, this.getDataDirPaths(this.config.datadir));
            }
            else {
                const datadir = (0, Utils_1.resolveHome)(this.config.datadir);
                if (!fs_1.default.existsSync(datadir)) {
                    fs_1.default.mkdirSync(datadir);
                }
                this.config.datadir = datadir;
                (0, Utils_1.deepMerge)(this.config, this.getDataDirPaths(this.config.datadir));
            }
            if (args.currencies) {
                const currencies = this.parseCurrency(args.currencies, this.config.currencies);
                args.currencies = currencies.currencies;
                (0, Utils_1.deepMerge)(this.config, currencies);
            }
            (0, Utils_1.deepMerge)(this.config, args);
            return this.config;
        };
        this.parseCurrency = (args, config) => {
            args.forEach((currency) => {
                const currencyJSON = JSON.parse(currency);
                for (let i = 0; i < config.length; i += 1) {
                    const confCurrency = config[i];
                    const hasSymbol = currencyJSON.symbol === confCurrency.symbol;
                    const hasNetwork = currencyJSON.network === confCurrency.network;
                    if (hasSymbol && hasNetwork) {
                        config[i] = { ...config[i], ...currencyJSON };
                        break;
                    }
                    else if (i === config.length - 1) {
                        config.push(currencyJSON);
                        break;
                    }
                }
            });
            return {
                currencies: config,
            };
        };
        this.parseTomlConfig = (filename) => {
            if (fs_1.default.existsSync(filename)) {
                try {
                    const tomlFile = fs_1.default.readFileSync(filename, 'utf-8');
                    const parsedToml = toml_1.default.parse(tomlFile);
                    parsedToml.configpath = filename;
                    return parsedToml;
                }
                catch (error) {
                    throw Errors_1.default.COULD_NOT_PARSE_CONFIG(filename, JSON.stringify(error));
                }
            }
        };
        this.getDataDirPaths = (dataDir) => {
            return {
                configpath: path_1.default.join(dataDir, Config.defaultConfigPath),
                mnemonicpath: path_1.default.join(dataDir, Config.defaultMnemonicPath),
                dbpath: path_1.default.join(dataDir, Config.defaultDbPath),
                logpath: path_1.default.join(dataDir, Config.defaultLogPath),
                backup: {
                    privatekeypath: path_1.default.join(dataDir, Config.defaultPrivatekeyPath),
                },
                notification: {
                    otpsecretpath: path_1.default.join(dataDir, Config.defaultOtpSecretPath),
                },
            };
        };
        this.resolveConfigPath = (configPath, fallback) => {
            return configPath ? (0, Utils_1.resolveHome)(configPath) : fallback;
        };
        this.getDefaultLogLevel = () => {
            return process.env.NODE_ENV === 'production' ? 'info' : 'debug';
        };
        this.dataDir = (0, Utils_1.getServiceDataDir)('boltz');
        const { dbpath, backup, logpath, configpath, mnemonicpath, notification } = this.getDataDirPaths(this.dataDir);
        this.config = {
            configpath,
            mnemonicpath,
            dbpath,
            logpath,
            datadir: this.dataDir,
            loglevel: this.getDefaultLogLevel(),
            retryInterval: 15,
            prepayminerfee: false,
            swapwitnessaddress: false,
            swap: {
                deferredClaimSymbols: ['L-BTC'],
                batchClaimInterval: '*/15 * * * *',
                expiryTolerance: 120,
            },
            marking: {
                ipV4Ranges: 'https://cdn.jsdelivr.net/npm/@ip-location-db/asn-country/asn-country-ipv4-num.csv',
                ipV6Ranges: 'https://cdn.jsdelivr.net/npm/@ip-location-db/asn-country/asn-country-ipv6-num.csv',
            },
            blocks: {},
            api: {
                host: '127.0.0.1',
                port: 9001,
                cors: '*',
            },
            grpc: {
                host: '127.0.0.1',
                port: 9000,
            },
            rates: {
                interval: 1,
            },
            backup: {
                interval: '0 0 * * *',
                gcloud: {
                    email: '',
                    privatekeypath: backup.privatekeypath,
                    bucketname: '',
                },
                webdav: {
                    url: '',
                    username: '',
                    password: '',
                },
            },
            notification: {
                token: '',
                channel: '',
                prefix: '',
                interval: 1,
                otpsecretpath: notification.otpsecretpath,
            },
            pairs: [
                {
                    base: 'LTC',
                    quote: 'BTC',
                    fee: 5,
                    minSwapAmount: 10000,
                    maxSwapAmount: 10000000,
                },
                {
                    base: 'BTC',
                    quote: 'BTC',
                    fee: 1,
                    rate: 1,
                    minSwapAmount: 1000,
                    maxSwapAmount: 10000000,
                },
                {
                    base: 'LTC',
                    quote: 'LTC',
                    fee: 1,
                    rate: 1,
                    minSwapAmount: 2000,
                    maxSwapAmount: 20000000,
                },
            ],
            currencies: [
                {
                    symbol: 'BTC',
                    network: Enums_1.Network.Testnet,
                    preferredWallet: 'lnd',
                    minWalletBalance: 1000000,
                    minLocalBalance: 500000,
                    minRemoteBalance: 500000,
                    maxZeroConfAmount: 200000,
                    chain: {
                        host: '127.0.0.1',
                        port: 18334,
                        cookie: 'docker/regtest/data/core/cookies/.bitcoin-cookie',
                    },
                    lnd: {
                        host: '127.0.0.1',
                        port: 10009,
                        certpath: path_1.default.join((0, Utils_1.getServiceDataDir)('lnd'), 'tls.cert'),
                        macaroonpath: path_1.default.join((0, Utils_1.getServiceDataDir)('lnd'), 'data', 'chain', 'bitcoin', Enums_1.Network.Testnet, 'admin.macaroon'),
                        maxPaymentFeeRatio: 0.03,
                    },
                },
                {
                    symbol: 'LTC',
                    network: Enums_1.Network.Testnet,
                    preferredWallet: 'lnd',
                    minWalletBalance: 100000000,
                    minLocalBalance: 50000000,
                    minRemoteBalance: 50000000,
                    maxZeroConfAmount: 20000000,
                    chain: {
                        host: '127.0.0.1',
                        port: 19334,
                        cookie: 'docker/regtest/data/core/cookies/.litecoin-cookie',
                    },
                    lnd: {
                        host: '127.0.0.1',
                        port: 11009,
                        certpath: path_1.default.join((0, Utils_1.getServiceDataDir)('lnd'), 'tls.cert'),
                        macaroonpath: path_1.default.join((0, Utils_1.getServiceDataDir)('lnd'), 'data', 'chain', 'litecoin', Enums_1.Network.Testnet, 'admin.macaroon'),
                        maxPaymentFeeRatio: 0.03,
                    },
                },
            ],
            ethereum: {
                providerEndpoint: '',
                infura: {
                    apiKey: '',
                    network: 'rinkeby',
                },
                alchemy: {
                    apiKey: '',
                    network: 'rinkeby',
                },
                etherSwapAddress: '',
                erc20SwapAddress: '',
                tokens: [],
            },
        };
    }
}
// Default paths
Config.defaultDataDir = (0, Utils_1.getServiceDataDir)('boltz');
Config.defaultConfigPath = 'boltz.conf';
Config.defaultMnemonicPath = 'seed.dat';
Config.defaultLogPath = 'boltz.log';
Config.defaultDbPath = 'boltz.db';
Config.defaultPrivatekeyPath = 'backupPrivatekey.pem';
Config.defaultOtpSecretPath = 'otpSecret.dat';
exports.default = Config;
//# sourceMappingURL=Config.js.map