"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const safe_1 = require("colors/safe");
const winston_1 = __importDefault(require("winston"));
const winston_loki_1 = __importDefault(require("winston-loki"));
const package_json_1 = require("../package.json");
const Utils_1 = require("./Utils");
class Logger {
    constructor(level, filename, lokiHost, lokiNetwork, disabled = false) {
        this.disabled = disabled;
        this.close = async () => {
            if (this.loki !== undefined) {
                await this.loki.flush();
                await this.loki.close();
            }
        };
        this.getLogFormat = (colorize) => {
            return winston_1.default.format.printf((info) => `${(0, Utils_1.getTsString)()} ${this.getLevel(info.level, colorize)}: ${info.message}`);
        };
        this.getLevel = (level, colorize) => {
            if (colorize) {
                switch (level) {
                    case 'error':
                        return (0, safe_1.red)(level);
                    case 'warn':
                        return (0, safe_1.yellow)(level);
                    case 'info':
                        return (0, safe_1.green)(level);
                    case 'verbose':
                        return (0, safe_1.cyan)(level);
                    case 'debug':
                        return (0, safe_1.blue)(level);
                    case 'silly':
                        return (0, safe_1.magenta)(level);
                }
            }
            return level;
        };
        this.error = (message) => {
            this.log('error', message);
        };
        this.warn = (message) => {
            this.log('warn', message);
        };
        this.info = (message) => {
            this.log('info', message);
        };
        this.verbose = (message) => {
            this.log('verbose', message);
        };
        this.debug = (message) => {
            this.log('debug', message);
        };
        this.silly = (message) => {
            this.log('silly', message);
        };
        this.log = (level, message) => {
            if (!this.disabled) {
                winston_1.default.log(level, message);
            }
        };
        if (disabled) {
            return;
        }
        const transports = [
            new winston_1.default.transports.Console({
                format: this.getLogFormat(true),
            }),
        ];
        if (filename) {
            transports.push(new winston_1.default.transports.File({
                filename,
                format: this.getLogFormat(false),
            }));
        }
        const lokiEnabled = [lokiHost, lokiNetwork].every((val) => val !== undefined);
        if (lokiEnabled) {
            this.loki = new winston_loki_1.default({
                host: lokiHost,
                labels: {
                    job: package_json_1.name,
                    network: lokiNetwork,
                },
            });
            transports.push(this.loki);
        }
        winston_1.default.configure({
            level,
            transports,
        });
        if (!lokiEnabled) {
            this.warn('Disabled loki logger because of invalid configuration');
        }
        else {
            this.debug('Enabled loki logger');
        }
    }
}
Logger.disabledLogger = new Logger('', undefined, undefined, undefined, true);
exports.default = Logger;
//# sourceMappingURL=Logger.js.map