/// <reference types="node" />
import bolt11, { RoutingInfo } from '@boltz/bolt11';
import { Transaction } from 'bitcoinjs-lib';
import { OutputType } from 'boltz-core';
import { ContractTransactionResponse } from 'ethers';
import { Transaction as LiquidTransaction } from 'liquidjs-lib';
import { IChainClient } from './chain/ChainClient';
import { OrderSide, SwapVersion } from './consts/Enums';
export declare const TAPROOT_NOT_SUPPORTED = "taproot not supported";
/**
 * Generate an ID for a swap
 *
 * @param length how many characters the id should have
 */
export declare const generateId: (length?: number) => string;
export declare const generateSwapId: (version: SwapVersion) => string;
/**
 * Stringify any object or array
 */
export declare const stringify: (object: unknown) => string;
export declare const saneStringify: (object: unknown) => string;
type MapKey = string | number | symbol;
type MapRecordType<T> = T extends Map<infer U, infer V> ? U extends MapKey ? Record<U, V extends Map<any, any> ? MapRecordType<V> : V> : never : T;
export declare const mapToObject: <T extends Map<MapKey, any>>(map: T) => MapRecordType<T>;
/**
 * Get the pair id of a pair
 */
export declare const getPairId: (pair: {
    base: string;
    quote: string;
}) => string;
/**
 * Get the quote and base asset of a pair id
 */
export declare const splitPairId: (pairId: string) => {
    base: string;
    quote: string;
};
/**
 * Convert minutes into milliseconds
 */
export declare const minutesToMilliseconds: (minutes: number) => number;
export declare const decodeInvoiceAmount: (invoice: string) => number;
export declare const decodeInvoice: (invoice: string) => bolt11.PaymentRequestObject & {
    satoshis: number;
    timeExpireDate: number;
    paymentHash: string | undefined;
    routingInfo: bolt11.RoutingInfo | undefined;
    minFinalCltvExpiry: number | undefined;
};
/**
 * Splits a derivation path into multiple parts
 */
export declare const splitDerivationPath: (path: string) => {
    master: string;
    sub: number[];
};
/**
 * Concat an error code and its prefix
 */
export declare const concatErrorCode: (prefix: number, code: number) => string;
/**
 * Capitalize the first letter of a string
 */
export declare const capitalizeFirstLetter: (input: string) => string;
/**
 * Resolve '~' on Linux and Unix-Like systems
 */
export declare const resolveHome: (filename: string) => string;
/**
 * Get a hex encoded Buffer from a string
 * @returns a hex encoded Buffer
 */
export declare const getHexBuffer: (input: string) => Buffer;
/**
 * Get a hex encoded string from a Buffer
 *
 * @returns a hex encoded string
 */
export declare const getHexString: (input: Buffer) => string;
/**
 * Check whether a variable is a non-array object
 */
export declare const isObject: (val: unknown) => boolean;
/**
 * Get the current date
 */
export declare const getTsString: (date?: Date) => string;
/**
 * Recursively merge properties from different sources into a target object, overriding any
 * existing properties.
 *
 * @param target The destination object to merge into.
 * @param sources The sources objects to copy from.
 */
export declare const deepMerge: (target: Record<string, any>, ...sources: any[]) => unknown;
/**
 * Split a string into host and port
 *
 * @param listen string of format host:port
 */
export declare const splitListen: (listen: string) => {
    host: string;
    port: string;
};
/**
 * Get directory of system home.
 */
export declare const getSystemHomeDir: () => string;
/**
 * Get the data directory of a service
 */
export declare const getServiceDataDir: (service: string) => string;
export declare const getOutputType: (type?: number) => OutputType;
export declare const getPubkeyHashFunction: (outputType: OutputType) => ((hash: Buffer) => Buffer) | ((hash: Buffer) => {
    redeemScript: Buffer;
    outputScript: Buffer;
});
export declare const getScriptHashFunction: (outputType: OutputType) => (scriptHex: Buffer) => Buffer;
export declare const reverseBuffer: (input: Buffer) => Buffer;
/**
 * The reversed version of the hex representation of a Buffer is not equal to the reversed Buffer.
 * Therefore we have to go full circle from a string to back to the Buffer, reverse that Buffer
 * and convert it back to a string to get the id of the transaction that is used by BTCD
 */
export declare const transactionHashToId: (transactionHash: Buffer) => string;
/**
 * Detects whether the transactions signals RBF explicitly
 *
 * @param transaction the transactions to scan
 */
export declare const transactionSignalsRbfExplicitly: (transaction: Transaction | LiquidTransaction) => boolean;
export declare const getSendingReceivingCurrency: (baseCurrency: string, quoteCurrency: string, orderSide: OrderSide) => {
    sending: string;
    receiving: string;
};
export declare const getRate: (rate: number, orderSide: OrderSide, isReverse: boolean) => number;
export declare const getChainCurrency: (base: string, quote: string, orderSide: OrderSide, isReverse: boolean) => string;
export declare const getLightningCurrency: (base: string, quote: string, orderSide: OrderSide, isReverse: boolean) => string;
/**
 * Gets the memo for the BIP21 payment request or the invoice of a swap
 *
 * @param sendingCurrency currency Boltz sends and the user is receiving
 * @param isReverse whether the swap is a reverse one
 */
export declare const getSwapMemo: (sendingCurrency: string, isReverse: boolean) => string;
export declare const getPrepayMinerFeeInvoiceMemo: (sendingCurrency: string) => string;
export declare const formatError: (error: unknown) => string;
export declare const getVersion: () => string;
export declare const getUnixTime: () => number;
/**
 * Calculates the miner fee of a transaction on a UTXO based chain
 */
export declare const calculateUtxoTransactionFee: (chainClient: IChainClient, transaction: Transaction) => Promise<number>;
export declare const calculateLiquidTransactionFee: (transaction: LiquidTransaction) => number;
/**
 * Calculates the transaction fee of an Ethereum contract interaction and rounds it to 10 ** -8 decimals
 */
export declare const calculateEthereumTransactionFee: (transaction: ContractTransactionResponse) => number;
/**
 * Hashes an arbitrary UTF-8 string with SHA256
 *
 * @returns hex encoded hash
 */
export declare const hashString: (input: string) => string;
export declare const createApiCredential: () => string;
export declare const splitChannelPoint: (channelPoint: string) => {
    id: string;
    vout: number;
};
export {};
