"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.splitChannelPoint = exports.createApiCredential = exports.hashString = exports.calculateEthereumTransactionFee = exports.calculateLiquidTransactionFee = exports.calculateUtxoTransactionFee = exports.getUnixTime = exports.getVersion = exports.formatError = exports.getPrepayMinerFeeInvoiceMemo = exports.getSwapMemo = exports.getLightningCurrency = exports.getChainCurrency = exports.getRate = exports.getSendingReceivingCurrency = exports.transactionSignalsRbfExplicitly = exports.transactionHashToId = exports.reverseBuffer = exports.getScriptHashFunction = exports.getPubkeyHashFunction = exports.getOutputType = exports.getServiceDataDir = exports.getSystemHomeDir = exports.splitListen = exports.deepMerge = exports.getTsString = exports.isObject = exports.getHexString = exports.getHexBuffer = exports.resolveHome = exports.capitalizeFirstLetter = exports.concatErrorCode = exports.splitDerivationPath = exports.decodeInvoice = exports.decodeInvoiceAmount = exports.minutesToMilliseconds = exports.splitPairId = exports.getPairId = exports.mapToObject = exports.saneStringify = exports.stringify = exports.generateSwapId = exports.generateId = exports.TAPROOT_NOT_SUPPORTED = void 0;
const bolt11_1 = __importDefault(require("@boltz/bolt11"));
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const boltz_core_1 = require("boltz-core");
const crypto_1 = require("crypto");
const liquidjs_lib_1 = require("liquidjs-lib");
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const package_json_1 = __importDefault(require("../package.json"));
const Version_1 = __importDefault(require("./Version"));
const Consts_1 = require("./consts/Consts");
const Enums_1 = require("./consts/Enums");
exports.TAPROOT_NOT_SUPPORTED = 'taproot not supported';
const { p2shOutput, p2wshOutput, p2pkhOutput, p2wpkhOutput, p2shP2wshOutput, p2shP2wpkhOutput, } = boltz_core_1.Scripts;
const idPossibilities = 'ABCDEFGHIJKLMNPQRSTUVWXYZabcdefghkmnopqrstuvwxyz123456789';
/**
 * Generate an ID for a swap
 *
 * @param length how many characters the id should have
 */
const generateId = (length = 6) => {
    let id = '';
    for (let i = 0; i < length; i += 1) {
        id += idPossibilities.charAt(Math.floor(Math.random() * idPossibilities.length));
    }
    return id;
};
exports.generateId = generateId;
const generateSwapId = (version) => (0, exports.generateId)(version === Enums_1.SwapVersion.Legacy ? undefined : 12);
exports.generateSwapId = generateSwapId;
/**
 * Stringify any object or array
 */
const stringify = (object) => {
    return JSON.stringify(object, undefined, 2);
};
exports.stringify = stringify;
const sanitizeObject = (object) => {
    if (typeof object !== 'object') {
        return;
    }
    const cpy = Object.assign({}, object);
    for (const [key, value] of Object.entries(cpy)) {
        switch (typeof value) {
            case 'object':
                cpy[key] = sanitizeObject(value);
                break;
            case 'string':
                if (value.length > 1000) {
                    cpy[key] = undefined;
                }
                break;
        }
    }
    return cpy;
};
const saneStringify = (object) => {
    return (0, exports.stringify)(sanitizeObject(object));
};
exports.saneStringify = saneStringify;
/**
 * Turn a map into an object
 */
const mapToObjectInternal = (map, recursionLevel = 0) => {
    if (recursionLevel > 10) {
        throw 'nested map recursion level too deep';
    }
    const object = {};
    map.forEach((value, index) => {
        object[index] =
            value instanceof Map
                ? mapToObjectInternal(value, recursionLevel + 1)
                : value;
    });
    return object;
};
const mapToObject = (map) => mapToObjectInternal(map);
exports.mapToObject = mapToObject;
/**
 * Get the pair id of a pair
 */
const getPairId = (pair) => {
    return `${pair.base}/${pair.quote}`;
};
exports.getPairId = getPairId;
/**
 * Get the quote and base asset of a pair id
 */
const splitPairId = (pairId) => {
    const split = pairId.split('/');
    return {
        base: split[0],
        quote: split[1],
    };
};
exports.splitPairId = splitPairId;
/**
 * Convert minutes into milliseconds
 */
const minutesToMilliseconds = (minutes) => {
    return minutes * 60 * 1000;
};
exports.minutesToMilliseconds = minutesToMilliseconds;
const coalesceInvoiceAmount = (decoded) => {
    const decodedMsat = decoded.millisatoshis
        ? Math.ceil(Number(decoded.millisatoshis) / 1000)
        : undefined;
    return decoded.satoshis || decodedMsat || 0;
};
const decodeInvoiceAmount = (invoice) => coalesceInvoiceAmount(bolt11_1.default.decode(invoice));
exports.decodeInvoiceAmount = decodeInvoiceAmount;
const decodeInvoice = (invoice) => {
    const decoded = bolt11_1.default.decode(invoice);
    let paymentHash;
    let routingInfo;
    let minFinalCltvExpiry;
    for (const tag of decoded.tags) {
        switch (tag.tagName) {
            case 'payment_hash':
                paymentHash = tag.data;
                break;
            case 'routing_info':
                routingInfo = tag.data;
                break;
            case 'min_final_cltv_expiry':
                minFinalCltvExpiry = tag.data;
                break;
        }
    }
    return {
        ...decoded,
        routingInfo,
        paymentHash,
        minFinalCltvExpiry,
        satoshis: coalesceInvoiceAmount(decoded),
        timeExpireDate: decoded.timeExpireDate || (decoded.timestamp || 0) + 3600,
    };
};
exports.decodeInvoice = decodeInvoice;
/**
 * Splits a derivation path into multiple parts
 */
const splitDerivationPath = (path) => {
    const split = path.split('/');
    const master = split.shift();
    const sub = [];
    split.forEach((part) => {
        sub.push(Number(part));
    });
    return {
        master,
        sub,
    };
};
exports.splitDerivationPath = splitDerivationPath;
/**
 * Concat an error code and its prefix
 */
const concatErrorCode = (prefix, code) => {
    return `${prefix}.${code}`;
};
exports.concatErrorCode = concatErrorCode;
/**
 * Capitalize the first letter of a string
 */
const capitalizeFirstLetter = (input) => {
    return input.charAt(0).toUpperCase() + input.slice(1);
};
exports.capitalizeFirstLetter = capitalizeFirstLetter;
/**
 * Resolve '~' on Linux and Unix-Like systems
 */
const resolveHome = (filename) => {
    if (os_1.default.platform() !== 'win32') {
        if (filename.charAt(0) === '~') {
            return path_1.default.join(process.env.HOME, filename.slice(1));
        }
    }
    return filename;
};
exports.resolveHome = resolveHome;
/**
 * Get a hex encoded Buffer from a string
 * @returns a hex encoded Buffer
 */
const getHexBuffer = (input) => {
    return Buffer.from(input, 'hex');
};
exports.getHexBuffer = getHexBuffer;
/**
 * Get a hex encoded string from a Buffer
 *
 * @returns a hex encoded string
 */
const getHexString = (input) => {
    return input.toString('hex');
};
exports.getHexString = getHexString;
/**
 * Check whether a variable is a non-array object
 */
const isObject = (val) => {
    return val !== undefined && typeof val === 'object' && !Array.isArray(val);
};
exports.isObject = isObject;
/**
 * Get the current date
 */
const getTsString = (date = new Date()) => {
    const pad = (input, maxLength = 2) => {
        return input.toString().padStart(maxLength, '0');
    };
    return (`${pad(date.getUTCDate())}/${pad(date.getUTCMonth() + 1)}/${date.getUTCFullYear()}` +
        ` ${pad(date.getUTCHours())}:${pad(date.getUTCMinutes())}:${pad(date.getUTCSeconds())}:` +
        `${pad(date.getUTCMilliseconds(), 3)}`);
};
exports.getTsString = getTsString;
/**
 * Recursively merge properties from different sources into a target object, overriding any
 * existing properties.
 *
 * @param target The destination object to merge into.
 * @param sources The sources objects to copy from.
 */
const deepMerge = (target, ...sources) => {
    if (!sources.length)
        return target;
    const source = sources.shift();
    if ((0, exports.isObject)(target) && (0, exports.isObject)(source)) {
        Object.keys(source).forEach((key) => {
            if ((0, exports.isObject)(source[key])) {
                if (!target[key])
                    Object.assign(target, { [key]: {} });
                (0, exports.deepMerge)(target[key], source[key]);
            }
            else if (source[key] !== undefined) {
                Object.assign(target, { [key]: source[key] });
            }
        });
    }
    return (0, exports.deepMerge)(target, ...sources);
};
exports.deepMerge = deepMerge;
/**
 * Split a string into host and port
 *
 * @param listen string of format host:port
 */
const splitListen = (listen) => {
    const split = listen.split(':');
    return {
        host: split[0],
        port: split[1],
    };
};
exports.splitListen = splitListen;
/**
 * Get directory of system home.
 */
const getSystemHomeDir = () => {
    switch (os_1.default.platform()) {
        case 'win32':
            return process.env.LOCALAPPDATA;
        case 'darwin':
            return path_1.default.join(process.env.HOME, 'Library', 'Application Support');
        default:
            return process.env.HOME;
    }
};
exports.getSystemHomeDir = getSystemHomeDir;
/**
 * Get the data directory of a service
 */
const getServiceDataDir = (service) => {
    const homeDir = (0, exports.getSystemHomeDir)();
    const serviceDir = service.toLowerCase();
    switch (os_1.default.platform()) {
        case 'win32':
        case 'darwin':
            return path_1.default.join(homeDir, (0, exports.capitalizeFirstLetter)(serviceDir));
        default:
            return path_1.default.join(homeDir, `.${serviceDir}`);
    }
};
exports.getServiceDataDir = getServiceDataDir;
const getOutputType = (type) => {
    if (type === undefined) {
        return boltz_core_1.OutputType.Legacy;
    }
    switch (type) {
        case 0:
            return boltz_core_1.OutputType.Bech32;
        case 1:
            return boltz_core_1.OutputType.Compatibility;
        case 2:
            return boltz_core_1.OutputType.Legacy;
        default:
            throw Error('type does not exist');
    }
};
exports.getOutputType = getOutputType;
const getPubkeyHashFunction = (outputType) => {
    switch (outputType) {
        case boltz_core_1.OutputType.Taproot:
            throw exports.TAPROOT_NOT_SUPPORTED;
        case boltz_core_1.OutputType.Bech32:
            return p2wpkhOutput;
        case boltz_core_1.OutputType.Compatibility:
            return p2shP2wpkhOutput;
        case boltz_core_1.OutputType.Legacy:
            return p2pkhOutput;
    }
};
exports.getPubkeyHashFunction = getPubkeyHashFunction;
const getScriptHashFunction = (outputType) => {
    switch (outputType) {
        case boltz_core_1.OutputType.Taproot:
            throw exports.TAPROOT_NOT_SUPPORTED;
        case boltz_core_1.OutputType.Bech32:
            return p2wshOutput;
        case boltz_core_1.OutputType.Compatibility:
            return p2shP2wshOutput;
        case boltz_core_1.OutputType.Legacy:
            return p2shOutput;
    }
};
exports.getScriptHashFunction = getScriptHashFunction;
const reverseBuffer = (input) => {
    const buffer = Buffer.allocUnsafe(input.length);
    for (let i = 0, j = input.length - 1; i <= j; i += 1, j -= 1) {
        buffer[i] = input[j];
        buffer[j] = input[i];
    }
    return buffer;
};
exports.reverseBuffer = reverseBuffer;
/**
 * The reversed version of the hex representation of a Buffer is not equal to the reversed Buffer.
 * Therefore we have to go full circle from a string to back to the Buffer, reverse that Buffer
 * and convert it back to a string to get the id of the transaction that is used by BTCD
 */
const transactionHashToId = (transactionHash) => {
    return (0, exports.getHexString)((0, exports.reverseBuffer)(transactionHash));
};
exports.transactionHashToId = transactionHashToId;
/**
 * Detects whether the transactions signals RBF explicitly
 *
 * @param transaction the transactions to scan
 */
const transactionSignalsRbfExplicitly = (transaction) => {
    for (const input of transaction.ins) {
        if (input.sequence < 0xffffffff - 1) {
            return true;
        }
    }
    return false;
};
exports.transactionSignalsRbfExplicitly = transactionSignalsRbfExplicitly;
const getSendingReceivingCurrency = (baseCurrency, quoteCurrency, orderSide) => {
    const isBuy = orderSide === Enums_1.OrderSide.BUY;
    return {
        sending: isBuy ? baseCurrency : quoteCurrency,
        receiving: isBuy ? quoteCurrency : baseCurrency,
    };
};
exports.getSendingReceivingCurrency = getSendingReceivingCurrency;
const getRate = (rate, orderSide, isReverse) => {
    if (isReverse) {
        return orderSide === Enums_1.OrderSide.BUY ? 1 / rate : rate;
    }
    else {
        return orderSide === Enums_1.OrderSide.BUY ? rate : 1 / rate;
    }
};
exports.getRate = getRate;
const getChainCurrency = (base, quote, orderSide, isReverse) => {
    if (isReverse) {
        return orderSide === Enums_1.OrderSide.BUY ? base : quote;
    }
    else {
        return orderSide === Enums_1.OrderSide.BUY ? quote : base;
    }
};
exports.getChainCurrency = getChainCurrency;
const getLightningCurrency = (base, quote, orderSide, isReverse) => {
    if (isReverse) {
        return orderSide === Enums_1.OrderSide.BUY ? quote : base;
    }
    else {
        return orderSide === Enums_1.OrderSide.BUY ? base : quote;
    }
};
exports.getLightningCurrency = getLightningCurrency;
/**
 * Gets the memo for the BIP21 payment request or the invoice of a swap
 *
 * @param sendingCurrency currency Boltz sends and the user is receiving
 * @param isReverse whether the swap is a reverse one
 */
const getSwapMemo = (sendingCurrency, isReverse) => {
    return `Send to ${sendingCurrency} ${isReverse ? 'address' : 'lightning'}`;
};
exports.getSwapMemo = getSwapMemo;
const getPrepayMinerFeeInvoiceMemo = (sendingCurrency) => {
    return `Miner fee for sending to ${sendingCurrency} address`;
};
exports.getPrepayMinerFeeInvoiceMemo = getPrepayMinerFeeInvoiceMemo;
const formatError = (error) => {
    if (typeof error === 'string') {
        return error;
    }
    else if ('message' in error) {
        return error['message'];
    }
    else {
        return JSON.stringify(error);
    }
};
exports.formatError = formatError;
const getVersion = () => {
    return `${package_json_1.default.version}${Version_1.default}`;
};
exports.getVersion = getVersion;
const getUnixTime = () => {
    return Math.round(new Date().getTime() / 1000);
};
exports.getUnixTime = getUnixTime;
/**
 * Calculates the miner fee of a transaction on a UTXO based chain
 */
const calculateUtxoTransactionFee = async (chainClient, transaction) => {
    let fee = 0n;
    for (const input of transaction.ins) {
        const inputId = (0, exports.transactionHashToId)(input.hash);
        const rawInputTransaction = await chainClient.getRawTransaction(inputId);
        const inputTransaction = bitcoinjs_lib_1.Transaction.fromHex(rawInputTransaction);
        const spentOutput = inputTransaction.outs[input.index];
        fee += BigInt(spentOutput.value);
    }
    transaction.outs.forEach((output) => {
        fee -= BigInt(output.value);
    });
    return Number(fee);
};
exports.calculateUtxoTransactionFee = calculateUtxoTransactionFee;
const calculateLiquidTransactionFee = (transaction) => {
    return liquidjs_lib_1.confidential.confidentialValueToSatoshi(transaction.outs.find((out) => out.script.length === 0).value);
};
exports.calculateLiquidTransactionFee = calculateLiquidTransactionFee;
/**
 * Calculates the transaction fee of an Ethereum contract interaction and rounds it to 10 ** -8 decimals
 */
const calculateEthereumTransactionFee = (transaction) => {
    return Number((transaction.gasLimit *
        (transaction.type === 2
            ? transaction.maxFeePerGas
            : transaction.gasPrice)) /
        Consts_1.etherDecimals);
};
exports.calculateEthereumTransactionFee = calculateEthereumTransactionFee;
/**
 * Hashes an arbitrary UTF-8 string with SHA256
 *
 * @returns hex encoded hash
 */
const hashString = (input) => {
    return (0, exports.getHexString)(bitcoinjs_lib_1.crypto.sha256(Buffer.from(input, 'utf-8')));
};
exports.hashString = hashString;
const createApiCredential = () => {
    return (0, crypto_1.randomBytes)(32).toString('hex');
};
exports.createApiCredential = createApiCredential;
const splitChannelPoint = (channelPoint) => {
    const split = channelPoint.split(':');
    return {
        id: split[0],
        vout: Number(split[1]),
    };
};
exports.splitChannelPoint = splitChannelPoint;
//# sourceMappingURL=Utils.js.map