"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cors_1 = __importDefault(require("cors"));
const express_1 = __importDefault(require("express"));
const Controller_1 = __importDefault(require("./Controller"));
const Utils_1 = require("./Utils");
const ApiV2_1 = __importDefault(require("./v2/ApiV2"));
const WebSocketHandler_1 = __importDefault(require("./v2/WebSocketHandler"));
class Api {
    constructor(logger, config, service, countryCodes) {
        this.logger = logger;
        this.config = config;
        this.init = async () => {
            await this.controller.init();
            await new Promise((resolve) => {
                const server = this.app.listen(this.config.port, this.config.host, () => {
                    this.logger.info(`API server listening on: ${this.config.host}:${this.config.port}`);
                    this.websocket.register(server);
                    resolve();
                });
            });
        };
        this.registerRoutes = (controller) => {
            // Static files
            ['/', '/index.html'].forEach((path) => {
                this.app.route(path).get(controller.serveFile('index.html'));
            });
            this.app.route('/favicon.ico').get(controller.serveFile('favicon.ico'));
            ['/swagger', '/swagger.html'].forEach((path) => {
                this.app.route(path).get(controller.serveFile('swagger.html'));
            });
            this.app
                .route('/swagger-spec.json')
                .get(controller.serveFile('swagger-spec.json'));
            // GET requests
            this.app.route('/version').get(controller.version);
            this.app.route('/getpairs').get(controller.getPairs);
            this.app.route('/getnodes').get(controller.getNodes);
            this.app.route('/nodestats').get(controller.getNodeStats);
            this.app.route('/timeouts').get(controller.getTimeouts);
            this.app.route('/getcontracts').get(controller.getContracts);
            this.app.route('/getfeeestimation').get(controller.getFeeEstimation);
            // POST requests
            this.app.route('/routinghints').post(controller.routingHints);
            this.app.route('/swapstatus').post(controller.swapStatus);
            this.app.route('/swaprates').post(controller.swapRates);
            this.app.route('/gettransaction').post(controller.getTransaction);
            this.app.route('/getswaptransaction').post(controller.getSwapTransaction);
            this.app
                .route('/broadcasttransaction')
                .post(controller.broadcastTransaction);
            this.app.route('/createswap').post(controller.createSwap);
            this.app.route('/setinvoice').post(controller.setInvoice);
            this.app.route('/referrals/query').get(controller.queryReferrals);
            // EventSource streams
            this.app.route('/streamswapstatus').get(controller.streamSwapStatus);
        };
        this.app = (0, express_1.default)();
        this.app.set('trust proxy', 'loopback');
        if (config.cors === undefined || config.cors.length !== 0) {
            this.app.use((0, cors_1.default)({
                origin: config.cors || '*',
                methods: 'GET,HEAD,PUT,PATCH,POST,DELETE',
                preflightContinue: false,
                optionsSuccessStatus: 204,
            }));
        }
        this.app.use(express_1.default.json({
            verify(req, _, buf, encoding) {
                if (buf && buf.length) {
                    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                    // @ts-ignore
                    req.rawBody = buf.toString(encoding || 'utf-8');
                }
            },
        }));
        // Catch the ugly errors generated by the body-parser
        this.app.use((error, req, res, next) => {
            if (error instanceof SyntaxError) {
                (0, Utils_1.errorResponse)(logger, req, res, error);
                return;
            }
            next(error);
        });
        this.controller = new Controller_1.default(logger, service, countryCodes);
        this.websocket = new WebSocketHandler_1.default(service, this.controller);
        new ApiV2_1.default(this.logger, service, this.controller, countryCodes).registerRoutes(this.app);
        this.registerRoutes(this.controller);
    }
}
exports.default = Api;
//# sourceMappingURL=Api.js.map