"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const ReferralStats_1 = __importDefault(require("../data/ReferralStats"));
const ChannelCreationRepository_1 = __importDefault(require("../db/repositories/ChannelCreationRepository"));
const ReverseSwapRepository_1 = __importDefault(require("../db/repositories/ReverseSwapRepository"));
const SwapRepository_1 = __importDefault(require("../db/repositories/SwapRepository"));
const LndClient_1 = __importDefault(require("../lightning/LndClient"));
const ClnClient_1 = __importDefault(require("../lightning/cln/ClnClient"));
const Errors_1 = __importDefault(require("../service/Errors"));
const NodeInfo_1 = __importDefault(require("../service/NodeInfo"));
const SwapNursery_1 = __importDefault(require("../swap/SwapNursery"));
const Bouncer_1 = __importDefault(require("./Bouncer"));
const Utils_2 = require("./Utils");
class Controller {
    constructor(logger, service, countryCodes) {
        this.logger = logger;
        this.service = service;
        this.countryCodes = countryCodes;
        // TODO: refactor
        // A map between the ids and statuses of the swaps
        this.pendingSwapInfos = new Map();
        // A map between the ids and HTTP streams of all pending swaps
        this.pendingSwapStreams = new Map();
        this.init = async () => {
            this.logger.verbose('Fetching swaps status from database');
            // Get the latest status of all swaps in the database
            const [swaps, reverseSwaps] = await Promise.all([
                SwapRepository_1.default.getSwaps(),
                ReverseSwapRepository_1.default.getReverseSwaps(),
            ]);
            for (const swap of swaps) {
                const status = swap.status;
                switch (status) {
                    case Enums_1.SwapUpdateEvent.ChannelCreated: {
                        const channelCreation = await ChannelCreationRepository_1.default.getChannelCreation({
                            swapId: swap.id,
                        });
                        this.pendingSwapInfos.set(swap.id, {
                            status,
                            channel: {
                                fundingTransactionId: channelCreation.fundingTransactionId,
                                fundingTransactionVout: channelCreation.fundingTransactionVout,
                            },
                        });
                        break;
                    }
                    case Enums_1.SwapUpdateEvent.TransactionZeroConfRejected:
                        this.pendingSwapInfos.set(swap.id, {
                            status: Enums_1.SwapUpdateEvent.TransactionMempool,
                            zeroConfRejected: true,
                        });
                        break;
                    default:
                        this.pendingSwapInfos.set(swap.id, {
                            status: swap.status,
                            failureReason: swap.failureReason !== null ? swap.failureReason : undefined,
                        });
                        break;
                }
            }
            for (const reverseSwap of reverseSwaps) {
                const status = reverseSwap.status;
                switch (status) {
                    case Enums_1.SwapUpdateEvent.TransactionMempool:
                    case Enums_1.SwapUpdateEvent.TransactionConfirmed: {
                        const { base, quote } = (0, Utils_1.splitPairId)(reverseSwap.pair);
                        const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, reverseSwap.orderSide, true);
                        try {
                            const transactionHex = await this.service.getTransaction(chainCurrency, reverseSwap.transactionId);
                            this.pendingSwapInfos.set(reverseSwap.id, {
                                status,
                                transaction: {
                                    hex: transactionHex,
                                    id: reverseSwap.transactionId,
                                    eta: status === Enums_1.SwapUpdateEvent.TransactionMempool
                                        ? SwapNursery_1.default.reverseSwapMempoolEta
                                        : undefined,
                                },
                            });
                        }
                        catch (error) {
                            // If the transaction can't be queried with the service it's either a transaction on the Ethereum network,
                            // or something is terribly wrong
                            if (error.message !==
                                Errors_1.default.NOT_SUPPORTED_BY_SYMBOL(chainCurrency).message) {
                                throw error;
                            }
                            this.pendingSwapInfos.set(reverseSwap.id, {
                                status,
                                transaction: {
                                    id: reverseSwap.transactionId,
                                },
                            });
                        }
                        break;
                    }
                    default:
                        this.pendingSwapInfos.set(reverseSwap.id, {
                            status: status,
                        });
                        break;
                }
            }
        };
        // Static files
        this.serveFile = (fileName) => {
            return (_, res) => {
                res.sendFile(path_1.default.join(__dirname, 'static', fileName));
            };
        };
        // GET requests
        this.version = (_, res) => {
            (0, Utils_2.successResponse)(res, {
                version: (0, Utils_1.getVersion)(),
            });
        };
        this.getPairs = (_, res) => {
            const data = this.service.getPairs();
            (0, Utils_2.successResponse)(res, {
                info: data.info,
                warnings: data.warnings,
                pairs: (0, Utils_1.mapToObject)(data.pairs),
            });
        };
        this.getNodes = (_, res) => {
            const result = Object.fromEntries(Array.from(this.service.getNodes().entries()).map(([symbol, nodeInfo]) => {
                return [
                    symbol,
                    nodeInfo.get(LndClient_1.default.serviceName) ||
                        nodeInfo.get(ClnClient_1.default.serviceName),
                ];
            }));
            (0, Utils_2.successResponse)(res, {
                nodes: result,
            });
        };
        this.getNodeStats = (_, res) => {
            const stats = this.service.getNodeStats();
            (0, Utils_2.successResponse)(res, {
                nodes: Object.fromEntries(Array.from(stats).map(([symbol, stats]) => {
                    return [symbol, stats.get(NodeInfo_1.default.totalStats)];
                })),
            });
        };
        this.getTimeouts = async (_, res) => {
            const timeouts = this.service.getTimeouts();
            (0, Utils_2.successResponse)(res, {
                timeouts: (0, Utils_1.mapToObject)(timeouts),
            });
        };
        this.getContracts = async (req, res) => {
            try {
                const contracts = await this.service.getContracts();
                const response = {};
                for (const [network, networkContracts] of Object.entries(contracts)) {
                    response[network] = {
                        network: networkContracts.network,
                        swapContracts: (0, Utils_1.mapToObject)(networkContracts.swapContracts),
                        tokens: (0, Utils_1.mapToObject)(networkContracts.tokens),
                    };
                }
                (0, Utils_2.successResponse)(res, response);
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error, 501);
            }
        };
        this.getFeeEstimation = async (_, res) => {
            const feeEstimation = await this.service.getFeeEstimation();
            (0, Utils_2.successResponse)(res, (0, Utils_1.mapToObject)(feeEstimation));
        };
        // POST requests
        this.routingHints = async (req, res) => {
            try {
                const { symbol, routingNode } = (0, Utils_2.validateRequest)(req.body, [
                    { name: 'symbol', type: 'string' },
                    { name: 'routingNode', type: 'string' },
                ]);
                const routingHints = await this.service.getRoutingHints(symbol, routingNode);
                (0, Utils_2.successResponse)(res, {
                    routingHints,
                });
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error);
            }
        };
        this.swapStatus = async (req, res) => {
            try {
                const { id } = (0, Utils_2.validateRequest)(req.body, [
                    { name: 'id', type: 'string' },
                ]);
                const response = this.pendingSwapInfos.get(id);
                if (response) {
                    (0, Utils_2.successResponse)(res, response);
                }
                else {
                    (0, Utils_2.errorResponse)(this.logger, req, res, `could not find swap with id: ${id}`, 404);
                }
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error);
            }
        };
        this.swapRates = async (req, res) => {
            try {
                const { id } = (0, Utils_2.validateRequest)(req.body, [
                    { name: 'id', type: 'string' },
                ]);
                const response = await this.service.getSwapRates(id);
                (0, Utils_2.successResponse)(res, response);
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error);
            }
        };
        this.getTransaction = async (req, res) => {
            try {
                const { currency, transactionId } = (0, Utils_2.validateRequest)(req.body, [
                    { name: 'currency', type: 'string' },
                    { name: 'transactionId', type: 'string' },
                ]);
                const response = await this.service.getTransaction(currency, transactionId);
                (0, Utils_2.successResponse)(res, { transactionHex: response });
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error);
            }
        };
        this.getSwapTransaction = async (req, res) => {
            try {
                const { id } = (0, Utils_2.validateRequest)(req.body, [
                    { name: 'id', type: 'string' },
                ]);
                const response = await this.service.getSwapTransaction(id);
                (0, Utils_2.successResponse)(res, response);
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error);
            }
        };
        this.broadcastTransaction = async (req, res) => {
            try {
                const { currency, transactionHex } = (0, Utils_2.validateRequest)(req.body, [
                    { name: 'currency', type: 'string' },
                    { name: 'transactionHex', type: 'string' },
                ]);
                const response = await this.service.broadcastTransaction(currency, transactionHex);
                (0, Utils_2.successResponse)(res, { transactionId: response });
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error);
            }
        };
        this.createSwap = async (req, res) => {
            try {
                const { type } = (0, Utils_2.validateRequest)(req.body, [
                    { name: 'type', type: 'string' },
                ]);
                const swapType = this.parseSwapType(type);
                switch (swapType) {
                    case Enums_1.SwapType.Submarine:
                        await this.createSubmarineSwap(req, res);
                        break;
                    case Enums_1.SwapType.ReverseSubmarine:
                        await this.createReverseSubmarineSwap(req, res);
                        break;
                }
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error);
            }
        };
        this.createSubmarineSwap = async (req, res) => {
            const { pairId, channel, invoice, pairHash, orderSide, referralId, preimageHash, refundPublicKey, } = (0, Utils_2.validateRequest)(req.body, [
                { name: 'pairId', type: 'string' },
                { name: 'orderSide', type: 'string' },
                { name: 'channel', type: 'object', optional: true },
                { name: 'invoice', type: 'string', optional: true },
                { name: 'pairHash', type: 'string', optional: true },
                { name: 'referralId', type: 'string', optional: true },
                { name: 'refundPublicKey', type: 'string', hex: true, optional: true },
                { name: 'preimageHash', type: 'string', hex: true, optional: true },
            ]);
            if (channel !== undefined) {
                (0, Utils_2.validateRequest)(channel, [
                    { name: 'auto', type: 'boolean' },
                    { name: 'private', type: 'boolean' },
                    { name: 'inboundLiquidity', type: 'number' },
                ]);
            }
            let response;
            if (invoice) {
                response = await this.service.createSwapWithInvoice(pairId, orderSide, refundPublicKey, invoice.toLowerCase(), pairHash, referralId, channel);
            }
            else {
                // Check that the preimage hash was set
                (0, Utils_2.validateRequest)(req.body, [
                    { name: 'preimageHash', type: 'string', hex: true },
                ]);
                (0, Utils_2.checkPreimageHashLength)(preimageHash);
                response = await this.service.createSwap({
                    pairId,
                    channel,
                    orderSide,
                    referralId,
                    preimageHash,
                    refundPublicKey,
                    version: Enums_1.SwapVersion.Legacy,
                });
            }
            await (0, Utils_2.markSwap)(this.countryCodes, req.ip, response.id);
            this.logger.verbose(`Created new Swap with id: ${response.id}`);
            this.logger.silly(`Swap ${response.id}: ${(0, Utils_1.stringify)(response)}`);
            delete response.canBeRouted;
            (0, Utils_2.createdResponse)(res, response);
        };
        this.createReverseSubmarineSwap = async (req, res) => {
            const { pairId, address, pairHash, orderSide, referralId, routingNode, claimAddress, preimageHash, invoiceAmount, onchainAmount, claimPublicKey, addressSignature, } = (0, Utils_2.validateRequest)(req.body, [
                { name: 'pairId', type: 'string' },
                { name: 'orderSide', type: 'string' },
                { name: 'address', type: 'string', optional: true },
                { name: 'preimageHash', type: 'string', hex: true },
                { name: 'pairHash', type: 'string', optional: true },
                { name: 'referralId', type: 'string', optional: true },
                { name: 'routingNode', type: 'string', optional: true },
                { name: 'claimAddress', type: 'string', optional: true },
                { name: 'invoiceAmount', type: 'number', optional: true },
                { name: 'onchainAmount', type: 'number', optional: true },
                { name: 'claimPublicKey', type: 'string', hex: true, optional: true },
                { name: 'addressSignature', type: 'string', hex: true, optional: true },
            ]);
            (0, Utils_2.checkPreimageHashLength)(preimageHash);
            const response = await this.service.createReverseSwap({
                pairId,
                pairHash,
                orderSide,
                referralId,
                routingNode,
                claimAddress,
                preimageHash,
                invoiceAmount,
                onchainAmount,
                claimPublicKey,
                userAddress: address,
                version: Enums_1.SwapVersion.Legacy,
                userAddressSignature: addressSignature,
            });
            await (0, Utils_2.markSwap)(this.countryCodes, req.ip, response.id);
            this.logger.verbose(`Created Reverse Swap with id: ${response.id}`);
            this.logger.silly(`Reverse swap ${response.id}: ${(0, Utils_1.stringify)(response)}`);
            (0, Utils_2.createdResponse)(res, response);
        };
        this.setInvoice = async (req, res) => {
            try {
                const { id, invoice, pairHash } = (0, Utils_2.validateRequest)(req.body, [
                    { name: 'id', type: 'string' },
                    { name: 'invoice', type: 'string' },
                    { name: 'pairHash', type: 'string', optional: true },
                ]);
                const response = await this.service.setInvoice(id, invoice.toLowerCase(), pairHash);
                (0, Utils_2.successResponse)(res, response);
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error);
            }
        };
        this.queryReferrals = async (req, res) => {
            try {
                const referral = await Bouncer_1.default.validateRequestAuthentication(req);
                const stats = await ReferralStats_1.default.getReferralFees(referral.id);
                (0, Utils_2.successResponse)(res, stats);
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error, 401);
            }
        };
        // EventSource streams
        this.streamSwapStatus = (req, res) => {
            try {
                const { id } = (0, Utils_2.validateRequest)(req.query, [
                    { name: 'id', type: 'string' },
                ]);
                res.writeHead(200, {
                    Connection: 'keep-alive',
                    'X-Accel-Buffering': 'no',
                    'Cache-Control': 'no-cache',
                    'Content-Type': 'text/event-stream',
                });
                res.setTimeout(0);
                const lastUpdate = this.pendingSwapInfos.get(id);
                if (lastUpdate) {
                    this.writeToSse(res, lastUpdate);
                }
                this.pendingSwapStreams.set(id, res);
                res.on('close', () => {
                    this.pendingSwapStreams.delete(id);
                });
            }
            catch (error) {
                (0, Utils_2.errorResponse)(this.logger, req, res, error);
            }
        };
        this.parseSwapType = (type) => {
            const lowerCaseType = type.toLowerCase();
            for (const swapType in Enums_1.SwapType) {
                if (lowerCaseType === Enums_1.SwapType[swapType]) {
                    return lowerCaseType;
                }
            }
            throw `could not find swap type: ${type}`;
        };
        this.writeToSse = (res, message) => {
            res.write(`data: ${JSON.stringify(message)}\n\n`);
        };
        this.service.eventHandler.on('swap.update', ({ id, status }) => {
            this.logger.debug(`Swap ${id} update: ${(0, Utils_1.saneStringify)(status)}`);
            this.pendingSwapInfos.set(id, status);
            const response = this.pendingSwapStreams.get(id);
            if (response) {
                this.writeToSse(response, status);
            }
        });
    }
}
exports.default = Controller;
//# sourceMappingURL=Controller.js.map