"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.markSwap = exports.checkPreimageHashLength = exports.setContentTypeJson = exports.writeErrorResponse = exports.createdResponse = exports.successResponse = exports.errorResponse = exports.validateRequest = void 0;
const Utils_1 = require("../Utils");
const MarkedSwapRepository_1 = __importDefault(require("../db/repositories/MarkedSwapRepository"));
const Errors_1 = __importDefault(require("../service/Errors"));
const Errors_2 = __importDefault(require("./Errors"));
// Some endpoints are getting spammed on mainnet, so we don't log the warnings for them
const errorsNotToLog = [
    Errors_1.default.SWAP_NO_LOCKUP().message,
    Errors_1.default.ETHEREUM_NOT_ENABLED().message,
];
/**
 * Validates that all required arguments were provided in the body correctly
 *
 * @returns the validated arguments
 */
const validateRequest = (body, argsToCheck) => {
    const response = {};
    argsToCheck.forEach((arg) => {
        const value = body[arg.name];
        if (value !== undefined) {
            if (typeof value === arg.type) {
                if (arg.hex && value !== '') {
                    const buffer = (0, Utils_1.getHexBuffer)(value);
                    if (buffer.length === 0) {
                        throw Errors_2.default.COULD_NOT_PARSE_HEX(arg.name);
                    }
                    response[arg.name] = buffer;
                }
                else {
                    response[arg.name] = value;
                }
            }
            else {
                throw Errors_2.default.INVALID_PARAMETER(arg.name);
            }
        }
        else if (!arg.optional) {
            throw Errors_2.default.UNDEFINED_PARAMETER(arg.name);
        }
    });
    return response;
};
exports.validateRequest = validateRequest;
const errorResponse = (logger, req, res, error, statusCode = 400) => {
    if (typeof error === 'string') {
        (0, exports.writeErrorResponse)(logger, req, res, statusCode, { error });
    }
    else {
        const errorObject = error;
        // Bitcoin Core related errors
        if (errorObject.details) {
            (0, exports.writeErrorResponse)(logger, req, res, statusCode, {
                error: errorObject.details,
            });
            // Custom error when broadcasting a refund transaction fails because
            // the locktime requirement has not been met yet
        }
        else if (errorObject.timeoutBlockHeight) {
            (0, exports.writeErrorResponse)(logger, req, res, statusCode, error);
            // Everything else
        }
        else {
            (0, exports.writeErrorResponse)(logger, req, res, statusCode, {
                error: errorObject.message,
            });
        }
    }
};
exports.errorResponse = errorResponse;
const successResponse = (res, data) => {
    (0, exports.setContentTypeJson)(res);
    res.status(200);
    if (typeof data === 'object') {
        res.json(data);
    }
    else {
        res.write(data);
        res.end();
    }
};
exports.successResponse = successResponse;
const createdResponse = (res, data) => {
    (0, exports.setContentTypeJson)(res);
    res.status(201).json(data);
};
exports.createdResponse = createdResponse;
const writeErrorResponse = (logger, req, res, statusCode, error) => {
    if (!errorsNotToLog.includes(error?.error || error)) {
        logger.warn(`Request ${req.method} ${req.originalUrl} ${req.body && Object.keys(req.body).length > 0
            ? `${JSON.stringify(req.body)} `
            : ''}failed: ${JSON.stringify(error)}`);
    }
    (0, exports.setContentTypeJson)(res);
    res.status(statusCode).json(error);
};
exports.writeErrorResponse = writeErrorResponse;
const setContentTypeJson = (res) => {
    res.set('Content-Type', 'application/json');
};
exports.setContentTypeJson = setContentTypeJson;
const checkPreimageHashLength = (preimageHash) => {
    if (preimageHash.length !== 32) {
        throw `invalid preimage hash length: ${preimageHash.length}`;
    }
};
exports.checkPreimageHashLength = checkPreimageHashLength;
const markSwap = async (countryCodes, ip, swapId) => {
    if (!countryCodes.isRelevantCountry(countryCodes.getCountryCode(ip))) {
        return;
    }
    await MarkedSwapRepository_1.default.addMarkedSwap(swapId);
};
exports.markSwap = markSwap;
//# sourceMappingURL=Utils.js.map