"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubscriptionChannel = exports.Operation = void 0;
const ws_1 = __importDefault(require("ws"));
const Utils_1 = require("../../Utils");
const DefaultMap_1 = __importDefault(require("../../consts/DefaultMap"));
const Errors_1 = __importDefault(require("../../service/Errors"));
var Operation;
(function (Operation) {
    // TODO: unsubscribe
    Operation["Subscribe"] = "subscribe";
    Operation["Update"] = "update";
})(Operation || (exports.Operation = Operation = {}));
var SubscriptionChannel;
(function (SubscriptionChannel) {
    SubscriptionChannel["SwapUpdate"] = "swap.update";
})(SubscriptionChannel || (exports.SubscriptionChannel = SubscriptionChannel = {}));
class WebSocketHandler {
    constructor(service, controller) {
        this.service = service;
        this.controller = controller;
        this.swapToSockets = new DefaultMap_1.default(() => new Set());
        this.socketToSwaps = new DefaultMap_1.default(() => new Set());
        this.register = (server) => {
            server.on('upgrade', (request, socket, head) => {
                this.ws.handleUpgrade(request, socket, head, (ws) => {
                    this.ws.emit('connection', ws, request);
                });
            });
            this.pingInterval = setInterval(() => {
                this.ws.clients.forEach((ws) => ws.ping());
            }, WebSocketHandler.pingIntervalMs);
        };
        this.close = () => {
            this.ws.close();
            clearInterval(this.pingInterval);
        };
        this.listenConnections = () => {
            this.ws.on('connection', (socket) => {
                socket.on('message', (msg) => this.handleMessage(socket, msg));
                socket.on('close', () => {
                    const ids = this.socketToSwaps.get(socket);
                    this.socketToSwaps.delete(socket);
                    for (const id of ids) {
                        const set = this.swapToSockets.get(id);
                        set.delete(socket);
                        if (set.size === 0) {
                            this.swapToSockets.delete(id);
                        }
                    }
                });
            });
        };
        this.handleMessage = (socket, message) => {
            try {
                const data = JSON.parse(message.toString('utf-8'));
                switch (data.op) {
                    case Operation.Subscribe:
                        this.handleSubscribe(socket, data);
                        break;
                    default:
                        this.sendToSocket(socket, { error: 'unknown operation' });
                        break;
                }
            }
            catch (e) {
                this.sendToSocket(socket, {
                    error: `could not parse message: ${(0, Utils_1.formatError)(e)}`,
                });
            }
        };
        this.handleSubscribe = (socket, data) => {
            const subscribeData = data;
            switch (subscribeData.channel) {
                case SubscriptionChannel.SwapUpdate: {
                    const idsWithSwaps = subscribeData.args.filter((id) => this.controller.pendingSwapInfos.has(id));
                    const existingSet = this.socketToSwaps.get(socket);
                    for (const id of idsWithSwaps) {
                        existingSet.add(id);
                    }
                    this.sendToSocket(socket, {
                        event: Operation.Subscribe,
                        channel: subscribeData.channel,
                        args: subscribeData.args,
                    });
                    const args = subscribeData.args.map((id) => {
                        const status = this.controller.pendingSwapInfos.get(id);
                        if (status === undefined) {
                            return {
                                id,
                                error: Errors_1.default.SWAP_NOT_FOUND(id).message,
                            };
                        }
                        this.swapToSockets.get(id).add(socket);
                        return {
                            id,
                            ...status,
                        };
                    });
                    this.sendToSocket(socket, {
                        event: Operation.Update,
                        channel: SubscriptionChannel.SwapUpdate,
                        args: args,
                    });
                    break;
                }
                default:
                    this.sendToSocket(socket, { error: 'unknown channel' });
                    return;
            }
        };
        this.listenSwapUpdates = () => {
            this.service.eventHandler.on('swap.update', ({ id, status }) => {
                const sockets = this.swapToSockets.getNoDefault(id);
                if (sockets === undefined) {
                    return;
                }
                for (const socket of sockets) {
                    this.sendToSocket(socket, {
                        event: Operation.Update,
                        channel: SubscriptionChannel.SwapUpdate,
                        args: [{ id, ...status }],
                    });
                }
            });
        };
        this.sendToSocket = (socket, msg) => {
            if (socket.readyState !== socket.OPEN) {
                return;
            }
            socket.send(JSON.stringify(msg));
        };
        this.ws = new ws_1.default.Server({
            noServer: true,
        });
        this.listenConnections();
        this.listenSwapUpdates();
    }
}
WebSocketHandler.pingIntervalMs = 15000;
exports.default = WebSocketHandler;
//# sourceMappingURL=WebSocketHandler.js.map