"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = require("express");
const Utils_1 = require("../../../Utils");
const Utils_2 = require("../../Utils");
const RouterBase_1 = __importDefault(require("./RouterBase"));
class ChainRouter extends RouterBase_1.default {
    constructor(logger, service) {
        super(logger, 'chain');
        this.service = service;
        this.getRouter = () => {
            const router = (0, express_1.Router)();
            /**
             * @openapi
             * tags:
             *   name: Chain
             *   description: Onchain data related endpoints
             */
            /**
             * @openapi
             * /chain/fees:
             *   get:
             *     description: Fee estimations for all supported chains
             *     tags: [Chain]
             *     responses:
             *       '200':
             *         description: Object of currency of chain -> fee estimation
             *         content:
             *           application/json:
             *             schema:
             *               type: object
             *               additionalProperties:
             *                 type: number
             *                 description: Fee estimation in sat/vbyte or GWEI
             *             examples:
             *               json:
             *                 value: '{"BTC":19,"L-BTC":0.11}'
             */
            router.get('/fees', this.handleError(this.getFees));
            /**
             * @openapi
             * /chain/heights:
             *   get:
             *     description: Block heights for all supported chains
             *     tags: [Chain]
             *     responses:
             *       '200':
             *         description: Object of currency of chain -> block height
             *         content:
             *           application/json:
             *             schema:
             *               type: object
             *               additionalProperties:
             *                 type: number
             *                 description: Block height of the chain
             *             examples:
             *               json:
             *                 value: '{"BTC":830311,"L-BTC":2725579}'
             */
            router.get('/heights', this.handleError(this.getHeights));
            /**
             * @openapi
             * components:
             *   schemas:
             *     Contracts:
             *       type: object
             *       properties:
             *         network:
             *           type: object
             *           description: Information about the network
             *           properties:
             *             chainId:
             *               type: number
             *               required: true
             *               description: ID of the chain
             *             name:
             *               type: string
             *               description: Name of the chain if applicable
             *         swapContracts:
             *           type: object
             *           required: true
             *           description: Mapping of the names of swap contracts to their address
             *           properties:
             *             EtherSwap:
             *               type: string
             *               description: Address of the EtherSwap contract
             *             ERC20Swap:
             *               type: string
             *               description: Address of the ERC20 contract
             *         tokens:
             *           type: object
             *           required: true
             *           description: Mapping of the symbol of tokens to their address
             *           additionalProperties:
             *             type: string
             *             description: Address of the token
             */
            /**
             * @openapi
             * /chain/contracts:
             *   get:
             *     tags: [Chain]
             *     description: Get the network information and contract addresses for all supported EVM chains
             *     responses:
             *       '200':
             *         description: Network details and contract addresses on the chain
             *         content:
             *           application/json:
             *             schema:
             *               type: object
             *               additionalProperties:
             *                 $ref: '#/components/schemas/Contracts'
             *             examples:
             *               json:
             *                 value: '{"rsk":{"network":{"chainId":31337},"tokens":{"USDT":"0x9fE46736679d2D9a65F0992F2272dE9f3c7fa6e0"},"swapContracts":{"EtherSwap":"0x5FbDB2315678afecb367f032d93F642f64180aa3","ERC20Swap":"0xe7f1725E7734CE288F8367e1Bb143E90bb3F0512"}}}'
             *       '400':
             *         description: Error that caused the request to fail
             *         content:
             *           application/json:
             *             schema:
             *               $ref: '#/components/schemas/ErrorResponse'
             */
            router.get('/contracts', this.handleError(this.getContracts));
            /**
             * @openapi
             * /chain/{currency}/fee:
             *   get:
             *     description: Fee estimations for a chain
             *     tags: [Chain]
             *     parameters:
             *       - in: path
             *         name: currency
             *         required: true
             *         schema:
             *           type: string
             *         description: Currency of the chain to get a fee estimation for
             *     responses:
             *       '200':
             *         description: Object containing the fee estimation
             *         content:
             *           application/json:
             *             schema:
             *               type: object
             *               properties:
             *                 fee:
             *                   type: number
             *                   required: true
             *                   description: Fee estimation in sat/vbyte or GWEI
             *       '400':
             *         description: Error that caused the request to fail
             *         content:
             *           application/json:
             *             schema:
             *               $ref: '#/components/schemas/ErrorResponse'
             */
            router.get('/:currency/fee', this.handleError(this.getFeeForChain));
            /**
             * @openapi
             * /chain/{currency}/height:
             *   get:
             *     description: Block height for a chain
             *     tags: [Chain]
             *     parameters:
             *       - in: path
             *         name: currency
             *         required: true
             *         schema:
             *           type: string
             *         description: Currency of the chain to get the block height for
             *     responses:
             *       '200':
             *         description: Object containing the block height
             *         content:
             *           application/json:
             *             schema:
             *               type: object
             *               properties:
             *                 height:
             *                   type: number
             *                   required: true
             *                   description: Block height of the chain
             *       '400':
             *         description: Error that caused the request to fail
             *         content:
             *           application/json:
             *             schema:
             *               $ref: '#/components/schemas/ErrorResponse'
             */
            router.get('/:currency/height', this.handleError(this.getHeightForChain));
            /**
             * @openapi
             * /chain/{currency}/transaction/{id}:
             *   get:
             *     tags: [Chain]
             *     description: Fetch a raw transaction by its id
             *     parameters:
             *       - in: path
             *         name: currency
             *         required: true
             *         schema:
             *           type: string
             *         description: Currency of the chain to query for
             *       - in: path
             *         name: id
             *         required: true
             *         schema:
             *           type: string
             *         description: Id of the transaction to query
             *     responses:
             *       '200':
             *         description: Raw transaction
             *         content:
             *           application/json:
             *             schema:
             *               type: object
             *               properties:
             *                 hex:
             *                   type: string
             *                   required: true
             *                   description: The transaction encoded as HEX
             *       '400':
             *         description: Error that caused the query for the transaction to fail
             *         content:
             *           application/json:
             *             schema:
             *               $ref: '#/components/schemas/ErrorResponse'
             */
            router.get('/:currency/transaction/:id', this.handleError(this.getTransaction));
            /**
             * @openapi
             * /chain/{currency}/transaction:
             *   post:
             *     tags: [Chain]
             *     description: Broadcast a transaction
             *     parameters:
             *       - in: path
             *         name: currency
             *         required: true
             *         schema:
             *           type: string
             *         description: Currency of the chain to broadcast on
             *     requestBody:
             *       required: true
             *       content:
             *         application/json:
             *           schema:
             *             type: object
             *             properties:
             *               hex:
             *                 type: string
             *                 required: true
             *                 description: The transaction to broadcast as raw HEX
             *     responses:
             *       '201':
             *         description: ID of the broadcast transaction
             *         content:
             *           application/json:
             *             schema:
             *               type: object
             *               properties:
             *                 id:
             *                   type: string
             *                   required: true
             *                   description: ID of the broadcast transaction
             *       '400':
             *         description: Error that caused the broadcast of the transaction to fail
             *         content:
             *           application/json:
             *             schema:
             *               $ref: '#/components/schemas/ErrorResponse'
             */
            router.post('/:currency/transaction', this.handleError(this.postTransaction));
            /**
             * @openapi
             * /chain/{currency}/contracts:
             *   get:
             *     tags: [Chain]
             *     description: Get the network information and contract addresses for a supported EVM chains
             *     parameters:
             *       - in: path
             *         name: currency
             *         required: true
             *         schema:
             *           type: string
             *         description: Currency of the chain to query for
             *     responses:
             *       '200':
             *         description: Raw transaction
             *         content:
             *           application/json:
             *             schema:
             *               $ref: '#/components/schemas/Contracts'
             *             examples:
             *               json:
             *                 value: '{"network":{"chainId":31337},"tokens":{"USDT":"0x9fE46736679d2D9a65F0992F2272dE9f3c7fa6e0"},"swapContracts":{"EtherSwap":"0x5FbDB2315678afecb367f032d93F642f64180aa3","ERC20Swap":"0xe7f1725E7734CE288F8367e1Bb143E90bb3F0512"}}'
             *       '400':
             *         description: Error that caused the query for the transaction to fail
             *         content:
             *           application/json:
             *             schema:
             *               $ref: '#/components/schemas/ErrorResponse'
             *       '404':
             *         description: Error when the currency is not on an EVM chain
             *         content:
             *           application/json:
             *             schema:
             *               $ref: '#/components/schemas/ErrorResponse'
             */
            router.get('/:currency/contracts', this.handleError(this.getContractsForCurrency));
            return router;
        };
        this.getFees = async (_, res) => (0, Utils_2.successResponse)(res, (0, Utils_1.mapToObject)(await this.service.getFeeEstimation()));
        this.getHeights = async (_, res) => (0, Utils_2.successResponse)(res, (0, Utils_1.mapToObject)(await this.service.getBlockHeights()));
        this.getContracts = async (_, res) => {
            const contracts = {};
            Object.entries(await this.service.getContracts()).forEach(([key, value]) => {
                contracts[key] = this.formatContracts(value);
            });
            (0, Utils_2.successResponse)(res, contracts);
        };
        this.getFeeForChain = async (req, res) => {
            const currency = this.getCurrencyFromPath(req);
            (0, Utils_2.successResponse)(res, {
                fee: (await this.service.getFeeEstimation(currency)).get(currency),
            });
        };
        this.getHeightForChain = async (req, res) => {
            const currency = this.getCurrencyFromPath(req);
            (0, Utils_2.successResponse)(res, {
                height: (await this.service.getBlockHeights(currency)).get(currency),
            });
        };
        this.getTransaction = async (req, res) => {
            const currency = this.getCurrencyFromPath(req);
            const { id } = (0, Utils_2.validateRequest)(req.params, [
                { name: 'id', type: 'string' },
            ]);
            const tx = await this.service.getTransaction(currency, id);
            (0, Utils_2.successResponse)(res, { hex: tx });
        };
        this.postTransaction = async (req, res) => {
            const currency = this.getCurrencyFromPath(req);
            const { hex } = (0, Utils_2.validateRequest)(req.body, [
                { name: 'hex', type: 'string' },
            ]);
            const id = await this.service.broadcastTransaction(currency, hex);
            (0, Utils_2.createdResponse)(res, { id });
        };
        this.getContractsForCurrency = async (req, res) => {
            const currency = this.getCurrencyFromPath(req);
            const manager = this.service.walletManager.ethereumManagers.find((manager) => manager.hasSymbol(currency));
            if (manager === undefined) {
                (0, Utils_2.errorResponse)(this.logger, req, res, 'chain does not have contracts', 404);
                return;
            }
            (0, Utils_2.successResponse)(res, this.formatContracts(await manager.getContractDetails()));
        };
        this.getCurrencyFromPath = (req) => (0, Utils_2.validateRequest)(req.params, [{ name: 'currency', type: 'string' }])
            .currency;
        this.formatContracts = (contracts) => ({
            network: contracts.network,
            tokens: (0, Utils_1.mapToObject)(contracts.tokens),
            swapContracts: (0, Utils_1.mapToObject)(contracts.swapContracts),
        });
    }
}
exports.default = ChainRouter;
//# sourceMappingURL=ChainRouter.js.map