"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = require("express");
const Utils_1 = require("../../Utils");
const RouterBase_1 = __importDefault(require("./RouterBase"));
class NodesRouter extends RouterBase_1.default {
    constructor(logger, service) {
        super(logger, 'nodes');
        this.service = service;
        this.getRouter = () => {
            /**
             * @openapi
             * tags:
             *   name: Nodes
             *   description: Lightning nodes
             */
            const router = (0, express_1.Router)();
            /**
             * @openapi
             * components:
             *   schemas:
             *     NodeInfo:
             *       type: object
             *       properties:
             *         publicKey:
             *           type: string
             *           required: true
             *         uris:
             *           type: array
             *           required: true
             *           items:
             *             type: string
             */
            /**
             * @openapi
             * /nodes:
             *   get:
             *     description: Information about the Lightning nodes the backend is connected to
             *     tags: [Nodes]
             *     responses:
             *       '200':
             *         description: Lightning node information
             *         content:
             *           application/json:
             *             schema:
             *               type: object
             *               additionalProperties:
             *                 type: object
             *                 additionalProperties:
             *                   $ref: '#/components/schemas/NodeInfo'
             *             examples:
             *               json:
             *                 value: '{"BTC":{"LND":{"publicKey":"026165850492521f4ac8abd9bd8088123446d126f648ca35e60f88177dc149ceb2","uris":["026165850492521f4ac8abd9bd8088123446d126f648ca35e60f88177dc149ceb2@45.86.229.190:9735","026165850492521f4ac8abd9bd8088123446d126f648ca35e60f88177dc149ceb2@d7kak4gpnbamm3b4ufq54aatgm3alhx3jwmu6kyy2bgjaauinkipz3id.onion:9735"]},"CLN":{"publicKey":"02d96eadea3d780104449aca5c93461ce67c1564e2e1d73225fa67dd3b997a6018","uris":["02d96eadea3d780104449aca5c93461ce67c1564e2e1d73225fa67dd3b997a6018@45.86.229.190:9736","02d96eadea3d780104449aca5c93461ce67c1564e2e1d73225fa67dd3b997a6018@2a10:1fc0:3::270:a9dc:9736","02d96eadea3d780104449aca5c93461ce67c1564e2e1d73225fa67dd3b997a6018@oo5tkbbpgnqjopdjxepyfavx3yemtylgzul67s7zzzxfeeqpde6yr7yd.onion:9736"]}}}'
             */
            router.get('/', this.handleError(this.getNodes));
            /**
             * @openapi
             * components:
             *   schemas:
             *     NodeStats:
             *       type: object
             *       properties:
             *         capacity:
             *           type: integer
             *           required: true
             *         channels:
             *           type: integer
             *           required: true
             *         peers:
             *           type: integer
             *           required: true
             *         oldestChannel:
             *           type: integer
             *           required: true
             *           description: UNIX timestamp of the block in which the funding transaction of the oldest channel was included
             */
            /**
             * @openapi
             * /nodes/stats:
             *   get:
             *     description: Statistics about the Lightning nodes the backend is connected to
             *     tags: [Nodes]
             *     responses:
             *       '200':
             *         description: Statistics about Lightning nodes
             *         content:
             *           application/json:
             *             schema:
             *               type: object
             *               additionalProperties:
             *                 type: object
             *                 properties:
             *                   total:
             *                     $ref: '#/components/schemas/NodeStats'
             *                 additionalProperties:
             *                   $ref: '#/components/schemas/NodeStats'
             *             examples:
             *               json:
             *                 value: '{"BTC":{"LND":{"capacity":5806395450,"channels":1052,"peers":2199,"oldestChannel":1553900632},"CLN":{"capacity":1062157049,"channels":79,"peers":124,"oldestChannel":1692910360},"total":{"capacity":6868552499,"channels":1131,"peers":2323,"oldestChannel":1553900632}}}'
             */
            router.get('/stats', this.handleError(this.getNodeStats));
            router.get('/:currency/:node/hints', this.handleError(this.getRoutingHints));
            return router;
        };
        this.getNodes = (_req, res) => {
            const result = Object.fromEntries(Array.from(this.service.getNodes().entries()).map(([symbol, nodeInfo]) => {
                return [
                    symbol,
                    Object.fromEntries(Array.from(nodeInfo.entries()).map(([name, info]) => {
                        return [
                            name,
                            {
                                publicKey: info.nodeKey,
                                uris: info.uris,
                            },
                        ];
                    })),
                ];
            }));
            (0, Utils_1.successResponse)(res, result);
        };
        this.getNodeStats = (_req, res) => {
            const stats = Object.fromEntries(Array.from(this.service.getNodeStats().entries()).map(([symbol, stats]) => [symbol, Object.fromEntries(stats.entries())]));
            (0, Utils_1.successResponse)(res, stats);
        };
        this.getRoutingHints = async (req, res) => {
            const { currency, node } = (0, Utils_1.validateRequest)(req.params, [
                { name: 'currency', type: 'string' },
                { name: 'node', type: 'string' },
            ]);
            (0, Utils_1.successResponse)(res, await this.service.swapManager.routingHints.getRoutingHints(currency, node));
        };
    }
}
exports.default = NodesRouter;
//# sourceMappingURL=NodesRouter.js.map