"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const node_schedule_1 = require("node-schedule");
const Utils_1 = require("../Utils");
const Database_1 = __importStar(require("../db/Database"));
const Errors_1 = __importDefault(require("./Errors"));
const GoogleCloud_1 = __importDefault(require("./providers/GoogleCloud"));
const Webdav_1 = __importDefault(require("./providers/Webdav"));
class BackupScheduler {
    constructor(logger, dbpath, postgresConfig, config, eventHandler) {
        this.logger = logger;
        this.dbpath = dbpath;
        this.postgresConfig = postgresConfig;
        this.config = config;
        this.eventHandler = eventHandler;
        this.providers = [];
        this.init = async () => {
            for (const provider of this.providers) {
                if (provider instanceof Webdav_1.default) {
                    await provider.init();
                }
            }
        };
        this.uploadDatabase = async (date) => {
            if (this.providers.length === 0) {
                throw Errors_1.default.BACKUP_DISABLED();
            }
            const dateString = BackupScheduler.getDate(date);
            this.logger.silly(`Backing up ${Database_1.default.type} database at: ${dateString}`);
            const backupPath = `backend/database-${dateString}.${Database_1.default.type === Database_1.DatabaseType.SQLite ? 'db' : 'sql.gz'}`;
            if (Database_1.default.type === Database_1.DatabaseType.SQLite) {
                await this.uploadFile(backupPath, this.dbpath);
            }
            else {
                const tempFilePath = `sql-backup-${Date.now().toString()}.temp`;
                return new Promise((resolve, reject) => {
                    const backupChild = (0, child_process_1.exec)(`PGPASSWORD="${this.postgresConfig.password}" pg_dump -U ${this.postgresConfig.username} -h ${this.postgresConfig.host} -p ${this.postgresConfig.port} -d ${this.postgresConfig.database} | gzip > ${tempFilePath}`);
                    backupChild.on('exit', (code) => {
                        if (code !== 0) {
                            reject(`creating ${Database_1.DatabaseType.PostgreSQL} backup failed with code: ${code}`);
                            return;
                        }
                        this.uploadFile(backupPath, tempFilePath)
                            .then(() => {
                            (0, fs_1.unlinkSync)(tempFilePath);
                            resolve();
                        })
                            .catch(reject);
                    });
                });
            }
        };
        this.uploadFile = async (path, file) => {
            try {
                await Promise.all(this.providers.map((provider) => provider.uploadFile(path, file)));
                this.logger.silly(`Uploaded file ${file} to: ${path}`);
            }
            catch (error) {
                this.logger.warn(`Could not upload file ${path}: ${error}`);
            }
        };
        this.uploadString = async (path, data) => {
            try {
                await Promise.all(this.providers.map((provider) => provider.uploadString(path, data)));
                this.logger.silly(`Uploaded data into file: ${path}`);
            }
            catch (error) {
                this.logger.warn(`Could not upload data to file ${path}: ${(0, Utils_1.formatError)(error)}`);
            }
        };
        this.subscribeChannelBackups = () => {
            this.eventHandler.on('channel.backup', async ({ currency, channelBackup }) => {
                const dateString = BackupScheduler.getDate(new Date());
                await this.uploadString(`lnd/${currency}/multiChannelBackup-${dateString}`, channelBackup);
            });
        };
        this.logProviderEnabled = (name) => {
            this.logger.verbose(`Enabled ${name} backup provider`);
        };
        try {
            if (config.gcloud && GoogleCloud_1.default.configValid(config.gcloud)) {
                this.providers.push(new GoogleCloud_1.default(config.gcloud));
                this.logProviderEnabled('Google Cloud Storage');
            }
            if (config.webdav && Webdav_1.default.configValid(config.webdav)) {
                this.providers.push(new Webdav_1.default(config.webdav));
                this.logProviderEnabled('WebDav');
            }
            if (this.providers.length === 0) {
                this.logger.warn('Disabled backups because no provider was specified');
                return;
            }
            this.subscribeChannelBackups();
            this.logger.info('Started channel backup subscription');
            this.logger.verbose(`Scheduling database backups: ${this.config.interval}`);
            (0, node_schedule_1.scheduleJob)(this.config.interval, async (date) => {
                await this.uploadDatabase(date);
            });
        }
        catch (error) {
            this.logger.warn(`Could not start backup scheduler: ${error}`);
        }
    }
}
BackupScheduler.getDate = (date) => {
    let str = '';
    for (const elem of [
        date.getFullYear(),
        date.getMonth(),
        date.getDate(),
        '-',
        date.getHours(),
        date.getMinutes(),
    ]) {
        str +=
            typeof elem === 'number' ? BackupScheduler.addLeadingZeros(elem) : elem;
    }
    return str;
};
/**
 * Adds a leading 0 to the provided number if it is smaller than 10
 */
BackupScheduler.addLeadingZeros = (number) => {
    return `${number}`.padStart(2, '0');
};
exports.default = BackupScheduler;
//# sourceMappingURL=BackupScheduler.js.map