/// <reference types="node" />
import { Transaction } from 'bitcoinjs-lib';
import BaseClient from '../BaseClient';
import { ChainConfig } from '../Config';
import Logger from '../Logger';
import { ClientStatus, CurrencyType } from '../consts/Enums';
import TypedEventEmitter from '../consts/TypedEventEmitter';
import { Block, BlockVerbose, BlockchainInfo, NetworkInfo, RawTransaction, UnspentUtxo, WalletInfo } from '../consts/Types';
import RpcClient from './RpcClient';
import ZmqClient, { SomeTransaction } from './ZmqClient';
declare enum AddressType {
    Legacy = "legacy",
    P2shegwit = "p2sh-segwit",
    Bech32 = "bech32",
    Taproot = "bech32m"
}
type BlockChainInfoScanned = BlockchainInfo & {
    scannedBlocks: number;
};
type ChainClientEvents<T extends SomeTransaction> = {
    'status.changed': ClientStatus;
    block: number;
    transaction: {
        transaction: T;
        confirmed: boolean;
    };
};
interface IChainClient<T extends SomeTransaction = SomeTransaction> extends TypedEventEmitter<ChainClientEvents<T>> {
    get symbol(): string;
    get currencyType(): CurrencyType;
    connect(): Promise<void>;
    rescanChain(startHeight: number): Promise<void>;
    addInputFilter(inputHash: Buffer): void;
    addOutputFilter(outputScript: Buffer): void;
    removeOutputFilter(outputScript: Buffer): void;
    removeInputFilter(inputHash: Buffer): void;
    getBlockchainInfo(): Promise<BlockChainInfoScanned>;
    getNetworkInfo(): Promise<NetworkInfo>;
    sendRawTransaction(transactionHex: string, isSwapRelated?: boolean): Promise<string>;
    getRawTransaction(transactionId: string): Promise<string>;
    getRawTransactionVerbose(transactionId: string): Promise<RawTransaction>;
    estimateFee(confTarget?: number): Promise<number>;
    listUnspent(minimalConfirmations?: number): Promise<UnspentUtxo[]>;
    getNewAddress(type: AddressType): Promise<string>;
    sendToAddress(address: string, amount: number, satPerVbyte?: number, subtractFeeFromAmount?: boolean): Promise<string>;
}
declare class ChainClient<T extends SomeTransaction = Transaction> extends BaseClient<ChainClientEvents<T>> implements IChainClient<T> {
    private readonly config;
    static readonly serviceName = "Core";
    static readonly decimals = 100000000;
    currencyType: CurrencyType;
    protected client: RpcClient;
    protected zmqClient: ZmqClient<T>;
    protected feeFloor: number;
    private readonly mempoolSpace?;
    constructor(logger: Logger, config: ChainConfig, symbol: string);
    serviceName: () => string;
    connect: () => Promise<void>;
    disconnect: () => void;
    getNetworkInfo: () => Promise<NetworkInfo>;
    getBlockchainInfo: () => Promise<{
        scannedBlocks: number;
        chain: string;
        blocks: number;
        headers: number;
        bestblockhash: string;
        difficulty: number;
        mediantime: number;
        verificationprogress: number;
        initialblockdownload: string;
        chainwork: string;
        size_on_disk: number;
        pruned: boolean;
    }>;
    getBlock: (hash: string) => Promise<Block>;
    getBlockVerbose: (hash: string) => Promise<BlockVerbose>;
    getBlockhash: (height: number) => Promise<string>;
    /**
     * Add an output to the list of relevant ones
     */
    addOutputFilter: (outputScript: Buffer) => void;
    /**
     * Removes an output from the list of relevant ones
     */
    removeOutputFilter: (outputScript: Buffer) => void;
    /**
     * Adds an input hash to the list of relevant ones
     */
    addInputFilter: (inputHash: Buffer) => void;
    /**
     * Removes an input hash from the list of relevant ones
     */
    removeInputFilter: (inputHash: Buffer) => void;
    rescanChain: (startHeight: number) => Promise<void>;
    sendRawTransaction: (rawTransaction: string) => Promise<string>;
    getRawTransaction: (id: string) => Promise<string>;
    getRawTransactionVerbose: (id: string) => Promise<RawTransaction>;
    getRawMempool: () => Promise<string[]>;
    estimateFee: (confTarget?: number) => Promise<number>;
    getWalletInfo: () => Promise<WalletInfo>;
    sendToAddress: (address: string, amount: number, satPerVbyte?: number, subtractFeeFromAmount?: boolean) => Promise<string>;
    listUnspent: (minimalConfirmations?: number) => Promise<UnspentUtxo[]>;
    generate: (blocks: number) => Promise<string[]>;
    getNewAddress: (type?: AddressType) => Promise<string>;
    protected estimateFeeWithFloor: (confTarget: number) => Promise<number>;
    private estimateFeeChainClient;
    private getZmqNotifications;
    private listenToZmq;
    private formatGetTransactionError;
}
export default ChainClient;
export { AddressType, BlockChainInfoScanned, ChainClientEvents, IChainClient };
