"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const BaseClient_1 = __importDefault(require("../BaseClient"));
const PromiseUtils_1 = require("../PromiseUtils");
const Enums_1 = require("../consts/Enums");
const ElementsClient_1 = __importDefault(require("./ElementsClient"));
class ElementsWrapper extends BaseClient_1.default {
    constructor(logger, config) {
        super(logger, ElementsClient_1.default.symbol);
        this.currencyType = Enums_1.CurrencyType.Liquid;
        this.clients = [];
        this.serviceName = () => 'ElementsWrapper';
        this.connect = async () => {
            await Promise.all(this.clients.map((c) => c.connect()));
            this.publicClient().on('status.changed', this.setClientStatus);
            this.publicClient().on('block', (blockHeight) => this.emit('block', blockHeight));
            // If we have a lowball client, bubble up only confirmed transactions
            // of the public client
            const hasLowball = this.lowballClient() !== undefined;
            this.publicClient().on('transaction', ({ transaction, confirmed }) => {
                if (hasLowball && !confirmed) {
                    return;
                }
                this.emit('transaction', { transaction, confirmed });
            });
            this.lowballClient()?.on('transaction', ({ transaction, confirmed }) => {
                if (confirmed) {
                    return;
                }
                this.emit('transaction', { transaction, confirmed });
            });
        };
        this.disconnect = () => this.clients.forEach((c) => c.disconnect());
        this.rescanChain = (startHeight) => 
        // Only rescan with the public client to avoid duplicate events
        this.publicClient().rescanChain(startHeight);
        this.addInputFilter = (inputHash) => this.clients.forEach((c) => c.addInputFilter(inputHash));
        this.addOutputFilter = (outputScript) => this.clients.forEach((c) => c.addOutputFilter(outputScript));
        this.removeOutputFilter = (outputScript) => this.clients.forEach((c) => c.removeOutputFilter(outputScript));
        this.removeInputFilter = (inputHash) => this.clients.forEach((c) => c.removeInputFilter(inputHash));
        this.getBlockchainInfo = () => this.annotateLowballInfo((c) => c.getBlockchainInfo());
        this.getNetworkInfo = () => this.annotateLowballInfo((c) => c.getNetworkInfo());
        this.sendRawTransaction = (transactionHex, isSwapRelated = true) => {
            if (isSwapRelated) {
                return (0, PromiseUtils_1.allSettledFirst)(this.clients.map((c) => c.sendRawTransaction(transactionHex)));
            }
            return this.publicClient().sendRawTransaction(transactionHex);
        };
        this.getRawTransaction = (transactionId) => (0, PromiseUtils_1.allSettledFirst)(this.clients.map((c) => c.getRawTransaction(transactionId)));
        this.getRawTransactionVerbose = (transactionId) => (0, PromiseUtils_1.allSettledFirst)(this.clients.map((c) => c.getRawTransactionVerbose(transactionId)));
        this.estimateFee = () => this.walletClient().estimateFee();
        this.listUnspent = (minimalConfirmations) => this.walletClient().listUnspent(minimalConfirmations);
        this.getAddressInfo = (address) => this.walletClient().getAddressInfo(address);
        this.getBalances = () => this.walletClient().getBalances();
        this.getNewAddress = (type) => this.walletClient().getNewAddress(type);
        this.dumpBlindingKey = (address) => this.walletClient().dumpBlindingKey(address);
        this.sendToAddress = (address, amount, satPerVbyte, subtractFeeFromAmount) => this.walletClient().sendToAddress(address, amount, satPerVbyte, subtractFeeFromAmount);
        this.walletClient = () => this.lowballClient() || this.publicClient();
        this.publicClient = () => this.clients.find((c) => !c.isLowball);
        // TODO: detect when lowball falls out of sync
        this.lowballClient = () => this.clients.find((c) => c.isLowball);
        this.annotateLowballInfo = async (fn) => {
            const infos = await Promise.all(this.clients.map(async (c) => [c.isLowball, await fn(c)]));
            return {
                ...infos.find(([isLowball]) => !isLowball)[1],
                lowball: infos.find(([isLowball]) => isLowball)?.[1],
            };
        };
        this.clients.push(new ElementsClient_1.default(this.logger, config, false));
        if (config.lowball !== undefined) {
            this.logger.info(`Using lowball for ${this.clients[0].serviceName()}`);
            this.clients.push(new ElementsClient_1.default(this.logger, config.lowball, true));
        }
    }
}
exports.default = ElementsWrapper;
//# sourceMappingURL=ElementsWrapper.js.map