"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MempoolSpaceClient = void 0;
const axios_1 = __importDefault(require("axios"));
const Utils_1 = require("../Utils");
class MempoolSpaceClient {
    constructor(logger, symbol, apiUrl) {
        this.logger = logger;
        this.symbol = symbol;
        this.apiUrl = apiUrl;
        this.init = async () => {
            await this.fetchRecommendedFees();
            this.fetchInterval = setInterval(async () => {
                await this.fetchRecommendedFees();
            }, MempoolSpaceClient.fetchInterval);
        };
        this.stop = () => {
            if (this.fetchInterval) {
                clearInterval(this.fetchInterval);
                this.fetchInterval = undefined;
            }
            this.latestFee = undefined;
        };
        this.fetchRecommendedFees = async () => {
            try {
                const response = await axios_1.default.get(`${this.apiUrl}/v1/fees/recommended`);
                if (typeof response.data.fastestFee !== 'number') {
                    this.handleCouldNotFetch('invalid response');
                    return;
                }
                this.logger.silly(`Fetched latest ${this.symbol} MempoolSpace fee estimations: ${(0, Utils_1.stringify)(response.data)}`);
                this.latestFee = Math.ceil(response.data.fastestFee * MempoolSpaceClient.factor);
            }
            catch (error) {
                this.handleCouldNotFetch(error);
            }
        };
        this.handleCouldNotFetch = (error) => {
            this.latestFee = undefined;
            this.logger.warn(`Could not fetch ${this.symbol} MempoolSpace fee estimations from ${this.apiUrl}: ${(0, Utils_1.formatError)(error)}`);
        };
    }
}
exports.MempoolSpaceClient = MempoolSpaceClient;
MempoolSpaceClient.factor = 1.1;
MempoolSpaceClient.fetchInterval = 2.5 * 60 * 1000;
class MempoolSpace {
    constructor(logger, symbol, apiUrls) {
        this.logger = logger;
        this.symbol = symbol;
        this.apis = [];
        this.init = async () => {
            await Promise.all(this.apis.map((api) => api.init()));
        };
        this.latestFee = () => {
            const fees = this.apis
                .map((c) => c.latestFee)
                .filter((val) => val !== undefined);
            if (fees.length === 0) {
                this.logger.warn(`All ${this.symbol} MempoolSpace endpoints failed`);
                return undefined;
            }
            return Math.max(...fees);
        };
        this.stop = () => {
            this.apis.forEach((api) => api.stop());
        };
        const apis = apiUrls.split(',').map((apiUrl) => apiUrl.trim());
        this.logger.info(`Enabling MempoolSpace fee estimations for ${this.symbol}: ${(0, Utils_1.stringify)(apis)}`);
        apis.forEach((api) => this.apis.push(new MempoolSpaceClient(this.logger, this.symbol, api)));
    }
}
exports.default = MempoolSpace;
//# sourceMappingURL=MempoolSpace.js.map