"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.filters = void 0;
const async_lock_1 = __importDefault(require("async-lock"));
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const zeromq_1 = __importDefault(require("zeromq"));
const Core_1 = require("../Core");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const TypedEventEmitter_1 = __importDefault(require("../consts/TypedEventEmitter"));
const Errors_1 = __importDefault(require("./Errors"));
const filters = {
    rawTx: 'pubrawtx',
    rawBlock: 'pubrawblock',
    hashBlock: 'pubhashblock',
};
exports.filters = filters;
class ZmqClient extends TypedEventEmitter_1.default {
    constructor(symbol, logger, chainClient) {
        super();
        this.symbol = symbol;
        this.logger = logger;
        this.chainClient = chainClient;
        // IDs of transactions that contain a UTXOs of Boltz
        this.utxos = new Set();
        this.relevantInputs = new Set();
        this.relevantOutputs = new Set();
        this.blockHeight = 0;
        this.bestBlockHash = '';
        this.sockets = [];
        this.compatibilityRescan = false;
        // Because the event handlers that process the blocks are doing work asynchronously
        // one has to use a lock to ensure the events get handled sequentially
        this.blockHandleLock = new async_lock_1.default();
        this.init = async (currencyType, notifications) => {
            this.currencyType = currencyType;
            const activeFilters = {};
            const { blocks, bestblockhash } = await this.chainClient.getBlockchainInfo();
            this.blockHeight = blocks;
            this.bestBlockHash = bestblockhash;
            for (const notification of notifications) {
                switch (notification.type) {
                    case filters.rawTx:
                        activeFilters.rawtx = true;
                        await this.initRawTransaction(notification.address);
                        break;
                    case filters.rawBlock:
                        activeFilters.rawBlock = true;
                        this.rawBlockAddress = notification.address;
                        break;
                    case filters.hashBlock:
                        activeFilters.hashBlock = true;
                        this.hashBlockAddress = notification.address;
                        break;
                }
            }
            if (!activeFilters.rawtx) {
                throw Errors_1.default.NO_RAWTX();
            }
            const logCouldNotSubscribe = (filter) => {
                this.logger.warn(`Could not find ${this.symbol} chain ZMQ filter: ${filter}`);
            };
            if (activeFilters.rawBlock) {
                await this.initRawBlock();
            }
            else {
                logCouldNotSubscribe(filters.rawBlock);
                if (!activeFilters.hashBlock) {
                    logCouldNotSubscribe(filters.hashBlock);
                    throw Errors_1.default.NO_BLOCK_NOTIFICATIONS();
                }
                else {
                    this.logger.warn(`Falling back to ${this.symbol} ${filters.hashBlock} ZMQ filter`);
                    await this.initHashBlock();
                }
            }
        };
        this.close = () => {
            this.sockets.forEach((socket) => {
                // Catch errors that are thrown if the socket is closed already
                try {
                    socket.close();
                }
                catch (error) {
                    this.logger.debug(`${this.symbol} socket already closed: ${(0, Utils_1.formatError)(error)}`);
                }
            });
        };
        this.rescanChain = async (startHeight) => {
            const checkTransaction = (transaction) => {
                if (this.isRelevantTransaction(transaction)) {
                    this.emit('transaction', { transaction, confirmed: true });
                }
            };
            try {
                for (let i = 0; startHeight + i <= this.blockHeight; i += 1) {
                    const hash = await this.chainClient.getBlockhash(startHeight + i);
                    if (!this.compatibilityRescan) {
                        const block = await this.chainClient.getBlockVerbose(hash);
                        for (const { hex } of block.tx) {
                            checkTransaction((0, Core_1.parseTransaction)(this.currencyType, hex));
                        }
                    }
                    else {
                        const block = await this.chainClient.getBlock(hash);
                        for (const tx of block.tx) {
                            const rawTransaction = await this.chainClient.getRawTransaction(tx);
                            checkTransaction((0, Core_1.parseTransaction)(this.currencyType, rawTransaction));
                        }
                    }
                }
            }
            catch (error) {
                if (!this.compatibilityRescan) {
                    this.logger.info(`Falling back to compatibility rescan for ${this.symbol} chain`);
                    this.compatibilityRescan = true;
                    await this.rescanChain(startHeight);
                }
                else {
                    throw error;
                }
            }
        };
        this.initRawTransaction = async (address) => {
            const socket = await this.createSocket(address, 'rawtx');
            socket.on('message', async (_, rawTransaction) => {
                const transaction = (0, Core_1.parseTransaction)(this.currencyType, rawTransaction);
                const id = transaction.getId();
                // If the client has already verified that the transaction is relevant for the wallet
                // when it got added to the mempool we can safely assume that it got included in a block
                // the second time the client receives the transaction
                if (this.utxos.has(id)) {
                    this.utxos.delete(id);
                    this.emit('transaction', { transaction, confirmed: true });
                    return;
                }
                if (this.isRelevantTransaction(transaction)) {
                    const transactionData = (await this.chainClient.getRawTransactionVerbose(id));
                    // Check whether the transaction got confirmed or added to the mempool
                    if (transactionData.confirmations) {
                        this.emit('transaction', { transaction, confirmed: true });
                    }
                    else {
                        this.utxos.add(id);
                        this.emit('transaction', { transaction, confirmed: false });
                    }
                }
            });
        };
        this.initRawBlock = async () => {
            // Elements raw block subscriptions are not supported
            if (this.currencyType === Enums_1.CurrencyType.Liquid ||
                this.rawBlockAddress === undefined) {
                return this.initHashBlock();
            }
            const socket = await this.createSocket(this.rawBlockAddress, 'rawblock');
            socket.on('disconnect', () => {
                socket.disconnect(this.rawBlockAddress);
                this.logger.warn(`${this.symbol} ${filters.rawBlock} ZMQ filter disconnected. Falling back to ${filters.hashBlock}`);
                this.initHashBlock();
            });
            socket.on('message', async (_, rawBlock) => {
                const previousBlockHash = (0, Utils_1.getHexString)((0, Utils_1.reverseBuffer)(rawBlock.slice(4, 36)));
                // To get the hash of a block one has to get the header (first 80 bytes),
                // hash it twice with SHA256 and reverse the resulting Buffer
                const hash = (0, Utils_1.getHexString)((0, Utils_1.reverseBuffer)(bitcoinjs_lib_1.crypto.sha256(bitcoinjs_lib_1.crypto.sha256(rawBlock.slice(0, 80)))));
                this.blockHandleLock.acquire(filters.rawBlock, async () => {
                    if (previousBlockHash === this.bestBlockHash) {
                        this.blockHeight += 1;
                        this.bestBlockHash = hash;
                        this.newChainTip();
                    }
                    else {
                        // If there are many blocks added to the chain at once, Bitcoin Core might
                        // take a few milliseconds to write all of them to the disk. Therefore,
                        // we just get the height of the previous block and increase it by 1
                        const previousBlock = await this.chainClient.getBlock(previousBlockHash);
                        const height = previousBlock.height + 1;
                        if (height > this.blockHeight) {
                            if (height > this.blockHeight + 1) {
                                for (let i = 1; height > this.blockHeight + i; i += 1) {
                                    this.emit('block', this.blockHeight + i);
                                }
                            }
                            this.blockHeight = height;
                            this.bestBlockHash = hash;
                            this.logReorganize();
                            this.newChainTip();
                        }
                        else {
                            this.logOrphanBlock(hash);
                        }
                    }
                }, () => { });
            });
        };
        this.initHashBlock = async () => {
            if (!this.hashBlockAddress) {
                throw Errors_1.default.NO_BLOCK_FALLBACK();
            }
            const lockKey = filters.hashBlock;
            const socket = await this.createSocket(this.hashBlockAddress, 'hashblock');
            const handleBlock = async (blockHash) => {
                const block = await this.chainClient.getBlock(blockHash);
                if (block.previousblockhash === this.bestBlockHash) {
                    this.blockHeight = block.height;
                    this.bestBlockHash = block.hash;
                    this.newChainTip();
                }
                else {
                    if (block.height > this.blockHeight) {
                        for (let i = 1; block.height > this.blockHeight + i; i += 1) {
                            this.emit('block', this.blockHeight + i);
                        }
                        this.blockHeight = block.height;
                        this.bestBlockHash = block.hash;
                        this.logReorganize();
                        this.newChainTip();
                    }
                    else {
                        this.logOrphanBlock(block.hash);
                    }
                }
            };
            socket.on('message', (_, blockHash) => {
                const blockHashString = (0, Utils_1.getHexString)(blockHash);
                this.blockHandleLock.acquire(lockKey, async () => {
                    try {
                        await handleBlock(blockHashString);
                    }
                    catch (error) {
                        if (error.message === 'Block not found on disk') {
                            // If there are many blocks added to the chain at once, Bitcoin Core might
                            // take a few milliseconds to write all of them to the disk. Therefore
                            // it just retries getting the block after a little delay
                            setTimeout(async () => {
                                await handleBlock(blockHashString);
                            }, 250);
                        }
                        else {
                            this.logger.error(`${this.symbol} ${filters.hashBlock} ZMQ filter threw: ${JSON.stringify(error, undefined, 2)}`);
                        }
                    }
                }, () => { });
            });
        };
        this.isRelevantTransaction = (transaction) => {
            for (const input of transaction.ins) {
                if (this.relevantInputs.has((0, Utils_1.getHexString)(input.hash))) {
                    return true;
                }
            }
            for (const output of transaction.outs) {
                if (this.relevantOutputs.has((0, Utils_1.getHexString)(output.script))) {
                    return true;
                }
            }
            return false;
        };
        this.newChainTip = () => {
            this.logger.silly(`New ${this.symbol} chain tip #${this.blockHeight}: ${this.bestBlockHash}`);
            this.emit('block', this.blockHeight);
        };
        this.logReorganize = () => {
            this.logger.info(`Reorganized ${this.symbol} chain to #${this.blockHeight}: ${this.bestBlockHash}`);
        };
        this.logOrphanBlock = (hash) => {
            this.logger.verbose(`Found ${this.symbol} orphan block: ${hash}`);
        };
        this.createSocket = (address, filter) => {
            return new Promise((resolve, reject) => {
                const socket = zeromq_1.default.socket('sub').monitor();
                this.sockets.push(socket);
                const timeoutHandle = setTimeout(() => reject(Errors_1.default.ZMQ_CONNECTION_TIMEOUT(this.symbol, filter, address)), ZmqClient.connectTimeout);
                socket.on('connect', () => {
                    this.logger.debug(`Connected to ${this.symbol} ZMQ filter ${filter} on: ${address}`);
                    clearTimeout(timeoutHandle);
                    resolve(socket);
                });
                socket.connect(address);
                socket.subscribe(filter);
            });
        };
    }
}
ZmqClient.connectTimeout = 1000;
exports.default = ZmqClient;
//# sourceMappingURL=ZmqClient.js.map