"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DatabaseType = void 0;
const pg_1 = require("pg");
const sequelize_1 = __importDefault(require("sequelize"));
const Migration_1 = __importDefault(require("./Migration"));
const ChainTip_1 = __importDefault(require("./models/ChainTip"));
const ChannelCreation_1 = __importDefault(require("./models/ChannelCreation"));
const DatabaseVersion_1 = __importDefault(require("./models/DatabaseVersion"));
const KeyProvider_1 = __importDefault(require("./models/KeyProvider"));
const MarkedSwap_1 = __importDefault(require("./models/MarkedSwap"));
const Pair_1 = __importDefault(require("./models/Pair"));
const PendingEthereumTransaction_1 = __importDefault(require("./models/PendingEthereumTransaction"));
const PendingLockupTransaction_1 = __importDefault(require("./models/PendingLockupTransaction"));
const Referral_1 = __importDefault(require("./models/Referral"));
const ReverseRoutingHint_1 = __importDefault(require("./models/ReverseRoutingHint"));
const ReverseSwap_1 = __importDefault(require("./models/ReverseSwap"));
const Swap_1 = __importDefault(require("./models/Swap"));
// To make sure that PostgreSQL types are parsed correctly
pg_1.types.setTypeParser(pg_1.types.builtins.INT8, parseInt);
pg_1.types.setTypeParser(pg_1.types.builtins.NUMERIC, parseFloat);
pg_1.types.setTypeParser(pg_1.types.builtins.FLOAT8, parseFloat);
var DatabaseType;
(function (DatabaseType) {
    DatabaseType[DatabaseType["SQLite"] = 0] = "SQLite";
    DatabaseType[DatabaseType["PostgreSQL"] = 1] = "PostgreSQL";
})(DatabaseType || (exports.DatabaseType = DatabaseType = {}));
class Database {
    /**
     * @param logger logger that should be used
     * @param sqlitePath the file path to the SQLite database; if ':memory:' the database will be stored in the memory
     * @param postgresConfig configuration of connection to a PostgreSQL database; takes precedence over SQLite if set
     */
    constructor(logger, sqlitePath, postgresConfig) {
        this.logger = logger;
        this.sqlitePath = sqlitePath;
        this.init = async () => {
            try {
                await Database.sequelize.authenticate();
                switch (Database.type) {
                    case DatabaseType.PostgreSQL:
                        this.logger.info('Connected to PostgreSQL database');
                        break;
                    case DatabaseType.SQLite:
                        this.logger.info(`Connected to database: ${this.sqlitePath === Database.memoryDatabase
                            ? 'in memory'
                            : this.sqlitePath}`);
                        break;
                }
            }
            catch (error) {
                this.logger.error(`Could not connect to database: ${error}`);
                throw error;
            }
            await Promise.all([
                Pair_1.default.sync(),
                ChainTip_1.default.sync(),
                Referral_1.default.sync(),
                KeyProvider_1.default.sync(),
                DatabaseVersion_1.default.sync(),
                PendingEthereumTransaction_1.default.sync(),
            ]);
            await Promise.all([Swap_1.default.sync(), ReverseSwap_1.default.sync()]);
            await Promise.all([
                MarkedSwap_1.default.sync(),
                ChannelCreation_1.default.sync(),
                ReverseRoutingHint_1.default.sync(),
                PendingLockupTransaction_1.default.sync(),
            ]);
        };
        this.migrate = async (currencies) => {
            await this.migration.migrate(currencies);
        };
        this.close = () => {
            return Database.sequelize.close();
        };
        this.loadModels = () => {
            Pair_1.default.load(Database.sequelize);
            Referral_1.default.load(Database.sequelize);
            Swap_1.default.load(Database.sequelize);
            ChainTip_1.default.load(Database.sequelize);
            ReverseSwap_1.default.load(Database.sequelize);
            MarkedSwap_1.default.load(Database.sequelize);
            KeyProvider_1.default.load(Database.sequelize);
            ChannelCreation_1.default.load(Database.sequelize);
            DatabaseVersion_1.default.load(Database.sequelize);
            ReverseRoutingHint_1.default.load(Database.sequelize);
            PendingLockupTransaction_1.default.load(Database.sequelize);
            PendingEthereumTransaction_1.default.load(Database.sequelize);
        };
        if (postgresConfig !== undefined &&
            [postgresConfig.host, postgresConfig.port, postgresConfig.database].every((value) => value !== undefined)) {
            Database.type = DatabaseType.PostgreSQL;
            Database.sequelize = new sequelize_1.default.Sequelize({
                host: postgresConfig.host,
                port: postgresConfig.port,
                database: postgresConfig.database,
                username: postgresConfig.username,
                password: postgresConfig.password,
                dialect: 'postgres',
                logging: this.logger.silly,
            });
        }
        else {
            Database.type = DatabaseType.SQLite;
            Database.sequelize = new sequelize_1.default.Sequelize({
                dialect: 'sqlite',
                storage: sqlitePath,
                logging: this.logger.silly,
                retry: {
                    max: 3,
                    match: ['SQLITE_BUSY'],
                },
            });
        }
        this.loadModels();
        this.migration = new Migration_1.default(this.logger, Database.sequelize);
    }
}
Database.memoryDatabase = ':memory:';
Database.type = DatabaseType.PostgreSQL;
exports.default = Database;
//# sourceMappingURL=Database.js.map