/// <reference types="node" />
import BaseClient from '../BaseClient';
import Logger from '../Logger';
import * as routerrpc from '../proto/lnd/router_pb';
import * as lndrpc from '../proto/lnd/rpc_pb';
import { WalletBalance } from '../wallet/providers/WalletProviderInterface';
import { ChannelInfo, DecodedInvoice, EventTypes, HopHint, Invoice, LightningClient, NodeInfo, PaymentResponse, Route } from './LightningClient';
/**
 * The configurable options for the LND client
 */
type LndConfig = {
    host: string;
    port: number;
    certpath: string;
    macaroonpath: string;
    maxPaymentFeeRatio: number;
};
/**
 * A class representing a client to interact with LND
 */
declare class LndClient extends BaseClient<EventTypes> implements LightningClient {
    readonly symbol: string;
    static readonly serviceName = "LND";
    static readonly paymentMinFee = 121;
    static readonly paymentMaxParts = 5;
    private static readonly paymentTimeout;
    private static readonly paymentTimePreference;
    private readonly uri;
    private readonly maxPaymentFeeRatio;
    private readonly credentials;
    private readonly meta;
    private router?;
    private invoices?;
    private lightning?;
    private peerEventSubscription?;
    private channelEventSubscription?;
    private channelBackupSubscription?;
    /**
     * Create an LND client
     */
    constructor(logger: Logger, symbol: string, config: LndConfig);
    private throwFilesNotFound;
    serviceName: () => string;
    /**
     * Returns a boolean determines whether LND is ready or not
     */
    connect: (startSubscriptions?: boolean) => Promise<boolean>;
    private reconnect;
    /**
     * End all subscriptions and reconnection attempts.
     */
    disconnect: () => void;
    private unaryInvoicesCall;
    private unaryRouterCall;
    private unaryLightningCall;
    getInfo: () => Promise<NodeInfo>;
    /**
     * Creates an invoice
     */
    addInvoice: (value: number, expiry?: number, memo?: string, routingHints?: HopHint[][]) => Promise<lndrpc.AddInvoiceResponse.AsObject>;
    /**
     * Creates a hold invoice with the supplied preimage hash
     */
    addHoldInvoice: (value: number, preimageHash: Buffer, cltvExpiry?: number, expiry?: number, memo?: string, routingHints?: HopHint[][]) => Promise<string>;
    lookupHoldInvoice: (preimageHash: Buffer) => Promise<Invoice>;
    lookupInvoice: (preimageHash: Buffer) => Promise<lndrpc.Invoice.AsObject>;
    trackPayment: (preimageHash: Buffer) => Promise<lndrpc.Payment.AsObject>;
    /**
     * Pay an invoice through the Lightning Network.
     *
     * @param invoice an invoice for a payment within the Lightning Network
     * @param cltvDelta CLTV delta limit for the payment
     * @param outgoingChannelId channel through which the invoice should be paid
     */
    sendPayment: (invoice: string, cltvDelta?: number, outgoingChannelId?: string) => Promise<PaymentResponse>;
    static formatPaymentFailureReason: (reason: lndrpc.PaymentFailureReason) => string;
    resetMissionControl: () => Promise<routerrpc.ResetMissionControlResponse.AsObject>;
    /**
     * Cancel a hold invoice
     */
    cancelHoldInvoice: (preimageHash: Buffer) => Promise<void>;
    /**
     * Settle a hold invoice with an already accepted HTLC
     */
    settleHoldInvoice: (preimage: Buffer) => Promise<void>;
    /**
     * Queries for a possible route to the target destination
     */
    queryRoutes: (destination: string, amt: number, cltvLimit?: number, finalCltvDelta?: number, routingHints?: HopHint[][]) => Promise<Route[]>;
    decodeInvoice: (invoice: string) => Promise<DecodedInvoice>;
    /**
     * Decode an encoded payment request
     *
     * @param paymentRequest encoded payment request
     */
    decodePayReq: (paymentRequest: string) => Promise<lndrpc.PayReq.AsObject>;
    decodePayReqRawResponse: (paymentRequest: string) => Promise<lndrpc.PayReq>;
    /**
     * Returns the latest advertised, aggregated, and authenticated channel information
     * for the specified node identified by its public key
     */
    getNodeInfo: (publicKey: string) => Promise<lndrpc.NodeInfo.AsObject>;
    /**
     * Establish a connection to a remote peer
     *
     * @param pubKey identity public key of the remote peer
     * @param host host of the remote peer
     */
    connectPeer: (pubKey: string, host: string) => Promise<lndrpc.ConnectPeerResponse.AsObject>;
    /**
     * Sends coins to a particular address
     *
     * @param address address to which coins should be sent
     * @param amount number of satoshis or litoshis to send
     * @param satPerByte satoshis or litoshis per byte that should be sent as fee
     */
    sendCoins: (address: string, amount: number, satPerByte?: number) => Promise<lndrpc.SendCoinsResponse.AsObject>;
    /**
     * Sends all coins of the wallet to a particular address
     *
     * @param address address to which coins should be sent
     * @param satPerByte satoshis or litoshis per byte that should be sent as fee
     */
    sweepWallet: (address: string, satPerByte?: number) => Promise<lndrpc.SendCoinsResponse.AsObject>;
    /**
     * Returns a list describing all the known transactions relevant to the wallet
     */
    getOnchainTransactions: (startHeight: number) => Promise<lndrpc.TransactionDetails.AsObject>;
    /**
     * Creates a new address
     *
     * @param addressType type of the address
     */
    newAddress: (addressType?: lndrpc.AddressType) => Promise<lndrpc.NewAddressResponse.AsObject>;
    /**
     * Attempts to open a channel to a remote peer
     *
     * @param pubKey identity public key of the remote peer
     * @param fundingAmount the number of satoshis the local wallet should commit
     * @param privateChannel whether the channel should be private
     * @param satPerByte sat/vbyte fee of the funding transaction
     */
    openChannel: (pubKey: string, fundingAmount: number, privateChannel: boolean, satPerByte: number) => Promise<lndrpc.ChannelPoint.AsObject>;
    /**
     * Gets a list of all open channels
     */
    listChannels: (activeOnly?: boolean, privateOnly?: boolean) => Promise<ChannelInfo[]>;
    /**
     * Gets the latest routing information of a given channel
     */
    getChannelInfo: (channelId: string) => Promise<lndrpc.ChannelEdge.AsObject>;
    /**
     * Gets a list of all peers
     */
    listPeers: () => Promise<lndrpc.ListPeersResponse.AsObject>;
    getBalance: () => Promise<WalletBalance>;
    /**
     * Gets the balance of the onchain wallet
     */
    getWalletBalance: () => Promise<lndrpc.WalletBalanceResponse.AsObject>;
    /**
     * Subscribe to events for a single invoice
     */
    subscribeSingleInvoice: (preimageHash: Buffer) => void;
    private static routingHintsToGrpc;
    private static routingHintsFromGrpc;
    private static invoiceStateFromGrpc;
    private static htlcFromGrpc;
    private static htlcStateFromGrpc;
    private handleSubscriptionError;
    private subscribePeerEvents;
    private subscribeChannelEvents;
    private subscribeChannelBackups;
}
export default LndClient;
export { LndConfig };
