"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BalanceType = void 0;
const DenominationConverter_1 = require("../DenominationConverter");
const LiquidTypes_1 = require("../consts/LiquidTypes");
const Markup_1 = require("./Markup");
var BalanceType;
(function (BalanceType) {
    BalanceType[BalanceType["Wallet"] = 0] = "Wallet";
    BalanceType[BalanceType["ChannelLocal"] = 1] = "ChannelLocal";
    BalanceType[BalanceType["ChannelRemote"] = 2] = "ChannelRemote";
})(BalanceType || (exports.BalanceType = BalanceType = {}));
class BalanceChecker {
    constructor(logger, service, notificationClient, currencyConfigs, tokenConfigs) {
        this.logger = logger;
        this.service = service;
        this.notificationClient = notificationClient;
        this.currencies = [];
        // These Sets contain the symbols for which an out of bounds alert notification was sent
        this.walletBalanceAlerts = new Set();
        this.localBalanceAlerts = new Set();
        this.remoteBalanceAlerts = new Set();
        this.check = async () => {
            const balances = (await this.service.getBalance()).getBalancesMap();
            for (const currency of this.currencies) {
                const balance = balances.get(currency.symbol);
                if (balance) {
                    await this.checkCurrency(currency, balance.toObject());
                }
            }
        };
        this.checkCurrency = async (currency, balances) => {
            for (const [service, balance] of balances.walletsMap) {
                await this.checkBalance(currency, service, BalanceType.Wallet, balance.confirmed + balance.unconfirmed, balances.walletsMap.length === 1);
            }
            for (const [service, lightningBalance] of balances.lightningMap) {
                await this.checkBalance(currency, service, BalanceType.ChannelLocal, lightningBalance.local);
                await this.checkBalance(currency, service, BalanceType.ChannelRemote, lightningBalance.remote);
            }
        };
        this.checkBalance = async (currency, service, type, balance, isOnlyWallet) => {
            let isInBounds;
            let isMainWallet = false;
            let notificationSet;
            if (type === BalanceType.Wallet) {
                notificationSet = this.walletBalanceAlerts;
                const { preferredWallet, minWalletBalance, maxWalletBalance, maxUnusedWalletBalance, } = currency;
                if (isOnlyWallet ||
                    (preferredWallet || 'lnd') === service.toLowerCase()) {
                    isMainWallet = true;
                    isInBounds =
                        minWalletBalance <= balance &&
                            balance <= (maxWalletBalance || Number.MAX_SAFE_INTEGER);
                }
                else if (maxUnusedWalletBalance !== undefined) {
                    isInBounds = balance <= maxUnusedWalletBalance;
                }
                else {
                    // Wallet does not need to be checked
                    return;
                }
            }
            else {
                notificationSet =
                    type === BalanceType.ChannelLocal
                        ? this.localBalanceAlerts
                        : this.remoteBalanceAlerts;
                const minThreshold = type === BalanceType.ChannelLocal
                    ? currency.minLocalBalance
                    : currency.minRemoteBalance;
                isInBounds = minThreshold <= balance;
            }
            const ident = `${currency.symbol}${service}`;
            if (!notificationSet.has(ident) && !isInBounds) {
                notificationSet.add(ident);
                await this.sendAlert(currency, type, service, isInBounds, isMainWallet, balance);
            }
            else if (notificationSet.has(ident) && isInBounds) {
                notificationSet.delete(ident);
                await this.sendAlert(currency, type, service, isInBounds, isMainWallet, balance);
            }
        };
        this.sendAlert = async (currency, type, service, isInBounds, isMainWallet, balance) => {
            const name = `${currency.symbol} ${service}`;
            let message;
            if (isInBounds) {
                if (type === BalanceType.Wallet) {
                    message = `${Markup_1.Emojis.Checkmark} ${name} wallet balance of ${(0, DenominationConverter_1.satoshisToSatcomma)(balance)} is in bounds again ${Markup_1.Emojis.Checkmark}`;
                }
                else {
                    message =
                        `${Markup_1.Emojis.Checkmark} ${name} ${type === BalanceType.ChannelLocal ? 'local' : 'remote'} channel balance ` +
                            `of ${(0, DenominationConverter_1.satoshisToSatcomma)(balance)} is more than expected ` +
                            `${(0, DenominationConverter_1.satoshisToSatcomma)(type === BalanceType.ChannelLocal
                                ? currency.minLocalBalance
                                : currency.minRemoteBalance)} again ${Markup_1.Emojis.Checkmark}`;
                }
            }
            else {
                if (type === BalanceType.Wallet) {
                    const limits = isMainWallet
                        ? `${currency.maxWalletBalance
                            ? `    Max: ${(0, DenominationConverter_1.satoshisToSatcomma)(currency.maxWalletBalance)}\n`
                            : ''}` + `    Min: ${(0, DenominationConverter_1.satoshisToSatcomma)(currency.minWalletBalance)}`
                        : `    Max: ${(0, DenominationConverter_1.satoshisToSatcomma)(currency.maxUnusedWalletBalance)}`;
                    message =
                        `${Markup_1.Emojis.RotatingLight} **${name} wallet balance is out of bounds** ${Markup_1.Emojis.RotatingLight}\n` +
                            `  Balance: ${(0, DenominationConverter_1.satoshisToSatcomma)(balance)}\n` +
                            limits;
                }
                else {
                    message =
                        `${Markup_1.Emojis.RotatingLight} **${name} ${type === BalanceType.ChannelLocal ? 'local' : 'remote'} channel balance ` +
                            `of ${(0, DenominationConverter_1.satoshisToSatcomma)(balance)} is less than expected ` +
                            `${(0, DenominationConverter_1.satoshisToSatcomma)(type === BalanceType.ChannelLocal
                                ? currency.minLocalBalance
                                : currency.minRemoteBalance)}** ${Markup_1.Emojis.RotatingLight}`;
                }
            }
            this.logger.warn(`Balance warning: ${message}`);
            await this.notificationClient.sendMessage(message, true);
        };
        currencyConfigs
            .filter((config) => config !== undefined)
            .filter((config) => config.minWalletBalance !== undefined)
            .forEach((config) => this.currencies.push({
            ...config,
            symbol: config.symbol || LiquidTypes_1.liquidSymbol,
        }));
        tokenConfigs.forEach((config) => this.currencies.push(config));
    }
}
exports.default = BalanceChecker;
//# sourceMappingURL=BalanceChecker.js.map