"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sequelize_1 = require("sequelize");
const DenominationConverter_1 = require("../DenominationConverter");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const ReferralStats_1 = __importDefault(require("../data/ReferralStats"));
const Stats_1 = __importDefault(require("../data/Stats"));
const ChannelCreationRepository_1 = __importDefault(require("../db/repositories/ChannelCreationRepository"));
const FeeRepository_1 = __importDefault(require("../db/repositories/FeeRepository"));
const ReverseSwapRepository_1 = __importDefault(require("../db/repositories/ReverseSwapRepository"));
const SwapRepository_1 = __importDefault(require("../db/repositories/SwapRepository"));
const Markup_1 = require("./Markup");
const OtpManager_1 = __importDefault(require("./OtpManager"));
var Command;
(function (Command) {
    Command["Help"] = "help";
    // Commands that retrieve information
    Command["GetFees"] = "getfees";
    Command["SwapInfo"] = "swapinfo";
    Command["GetStats"] = "getstats";
    Command["GetBalance"] = "getbalance";
    Command["LockedFunds"] = "lockedfunds";
    Command["PendingSwaps"] = "pendingswaps";
    Command["GetReferrals"] = "getreferrals";
    Command["PendingSweeps"] = "pendingsweeps";
    // Commands that generate a value or trigger a function
    Command["Backup"] = "backup";
    Command["Withdraw"] = "withdraw";
    Command["GetAddress"] = "getaddress";
    Command["ToggleReverseSwaps"] = "togglereverse";
})(Command || (Command = {}));
class CommandHandler {
    constructor(logger, config, notificationClient, service, backupScheduler) {
        this.logger = logger;
        this.notificationClient = notificationClient;
        this.service = service;
        this.backupScheduler = backupScheduler;
        /*
         * Command executors
         */
        this.help = async (args) => {
            if (args.length === 0) {
                let message = 'Commands:\n';
                this.commands.forEach((info, command) => {
                    message += `\n**${command}**: ${info.description}`;
                });
                await this.notificationClient.sendMessage(message);
            }
            else {
                const command = args[0].toLowerCase();
                const info = this.commands.get(command);
                if (!info) {
                    await this.notificationClient.sendMessage(`Could not find command: ${command}`);
                    return;
                }
                let message = `\`${command}\`: ${info.description}`;
                if (info.usage) {
                    message += '\n\n**Usage:**\n';
                    for (const usage of info.usage) {
                        message += `\n\`${usage.command}\`: ${usage.description}`;
                    }
                }
                await this.notificationClient.sendMessage(message);
            }
        };
        this.getFees = async () => {
            let message = 'Fees:\n';
            (await FeeRepository_1.default.getFees()).forEach(({ asset, sum }) => {
                message += `\n**${asset}**: ${(0, DenominationConverter_1.satoshisToSatcomma)(sum)} ${asset}`;
            });
            await this.notificationClient.sendMessage(message);
        };
        this.swapInfo = async (args) => {
            if (args.length === 0) {
                await this.sendCouldNotFindSwap('');
                return;
            }
            const identifier = args[0];
            const swaps = await SwapRepository_1.default.getSwaps({
                [sequelize_1.Op.or]: {
                    id: identifier,
                    invoice: identifier,
                    preimageHash: identifier,
                    lockupAddress: identifier,
                    lockupTransactionId: identifier,
                },
            });
            for (const swap of swaps) {
                const channelCreation = await ChannelCreationRepository_1.default.getChannelCreation({
                    swapId: swap.id,
                });
                await this.sendSwapInfo(swap, false, channelCreation);
            }
            const reverseSwaps = await ReverseSwapRepository_1.default.getReverseSwaps({
                [sequelize_1.Op.or]: {
                    id: identifier,
                    invoice: identifier,
                    preimageHash: identifier,
                    lockupAddress: identifier,
                    transactionId: identifier,
                },
            });
            for (const reverseSwap of reverseSwaps) {
                await this.sendSwapInfo(reverseSwap, true);
            }
            if (swaps.length === 0 && reverseSwaps.length === 0) {
                await this.sendCouldNotFindSwap(identifier);
            }
        };
        this.getStats = async (args) => {
            let minYear;
            let minMonth;
            if (args.length !== 0) {
                switch (args[0].toLowerCase()) {
                    case 'all':
                        minYear = 0;
                        minMonth = 0;
                        break;
                    default:
                        await this.notificationClient.sendMessage(`Invalid parameter: ${args[0]}`);
                        return;
                }
            }
            else {
                const date = new Date();
                // Only show the last 6 months by default
                date.setMonth(date.getMonth() - 5);
                minYear = date.getUTCFullYear();
                minMonth = date.getUTCMonth();
            }
            await this.notificationClient.sendMessage(`${Markup_1.codeBlock}${(0, Utils_1.stringify)(await Stats_1.default.generate(minYear, minMonth))}${Markup_1.codeBlock}`);
        };
        this.getBalance = async () => {
            const balances = (await this.service.getBalance()).toObject().balancesMap;
            let message = 'Balances:';
            balances.forEach(([symbol, bals]) => {
                message += `\n\n**${symbol}**\n`;
                bals.walletsMap.forEach(([service, walletBals]) => {
                    message += `\n${service} Wallet: ${(0, DenominationConverter_1.satoshisToSatcomma)(walletBals.confirmed + walletBals.unconfirmed)} ${symbol}`;
                });
                if (bals.lightningMap.length > 0) {
                    message += '\n';
                }
                bals.lightningMap.forEach(([service, lightningBals]) => {
                    message +=
                        `\n${service}:\n` +
                            `  Local: ${(0, DenominationConverter_1.satoshisToSatcomma)(lightningBals.local)} ${symbol}\n` +
                            `  Remote: ${(0, DenominationConverter_1.satoshisToSatcomma)(lightningBals.remote)} ${symbol}`;
                });
            });
            await this.notificationClient.sendMessage(message);
        };
        this.lockedFunds = async () => {
            const pendingReverseSwapsByChain = await this.service.getLockedFunds();
            let message = '**Locked up funds:**\n';
            for (const [symbol, chainArray] of pendingReverseSwapsByChain) {
                message += `\n**${symbol}**`;
                let symbolTotal = 0;
                for (const pendingReverseSwap of chainArray) {
                    symbolTotal += pendingReverseSwap.onchainAmount;
                    message += `\n  - \`${pendingReverseSwap.id}\`: ${(0, DenominationConverter_1.satoshisToSatcomma)(pendingReverseSwap.onchainAmount)}`;
                }
                message += `\n\nTotal: ${(0, DenominationConverter_1.satoshisToSatcomma)(symbolTotal)}\n`;
            }
            await this.notificationClient.sendMessage(message);
        };
        this.pendingSwaps = async () => {
            const [pendingSwaps, pendingReverseSwaps] = await Promise.all([
                SwapRepository_1.default.getSwaps({
                    status: {
                        [sequelize_1.Op.notIn]: Enums_1.NotPendingSwapEvents,
                    },
                }),
                ReverseSwapRepository_1.default.getReverseSwaps({
                    status: {
                        [sequelize_1.Op.notIn]: Enums_1.NotPendingReverseSwapEvents,
                    },
                }),
            ]);
            let message = '';
            const formatSwapIds = (isReverse) => {
                const swaps = isReverse ? pendingReverseSwaps : pendingSwaps;
                if (swaps.length > 0) {
                    message += `\n${message.endsWith('\n') ? '' : '\n'}**Pending${isReverse ? ' reverse' : ''} Swaps:**\n\n`;
                    for (const swap of swaps) {
                        message += `- \`${swap.id}\`\n`;
                    }
                }
            };
            formatSwapIds(false);
            formatSwapIds(true);
            await this.notificationClient.sendMessage(message);
        };
        this.pendingSweeps = async () => {
            await this.notificationClient.sendMessage(`${Markup_1.codeBlock}${(0, Utils_1.stringify)((0, Utils_1.mapToObject)(this.service.swapManager.deferredClaimer.pendingSweeps()))}${Markup_1.codeBlock}`);
        };
        this.getReferrals = async () => {
            await this.notificationClient.sendMessage(`${Markup_1.codeBlock}${(0, Utils_1.stringify)(await ReferralStats_1.default.getReferralFees())}${Markup_1.codeBlock}`);
        };
        this.backup = async () => {
            try {
                await this.backupScheduler.uploadDatabase(new Date());
                await this.notificationClient.sendMessage('Uploaded backup of Boltz database');
            }
            catch (error) {
                await this.notificationClient.sendMessage(`Could not upload backup: ${error}`);
            }
        };
        this.getAddress = async (args) => {
            const sendError = (error) => {
                return this.notificationClient.sendMessage(`Could not get address: ${(0, Utils_1.formatError)(error)}`);
            };
            try {
                if (args.length === 0) {
                    await sendError('no currency was specified');
                    return;
                }
                const currency = args[0].toUpperCase();
                const response = await this.service.getAddress(currency);
                await this.notificationClient.sendMessage(`\`${response}\``);
            }
            catch (error) {
                await sendError(error);
            }
        };
        this.withdraw = async (args) => {
            if (args.length !== 3 && args.length !== 4) {
                await this.notificationClient.sendMessage('Invalid number of arguments');
                return;
            }
            const validToken = this.optManager.verify(args[0]);
            if (!validToken) {
                await this.notificationClient.sendMessage('Invalid OTP token');
                return;
            }
            const symbol = args[1].toUpperCase();
            // Three arguments mean that just the OTP token, the symbol and a lightning invoice was provided
            if (args.length === 3) {
                try {
                    const response = await this.service.payInvoice(symbol, args[2]);
                    await this.notificationClient.sendMessage(`Paid lightning invoice\nPreimage: ${(0, Utils_1.getHexString)(response.preimage)}`);
                }
                catch (error) {
                    await this.notificationClient.sendMessage(`Could not pay lightning invoice: ${(0, Utils_1.formatError)(error)}`);
                }
                // Four arguments mean that the OTP token, the symbol, an address and the amount were provided
            }
            else if (args.length === 4) {
                try {
                    const sendAll = args[3] === 'all';
                    const amount = sendAll ? 0 : (0, DenominationConverter_1.coinsToSatoshis)(Number(args[3]));
                    const response = await this.service.sendCoins({
                        amount,
                        symbol,
                        sendAll,
                        address: args[2],
                    });
                    await this.notificationClient.sendMessage(`Sent transaction: ${response.transactionId}:${response.vout}`);
                }
                catch (error) {
                    await this.notificationClient.sendMessage(`Could not send coins: ${(0, Utils_1.formatError)(error)}`);
                }
            }
        };
        this.toggleReverseSwaps = async () => {
            this.service.allowReverseSwaps = !this.service.allowReverseSwaps;
            const message = `${this.service.allowReverseSwaps ? 'Enabled' : 'Disabled'} Reverse Swaps`;
            this.logger.info(message);
            await this.notificationClient.sendMessage(message);
        };
        /*
         * Helper functions
         */
        this.sendSwapInfo = async (swap, isReverse, channelCreation) => {
            const hasChannelCreation = channelCreation !== null && channelCreation !== undefined;
            let name;
            if (hasChannelCreation) {
                name = 'Channel Creation';
            }
            else {
                if (isReverse) {
                    name = 'Reverse Swap';
                }
                else {
                    name = 'Swap';
                }
            }
            await this.notificationClient.sendMessage(`${name} \`${swap.id}\`:\n` +
                `${Markup_1.codeBlock}${(0, Utils_1.stringify)(swap)}${hasChannelCreation ? '\n' + (0, Utils_1.stringify)(channelCreation) : ''}${Markup_1.codeBlock}`);
        };
        this.sendCouldNotFindSwap = async (id) => {
            await this.notificationClient.sendMessage(`Could not find swap with id: ${id}`);
        };
        this.commands = new Map([
            [
                Command.Help,
                {
                    usage: [
                        {
                            command: 'help [command]',
                            description: 'gets more information about a single command',
                        },
                    ],
                    executor: this.help,
                    description: 'gets a list of all available commands',
                },
            ],
            [
                Command.GetFees,
                {
                    executor: this.getFees,
                    description: 'gets accumulated fees',
                },
            ],
            [
                Command.SwapInfo,
                {
                    usage: [
                        {
                            command: 'swapinfo <id>',
                            description: 'gets all available information about the (reverse) swap with the id `<id>`',
                        },
                    ],
                    executor: this.swapInfo,
                    description: 'gets all available information about a (reverse) swap',
                },
            ],
            [
                Command.GetStats,
                {
                    executor: this.getStats,
                    description: 'gets statistics grouped by year and month for the current and last 6 months',
                    usage: [
                        {
                            command: `${Command.GetStats} all`,
                            description: 'get statistics for all swaps in the database',
                        },
                    ],
                },
            ],
            [
                Command.GetBalance,
                {
                    executor: this.getBalance,
                    description: 'gets the balance of the wallets and channels',
                },
            ],
            [
                Command.LockedFunds,
                {
                    executor: this.lockedFunds,
                    description: 'gets funds locked up by Boltz',
                },
            ],
            [
                Command.PendingSwaps,
                {
                    executor: this.pendingSwaps,
                    description: 'gets a list of pending (reverse) swaps',
                },
            ],
            [
                Command.PendingSweeps,
                {
                    executor: this.pendingSweeps,
                    description: 'gets all pending sweeps',
                },
            ],
            [
                Command.GetReferrals,
                {
                    executor: this.getReferrals,
                    description: 'gets stats for all referral IDs',
                },
            ],
            [
                Command.Backup,
                {
                    executor: this.backup,
                    description: 'uploads a backup of the databases',
                },
            ],
            [
                Command.Withdraw,
                {
                    usage: [
                        {
                            command: 'withdraw <OTP token> <currency> <invoice>',
                            description: 'withdraws lightning funds',
                        },
                        {
                            command: 'withdraw <OTP token> <currency> <address> <amount in whole coins>',
                            description: 'withdraws a specific amount of onchain coins',
                        },
                        {
                            command: 'withdraw <OTP token> <currency> <address> all',
                            description: 'withdraws all onchain coins',
                        },
                    ],
                    executor: this.withdraw,
                    description: 'withdraws coins from Boltz',
                },
            ],
            [
                Command.GetAddress,
                {
                    usage: [
                        {
                            command: 'getaddress <currency>',
                            description: 'gets an address for the currency `<currency>`',
                        },
                    ],
                    executor: this.getAddress,
                    description: 'gets an address for a currency',
                },
            ],
            [
                Command.ToggleReverseSwaps,
                {
                    executor: this.toggleReverseSwaps,
                    description: 'enables or disables reverse swaps',
                },
            ],
        ]);
        this.optManager = new OtpManager_1.default(this.logger, config);
        this.notificationClient.on('message', async (message) => {
            const args = message.split(' ');
            // Get the command and remove the first argument from the array which is the command itself
            const command = args.shift();
            if (command) {
                const commandInfo = this.commands.get(command.toLowerCase());
                if (commandInfo) {
                    this.logger.debug(`Executing Discord command: ${command} ${args.join(', ')}`);
                    await commandInfo.executor(args);
                }
            }
        });
    }
}
exports.default = CommandHandler;
//# sourceMappingURL=CommandHandler.js.map