"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const DenominationConverter_1 = require("../DenominationConverter");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const BalanceChecker_1 = __importDefault(require("./BalanceChecker"));
const CommandHandler_1 = __importDefault(require("./CommandHandler"));
const DiskUsageChecker_1 = __importDefault(require("./DiskUsageChecker"));
const Markup_1 = require("./Markup");
const DiscordClient_1 = __importDefault(require("./clients/DiscordClient"));
const MattermostClient_1 = __importDefault(require("./clients/MattermostClient"));
// TODO: test balance and service alerts
// TODO: use events instead of intervals to check connections and balances
class NotificationProvider {
    constructor(logger, service, walletManager, backup, config, currencies, tokenConfigs) {
        this.logger = logger;
        this.service = service;
        this.walletManager = walletManager;
        this.backup = backup;
        this.config = config;
        this.disconnected = new Set();
        this.init = async () => {
            try {
                await this.client.init();
                await this.client.sendMessage('Started Boltz instance');
                this.logger.verbose(`Connected to ${this.client.serviceName}`);
                for (const [symbol, currency] of this.service.currencies) {
                    [currency.lndClient, currency.clnClient]
                        .filter((client) => client !== undefined)
                        .forEach((client) => {
                        client.on('status.changed', async (status) => {
                            switch (status) {
                                case Enums_1.ClientStatus.Connected:
                                    await this.sendReconnected(`${client.serviceName()} ${symbol}`);
                                    break;
                                case Enums_1.ClientStatus.Disconnected:
                                    await this.sendLostConnection(`${client.serviceName()} ${symbol}`);
                                    break;
                            }
                        });
                        client.on('subscription.error', async (subscription) => {
                            await this.sendLostConnection(`${client.serviceName()} ${symbol}`, subscription);
                        });
                        client.on('subscription.reconnected', async () => await this.sendReconnected(`${client.serviceName()} ${symbol}`));
                    });
                }
                this.service.lockupTransactionTracker.on('zeroConf.disabled', async (symbol) => await this.client.sendMessage(`${Markup_1.Emojis.RotatingLight} **Disabled 0-conf for ${symbol}** ${Markup_1.Emojis.RotatingLight}`, true));
                const check = async () => {
                    await Promise.all([
                        this.checkConnections(),
                        this.balanceChecker.check(),
                        this.diskUsageChecker.checkUsage(),
                    ]);
                };
                await check();
                this.logger.debug(`Checking balances and connection status every ${this.config.interval} minutes`);
                this.timer = setInterval(async () => {
                    await check();
                }, (0, Utils_1.minutesToMilliseconds)(this.config.interval));
            }
            catch (error) {
                this.logger.warn(`Could not start notification service: ${error}`);
            }
        };
        this.disconnect = () => {
            clearInterval(this.timer);
        };
        this.checkConnections = async () => {
            const info = (await this.service.getInfo()).toObject();
            const promises = [];
            info.chainsMap.forEach(([symbol, currency]) => {
                currency.lightningMap.forEach(([service, lnInfo]) => {
                    promises.push(this.checkConnection(`${symbol} ${service}`, lnInfo));
                });
                promises.push(this.checkConnection(`${symbol} node`, currency.chain));
            });
            await Promise.all(promises);
        };
        this.checkConnection = async (service, object) => {
            if (object !== undefined) {
                if (object.error === '') {
                    await this.sendReconnected(service);
                    return;
                }
            }
            await this.sendLostConnection(service);
        };
        this.listenToDiscord = () => {
            this.client.on('error', (error) => {
                this.logger.warn(`Discord client threw: ${(0, Utils_1.formatError)(error)}`);
            });
        };
        this.listenToService = () => {
            const getSwapTitle = (pair, orderSide, isReverse) => {
                const { base, quote } = (0, Utils_1.splitPairId)(pair);
                const { sending, receiving } = (0, Utils_1.getSendingReceivingCurrency)(base, quote, orderSide);
                return `${receiving}${isReverse ? ` ${Markup_1.Emojis.Zap}` : ''} -> ${sending}${!isReverse ? ` ${Markup_1.Emojis.Zap}` : ''}`;
            };
            const getBasicSwapInfo = (swap, onchainSymbol, lightningSymbol) => {
                let message = `ID: ${swap.id}\n` +
                    `Pair: ${swap.pair}\n` +
                    `Order side: ${swap.orderSide === Enums_1.OrderSide.BUY ? 'buy' : 'sell'}`;
                if (swap.invoice) {
                    const lightningAmount = (0, Utils_1.decodeInvoice)(swap.invoice).satoshis;
                    message +=
                        `${swap.onchainAmount
                            ? `\nOnchain amount: ${(0, DenominationConverter_1.satoshisToSatcomma)(swap.onchainAmount)} ${onchainSymbol}`
                            : ''}` +
                            `\nLightning amount: ${(0, DenominationConverter_1.satoshisToSatcomma)(lightningAmount)} ${lightningSymbol}`;
                }
                return message;
            };
            const getSymbols = (pairId, orderSide, isReverse) => {
                const { base, quote } = (0, Utils_1.splitPairId)(pairId);
                return {
                    onchainSymbol: (0, Utils_1.getChainCurrency)(base, quote, orderSide, isReverse),
                    lightningSymbol: (0, Utils_1.getLightningCurrency)(base, quote, orderSide, isReverse),
                };
            };
            this.service.eventHandler.on('swap.success', async ({ swap, isReverse, channelCreation }) => {
                const { onchainSymbol, lightningSymbol } = getSymbols(swap.pair, swap.orderSide, isReverse);
                const hasChannelCreation = channelCreation !== null &&
                    channelCreation !== undefined &&
                    channelCreation.fundingTransactionId !== null;
                let message = `**Swap ${getSwapTitle(swap.pair, swap.orderSide, isReverse)}${hasChannelCreation ? ' :construction_site:' : ''}**\n` +
                    `${getBasicSwapInfo(swap, onchainSymbol, lightningSymbol)}\n` +
                    `Fees earned: ${(0, DenominationConverter_1.satoshisToSatcomma)(swap.fee)} ${onchainSymbol}\n` +
                    `Miner fees: ${(0, DenominationConverter_1.satoshisToSatcomma)(swap.minerFee)} ${this.getMinerFeeSymbol(onchainSymbol)}`;
                if (!isReverse) {
                    // The routing fees are denominated in millisatoshi
                    message += `\nRouting fees: ${swap.routingFee / 1000} ${this.getSmallestDenomination(lightningSymbol)}`;
                }
                if (hasChannelCreation) {
                    message +=
                        '\n\n**Channel Creation:**\n' +
                            `Private: ${channelCreation.private}\n` +
                            `Inbound: ${channelCreation.inboundLiquidity}%\n` +
                            `Node: ${channelCreation.nodePublicKey}\n` +
                            `Funding: ${channelCreation.fundingTransactionId}:${channelCreation.fundingTransactionVout}`;
                }
                await this.client.sendMessage(`${message}${NotificationProvider.trailingWhitespace}`);
            });
            this.service.eventHandler.on('swap.failure', async ({ swap, isReverse, reason }) => {
                const { onchainSymbol, lightningSymbol } = getSymbols(swap.pair, swap.orderSide, isReverse);
                let message = `**Swap ${getSwapTitle(swap.pair, swap.orderSide, isReverse)} failed: ${reason}**\n` +
                    `${getBasicSwapInfo(swap, onchainSymbol, lightningSymbol)}`;
                if (isReverse) {
                    if (swap.minerFee) {
                        message += `\nMiner fees: ${(0, DenominationConverter_1.satoshisToSatcomma)(swap.minerFee)} ${onchainSymbol}`;
                    }
                }
                else if (swap.invoice) {
                    message += `\nInvoice: ${swap.invoice}`;
                }
                await this.client.sendMessage(`${message}${NotificationProvider.trailingWhitespace}`);
            });
        };
        this.sendLostConnection = async (service, subscription) => {
            if (this.disconnected.has(service)) {
                return;
            }
            if (subscription === undefined) {
                this.disconnected.add(service);
            }
            await this.client.sendMessage(`**Lost connection to ${service}${subscription ? ` ${subscription} subscription` : ''}**`, true);
        };
        this.sendReconnected = async (service) => {
            if (this.disconnected.has(service)) {
                this.disconnected.delete(service);
                await this.client.sendMessage(`Reconnected to ${service}`, true);
            }
        };
        this.getSmallestDenomination = (symbol) => {
            switch (symbol) {
                case 'LTC':
                    return 'litoshi';
                default:
                    return 'satoshi';
            }
        };
        this.getMinerFeeSymbol = (symbol) => {
            if (this.service.currencies.get(symbol).type === Enums_1.CurrencyType.ERC20) {
                return (this.walletManager.ethereumManagers.find((manager) => manager.hasSymbol(symbol))?.networkDetails.symbol || '');
            }
            else {
                return symbol;
            }
        };
        if (this.config.mattermostUrl === undefined) {
            this.client = new DiscordClient_1.default(this.logger, config);
        }
        else {
            this.client = new MattermostClient_1.default(this.logger, config);
        }
        this.listenToDiscord();
        this.listenToService();
        new CommandHandler_1.default(this.logger, this.config, this.client, this.service, this.backup);
        this.balanceChecker = new BalanceChecker_1.default(this.logger, this.service, this.client, currencies, tokenConfigs);
        this.diskUsageChecker = new DiskUsageChecker_1.default(this.logger, this.client);
    }
}
// This is a Discord hack to add trailing whitespace which is trimmed by default
NotificationProvider.trailingWhitespace = '\n** **';
exports.default = NotificationProvider;
//# sourceMappingURL=NotificationProvider.js.map