"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const v10_1 = require("discord-api-types/v10");
const discord_js_1 = require("discord.js");
const Markup_1 = require("../Markup");
const NotificationClient_1 = __importDefault(require("./NotificationClient"));
class DiscordClient extends NotificationClient_1.default {
    constructor(logger, config) {
        super(DiscordClient.serviceName, logger, config);
        this.channel = undefined;
        this.channelAlerts = undefined;
        this.init = async () => {
            if (this.config.token === '') {
                throw 'no API token provided';
            }
            return new Promise((resolve, reject) => {
                this.client.on('ready', () => {
                    for (const [, channel] of this.client.channels.cache) {
                        if (channel instanceof discord_js_1.TextChannel) {
                            switch (channel.name) {
                                case this.config.channel:
                                    this.channel = channel;
                                    break;
                                case this.config.channelAlerts:
                                    this.channelAlerts = channel;
                                    break;
                            }
                        }
                    }
                    if (!this.channel) {
                        reject(`Could not find Discord channel: ${this.config.channel}`);
                    }
                    if (this.config.channelAlerts !== undefined &&
                        this.channelAlerts === undefined) {
                        this.logger.warn(`Discord alert channel "${this.config.channelAlerts}" configured but could not be found`);
                    }
                    this.listenForMessages();
                    resolve();
                });
                this.client.login(this.config.token).catch((error) => reject(error));
            });
        };
        this.destroy = () => {
            this.channel = undefined;
            this.channelAlerts = undefined;
            if (this.client.isReady()) {
                this.client.destroy().then();
            }
        };
        this.sendMessage = async (message, isAlert) => {
            const channel = this.selectChannel(this.channel, this.channelAlerts, isAlert);
            if (channel === undefined) {
                return;
            }
            if (message.length + this.prefix.length <= DiscordClient.maxMessageLen) {
                await channel.send(`${this.prefix}${message}`);
            }
            else {
                let toSplit = message;
                let maxPartLen = DiscordClient.maxMessageLen;
                const isCodeBlock = message.startsWith(Markup_1.codeBlock) && message.endsWith(Markup_1.codeBlock);
                if (isCodeBlock) {
                    // When splitting code blocks we need to account for the length the code block markup needs
                    maxPartLen -= Markup_1.codeBlock.length * 2;
                    // Trim the code block markup from the original message that will be split
                    toSplit = toSplit.substring(Markup_1.codeBlock.length, toSplit.length - Markup_1.codeBlock.length);
                }
                await channel.send(this.prefix);
                for (const part of this.splitString(toSplit, maxPartLen)) {
                    const getBlockMarkup = () => (isCodeBlock ? Markup_1.codeBlock : '');
                    await channel.send(`${getBlockMarkup()}${part}${getBlockMarkup()}`);
                }
            }
        };
        this.listenForMessages = () => {
            if (this.channel) {
                this.client.on('messageCreate', (message) => {
                    if (message.author.bot)
                        return;
                    if (message.channel.id === this.channel.id) {
                        this.emit('message', message.content);
                    }
                });
            }
            this.client.on('error', (error) => {
                this.emit('error', error);
            });
        };
        this.splitString = (toSplit, length) => {
            const splitRegex = new RegExp(`[\\s\\S]{1,${length}}`, 'g');
            return toSplit.match(splitRegex) || [];
        };
        this.client = new discord_js_1.Client({
            intents: [
                v10_1.GatewayIntentBits.Guilds,
                v10_1.GatewayIntentBits.GuildMessages,
                v10_1.GatewayIntentBits.MessageContent,
            ],
        });
    }
}
DiscordClient.serviceName = 'Discord';
DiscordClient.maxMessageLen = 2000;
exports.default = DiscordClient;
//# sourceMappingURL=DiscordClient.js.map