"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../Utils");
const ElementsClient_1 = __importDefault(require("../chain/ElementsClient"));
const Consts_1 = require("../consts/Consts");
const Enums_1 = require("../consts/Enums");
const EvmNetworks_1 = require("../wallet/ethereum/EvmNetworks");
class FeeProvider {
    constructor(logger, walletManager, dataAggregator, getFeeEstimation) {
        this.logger = logger;
        this.walletManager = walletManager;
        this.dataAggregator = dataAggregator;
        this.getFeeEstimation = getFeeEstimation;
        // A map between the symbols of the pairs and their percentage fees
        this.percentageFees = new Map();
        this.percentageSwapInFees = new Map();
        this.minerFees = new Map();
        this.init = (pairs) => {
            const feesToPrint = {};
            pairs.forEach((pair) => {
                const pairId = (0, Utils_1.getPairId)(pair);
                // Set the configured fee or fallback to 1% if it is not defined
                let percentage = pair.fee;
                if (percentage === undefined) {
                    percentage = FeeProvider.defaultFee;
                    this.logger.warn(`Setting default fee of ${percentage}% for pair ${pairId} because none was specified`);
                }
                this.percentageFees.set(pairId, percentage / 100);
                if (pair.swapInFee) {
                    this.percentageSwapInFees.set(pairId, pair.swapInFee / 100);
                }
                feesToPrint[pairId] = this.getPercentageFees(pairId);
            });
            this.logger.debug(`Prepared data for fee estimations: ${(0, Utils_1.stringify)(feesToPrint)}`);
        };
        this.getPercentageFees = (pairId) => {
            const percentage = this.percentageFees.get(pairId);
            const percentageSwapIn = this.percentageSwapInFees.get(pairId) || percentage;
            return {
                percentage: percentage * 100,
                percentageSwapIn: percentageSwapIn * 100,
            };
        };
        this.getPercentageFee = (pair, isReverse) => {
            if (!isReverse && this.percentageSwapInFees.has(pair)) {
                return this.percentageSwapInFees.get(pair);
            }
            return this.percentageFees.get(pair) || 0;
        };
        this.getFees = (pair, swapVersion, rate, orderSide, amount, type) => {
            const isReverse = type !== Enums_1.BaseFeeType.NormalClaim;
            let percentageFee = this.getPercentageFee(pair, isReverse);
            if (percentageFee !== 0) {
                percentageFee = percentageFee * amount * rate;
            }
            const { base, quote } = (0, Utils_1.splitPairId)(pair);
            const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, orderSide, isReverse);
            return {
                percentageFee: Math.ceil(percentageFee),
                baseFee: this.getBaseFee(chainCurrency, swapVersion, type),
            };
        };
        this.getBaseFee = (chainCurrency, swapVersion, type) => {
            const minerFees = this.minerFees.get(chainCurrency)[swapVersion];
            switch (type) {
                case Enums_1.BaseFeeType.NormalClaim:
                    return minerFees.normal;
                case Enums_1.BaseFeeType.ReverseClaim:
                    return minerFees.reverse.claim;
                case Enums_1.BaseFeeType.ReverseLockup:
                    return minerFees.reverse.lockup;
            }
        };
        this.updateMinerFees = async (chainCurrency) => {
            const feeMap = await this.getFeeEstimation(chainCurrency);
            // TODO: avoid hard coding symbols
            switch (chainCurrency) {
                case 'BTC':
                case 'LTC':
                case ElementsClient_1.default.symbol: {
                    const relativeFee = feeMap.get(chainCurrency);
                    const calculateMinerFeesForVersion = (sizes) => ({
                        normal: Math.ceil(relativeFee * sizes.normalClaim),
                        reverse: {
                            claim: Math.ceil(relativeFee * sizes.reverseClaim),
                            lockup: Math.ceil(relativeFee * sizes.reverseLockup),
                        },
                    });
                    const sizes = chainCurrency === ElementsClient_1.default.symbol
                        ? FeeProvider.transactionSizes[Enums_1.CurrencyType.Liquid]
                        : FeeProvider.transactionSizes[Enums_1.CurrencyType.BitcoinLike];
                    this.minerFees.set(chainCurrency, {
                        [Enums_1.SwapVersion.Taproot]: calculateMinerFeesForVersion(sizes[Enums_1.SwapVersion.Taproot]),
                        [Enums_1.SwapVersion.Legacy]: calculateMinerFeesForVersion(sizes[Enums_1.SwapVersion.Legacy]),
                    });
                    break;
                }
                case EvmNetworks_1.Ethereum.symbol:
                case EvmNetworks_1.Rsk.symbol: {
                    const relativeFee = feeMap.get(chainCurrency);
                    const claimCost = FeeProvider.calculateEtherGasCost(relativeFee, FeeProvider.gasUsage.EtherSwap.claim);
                    const fees = {
                        normal: claimCost,
                        reverse: {
                            claim: claimCost,
                            lockup: FeeProvider.calculateEtherGasCost(relativeFee, FeeProvider.gasUsage.EtherSwap.lockup),
                        },
                    };
                    this.minerFees.set(chainCurrency, {
                        [Enums_1.SwapVersion.Legacy]: fees,
                        [Enums_1.SwapVersion.Taproot]: fees,
                    });
                    break;
                }
                // If it is not BTC, LTC or ETH, it is an ERC20 token
                default: {
                    const networkSymbol = this.walletManager.ethereumManagers.find((manager) => manager.hasSymbol(chainCurrency)).networkDetails.symbol;
                    const relativeFee = feeMap.get(networkSymbol);
                    const rate = this.dataAggregator.latestRates.get((0, Utils_1.getPairId)({ base: networkSymbol, quote: chainCurrency }));
                    const claimCost = FeeProvider.calculateTokenGasCosts(rate, relativeFee, FeeProvider.gasUsage.ERC20Swap.claim);
                    const fees = {
                        normal: claimCost,
                        reverse: {
                            claim: claimCost,
                            lockup: FeeProvider.calculateTokenGasCosts(rate, relativeFee, FeeProvider.gasUsage.ERC20Swap.lockup),
                        },
                    };
                    this.minerFees.set(chainCurrency, {
                        [Enums_1.SwapVersion.Legacy]: fees,
                        [Enums_1.SwapVersion.Taproot]: fees,
                    });
                    break;
                }
            }
        };
    }
}
FeeProvider.transactionSizes = {
    [Enums_1.CurrencyType.BitcoinLike]: {
        [Enums_1.SwapVersion.Taproot]: {
            normalClaim: 151,
            reverseLockup: 154,
            reverseClaim: 111,
        },
        [Enums_1.SwapVersion.Legacy]: {
            // The claim transaction which spends a nested SegWit swap output and sends it to a P2WPKH address has about 170 vbytes
            normalClaim: 170,
            // We cannot know what kind of address the user will claim to, so we just assume the worst case: P2PKH
            // Claiming a P2WSH to a P2PKH address is about 138 bytes
            reverseClaim: 138,
            // The lockup transaction which spends a P2WPKH output (possibly more, but we assume a best case scenario here),
            // locks up funds in a P2WSH swap and sends the change back to a P2WKH output has about 153 vbytes
            reverseLockup: 153,
        },
    },
    [Enums_1.CurrencyType.Liquid]: {
        [Enums_1.SwapVersion.Taproot]: {
            normalClaim: 1337,
            reverseLockup: 2503,
            reverseClaim: 1309,
        },
        [Enums_1.SwapVersion.Legacy]: {
            normalClaim: 1333,
            reverseLockup: 2503,
            reverseClaim: 1378,
        },
    },
};
// TODO: query those estimations from the provider
FeeProvider.gasUsage = {
    EtherSwap: {
        lockup: 46460,
        claim: 24924,
        refund: 23372,
    },
    ERC20Swap: {
        lockup: 86980,
        claim: 24522,
        refund: 23724,
    },
};
FeeProvider.defaultFee = 1;
FeeProvider.calculateTokenGasCosts = (rate, gasPrice, gasUsage) => {
    return Math.ceil(rate * FeeProvider.calculateEtherGasCost(gasPrice, gasUsage));
};
FeeProvider.calculateEtherGasCost = (gasPrice, gasUsage) => {
    return Number((BigInt(Math.round(gasPrice * Number(Consts_1.gweiDecimals))) * BigInt(gasUsage)) /
        Consts_1.etherDecimals);
};
exports.default = FeeProvider;
//# sourceMappingURL=FeeProvider.js.map