"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const async_lock_1 = __importDefault(require("async-lock"));
const sequelize_1 = require("sequelize");
const Utils_1 = require("../Utils");
const TypedEventEmitter_1 = __importDefault(require("../consts/TypedEventEmitter"));
const PendingLockupTransactionRepository_1 = __importDefault(require("../db/repositories/PendingLockupTransactionRepository"));
const SwapRepository_1 = __importDefault(require("../db/repositories/SwapRepository"));
const Errors_1 = __importDefault(require("./Errors"));
class LockupTransactionTracker extends TypedEventEmitter_1.default {
    constructor(logger, currencies, rateProvider) {
        super();
        this.logger = logger;
        this.rateProvider = rateProvider;
        this.lock = new async_lock_1.default();
        this.zeroConfAcceptedMap = new Map();
        this.zeroConfAccepted = (symbol) => this.zeroConfAcceptedMap.get(symbol) || false;
        this.addPendingTransactionToTrack = async (swap) => {
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
            if (!this.zeroConfAcceptedMap.has(chainCurrency)) {
                throw Errors_1.default.SYMBOL_LOCKUPS_NOT_BEING_TRACKED(chainCurrency);
            }
            await PendingLockupTransactionRepository_1.default.create(swap.id, chainCurrency);
        };
        this.listenToBlocks = (chainClient) => {
            chainClient.on('block', async () => {
                await this.lock.acquire(chainClient.symbol, () => this.checkPendingLockupsForChain(chainClient));
            });
        };
        this.checkPendingLockupsForChain = async (chainClient) => {
            const pendingLockups = await PendingLockupTransactionRepository_1.default.getForChain(chainClient.symbol);
            const swaps = await SwapRepository_1.default.getSwaps({
                id: {
                    [sequelize_1.Op.in]: pendingLockups.map((p) => p.swapId),
                },
            });
            for (const swap of swaps) {
                try {
                    const info = await chainClient.getRawTransactionVerbose(swap.lockupTransactionId);
                    if (info.confirmations !== undefined && info.confirmations > 0) {
                        this.logger.debug(`Pending lockup transaction of Submarine Swap ${swap.id} (${swap.lockupTransactionId}) confirmed`);
                        await PendingLockupTransactionRepository_1.default.destroy(swap.id);
                    }
                }
                catch (e) {
                    this.logger.warn(`Could not find pending lockup transaction of Submarine Swap ${swap.id} (${swap.lockupTransactionId}): ${(0, Utils_1.formatError)(e)}`);
                    this.logger.warn(`Disabling 0-conf for ${chainClient.symbol}`);
                    this.zeroConfAcceptedMap.set(chainClient.symbol, false);
                    await this.rateProvider.setZeroConfAmount(chainClient.symbol, 0);
                    this.emit('zeroConf.disabled', chainClient.symbol);
                }
            }
        };
        for (const currency of currencies.values()) {
            if (currency.chainClient === undefined) {
                continue;
            }
            this.listenToBlocks(currency.chainClient);
            this.zeroConfAcceptedMap.set(currency.chainClient.symbol, true);
        }
    }
}
exports.default = LockupTransactionTracker;
//# sourceMappingURL=LockupTransactionTracker.js.map