"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const Errors_1 = __importDefault(require("./Errors"));
const FeeProvider_1 = __importDefault(require("./FeeProvider"));
const RateCalculator_1 = __importDefault(require("./RateCalculator"));
const DataAggregator_1 = __importDefault(require("./data/DataAggregator"));
const RateProviderLegacy_1 = __importDefault(require("./providers/RateProviderLegacy"));
const RateProviderTaproot_1 = __importDefault(require("./providers/RateProviderTaproot"));
class RateProvider {
    constructor(logger, rateUpdateInterval, currencies, walletManager, getFeeEstimation) {
        this.logger = logger;
        this.rateUpdateInterval = rateUpdateInterval;
        this.currencies = currencies;
        this.walletManager = walletManager;
        this.dataAggregator = new DataAggregator_1.default();
        this.rateCalculator = new RateCalculator_1.default(this.dataAggregator);
        // A list of all pairs that are specified in the config
        this.configPairs = new Set();
        // A map of all pairs with hardcoded rates
        this.hardcodedPairs = new Set();
        this.zeroConfAmounts = new Map();
        this.pairConfigs = new Map();
        this.init = async (pairs) => {
            pairs.forEach((pair) => {
                this.pairConfigs.set((0, Utils_1.getPairId)(pair), pair);
            });
            await this.updateMinerFees();
            pairs.forEach((pair) => {
                const id = (0, Utils_1.getPairId)(pair);
                this.configPairs.add(id);
                if (pair.maxSwapAmount === undefined) {
                    throw Errors_1.default.CONFIGURATION_INCOMPLETE(id, 'maxSwapAmount');
                }
                if (pair.minSwapAmount === undefined) {
                    throw Errors_1.default.CONFIGURATION_INCOMPLETE(id, 'minSwapAmount');
                }
                // If a pair has a hardcoded rate the rate doesn't have to be queried from the exchanges
                if (pair.rate) {
                    this.logger.debug(`Setting hardcoded rate for pair ${id}: ${pair.rate}`);
                    this.hardcodedPairs.add(id);
                    this.forProviders((provider) => provider.setHardcodedPair(pair));
                }
                else {
                    this.dataAggregator.registerPair(pair.base, pair.quote);
                    // TODO: find way to get <chain fee asset>/<token> rate without having to hardcode it here
                    const checkAndRegisterToken = (symbol) => {
                        if (this.currencies.get(symbol).type !== Enums_1.CurrencyType.ERC20) {
                            return;
                        }
                        const chainFeeSymbol = this.walletManager.ethereumManagers.find((manager) => manager.hasSymbol(symbol)).networkDetails.symbol;
                        this.dataAggregator.registerPair(chainFeeSymbol, symbol);
                    };
                    checkAndRegisterToken(pair.base);
                    checkAndRegisterToken(pair.quote);
                }
            });
            if (this.dataAggregator.pairs.size > 0) {
                const pairsToQuery = [];
                this.dataAggregator.pairs.forEach(([base, quote]) => {
                    pairsToQuery.push((0, Utils_1.getPairId)({ base, quote }));
                });
                this.logger.debug(`Prepared data for requests to exchanges: \n  - ${pairsToQuery.join('\n  - ')}`);
            }
            await this.updateRates();
            this.logger.debug(`Updating rates every ${this.rateUpdateInterval} minutes`);
            this.timer = setInterval(async () => {
                await this.updateRates();
            }, (0, Utils_1.minutesToMilliseconds)(this.rateUpdateInterval));
        };
        this.disconnect = () => {
            clearInterval(this.timer);
        };
        /**
         * Returns whether 0-conf should be accepted for a specific amount on a specified chain
         */
        this.acceptZeroConf = (chainCurrency, amount) => {
            const limits = this.zeroConfAmounts.get(chainCurrency);
            if (limits) {
                return amount <= limits;
            }
            else {
                return false;
            }
        };
        this.has = (pairId) => this.configPairs.has(pairId);
        this.setZeroConfAmount = async (symbol, amount) => {
            this.zeroConfAmounts.set(symbol, amount);
            await this.updateRates();
        };
        this.updateRates = async () => {
            // The fees for the ERC20 tokens depend on the rates
            // Updating rates and fees at the same time would result in a race condition
            // that could leave the fee estimations for the ERC20 tokens outdated or even
            // "null" on the very first run of this function
            const updatedRates = await this.dataAggregator.fetchPairs();
            await this.updateMinerFees();
            for (const [pairId, rate] of updatedRates) {
                // Filter pairs that are fetched (for example to calculate gas fees for a BTC/<token> pair)
                // but not specified in the config
                if (!this.configPairs.has(pairId)) {
                    continue;
                }
                // If the rate returned is "undefined" or "NaN" that means that all requests to the APIs of the exchanges
                // failed and that the pairs and limits don't have to be updated
                if (rate && !isNaN(rate)) {
                    this.forProviders((provider) => provider.updatePair(pairId, rate));
                }
                else {
                    this.logger.warn(`Could not fetch rates of ${pairId}`);
                }
            }
            // Update the miner fees of the pairs with a hardcoded rate
            this.hardcodedPairs.forEach((pairId) => this.forProviders((provider) => provider.updateHardcodedPair(pairId)));
            this.logger.silly('Updated rates');
        };
        this.parseCurrencies = (currencies) => {
            for (const currency of currencies) {
                if (currency.limits.maxZeroConfAmount === undefined) {
                    this.logger.warn(`Maximal 0-conf amount not set for ${currency.symbol}`);
                }
                // Set the maximal 0-conf amount to 0 if it wasn't set explicitly
                this.zeroConfAmounts.set(currency.symbol, currency.limits.maxZeroConfAmount || 0);
            }
        };
        this.updateMinerFees = async () => {
            await Promise.all(Array.from(this.currencies.keys()).map((currency) => this.feeProvider.updateMinerFees(currency)));
        };
        this.forProviders = (fn) => Object.values(this.providers).forEach(fn);
        this.feeProvider = new FeeProvider_1.default(this.logger, walletManager, this.dataAggregator, getFeeEstimation);
        this.parseCurrencies(Array.from(currencies.values()));
        this.providers = {
            [Enums_1.SwapVersion.Taproot]: new RateProviderTaproot_1.default(this.currencies, this.feeProvider, this.pairConfigs, this.zeroConfAmounts),
            [Enums_1.SwapVersion.Legacy]: new RateProviderLegacy_1.default(this.currencies, this.feeProvider, this.pairConfigs, this.zeroConfAmounts),
        };
    }
}
exports.default = RateProvider;
//# sourceMappingURL=RateProvider.js.map