"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../../Utils");
const Binance_1 = __importDefault(require("./exchanges/Binance"));
const Bitfinex_1 = __importDefault(require("./exchanges/Bitfinex"));
const CoinbasePro_1 = __importDefault(require("./exchanges/CoinbasePro"));
const Kraken_1 = __importDefault(require("./exchanges/Kraken"));
class DataAggregator {
    constructor() {
        this.exchanges = [
            new Kraken_1.default(),
            new Binance_1.default(),
            new Bitfinex_1.default(),
            new CoinbasePro_1.default(),
        ];
        this.pairs = new Set();
        this.latestRates = new Map();
        this.registerPair = (baseAsset, quoteAsset) => {
            this.pairs.add([baseAsset, quoteAsset]);
        };
        this.fetchPairs = async () => {
            const rateMap = new Map();
            const queryPromises = [];
            const queryRate = async (base, quote) => {
                const pairId = (0, Utils_1.getPairId)({ base, quote });
                const rate = await this.getRate(base, quote);
                if (rate && !isNaN(rate)) {
                    rateMap.set(pairId, rate);
                }
                else {
                    // If the rate couldn't be fetched, the latest one should be used
                    rateMap.set(pairId, this.latestRates.get(pairId) || NaN);
                }
            };
            this.pairs.forEach(([baseAsset, quoteAsset]) => {
                queryPromises.push(queryRate(baseAsset, quoteAsset));
            });
            await Promise.all(queryPromises);
            this.latestRates = rateMap;
            return rateMap;
        };
        this.getRate = async (baseAsset, quoteAsset) => {
            const promises = [];
            this.exchanges.forEach((exchange) => promises.push(exchange.getPrice(baseAsset, quoteAsset)));
            const results = await Promise.all(promises.map((promise) => promise.catch((error) => error)));
            // Filter all results that are not numeric (failed requests)
            const validResults = results.filter((result) => !isNaN(Number(result)));
            validResults.sort((a, b) => a - b);
            const middle = (validResults.length - 1) / 2;
            if (validResults.length % 2 === 0) {
                return ((validResults[Math.ceil(middle)] + validResults[Math.floor(middle)]) / 2);
            }
            else {
                return validResults[middle];
            }
        };
    }
}
exports.default = DataAggregator;
//# sourceMappingURL=DataAggregator.js.map