"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../../Utils");
const Enums_1 = require("../../consts/Enums");
const Errors_1 = __importDefault(require("../../service/Errors"));
const NodeSwitch_1 = __importDefault(require("../../swap/NodeSwitch"));
const RateProviderBase_1 = __importDefault(require("./RateProviderBase"));
class RateProviderTaproot extends RateProviderBase_1.default {
    constructor(currencies, feeProvider, pairConfigs, zeroConfAmounts) {
        super(currencies, feeProvider);
        this.pairConfigs = pairConfigs;
        this.zeroConfAmounts = zeroConfAmounts;
        this.submarinePairs = new Map();
        this.reversePairs = new Map();
        this.setHardcodedPair = (pair) => {
            const id = (0, Utils_1.getPairId)(pair);
            for (const orderSide of [Enums_1.OrderSide.BUY, Enums_1.OrderSide.SELL]) {
                const rate = orderSide === Enums_1.OrderSide.BUY ? pair.rate : 1 / pair.rate;
                this.setPair(id, orderSide, false, rate, 0);
                this.setPair(id, orderSide, true, pair.rate, {
                    claim: 0,
                    lockup: 0,
                });
            }
        };
        this.updatePair = (pairId, rawRate) => {
            for (const orderSide of [Enums_1.OrderSide.BUY, Enums_1.OrderSide.SELL]) {
                const rate = orderSide === Enums_1.OrderSide.BUY ? rawRate : 1 / rawRate;
                this.setPair(pairId, orderSide, false, rate);
                this.setPair(pairId, orderSide, true, rate);
            }
        };
        this.updateHardcodedPair = (pairId) => {
            for (const orderSide of [Enums_1.OrderSide.BUY, Enums_1.OrderSide.SELL]) {
                this.setPair(pairId, orderSide, false);
                this.setPair(pairId, orderSide, true);
            }
        };
        this.validatePairHash = (hash, pairId, orderSide, isReverse) => {
            const nested = this.getToMap(pairId, orderSide, isReverse);
            if (nested === undefined) {
                throw Errors_1.default.PAIR_NOT_FOUND(pairId);
            }
            const pair = nested.toMap.get(nested.toAsset);
            if (pair === undefined) {
                throw Errors_1.default.PAIR_NOT_FOUND(pairId);
            }
            if (hash !== pair.hash) {
                throw Errors_1.default.INVALID_PAIR_HASH();
            }
        };
        this.getToMap = (pairId, orderSide, isReverse, create = false) => {
            const { base, quote } = (0, Utils_1.splitPairId)(pairId);
            const fromAsset = isReverse
                ? (0, Utils_1.getLightningCurrency)(base, quote, orderSide, true)
                : (0, Utils_1.getChainCurrency)(base, quote, orderSide, false);
            const toAsset = isReverse
                ? (0, Utils_1.getChainCurrency)(base, quote, orderSide, true)
                : (0, Utils_1.getLightningCurrency)(base, quote, orderSide, false);
            const toMap = (isReverse ? this.reversePairs : this.submarinePairs).get(fromAsset);
            if (toMap === undefined) {
                if (!create) {
                    return undefined;
                }
                const newMap = new Map();
                (isReverse ? this.reversePairs : this.submarinePairs).set(fromAsset, newMap);
                return {
                    toAsset,
                    fromAsset,
                    toMap: newMap,
                };
            }
            return {
                toAsset,
                fromAsset,
                toMap: toMap,
            };
        };
        this.hashPair = (pair) => (0, Utils_1.hashString)(JSON.stringify({
            rate: pair.rate,
            fees: pair.fees,
            limits: pair.limits,
        }));
        this.setPair = (pairId, orderSide, isReverse, rate, minerFees) => {
            const nested = this.getToMap(pairId, orderSide, isReverse, true);
            if (!this.isPossibleCombination(isReverse, nested.fromAsset, nested.toAsset)) {
                return;
            }
            if (rate === undefined) {
                rate = nested.toMap.get(nested.toAsset)?.rate;
                if (rate === undefined) {
                    return;
                }
            }
            if (minerFees === undefined) {
                const { base, quote } = (0, Utils_1.splitPairId)(pairId);
                const minerFeesObj = this.feeProvider.minerFees.get((0, Utils_1.getChainCurrency)(base, quote, orderSide, isReverse))[Enums_1.SwapVersion.Taproot];
                minerFees = isReverse ? minerFeesObj.reverse : minerFeesObj.normal;
            }
            const percentageFees = this.feeProvider.getPercentageFees(pairId);
            const pair = {
                hash: '',
                rate: rate,
                limits: this.getLimits(pairId, orderSide, isReverse, rate),
                fees: {
                    percentage: isReverse
                        ? percentageFees.percentage
                        : percentageFees.percentageSwapIn,
                    minerFees,
                },
            };
            pair.hash = this.hashPair(pair);
            nested.toMap.set(nested.toAsset, pair);
        };
        this.getLimits = (pair, orderSide, isReverse, rate) => {
            const config = this.pairConfigs.get(pair);
            if (config === undefined) {
                throw `Could not get limits for pair: ${pair}`;
            }
            const { base, quote } = (0, Utils_1.splitPairId)(pair);
            const result = {
                maximal: config.maxSwapAmount,
                minimal: this.adjustMinimaForFees(base, quote, rate, config.minSwapAmount, orderSide, isReverse),
            };
            if (isReverse) {
                return result;
            }
            const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, orderSide, false);
            return {
                ...result,
                maximalZeroConf: this.zeroConfAmounts.get(chainCurrency),
            };
        };
        this.isPossibleCombination = (isReverse, fromAsset, toAsset) => {
            if (isReverse) {
                return this.canLightning(fromAsset) && this.canOnchain(toAsset);
            }
            return this.canOnchain(fromAsset) && this.canLightning(toAsset);
        };
        this.canLightning = (currency) => {
            const cur = this.currencies.get(currency);
            if (cur === undefined) {
                return false;
            }
            return NodeSwitch_1.default.hasClient(cur);
        };
        this.canOnchain = (currency) => {
            const cur = this.currencies.get(currency);
            if (cur === undefined) {
                return false;
            }
            return cur.chainClient !== undefined || cur.provider !== undefined;
        };
    }
}
RateProviderTaproot.serializePairs = (map) => {
    const obj = (0, Utils_1.mapToObject)(map);
    // Remove empty "from" currencies
    Object.entries(obj).forEach(([key, value]) => {
        if (Object.values(value).length === 0) {
            delete obj[key];
        }
    });
    return obj;
};
exports.default = RateProviderTaproot;
//# sourceMappingURL=RateProviderTaproot.js.map