"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const liquidjs_lib_1 = require("liquidjs-lib");
const Enums_1 = require("../consts/Enums");
const TypedEventEmitter_1 = __importDefault(require("../consts/TypedEventEmitter"));
const SwapNursery_1 = __importDefault(require("../swap/SwapNursery"));
class EventHandler extends TypedEventEmitter_1.default {
    constructor(logger, currencies, nursery) {
        super();
        this.logger = logger;
        this.currencies = currencies;
        this.nursery = nursery;
        this.emitSwapCreation = (id) => {
            this.emit('swap.update', {
                id,
                status: {
                    status: Enums_1.SwapUpdateEvent.SwapCreated,
                },
            });
        };
        this.emitSwapInvoiceSet = (id) => {
            this.emit('swap.update', {
                id,
                status: { status: Enums_1.SwapUpdateEvent.InvoiceSet },
            });
        };
        /**
         * Subscribes transaction related swap events
         */
        this.subscribeTransactions = () => {
            this.nursery.on('transaction', ({ swap, transaction, confirmed, isReverse }) => {
                if (!isReverse) {
                    this.emit('swap.update', {
                        id: swap.id,
                        status: {
                            status: confirmed
                                ? Enums_1.SwapUpdateEvent.TransactionConfirmed
                                : Enums_1.SwapUpdateEvent.TransactionMempool,
                        },
                    });
                }
                else {
                    // Reverse Swaps only emit the "transaction.confirmed" event
                    // "transaction.mempool" is handled by the event "coins.sent"
                    if (transaction instanceof bitcoinjs_lib_1.Transaction ||
                        transaction instanceof liquidjs_lib_1.Transaction) {
                        this.emit('swap.update', {
                            id: swap.id,
                            status: {
                                status: Enums_1.SwapUpdateEvent.TransactionConfirmed,
                                transaction: {
                                    id: transaction.getId(),
                                    hex: transaction.toHex(),
                                },
                            },
                        });
                    }
                    else {
                        this.emit('swap.update', {
                            id: swap.id,
                            status: {
                                status: Enums_1.SwapUpdateEvent.TransactionConfirmed,
                                transaction: {
                                    id: transaction,
                                },
                            },
                        });
                    }
                }
            });
        };
        /**
         * Subscribes to invoice related Swap events
         */
        this.subscribeInvoices = () => {
            this.nursery.on('invoice.settled', (swap) => {
                this.logger.verbose(`Reverse swap ${swap.id} succeeded`);
                this.emit('swap.update', {
                    id: swap.id,
                    status: {
                        status: Enums_1.SwapUpdateEvent.InvoiceSettled,
                    },
                });
                this.emit('swap.success', { swap, isReverse: true });
            });
            this.nursery.on('invoice.pending', (swap) => {
                this.emit('swap.update', {
                    id: swap.id,
                    status: {
                        status: Enums_1.SwapUpdateEvent.InvoicePending,
                    },
                });
            });
            this.nursery.on('invoice.failedToPay', (swap) => {
                this.handleFailedSwap(swap, Enums_1.SwapUpdateEvent.InvoiceFailedToPay, swap.failureReason);
            });
            this.nursery.on('invoice.paid', (swap) => {
                this.emit('swap.update', {
                    id: swap.id,
                    status: {
                        status: Enums_1.SwapUpdateEvent.InvoicePaid,
                    },
                });
            });
            this.nursery.on('invoice.expired', (reverseSwap) => {
                this.emit('swap.update', {
                    id: reverseSwap.id,
                    status: {
                        status: Enums_1.SwapUpdateEvent.InvoiceExpired,
                    },
                });
            });
        };
        /**
         * Subscribes Swap events
         */
        this.subscribeSwapEvents = () => {
            this.nursery.on('zeroconf.rejected', (swap) => {
                this.emit('swap.update', {
                    id: swap.id,
                    status: {
                        status: Enums_1.SwapUpdateEvent.TransactionMempool,
                        zeroConfRejected: true,
                    },
                });
            });
            this.nursery.on('claim', ({ swap, channelCreation }) => {
                this.logger.verbose(`Swap ${swap.id} succeeded`);
                this.emit('swap.update', {
                    id: swap.id,
                    status: {
                        status: Enums_1.SwapUpdateEvent.TransactionClaimed,
                    },
                });
                this.emit('swap.success', {
                    swap,
                    channelCreation,
                    isReverse: false,
                });
            });
            this.nursery.on('claim.pending', (swap) => {
                this.emit('swap.update', {
                    id: swap.id,
                    status: {
                        status: Enums_1.SwapUpdateEvent.TransactionClaimPending,
                    },
                });
            });
            this.nursery.on('expiration', ({ swap, isReverse }) => {
                const newStatus = Enums_1.SwapUpdateEvent.SwapExpired;
                if (isReverse) {
                    this.handleFailedReverseSwap(swap, newStatus, swap.failureReason);
                }
                else {
                    this.handleFailedSwap(swap, newStatus, swap.failureReason);
                }
            });
            this.nursery.on('minerfee.paid', (reverseSwap) => {
                this.emit('swap.update', {
                    id: reverseSwap.id,
                    status: {
                        status: Enums_1.SwapUpdateEvent.MinerFeePaid,
                    },
                });
            });
            this.nursery.on('coins.sent', ({ reverseSwap, transaction }) => {
                if (transaction instanceof bitcoinjs_lib_1.Transaction ||
                    transaction instanceof liquidjs_lib_1.Transaction) {
                    this.emit('swap.update', {
                        id: reverseSwap.id,
                        status: {
                            status: Enums_1.SwapUpdateEvent.TransactionMempool,
                            transaction: {
                                id: transaction.getId(),
                                hex: transaction.toHex(),
                                eta: SwapNursery_1.default.reverseSwapMempoolEta,
                            },
                        },
                    });
                }
                else {
                    this.emit('swap.update', {
                        id: reverseSwap.id,
                        status: {
                            status: Enums_1.SwapUpdateEvent.TransactionMempool,
                            transaction: {
                                id: transaction,
                            },
                        },
                    });
                }
            });
            this.nursery.on('coins.failedToSend', (reverseSwap) => {
                this.handleFailedReverseSwap(reverseSwap, Enums_1.SwapUpdateEvent.TransactionFailed, reverseSwap.failureReason);
            });
            this.nursery.on('refund', ({ reverseSwap }) => {
                this.handleFailedReverseSwap(reverseSwap, Enums_1.SwapUpdateEvent.TransactionRefunded, reverseSwap.failureReason);
            });
            this.nursery.channelNursery.on('channel.created', ({ swap, channelCreation }) => {
                this.emit('swap.update', {
                    id: swap.id,
                    status: {
                        status: Enums_1.SwapUpdateEvent.ChannelCreated,
                        channel: {
                            fundingTransactionId: channelCreation.fundingTransactionId,
                            fundingTransactionVout: channelCreation.fundingTransactionVout,
                        },
                    },
                });
            });
            this.nursery.on('lockup.failed', (swap) => {
                this.emit('swap.update', {
                    id: swap.id,
                    status: {
                        status: Enums_1.SwapUpdateEvent.TransactionLockupFailed,
                        failureReason: swap.failureReason,
                    },
                });
            });
        };
        /**
         * Subscribes to a stream of channel backups
         */
        this.subscribeChannelBackups = () => {
            this.currencies.forEach((currency) => {
                // TODO: also do backups for CLN here?
                if (currency.lndClient) {
                    const { symbol, lndClient } = currency;
                    lndClient.on('channel.backup', (channelBackup) => {
                        this.emit('channel.backup', { channelBackup, currency: symbol });
                    });
                }
            });
        };
        this.handleFailedSwap = (swap, status, failureReason) => {
            this.logger.warn(`Swap ${swap.id} failed: ${failureReason}`);
            this.emit('swap.update', {
                id: swap.id,
                status: {
                    status,
                    failureReason,
                },
            });
            this.emit('swap.failure', {
                swap,
                isReverse: false,
                reason: failureReason,
            });
        };
        this.handleFailedReverseSwap = (reverseSwap, status, failureReason) => {
            this.logger.warn(`Reverse swap ${reverseSwap.id} failed: ${failureReason}`);
            this.emit('swap.update', {
                id: reverseSwap.id,
                status: { status, failureReason },
            });
            this.emit('swap.failure', {
                swap: reverseSwap,
                isReverse: true,
                reason: failureReason,
            });
        };
        this.subscribeInvoices();
        this.subscribeSwapEvents();
        this.subscribeTransactions();
        this.subscribeChannelBackups();
    }
}
exports.default = EventHandler;
//# sourceMappingURL=EventHandler.js.map