"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_schedule_1 = require("node-schedule");
const NodeSwitch_1 = __importDefault(require("../swap/NodeSwitch"));
class NodeInfo {
    constructor(logger, currencies) {
        this.logger = logger;
        this.currencies = currencies;
        this.stats = new Map();
        this.uris = new Map();
        this.pubkeys = new Set();
        this.init = () => {
            this.job = (0, node_schedule_1.scheduleJob)('0 0 * * *', async () => {
                this.logger.debug('Updating lightning node stats');
                await this.update();
            });
            return this.update();
        };
        this.stopSchedule = () => {
            if (this.job) {
                this.job.cancel();
                this.job = undefined;
            }
        };
        this.isOurNode = (pubkey) => this.pubkeys.has(pubkey);
        this.update = async () => {
            for (const [symbol, currency] of this.currencies) {
                if (!NodeSwitch_1.default.hasClient(currency)) {
                    continue;
                }
                const clients = [currency.lndClient, currency.clnClient].filter((client) => client !== undefined);
                const infos = await Promise.all(clients.map(async (client) => [
                    client.serviceName(),
                    await client.getInfo(),
                    await client?.listChannels(),
                ]));
                infos.forEach(([, info]) => this.pubkeys.add(info.pubkey));
                this.uris.set(symbol, new Map(infos.map(([name, info]) => [
                    name,
                    {
                        uris: info.uris,
                        nodeKey: info.pubkey,
                    },
                ])));
                const stats = await Promise.all(infos.map(async ([name, info, channels]) => [
                    name,
                    await this.calculateNodeStats(currency, channels, info.peers),
                ]));
                stats.push([
                    NodeInfo.totalStats,
                    {
                        capacity: stats.reduce((sum, [, stat]) => sum + stat.capacity, 0),
                        channels: stats.reduce((sum, [, stat]) => sum + stat.channels, 0),
                        peers: stats.reduce((sum, [, stat]) => sum + stat.peers, 0),
                        oldestChannel: stats.reduce((oldest, [, stat]) => {
                            if (stat.oldestChannel === undefined) {
                                return oldest;
                            }
                            if (oldest === undefined) {
                                return stat.oldestChannel;
                            }
                            return oldest < stat.oldestChannel ? oldest : stat.oldestChannel;
                        }, undefined),
                    },
                ]);
                this.stats.set(symbol, new Map(stats));
            }
        };
        this.calculateNodeStats = async (currency, channels, peers) => {
            let oldestChannelBlockTime;
            if (channels.length > 0) {
                const oldestChannel = channels.reduce((prev, cur) => {
                    return Number(prev.chanId) < Number(cur.chanId) ? prev : cur;
                });
                oldestChannelBlockTime = (await currency.chainClient.getRawTransactionVerbose(oldestChannel.fundingTransactionId)).blocktime;
            }
            const publicChannels = channels.filter((chan) => !chan.private);
            return {
                capacity: publicChannels.reduce((sum, chan) => sum + chan.capacity, 0),
                channels: publicChannels.length,
                peers: peers,
                oldestChannel: oldestChannelBlockTime,
            };
        };
    }
}
NodeInfo.totalStats = 'total';
exports.default = NodeInfo;
//# sourceMappingURL=NodeInfo.js.map