/// <reference types="node" />
import { SwapTreeSerializer } from 'boltz-core';
import { ConfigType } from '../Config';
import Logger from '../Logger';
import { ServiceInfo, ServiceWarning, SwapVersion } from '../consts/Enums';
import { PairConfig } from '../consts/Types';
import ReverseSwap from '../db/models/ReverseSwap';
import { HopHint, PaymentResponse } from '../lightning/LightningClient';
import { DeriveKeysResponse, GetBalanceResponse, GetInfoResponse } from '../proto/boltzrpc_pb';
import LockupTransactionTracker from '../rates/LockupTransactionTracker';
import RateProvider from '../rates/RateProvider';
import { PairTypeLegacy } from '../rates/providers/RateProviderLegacy';
import NodeSwitch from '../swap/NodeSwitch';
import SwapManager, { ChannelCreationInfo } from '../swap/SwapManager';
import WalletManager, { Currency } from '../wallet/WalletManager';
import Blocks from './Blocks';
import ElementsService from './ElementsService';
import EventHandler from './EventHandler';
import { PairTimeoutBlocksDelta } from './TimeoutDeltaProvider';
import { SwapToClaim } from './cooperative/DeferredClaimer';
import EipSigner from './cooperative/EipSigner';
import MusigSigner from './cooperative/MusigSigner';
type NetworkContracts = {
    network: {
        chainId: number;
        name?: string;
    };
    swapContracts: Map<string, string>;
    tokens: Map<string, string>;
};
type Contracts = {
    ethereum?: NetworkContracts;
    rsk?: NetworkContracts;
};
type ReverseTransaction = {
    transactionId: string;
    timeoutBlockHeight: number;
    transactionHex?: string;
};
type SwapTransaction = ReverseTransaction & {
    timeoutEta?: number;
};
declare class Service {
    private logger;
    walletManager: WalletManager;
    private nodeSwitch;
    currencies: Map<string, Currency>;
    allowReverseSwaps: boolean;
    private nodeInfo;
    swapManager: SwapManager;
    eventHandler: EventHandler;
    elementsService: ElementsService;
    readonly eipSigner: EipSigner;
    readonly musigSigner: MusigSigner;
    readonly rateProvider: RateProvider;
    readonly lockupTransactionTracker: LockupTransactionTracker;
    private prepayMinerFee;
    private readonly paymentRequestUtils;
    private readonly timeoutDeltaProvider;
    private static MinInboundLiquidity;
    private static MaxInboundLiquidity;
    constructor(logger: Logger, config: ConfigType, walletManager: WalletManager, nodeSwitch: NodeSwitch, currencies: Map<string, Currency>, blocks: Blocks);
    init: (configPairs: PairConfig[]) => Promise<void>;
    convertToPairAndSide: (from: string, to: string) => {
        pairId: string;
        orderSide: string;
    };
    /**
     * Gets general information about this Boltz instance and the nodes it is connected to
     */
    getInfo: () => Promise<GetInfoResponse>;
    rescan: (symbol: string, startHeight: number) => Promise<number>;
    /**
     * Gets the balance for either all wallets or just a single one if specified
     */
    getBalance: () => Promise<GetBalanceResponse>;
    /**
     * Gets all supported pairs and their conversion rates
     */
    getPairs: () => {
        info: ServiceInfo[];
        warnings: ServiceWarning[];
        pairs: Map<string, PairTypeLegacy>;
    };
    getInfos: () => ServiceInfo[];
    getWarnings: () => ServiceWarning[];
    getNodes: () => Map<string, Map<string, import("./NodeInfo").LightningNodeInfo>>;
    getNodeStats: () => Map<string, Map<string, import("./NodeInfo").Stats>>;
    getRoutingHints: (symbol: string, routingNode: string) => Promise<{
        hopHintsList: HopHint[];
    }[]>;
    getTimeouts: () => Map<string, {
        base: PairTimeoutBlocksDelta;
        quote: PairTimeoutBlocksDelta;
    }>;
    /**
     * Gets the contract address used by the Boltz instance
     */
    getContracts: () => Promise<Contracts>;
    /**
     * Gets a hex encoded transaction from a transaction hash on the specified network
     */
    getTransaction: (symbol: string, transactionHash: string) => Promise<string>;
    /**
     * Gets the hex encoded lockup transaction of a Submarine Swap, the block height
     * at which it will time out and the expected ETA for that block
     */
    getSwapTransaction: (id: string) => Promise<SwapTransaction>;
    getReverseSwapTransaction: (id: string) => Promise<ReverseTransaction>;
    getReverseBip21: (invoice: string) => Promise<{
        bip21: string;
        signature: string;
    } | undefined>;
    deriveKeys: (symbol: string, index: number) => DeriveKeysResponse;
    /**
     * Gets an address of a specified wallet
     */
    getAddress: (symbol: string) => Promise<string>;
    getBlockHeights: (symbol?: string) => Promise<Map<string, number>>;
    /**
     * Gets a fee estimation in satoshis per vbyte or GWEI for either all currencies or just a single one if specified
     */
    getFeeEstimation: (symbol?: string, blocks?: number) => Promise<Map<string, number>>;
    /**
     * Broadcast a hex encoded transaction on the specified network
     */
    broadcastTransaction: (symbol: string, transactionHex: string) => Promise<string>;
    /**
     * Updates the timeout block delta of a pair
     */
    updateTimeoutBlockDelta: (pairId: string, newDeltas: PairTimeoutBlocksDelta) => void;
    addReferral: (referral: {
        id: string;
        feeShare: number;
        routingNode?: string;
    }) => Promise<{
        apiKey: string;
        apiSecret: string;
    }>;
    setSwapStatus: (id: string, status: string) => Promise<void>;
    getLockedFunds: () => Promise<Map<string, ReverseSwap[]>>;
    getPendingSweeps: () => Map<string, SwapToClaim[]>;
    /**
     * Creates a new Swap from the chain to Lightning
     */
    createSwap: (args: {
        pairId: string;
        orderSide: string;
        preimageHash: Buffer;
        version: SwapVersion;
        channel?: ChannelCreationInfo;
        referralId?: string;
        refundPublicKey?: Buffer;
        invoice?: string;
    }) => Promise<{
        id: string;
        address: string;
        canBeRouted: boolean;
        timeoutBlockHeight: number;
        redeemScript?: string | undefined;
        claimPublicKey?: string | undefined;
        swapTree?: SwapTreeSerializer.SerializedTree | undefined;
        claimAddress?: string | undefined;
        blindingKey?: string | undefined;
    }>;
    /**
     * Gets the rates for a Submarine Swap that has coins in its lockup address but no invoice yet
     */
    getSwapRates: (id: string) => Promise<{
        onchainAmount: number;
        submarineSwap: {
            invoiceAmount: number;
        };
    }>;
    setInvoice: (id: string, invoice: string, pairHash?: string) => Promise<{
        bip21: string;
        expectedAmount: number;
        acceptZeroConf: boolean;
    }>;
    /**
     * Sets the invoice for Submarine Swap
     */
    private setSwapInvoice;
    /**
     * Creates a Submarine Swap with an invoice
     *
     * This method combines "createSwap" and "setSwapInvoice"
     */
    createSwapWithInvoice: (pairId: string, orderSide: string, refundPublicKey: Buffer, invoice: string, pairHash?: string, referralId?: string, channel?: ChannelCreationInfo, version?: SwapVersion) => Promise<{
        id: string;
        bip21: string;
        address: string;
        expectedAmount: number;
        acceptZeroConf: boolean;
        timeoutBlockHeight: number;
        redeemScript?: string | undefined;
        claimPublicKey?: string | undefined;
        swapTree?: SwapTreeSerializer.SerializedTree | undefined;
        claimAddress?: string | undefined;
        blindingKey?: string | undefined;
    }>;
    /**
     * Creates a new Swap from Lightning to the chain
     */
    createReverseSwap: (args: {
        pairId: string;
        version: SwapVersion;
        pairHash?: string;
        orderSide: string;
        preimageHash: Buffer;
        invoiceAmount?: number;
        onchainAmount?: number;
        routingNode?: string;
        referralId?: string;
        claimPublicKey?: Buffer;
        claimAddress?: string;
        prepayMinerFee?: boolean;
        userAddress?: string;
        userAddressSignature?: Buffer;
        claimCovenant?: boolean;
    }) => Promise<{
        id: string;
        invoice: string;
        blindingKey?: string | undefined;
        lockupAddress: string;
        redeemScript?: string | undefined;
        refundPublicKey?: string | undefined;
        swapTree?: SwapTreeSerializer.SerializedTree | undefined;
        refundAddress?: string | undefined;
        onchainAmount?: number | undefined;
        minerFeeInvoice?: string | undefined;
        timeoutBlockHeight: number;
        prepayMinerFeeAmount?: number | undefined;
    }>;
    /**
     * Pays a lightning invoice
     */
    payInvoice: (symbol: string, invoice: string) => Promise<PaymentResponse>;
    /**
     * Sends coins to a specified address
     */
    sendCoins: (args: {
        symbol: string;
        address: string;
        amount: number;
        sendAll?: boolean;
        fee?: number;
    }) => Promise<{
        vout: number;
        transactionId: string;
    }>;
    private getGasPrice;
    private getReferralId;
    /**
     * Verifies that the requested amount is neither above the maximal nor beneath the minimal
     */
    private verifyAmount;
    /**
     * Calculates the amount of an invoice for a Submarine Swap
     */
    private calculateInvoiceAmount;
    private getPair;
    private getCurrency;
    private getOrderSide;
    private calculateTimeoutDate;
    private checkWholeNumber;
}
export declare const cancelledViaCliFailureReason = "payment has been cancelled";
export default Service;
export { Contracts, NetworkContracts };
