"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cancelledViaCliFailureReason = void 0;
const boltz_core_1 = require("boltz-core");
const ethers_1 = require("ethers");
const sequelize_1 = require("sequelize");
const Core_1 = require("../Core");
const Utils_1 = require("../Utils");
const Errors_1 = __importDefault(require("../api/Errors"));
const Utils_2 = require("../api/Utils");
const ElementsClient_1 = __importDefault(require("../chain/ElementsClient"));
const Consts_1 = require("../consts/Consts");
const DefaultMap_1 = __importDefault(require("../consts/DefaultMap"));
const Enums_1 = require("../consts/Enums");
const ChannelCreationRepository_1 = __importDefault(require("../db/repositories/ChannelCreationRepository"));
const PairRepository_1 = __importDefault(require("../db/repositories/PairRepository"));
const ReferralRepository_1 = __importDefault(require("../db/repositories/ReferralRepository"));
const ReverseRoutingHintRepository_1 = __importDefault(require("../db/repositories/ReverseRoutingHintRepository"));
const ReverseSwapRepository_1 = __importDefault(require("../db/repositories/ReverseSwapRepository"));
const SwapRepository_1 = __importDefault(require("../db/repositories/SwapRepository"));
const LightningClient_1 = require("../lightning/LightningClient");
const boltzrpc_pb_1 = require("../proto/boltzrpc_pb");
const LockupTransactionTracker_1 = __importDefault(require("../rates/LockupTransactionTracker"));
const RateProvider_1 = __importDefault(require("../rates/RateProvider"));
const Errors_2 = __importDefault(require("../swap/Errors"));
const NodeSwitch_1 = __importDefault(require("../swap/NodeSwitch"));
const SwapManager_1 = __importDefault(require("../swap/SwapManager"));
const SwapOutputType_1 = __importDefault(require("../swap/SwapOutputType"));
const ElementsService_1 = __importDefault(require("./ElementsService"));
const Errors_3 = __importDefault(require("./Errors"));
const EventHandler_1 = __importDefault(require("./EventHandler"));
const NodeInfo_1 = __importDefault(require("./NodeInfo"));
const PaymentRequestUtils_1 = __importDefault(require("./PaymentRequestUtils"));
const TimeoutDeltaProvider_1 = __importDefault(require("./TimeoutDeltaProvider"));
const EipSigner_1 = __importDefault(require("./cooperative/EipSigner"));
const MusigSigner_1 = __importDefault(require("./cooperative/MusigSigner"));
class Service {
    constructor(logger, config, walletManager, nodeSwitch, currencies, blocks) {
        this.logger = logger;
        this.walletManager = walletManager;
        this.nodeSwitch = nodeSwitch;
        this.currencies = currencies;
        this.allowReverseSwaps = true;
        this.init = async (configPairs) => {
            const dbPairSet = new Set();
            const dbPairs = await PairRepository_1.default.getPairs();
            dbPairs.forEach((dbPair) => {
                dbPairSet.add(dbPair.id);
            });
            const checkCurrency = (symbol) => {
                if (!this.currencies.has(symbol)) {
                    throw Errors_3.default.CURRENCY_NOT_FOUND(symbol);
                }
            };
            for (const configPair of configPairs) {
                const id = (0, Utils_1.getPairId)(configPair);
                checkCurrency(configPair.base);
                checkCurrency(configPair.quote);
                if (!dbPairSet.has(id)) {
                    await PairRepository_1.default.addPair({
                        id,
                        ...configPair,
                    });
                    this.logger.silly(`Added pair to database: ${id}`);
                }
            }
            this.logger.verbose('Updated pairs in the database');
            this.timeoutDeltaProvider.init(configPairs, this.walletManager.ethereumManagers);
            this.rateProvider.feeProvider.init(configPairs);
            await this.rateProvider.init(configPairs);
            await this.nodeInfo.init();
        };
        this.convertToPairAndSide = (from, to) => {
            const pair = [
                [(0, Utils_1.getPairId)({ base: from, quote: to }), false],
                [(0, Utils_1.getPairId)({ base: to, quote: from }), true],
            ].find(([val]) => this.rateProvider.has(val));
            if (pair === undefined) {
                throw Errors_3.default.PAIR_NOT_FOUND((0, Utils_1.getPairId)({ base: from, quote: to }));
            }
            return {
                pairId: pair[0],
                orderSide: pair[1] ? 'buy' : 'sell',
            };
        };
        /**
         * Gets general information about this Boltz instance and the nodes it is connected to
         */
        this.getInfo = async () => {
            const response = new boltzrpc_pb_1.GetInfoResponse();
            const map = response.getChainsMap();
            response.setVersion((0, Utils_1.getVersion)());
            for (const [symbol, currency] of this.currencies) {
                const chain = new boltzrpc_pb_1.ChainInfo();
                if (currency.chainClient) {
                    try {
                        const [networkInfo, blockchainInfo] = await Promise.all([
                            currency.chainClient.getNetworkInfo(),
                            currency.chainClient.getBlockchainInfo(),
                        ]);
                        chain.setVersion(networkInfo.version);
                        chain.setConnections(networkInfo.connections);
                        chain.setBlocks(blockchainInfo.blocks);
                        chain.setScannedBlocks(blockchainInfo.scannedBlocks);
                    }
                    catch (error) {
                        chain.setError((0, Utils_1.formatError)(error));
                    }
                }
                else if (currency.provider) {
                    try {
                        const blockNumber = await currency.provider.getBlockNumber();
                        chain.setBlocks(blockNumber);
                        chain.setScannedBlocks(blockNumber);
                    }
                    catch (error) {
                        chain.setError((0, Utils_1.formatError)(error));
                    }
                }
                const currencyInfo = new boltzrpc_pb_1.CurrencyInfo();
                currencyInfo.setChain(chain);
                await Promise.all([currency.lndClient, currency.clnClient]
                    .filter((client) => client !== undefined)
                    .map(async (client) => {
                    const info = new boltzrpc_pb_1.LightningInfo();
                    try {
                        const infoRes = await client.getInfo();
                        const channels = new boltzrpc_pb_1.LightningInfo.Channels();
                        channels.setActive(infoRes.channels.active);
                        channels.setInactive(infoRes.channels.inactive);
                        channels.setPending(infoRes.channels.pending);
                        info.setChannels(channels);
                        info.setVersion(infoRes.version);
                        info.setBlockHeight(infoRes.blockHeight);
                    }
                    catch (error) {
                        info.setError(typeof error === 'object' ? error.details : error);
                    }
                    currencyInfo.getLightningMap().set(client.serviceName(), info);
                }));
                map.set(symbol, currencyInfo);
            }
            return response;
        };
        this.rescan = async (symbol, startHeight) => {
            const currency = this.getCurrency(symbol);
            let endHeight;
            if (currency.chainClient) {
                endHeight = (await currency.chainClient.getBlockchainInfo()).blocks;
                await currency.chainClient.rescanChain(startHeight);
            }
            else if (currency.provider) {
                const manager = this.walletManager.ethereumManagers.find((manager) => manager.hasSymbol(symbol));
                if (manager === undefined) {
                    throw Errors_3.default.NO_CHAIN_FOR_SYMBOL();
                }
                endHeight = await manager.provider.getBlockNumber();
                await manager.contractEventHandler.rescan(startHeight);
            }
            else {
                throw Errors_3.default.NO_CHAIN_FOR_SYMBOL();
            }
            return endHeight;
        };
        /**
         * Gets the balance for either all wallets or just a single one if specified
         */
        this.getBalance = async () => {
            const response = new boltzrpc_pb_1.GetBalanceResponse();
            const map = response.getBalancesMap();
            for (const [symbol, wallet] of this.walletManager.wallets) {
                const balances = new boltzrpc_pb_1.Balances();
                const currency = this.currencies.get(symbol);
                const lightningClients = currency
                    ? [currency.lndClient, currency.clnClient].filter((client) => client !== undefined)
                    : [];
                await Promise.all([wallet, ...lightningClients].map(async (bf) => {
                    const res = await bf.getBalance();
                    const walletBal = new boltzrpc_pb_1.Balances.WalletBalance();
                    walletBal.setConfirmed(res.confirmedBalance);
                    walletBal.setUnconfirmed(res.unconfirmedBalance);
                    balances.getWalletsMap().set(bf.serviceName(), walletBal);
                }));
                await Promise.all(lightningClients.map(async (client) => {
                    const lightningBalance = new boltzrpc_pb_1.Balances.LightningBalance();
                    const channelsList = await client.listChannels();
                    let localBalance = 0n;
                    let remoteBalance = 0n;
                    channelsList.forEach((channel) => {
                        localBalance += BigInt(channel.localBalance);
                        remoteBalance += BigInt(channel.remoteBalance);
                    });
                    lightningBalance.setLocal(Number(localBalance));
                    lightningBalance.setRemote(Number(remoteBalance));
                    balances
                        .getLightningMap()
                        .set(client.serviceName(), lightningBalance);
                }));
                map.set(symbol, balances);
            }
            return response;
        };
        /**
         * Gets all supported pairs and their conversion rates
         */
        this.getPairs = () => {
            return {
                info: this.getInfos(),
                warnings: this.getWarnings(),
                pairs: this.rateProvider.providers[Enums_1.SwapVersion.Legacy].pairs,
            };
        };
        this.getInfos = () => {
            const infos = [];
            if (this.prepayMinerFee) {
                infos.push(Enums_1.ServiceInfo.PrepayMinerFee);
            }
            return infos;
        };
        this.getWarnings = () => {
            const warnings = [];
            if (!this.allowReverseSwaps) {
                warnings.push(Enums_1.ServiceWarning.ReverseSwapsDisabled);
            }
            return warnings;
        };
        this.getNodes = () => this.nodeInfo.uris;
        this.getNodeStats = () => this.nodeInfo.stats;
        this.getRoutingHints = async (symbol, routingNode) => {
            const hints = await this.swapManager.routingHints.getRoutingHints(symbol, routingNode);
            return hints.map((hop) => ({
                hopHintsList: hop,
            }));
        };
        this.getTimeouts = () => {
            return this.timeoutDeltaProvider.timeoutDeltas;
        };
        /**
         * Gets the contract address used by the Boltz instance
         */
        this.getContracts = async () => {
            if (this.walletManager.ethereumManagers.length === 0) {
                throw Errors_3.default.ETHEREUM_NOT_ENABLED();
            }
            const result = {};
            const transformManager = async (manager) => {
                result[manager.networkDetails.name.toLowerCase()] =
                    await manager.getContractDetails();
            };
            await Promise.all(this.walletManager.ethereumManagers.map((manager) => transformManager(manager)));
            return result;
        };
        /**
         * Gets a hex encoded transaction from a transaction hash on the specified network
         */
        this.getTransaction = async (symbol, transactionHash) => {
            const currency = this.getCurrency(symbol);
            if (currency.chainClient === undefined) {
                throw Errors_3.default.NOT_SUPPORTED_BY_SYMBOL(symbol);
            }
            return await currency.chainClient.getRawTransaction(transactionHash);
        };
        /**
         * Gets the hex encoded lockup transaction of a Submarine Swap, the block height
         * at which it will time out and the expected ETA for that block
         */
        this.getSwapTransaction = async (id) => {
            const swap = await SwapRepository_1.default.getSwap({
                id,
            });
            if (!swap) {
                throw Errors_3.default.SWAP_NOT_FOUND(id);
            }
            if (!swap.lockupTransactionId) {
                throw Errors_3.default.SWAP_NO_LOCKUP();
            }
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
            const currency = this.getCurrency(chainCurrency);
            const response = {
                transactionId: swap.lockupTransactionId,
                timeoutBlockHeight: swap.timeoutBlockHeight,
            };
            let blocks = 0;
            if (currency.chainClient) {
                const chainInfo = await currency.chainClient.getBlockchainInfo();
                blocks = chainInfo.blocks;
                response.transactionHex = await currency.chainClient.getRawTransaction(swap.lockupTransactionId);
            }
            else if (currency.provider) {
                blocks = await currency.provider.getBlockNumber();
            }
            else {
                throw Errors_3.default.NOT_SUPPORTED_BY_SYMBOL(currency.symbol);
            }
            if (blocks < swap.timeoutBlockHeight) {
                response.timeoutEta = this.calculateTimeoutDate(chainCurrency, swap.timeoutBlockHeight - blocks);
            }
            return response;
        };
        this.getReverseSwapTransaction = async (id) => {
            const reverseSwap = await ReverseSwapRepository_1.default.getReverseSwap({
                id,
            });
            if (!reverseSwap) {
                throw Errors_3.default.SWAP_NOT_FOUND(id);
            }
            if (!reverseSwap.transactionId) {
                throw Errors_3.default.SWAP_NO_LOCKUP();
            }
            const { base, quote } = (0, Utils_1.splitPairId)(reverseSwap.pair);
            const currency = this.getCurrency((0, Utils_1.getChainCurrency)(base, quote, reverseSwap.orderSide, true));
            const response = {
                transactionId: reverseSwap.transactionId,
                timeoutBlockHeight: reverseSwap.timeoutBlockHeight,
            };
            if (currency.chainClient) {
                response.transactionHex = await currency.chainClient.getRawTransaction(reverseSwap.transactionId);
            }
            return response;
        };
        this.getReverseBip21 = async (invoice) => {
            const reverseSwap = await ReverseSwapRepository_1.default.getReverseSwap({
                invoice,
            });
            if (!reverseSwap) {
                return undefined;
            }
            const hint = await ReverseRoutingHintRepository_1.default.getHint(reverseSwap.id);
            if (!hint) {
                return undefined;
            }
            return {
                bip21: hint.bip21,
                signature: hint.signature,
            };
        };
        this.deriveKeys = (symbol, index) => {
            const wallet = this.walletManager.wallets.get(symbol.toUpperCase());
            if (wallet === undefined) {
                throw Errors_3.default.CURRENCY_NOT_FOUND(symbol);
            }
            const keys = wallet.getKeysByIndex(index);
            const response = new boltzrpc_pb_1.DeriveKeysResponse();
            response.setPublicKey((0, Utils_1.getHexString)(keys.publicKey));
            response.setPrivateKey((0, Utils_1.getHexString)(keys.privateKey));
            return response;
        };
        /**
         * Gets an address of a specified wallet
         */
        this.getAddress = async (symbol) => {
            const wallet = this.walletManager.wallets.get(symbol);
            if (wallet !== undefined) {
                return wallet.getAddress();
            }
            throw Errors_3.default.CURRENCY_NOT_FOUND(symbol);
        };
        this.getBlockHeights = async (symbol) => {
            const currencies = symbol
                ? [this.getCurrency(symbol)]
                : Array.from(this.currencies.values());
            return new Map((await Promise.all(currencies.map(async (currency) => {
                if (currency.chainClient) {
                    return [
                        currency.symbol,
                        (await currency.chainClient.getBlockchainInfo()).blocks,
                    ];
                }
                else {
                    return [
                        currency.symbol,
                        (await currency.provider?.getBlockNumber()) || 0,
                    ];
                }
            }))));
        };
        /**
         * Gets a fee estimation in satoshis per vbyte or GWEI for either all currencies or just a single one if specified
         */
        this.getFeeEstimation = async (symbol, blocks) => {
            const map = new Map();
            const numBlocks = blocks === undefined ? 2 : blocks;
            const estimateFeeForProvider = async (provider) => Number(await this.getGasPrice(provider)) / Number(Consts_1.gweiDecimals);
            const estimateFee = async (currency) => {
                if (currency.chainClient) {
                    return currency.chainClient.estimateFee(numBlocks);
                }
                else if (currency.provider) {
                    return estimateFeeForProvider(currency.provider);
                }
                else {
                    throw Errors_3.default.NOT_SUPPORTED_BY_SYMBOL(currency.symbol);
                }
            };
            if (symbol !== undefined) {
                const currency = this.getCurrency(symbol);
                if (currency.type !== Enums_1.CurrencyType.ERC20) {
                    map.set(symbol, await estimateFee(currency));
                }
                else {
                    const manager = this.walletManager.ethereumManagers.find((manager) => manager.hasSymbol(currency.symbol));
                    map.set(manager.networkDetails.symbol, await estimateFee(currency));
                }
            }
            else {
                await Promise.all(this.walletManager.ethereumManagers.map(async (manager) => {
                    map.set(manager.networkDetails.symbol, await estimateFeeForProvider(manager.provider));
                }));
                for (const [symbol, currency] of this.currencies) {
                    if (currency.type === Enums_1.CurrencyType.ERC20) {
                        continue;
                    }
                    map.set(symbol, await estimateFee(currency));
                }
            }
            return map;
        };
        /**
         * Broadcast a hex encoded transaction on the specified network
         */
        this.broadcastTransaction = async (symbol, transactionHex) => {
            const currency = this.getCurrency(symbol);
            if (currency.chainClient === undefined) {
                throw Errors_3.default.NOT_SUPPORTED_BY_SYMBOL(symbol);
            }
            const wallet = this.walletManager.wallets.get(symbol);
            const transaction = (0, Core_1.parseTransaction)(currency.type, transactionHex);
            const inputIds = transaction.ins.map((input) => (0, Utils_1.getHexString)((0, Utils_1.reverseBuffer)(input.hash)));
            const [swapsLocked, reverseSwapsClaimed, swapsRefunded] = await Promise.all([
                wallet !== undefined
                    ? SwapRepository_1.default.getSwaps({
                        lockupAddress: {
                            [sequelize_1.Op.in]: transaction.outs
                                // Filter Liquid fee outputs
                                .filter((out) => out.script.length > 0)
                                .map((out) => wallet.encodeAddress(out.script)),
                        },
                    })
                    : new Promise((resolve) => resolve([])),
                ReverseSwapRepository_1.default.getReverseSwaps({
                    transactionId: {
                        [sequelize_1.Op.in]: inputIds,
                    },
                }),
                SwapRepository_1.default.getSwaps({
                    lockupTransactionId: {
                        [sequelize_1.Op.in]: inputIds,
                    },
                }),
            ]);
            const swapsSpent = reverseSwapsClaimed.concat(swapsRefunded);
            // Only allow lowball when Reverse Swaps are being claimed or Submarine Swaps being refunded
            // and no Submarine Swap lockup happens
            // That prevents transactions that we could accept 0-conf for from being broadcast through our API
            const isSwapRelated = swapsLocked.length === 0 && swapsSpent.length > 0;
            if (isSwapRelated) {
                this.logger.debug(`Broadcasting transaction related to Swaps (${swapsSpent.map((r) => r.id).join(', ')}): ${transaction.getId()}`);
            }
            try {
                return await currency.chainClient.sendRawTransaction(transactionHex, isSwapRelated);
            }
            catch (error) {
                // This special error is thrown when a Submarine Swap that has not timed out yet is refunded
                // To improve the UX we will throw not only the error but also some additional information
                // regarding when the Submarine Swap can be refunded
                if (error.code === -26 &&
                    error.message.startsWith('non-mandatory-script-verify-flag (Locktime requirement not satisfied)')) {
                    if (swapsRefunded.length === 0) {
                        throw error;
                    }
                    const { blocks } = await currency.chainClient.getBlockchainInfo();
                    throw {
                        error: error.message,
                        timeoutBlockHeight: swapsRefunded[0].timeoutBlockHeight,
                        // Here we don't need to check whether the Swap has timed out yet because
                        // if the error above has been thrown, we can be sure that this is not the case
                        timeoutEta: this.calculateTimeoutDate(symbol, swapsRefunded[0].timeoutBlockHeight - blocks),
                    };
                }
                else {
                    throw error;
                }
            }
        };
        /**
         * Updates the timeout block delta of a pair
         */
        this.updateTimeoutBlockDelta = (pairId, newDeltas) => {
            this.timeoutDeltaProvider.setTimeout(pairId, newDeltas);
            this.logger.info(`Updated timeout block delta of ${pairId} to ${(0, Utils_1.stringify)(newDeltas)}`);
        };
        this.addReferral = async (referral) => {
            if (referral.id === '') {
                throw new Error('referral IDs cannot be empty');
            }
            if (referral.feeShare > 100 || referral.feeShare < 0) {
                throw new Error('referral fee share must be between 0 and 100');
            }
            const apiKey = (0, Utils_1.createApiCredential)();
            const apiSecret = (0, Utils_1.createApiCredential)();
            await ReferralRepository_1.default.addReferral({
                ...referral,
                apiKey,
                apiSecret,
            });
            this.logger.info(`Added referral ${referral.id} with ${referral.routingNode !== undefined
                ? `routing node ${referral.routingNode} and `
                : ''}fee share ${referral.feeShare}%`);
            return {
                apiKey,
                apiSecret,
            };
        };
        this.setSwapStatus = async (id, status) => {
            if (![
                Enums_1.SwapUpdateEvent.InvoiceFailedToPay,
                Enums_1.SwapUpdateEvent.InvoicePending,
            ].includes(status)) {
                throw Errors_3.default.SET_SWAP_UPDATE_EVENT_NOT_ALLOWED(status);
            }
            const swap = await SwapRepository_1.default.getSwap({ id });
            if (!swap) {
                throw Errors_3.default.SWAP_NOT_FOUND(id);
            }
            await SwapRepository_1.default.setSwapStatus(swap, status, status === Enums_1.SwapUpdateEvent.InvoiceFailedToPay
                ? exports.cancelledViaCliFailureReason
                : undefined);
            // Not the nicest way to do it, but works for the 2 whitelisted events
            this.swapManager.nursery.emit(status, swap);
        };
        this.getLockedFunds = async () => {
            const pendingReverseSwaps = await ReverseSwapRepository_1.default.getReverseSwaps({
                status: {
                    [sequelize_1.Op.or]: [
                        Enums_1.SwapUpdateEvent.TransactionMempool,
                        Enums_1.SwapUpdateEvent.TransactionConfirmed,
                    ],
                },
            });
            return pendingReverseSwaps.reduce((acc, pending) => {
                const pair = (0, Utils_1.splitPairId)(pending.pair);
                acc
                    .get((0, Utils_1.getChainCurrency)(pair.base, pair.quote, pending.orderSide, true))
                    .push(pending);
                return acc;
            }, new DefaultMap_1.default(() => []));
        };
        this.getPendingSweeps = () => {
            return this.swapManager.deferredClaimer.pendingSweepsValues();
        };
        /**
         * Creates a new Swap from the chain to Lightning
         */
        this.createSwap = async (args) => {
            const swap = await SwapRepository_1.default.getSwap({
                preimageHash: (0, Utils_1.getHexString)(args.preimageHash),
            });
            if (swap) {
                throw Errors_3.default.SWAP_WITH_PREIMAGE_EXISTS();
            }
            const { base, quote } = this.getPair(args.pairId);
            const orderSide = this.getOrderSide(args.orderSide);
            switch (this.getCurrency((0, Utils_1.getChainCurrency)(base, quote, orderSide, false)).type) {
                case Enums_1.CurrencyType.BitcoinLike:
                case Enums_1.CurrencyType.Liquid:
                    if (args.refundPublicKey === undefined) {
                        throw Errors_1.default.UNDEFINED_PARAMETER('refundPublicKey');
                    }
                    break;
                case Enums_1.CurrencyType.Ether:
                case Enums_1.CurrencyType.ERC20:
                    break;
                default:
                    if (args.version !== Enums_1.SwapVersion.Legacy) {
                        throw Errors_3.default.UNSUPPORTED_SWAP_VERSION();
                    }
            }
            const lightningCurrency = (0, Utils_1.getLightningCurrency)(base, quote, orderSide, false);
            if (!NodeSwitch_1.default.hasClient(this.getCurrency(lightningCurrency))) {
                throw Errors_2.default.NO_LIGHTNING_SUPPORT(lightningCurrency);
            }
            if (args.channel) {
                if (args.channel.inboundLiquidity > Service.MaxInboundLiquidity) {
                    throw Errors_3.default.EXCEEDS_MAX_INBOUND_LIQUIDITY(args.channel.inboundLiquidity, Service.MaxInboundLiquidity);
                }
                if (args.channel.inboundLiquidity < Service.MinInboundLiquidity) {
                    throw Errors_3.default.BENEATH_MIN_INBOUND_LIQUIDITY(args.channel.inboundLiquidity, Service.MinInboundLiquidity);
                }
            }
            const [timeoutBlockDelta, canBeRouted] = await this.timeoutDeltaProvider.getTimeout(args.pairId, orderSide, false, args.version, args.invoice, args.referralId);
            if (!canBeRouted) {
                this.logger.warn(`Could not query ${lightningCurrency} routes for: ${args.invoice}`);
            }
            const referralId = await this.getReferralId(args.referralId);
            const { id, address, swapTree, timeoutBlockHeight, blindingKey, redeemScript, claimAddress, claimPublicKey, } = await this.swapManager.createSwap({
                orderSide,
                referralId,
                timeoutBlockDelta,
                baseCurrency: base,
                quoteCurrency: quote,
                version: args.version,
                channel: args.channel,
                preimageHash: args.preimageHash,
                refundPublicKey: args.refundPublicKey,
            });
            this.eventHandler.emitSwapCreation(id);
            return {
                id,
                address,
                swapTree,
                canBeRouted,
                blindingKey,
                redeemScript,
                claimAddress,
                claimPublicKey,
                timeoutBlockHeight,
            };
        };
        /**
         * Gets the rates for a Submarine Swap that has coins in its lockup address but no invoice yet
         */
        this.getSwapRates = async (id) => {
            const swap = await SwapRepository_1.default.getSwap({
                id,
            });
            if (!swap) {
                throw Errors_3.default.SWAP_NOT_FOUND(id);
            }
            if (!swap.onchainAmount) {
                throw Errors_3.default.SWAP_NO_LOCKUP();
            }
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const onchainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
            const rate = (0, Utils_1.getRate)(swap.rate, swap.orderSide, false);
            const percentageFee = this.rateProvider.feeProvider.getPercentageFee(swap.pair, false);
            const baseFee = this.rateProvider.feeProvider.getBaseFee(onchainCurrency, swap.version, Enums_1.BaseFeeType.NormalClaim);
            const invoiceAmount = this.calculateInvoiceAmount(swap.orderSide, rate, swap.onchainAmount, baseFee, percentageFee);
            this.verifyAmount(swap.pair, rate, invoiceAmount, swap.orderSide, false);
            return {
                onchainAmount: swap.onchainAmount,
                submarineSwap: {
                    invoiceAmount,
                },
            };
        };
        this.setInvoice = async (id, invoice, pairHash) => {
            const swap = await SwapRepository_1.default.getSwap({
                id,
            });
            if (!swap) {
                throw Errors_3.default.SWAP_NOT_FOUND(id);
            }
            if (swap.invoice) {
                throw Errors_3.default.SWAP_HAS_INVOICE_ALREADY(id);
            }
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const lightningCurrency = (0, Utils_1.getLightningCurrency)(base, quote, swap.orderSide, false);
            swap.invoiceAmount = (0, Utils_1.decodeInvoiceAmount)(invoice);
            const lightningClient = this.nodeSwitch.getSwapNode(this.currencies.get(lightningCurrency), swap);
            const [cltvLimit, decodedInvoice] = await Promise.all([
                this.timeoutDeltaProvider.getCltvLimit(swap),
                lightningClient.decodeInvoice(invoice),
            ]);
            const requiredTimeout = await this.timeoutDeltaProvider.checkRoutability(lightningClient, decodedInvoice, cltvLimit);
            if (requiredTimeout == TimeoutDeltaProvider_1.default.noRoutes) {
                throw Errors_2.default.NO_ROUTE_FOUND();
            }
            return this.setSwapInvoice(swap, invoice, true, pairHash);
        };
        /**
         * Sets the invoice for Submarine Swap
         */
        this.setSwapInvoice = async (swap, invoice, canBeRouted, pairHash) => {
            const { base, quote, rate: pairRate } = this.getPair(swap.pair);
            if (pairHash !== undefined) {
                this.rateProvider.providers[swap.version].validatePairHash(pairHash, swap.pair, swap.orderSide, false);
            }
            const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
            const lightningCurrency = (0, Utils_1.getLightningCurrency)(base, quote, swap.orderSide, false);
            const decodedInvoice = (0, Utils_1.decodeInvoice)(invoice);
            swap.invoiceAmount = decodedInvoice.satoshis;
            const { destination, features } = await this.nodeSwitch
                .getSwapNode(this.getCurrency(lightningCurrency), swap)
                .decodeInvoice(invoice);
            if (this.nodeInfo.isOurNode(destination)) {
                throw Errors_3.default.DESTINATION_BOLTZ_NODE();
            }
            if (features.has(LightningClient_1.InvoiceFeature.AMP)) {
                throw Errors_3.default.AMP_INVOICES_NOT_SUPPORTED();
            }
            const rate = swap.rate || (0, Utils_1.getRate)(pairRate, swap.orderSide, false);
            this.verifyAmount(swap.pair, rate, swap.invoiceAmount, swap.orderSide, false);
            const { baseFee, percentageFee } = this.rateProvider.feeProvider.getFees(swap.pair, swap.version, rate, swap.orderSide, swap.invoiceAmount, Enums_1.BaseFeeType.NormalClaim);
            const expectedAmount = Math.floor(swap.invoiceAmount * rate) + baseFee + percentageFee;
            if (swap.onchainAmount && expectedAmount > swap.onchainAmount) {
                const maxInvoiceAmount = this.calculateInvoiceAmount(swap.orderSide, rate, swap.onchainAmount, baseFee, this.rateProvider.feeProvider.getPercentageFee(swap.pair, false));
                throw Errors_3.default.INVALID_INVOICE_AMOUNT(maxInvoiceAmount);
            }
            const acceptZeroConf = this.rateProvider.acceptZeroConf(chainCurrency, expectedAmount);
            // When we do not accept 0-conf, we make sure there is enough time for a lockup transaction to confirm
            if (!acceptZeroConf) {
                const minutesUntilExpiry = (decodedInvoice.timeExpireDate - (0, Utils_1.getUnixTime)()) / 60;
                if ((TimeoutDeltaProvider_1.default.blockTimes.get(chainCurrency) || 0) * 2 >
                    minutesUntilExpiry) {
                    throw Errors_3.default.INVOICE_EXPIRY_TOO_SHORT();
                }
            }
            await this.swapManager.setSwapInvoice(swap, invoice, swap.invoiceAmount, expectedAmount, percentageFee, acceptZeroConf, canBeRouted, this.eventHandler.emitSwapInvoiceSet);
            return {
                expectedAmount,
                acceptZeroConf,
                bip21: this.paymentRequestUtils.encodeBip21(chainCurrency, swap.lockupAddress, expectedAmount, (0, Utils_1.getSwapMemo)(lightningCurrency, false)),
            };
        };
        /**
         * Creates a Submarine Swap with an invoice
         *
         * This method combines "createSwap" and "setSwapInvoice"
         */
        this.createSwapWithInvoice = async (pairId, orderSide, refundPublicKey, invoice, pairHash, referralId, channel, version = Enums_1.SwapVersion.Legacy) => {
            let swap = await SwapRepository_1.default.getSwap({
                invoice,
            });
            if (swap) {
                throw Errors_3.default.SWAP_WITH_INVOICE_EXISTS();
            }
            const preimageHash = (0, Utils_1.getHexBuffer)((0, Utils_1.decodeInvoice)(invoice).paymentHash);
            (0, Utils_2.checkPreimageHashLength)(preimageHash);
            const { id, address, swapTree, canBeRouted, claimPublicKey, blindingKey, claimAddress, redeemScript, timeoutBlockHeight, } = await this.createSwap({
                pairId,
                channel,
                version,
                invoice,
                orderSide,
                referralId,
                preimageHash,
                refundPublicKey,
            });
            try {
                const { bip21, acceptZeroConf, expectedAmount } = await this.setSwapInvoice((await SwapRepository_1.default.getSwap({
                    id,
                })), invoice, canBeRouted, pairHash);
                return {
                    id,
                    bip21,
                    address,
                    swapTree,
                    blindingKey,
                    claimAddress,
                    redeemScript,
                    acceptZeroConf,
                    expectedAmount,
                    claimPublicKey,
                    timeoutBlockHeight,
                };
            }
            catch (error) {
                const channelCreation = await ChannelCreationRepository_1.default.getChannelCreation({
                    swapId: id,
                });
                await channelCreation?.destroy();
                swap = await SwapRepository_1.default.getSwap({
                    id,
                });
                await swap?.destroy();
                throw error;
            }
        };
        /**
         * Creates a new Swap from Lightning to the chain
         */
        this.createReverseSwap = async (args) => {
            if (!this.allowReverseSwaps) {
                throw Errors_3.default.REVERSE_SWAPS_DISABLED();
            }
            if ((await ReverseSwapRepository_1.default.getReverseSwap({
                preimageHash: (0, Utils_1.getHexString)(args.preimageHash),
            })) !== null) {
                throw Errors_3.default.SWAP_WITH_PREIMAGE_EXISTS();
            }
            const side = this.getOrderSide(args.orderSide);
            const { base, quote, rate: pairRate } = this.getPair(args.pairId);
            if (args.pairHash !== undefined) {
                this.rateProvider.providers[args.version].validatePairHash(args.pairHash, args.pairId, side, true);
            }
            const { sending, receiving } = (0, Utils_1.getSendingReceivingCurrency)(base, quote, side);
            const sendingCurrency = this.getCurrency(sending);
            // Not the pretties way and also not the right spot to do input validation but
            // only at this point in time the type of the sending currency is known
            switch (sendingCurrency.type) {
                case Enums_1.CurrencyType.BitcoinLike:
                    if (args.claimPublicKey === undefined) {
                        throw Errors_1.default.UNDEFINED_PARAMETER('claimPublicKey');
                    }
                    if (args.prepayMinerFee === true) {
                        throw Errors_1.default.UNSUPPORTED_PARAMETER(sending, 'prepayMinerFee');
                    }
                    break;
                case Enums_1.CurrencyType.Ether:
                case Enums_1.CurrencyType.ERC20:
                    if (args.claimAddress === undefined) {
                        throw Errors_1.default.UNDEFINED_PARAMETER('claimAddress');
                    }
                    try {
                        // Get a checksum address and verify that the address is valid
                        args.claimAddress = (0, ethers_1.getAddress)(args.claimAddress);
                    }
                    catch (error) {
                        throw Errors_3.default.INVALID_ETHEREUM_ADDRESS();
                    }
                    break;
            }
            const [onchainTimeoutBlockDelta] = await this.timeoutDeltaProvider.getTimeout(args.pairId, side, true, args.version);
            let lightningTimeoutBlockDelta = TimeoutDeltaProvider_1.default.convertBlocks(sending, receiving, onchainTimeoutBlockDelta);
            // Add 15 blocks to the delta for same currency swaps and 25% for cross chain ones as buffer
            lightningTimeoutBlockDelta +=
                sending === receiving ? 15 : Math.ceil(lightningTimeoutBlockDelta * 0.25);
            const rate = (0, Utils_1.getRate)(pairRate, side, true);
            const feePercent = this.rateProvider.feeProvider.getPercentageFee(args.pairId, true);
            const baseFee = this.rateProvider.feeProvider.getBaseFee(sendingCurrency.symbol, args.version, Enums_1.BaseFeeType.ReverseLockup);
            let onchainAmount;
            let holdInvoiceAmount;
            let percentageFee;
            // True when the invoice amount was set in the request, false when the onchain amount was set
            let invoiceAmountDefined;
            if (args.invoiceAmount !== undefined && args.onchainAmount !== undefined) {
                throw Errors_3.default.INVOICE_AND_ONCHAIN_AMOUNT_SPECIFIED();
            }
            else if (args.invoiceAmount !== undefined) {
                invoiceAmountDefined = true;
                this.checkWholeNumber(args.invoiceAmount);
                holdInvoiceAmount = args.invoiceAmount;
                onchainAmount = args.invoiceAmount * rate;
                percentageFee = Math.ceil(feePercent * onchainAmount);
                onchainAmount -= percentageFee + baseFee;
                onchainAmount = Math.floor(onchainAmount);
            }
            else if (args.onchainAmount !== undefined) {
                invoiceAmountDefined = false;
                this.checkWholeNumber(args.onchainAmount);
                onchainAmount = args.onchainAmount;
                holdInvoiceAmount = (args.onchainAmount + baseFee) / rate;
                holdInvoiceAmount = holdInvoiceAmount / (1 - feePercent);
                holdInvoiceAmount = Math.ceil(holdInvoiceAmount);
                percentageFee = Math.ceil(holdInvoiceAmount * rate * feePercent);
            }
            else {
                throw Errors_3.default.NO_AMOUNT_SPECIFIED();
            }
            this.verifyAmount(args.pairId, rate, holdInvoiceAmount, side, true);
            let prepayMinerFeeInvoiceAmount = undefined;
            let prepayMinerFeeOnchainAmount = undefined;
            const swapIsPrepayMinerFee = this.prepayMinerFee || args.prepayMinerFee === true;
            if (swapIsPrepayMinerFee) {
                if (sendingCurrency.type === Enums_1.CurrencyType.BitcoinLike ||
                    sendingCurrency.type === Enums_1.CurrencyType.Liquid) {
                    prepayMinerFeeInvoiceAmount = Math.ceil(baseFee / rate);
                    holdInvoiceAmount = Math.floor(holdInvoiceAmount - prepayMinerFeeInvoiceAmount);
                }
                else {
                    const gasPrice = await this.getGasPrice(sendingCurrency.provider);
                    prepayMinerFeeOnchainAmount = Number((gasPrice * Consts_1.ethereumPrepayMinerFeeGasLimit) / Consts_1.etherDecimals);
                    const chainFeeSymbol = this.walletManager.ethereumManagers.find((manager) => manager.hasSymbol(sendingCurrency.symbol)).networkDetails.symbol;
                    const sendingAmountRate = sending === chainFeeSymbol
                        ? 1
                        : this.rateProvider.rateCalculator.calculateRate(chainFeeSymbol, sending);
                    const receivingAmountRate = receiving === chainFeeSymbol
                        ? 1
                        : this.rateProvider.rateCalculator.calculateRate(chainFeeSymbol, receiving);
                    prepayMinerFeeInvoiceAmount = Math.ceil(prepayMinerFeeOnchainAmount * receivingAmountRate);
                    // If the invoice amount was specified, the onchain and hold invoice amounts need to be adjusted
                    if (invoiceAmountDefined) {
                        onchainAmount -= Math.ceil(prepayMinerFeeOnchainAmount * sendingAmountRate);
                        holdInvoiceAmount = Math.floor(holdInvoiceAmount - prepayMinerFeeInvoiceAmount);
                    }
                }
            }
            if (onchainAmount < 1) {
                throw Errors_3.default.ONCHAIN_AMOUNT_TOO_LOW();
            }
            const referralId = await this.getReferralId(args.referralId, args.routingNode);
            const { id, invoice, swapTree, blindingKey, redeemScript, refundAddress, lockupAddress, refundPublicKey, minerFeeInvoice, timeoutBlockHeight, } = await this.swapManager.createReverseSwap({
                referralId,
                percentageFee,
                onchainAmount,
                holdInvoiceAmount,
                onchainTimeoutBlockDelta,
                lightningTimeoutBlockDelta,
                prepayMinerFeeInvoiceAmount,
                prepayMinerFeeOnchainAmount,
                orderSide: side,
                baseCurrency: base,
                quoteCurrency: quote,
                version: args.version,
                routingNode: args.routingNode,
                userAddress: args.userAddress,
                claimAddress: args.claimAddress,
                preimageHash: args.preimageHash,
                claimPublicKey: args.claimPublicKey,
                claimCovenant: args.claimCovenant || false,
                userAddressSignature: args.userAddressSignature,
            });
            this.eventHandler.emitSwapCreation(id);
            const response = {
                id,
                invoice,
                swapTree,
                blindingKey,
                redeemScript,
                refundAddress,
                lockupAddress,
                refundPublicKey,
                timeoutBlockHeight,
            };
            if (swapIsPrepayMinerFee) {
                response.minerFeeInvoice = minerFeeInvoice;
                response.prepayMinerFeeAmount = prepayMinerFeeOnchainAmount;
            }
            if (invoiceAmountDefined) {
                response.onchainAmount = onchainAmount;
            }
            return response;
        };
        /**
         * Pays a lightning invoice
         */
        this.payInvoice = async (symbol, invoice) => {
            const currency = this.getCurrency(symbol);
            const lightningClient = currency.lndClient || currency.clnClient;
            if (lightningClient === undefined) {
                throw Errors_2.default.NO_LIGHTNING_SUPPORT(symbol);
            }
            return lightningClient.sendPayment(invoice);
        };
        /**
         * Sends coins to a specified address
         */
        this.sendCoins = async (args) => {
            const { fee, amount, symbol, sendAll, address } = args;
            const wallet = this.walletManager.wallets.get(symbol);
            if (wallet !== undefined) {
                const { transactionId, vout } = sendAll
                    ? await wallet.sweepWallet(address, fee)
                    : await wallet.sendToAddress(address, amount, fee);
                return {
                    transactionId,
                    vout: vout,
                };
            }
            throw Errors_3.default.CURRENCY_NOT_FOUND(symbol);
        };
        this.getGasPrice = async (provider) => {
            const feeData = await provider.getFeeData();
            return (feeData.gasPrice || feeData.maxFeePerGas);
        };
        this.getReferralId = async (referralId, routingNode) => {
            // An explicitly set referral ID trumps the routing node
            if (referralId) {
                return referralId;
            }
            if (routingNode) {
                const referral = await ReferralRepository_1.default.getReferralByRoutingNode(routingNode);
                if (referral) {
                    return referral.id;
                }
            }
            return;
        };
        /**
         * Verifies that the requested amount is neither above the maximal nor beneath the minimal
         */
        this.verifyAmount = (pairId, rate, amount, orderSide, isReverse) => {
            if ((!isReverse && orderSide === Enums_1.OrderSide.BUY) ||
                (isReverse && orderSide === Enums_1.OrderSide.SELL)) {
                amount = Math.floor(amount * rate);
            }
            const { limits } = this.getPair(pairId);
            if (limits) {
                if (Math.floor(amount) > limits.maximal)
                    throw Errors_3.default.EXCEED_MAXIMAL_AMOUNT(amount, limits.maximal);
                if (Math.ceil(amount) < limits.minimal)
                    throw Errors_3.default.BENEATH_MINIMAL_AMOUNT(amount, limits.minimal);
            }
            else {
                throw Errors_3.default.PAIR_NOT_FOUND(pairId);
            }
        };
        /**
         * Calculates the amount of an invoice for a Submarine Swap
         */
        this.calculateInvoiceAmount = (orderSide, rate, onchainAmount, baseFee, percentageFee) => {
            if (orderSide === Enums_1.OrderSide.BUY) {
                rate = 1 / rate;
            }
            return Math.floor(((onchainAmount - baseFee) * rate) / (1 + percentageFee));
        };
        this.getPair = (pairId) => {
            const { base, quote } = (0, Utils_1.splitPairId)(pairId);
            const pair = this.rateProvider.providers[Enums_1.SwapVersion.Legacy].pairs.get(pairId);
            if (!pair) {
                throw Errors_3.default.PAIR_NOT_FOUND(pairId);
            }
            return {
                base,
                quote,
                ...pair,
            };
        };
        this.getCurrency = (symbol) => {
            const currency = this.currencies.get(symbol);
            if (!currency) {
                throw Errors_3.default.CURRENCY_NOT_FOUND(symbol);
            }
            return currency;
        };
        this.getOrderSide = (side) => {
            switch (side.toLowerCase()) {
                case 'buy':
                    return Enums_1.OrderSide.BUY;
                case 'sell':
                    return Enums_1.OrderSide.SELL;
                default:
                    throw Errors_3.default.ORDER_SIDE_NOT_FOUND(side);
            }
        };
        this.calculateTimeoutDate = (chain, blocksMissing) => {
            return ((0, Utils_1.getUnixTime)() +
                blocksMissing * TimeoutDeltaProvider_1.default.blockTimes.get(chain) * 60);
        };
        this.checkWholeNumber = (input) => {
            if (input % 1 !== 0) {
                throw Errors_3.default.NOT_WHOLE_NUMBER(input);
            }
        };
        this.prepayMinerFee = config.prepayminerfee;
        this.logger.debug(`Prepay miner fee for Reverse Swaps is ${this.prepayMinerFee ? 'enabled' : 'disabled'}`);
        this.paymentRequestUtils = new PaymentRequestUtils_1.default(this.currencies.get(ElementsClient_1.default.symbol));
        this.timeoutDeltaProvider = new TimeoutDeltaProvider_1.default(this.logger, config, currencies, this.nodeSwitch);
        this.rateProvider = new RateProvider_1.default(this.logger, config.rates.interval, currencies, this.walletManager, this.getFeeEstimation);
        this.logger.debug(`Using ${config.swapwitnessaddress ? 'P2WSH' : 'P2SH nested P2WSH'} addresses for legacy Bitcoin Submarine Swaps`);
        this.lockupTransactionTracker = new LockupTransactionTracker_1.default(this.logger, this.currencies, this.rateProvider);
        this.swapManager = new SwapManager_1.default(this.logger, this.walletManager, this.nodeSwitch, this.rateProvider, this.timeoutDeltaProvider, this.paymentRequestUtils, new SwapOutputType_1.default(config.swapwitnessaddress
            ? boltz_core_1.OutputType.Bech32
            : boltz_core_1.OutputType.Compatibility), config.retryInterval, blocks, config.swap, this.lockupTransactionTracker);
        this.eventHandler = new EventHandler_1.default(this.logger, this.currencies, this.swapManager.nursery);
        this.nodeInfo = new NodeInfo_1.default(this.logger, this.currencies);
        this.elementsService = new ElementsService_1.default(this.currencies, this.walletManager);
        this.eipSigner = new EipSigner_1.default(this.logger, this.currencies, this.walletManager);
        this.musigSigner = new MusigSigner_1.default(this.logger, this.currencies, this.walletManager, this.swapManager.nursery);
    }
}
Service.MinInboundLiquidity = 10;
Service.MaxInboundLiquidity = 50;
exports.cancelledViaCliFailureReason = 'payment has been cancelled';
exports.default = Service;
//# sourceMappingURL=Service.js.map