"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const toml_1 = __importDefault(require("@iarna/toml"));
const fs_1 = __importDefault(require("fs"));
const Utils_1 = require("../Utils");
const ElementsClient_1 = __importDefault(require("../chain/ElementsClient"));
const Enums_1 = require("../consts/Enums");
const LightningClient_1 = require("../lightning/LightningClient");
const LndClient_1 = __importDefault(require("../lightning/LndClient"));
const NodeSwitch_1 = __importDefault(require("../swap/NodeSwitch"));
const EvmNetworks_1 = require("../wallet/ethereum/EvmNetworks");
const Errors_1 = __importDefault(require("./Errors"));
const RoutingOffsets_1 = __importDefault(require("./RoutingOffsets"));
class TimeoutDeltaProvider {
    constructor(logger, config, currencies, nodeSwitch) {
        this.logger = logger;
        this.config = config;
        this.currencies = currencies;
        this.nodeSwitch = nodeSwitch;
        this.timeoutDeltas = new Map();
        this.init = (pairs, ethereumManagers) => {
            // Set the block time of the chain for its tokens
            ethereumManagers.forEach((manager) => {
                const blockTime = TimeoutDeltaProvider.blockTimes.get(manager.networkDetails.symbol);
                for (const token of manager.tokenAddresses.keys()) {
                    TimeoutDeltaProvider.blockTimes.set(token, blockTime);
                }
            });
            for (const pair of pairs) {
                const pairId = (0, Utils_1.getPairId)(pair);
                if (pair.timeoutDelta !== undefined) {
                    // Compatibility mode with legacy config
                    if (typeof pair.timeoutDelta === 'number') {
                        pair.timeoutDelta = {
                            reverse: pair.timeoutDelta,
                            swapMaximal: pair.timeoutDelta,
                            swapMinimal: pair.timeoutDelta,
                            swapTaproot: pair.timeoutDelta,
                        };
                    }
                    this.logger.debug(`Setting timeout block delta of ${pairId} to minutes: ${(0, Utils_1.stringify)(pair.timeoutDelta)}`);
                    this.timeoutDeltas.set(pairId, this.minutesToBlocks(pairId, pair.timeoutDelta));
                }
                else {
                    throw Errors_1.default.NO_TIMEOUT_DELTA(pairId);
                }
            }
        };
        this.setTimeout = (pairId, newDeltas) => {
            if (this.timeoutDeltas.has(pairId)) {
                const blocks = this.minutesToBlocks(pairId, newDeltas);
                this.timeoutDeltas.set(pairId, blocks);
                for (let i = 0; i < this.config.pairs.length; i += 1) {
                    if ((0, Utils_1.getPairId)(this.config.pairs[i]) === pairId) {
                        this.config.pairs[i].timeoutDelta = newDeltas;
                        break;
                    }
                }
                // Write the new config to the disk
                const newConfig = toml_1.default.stringify(this.config);
                fs_1.default.writeFileSync(this.config.configpath, newConfig);
            }
            else {
                throw Errors_1.default.PAIR_NOT_FOUND(pairId);
            }
        };
        this.getCltvLimit = async (swap) => {
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const chainCurrency = this.currencies.get((0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false));
            const currentBlock = chainCurrency.chainClient
                ? (await chainCurrency.chainClient.getBlockchainInfo()).blocks
                : await chainCurrency.provider.getBlockNumber();
            const blockLeft = TimeoutDeltaProvider.convertBlocks(chainCurrency.symbol, (0, Utils_1.getLightningCurrency)(base, quote, swap.orderSide, false), swap.timeoutBlockHeight - currentBlock);
            return Math.floor(blockLeft - 2);
        };
        this.getTimeout = async (pairId, orderSide, isReverse, version, invoice, referralId) => {
            const timeouts = this.timeoutDeltas.get(pairId);
            if (!timeouts) {
                throw Errors_1.default.PAIR_NOT_FOUND(pairId);
            }
            if (isReverse) {
                return [
                    orderSide === Enums_1.OrderSide.BUY
                        ? timeouts.base.reverse
                        : timeouts.quote.reverse,
                    false,
                ];
            }
            else {
                const { base, quote } = (0, Utils_1.splitPairId)(pairId);
                const chain = (0, Utils_1.getChainCurrency)(base, quote, orderSide, false);
                const lightning = (0, Utils_1.getLightningCurrency)(base, quote, orderSide, false);
                const chainDeltas = orderSide === Enums_1.OrderSide.BUY ? timeouts.quote : timeouts.base;
                const lightningDeltas = orderSide === Enums_1.OrderSide.BUY ? timeouts.base : timeouts.quote;
                return invoice
                    ? await this.getTimeoutInvoice(pairId, chain, lightning, chainDeltas, lightningDeltas, version, invoice, referralId)
                    : [chainDeltas.swapMinimal, true];
            }
        };
        this.checkRoutability = async (lightningClient, decodedInvoice, cltvLimit) => {
            try {
                // Check whether the receiving side supports MPP and if so,
                // query a route for the number of sats of the invoice divided
                // by the max payment parts we tell to LND to use
                const supportsMpp = decodedInvoice.features.has(LightningClient_1.InvoiceFeature.MPP);
                // TODO: CLN adjustments
                const amountToQuery = Math.max(supportsMpp
                    ? Math.ceil(decodedInvoice.value / LndClient_1.default.paymentMaxParts)
                    : decodedInvoice.value, 1);
                const routes = await lightningClient.queryRoutes(decodedInvoice.destination, amountToQuery, cltvLimit, decodedInvoice.cltvExpiry, decodedInvoice.routingHints);
                return routes.reduce((highest, r) => (highest > r.ctlv ? highest : r.ctlv), TimeoutDeltaProvider.noRoutes);
            }
            catch (error) {
                this.logger.debug(`Could not query routes: ${(0, Utils_1.formatError)(error)}`);
                return TimeoutDeltaProvider.noRoutes;
            }
        };
        this.getTimeoutInvoice = async (pair, chainCurrency, lightningCurrency, chainTimeout, lightningTimeout, version, invoice, referralId) => {
            const currency = this.currencies.get(lightningCurrency);
            const decodedInvoice = await NodeSwitch_1.default.fallback(currency).decodeInvoice(invoice);
            const lightningClient = this.nodeSwitch.getSwapNode(currency, {
                referral: referralId,
                invoiceAmount: decodedInvoice.value,
            });
            const lightningCltv = version === Enums_1.SwapVersion.Taproot
                ? lightningTimeout.swapTaproot
                : lightningTimeout.swapMaximal;
            const [routeTimeLock, chainInfo] = await Promise.all([
                this.checkRoutability(lightningClient, decodedInvoice, lightningCltv),
                currency.chainClient.getBlockchainInfo(),
            ]);
            if (routeTimeLock === TimeoutDeltaProvider.noRoutes) {
                return [chainTimeout.swapMaximal, false];
            }
            const routeDeltaRelative = lightningClient instanceof LndClient_1.default
                ? routeTimeLock - chainInfo.blocks
                : routeTimeLock;
            this.logger.debug(`CLTV needed to route: ${routeDeltaRelative} ${lightningCurrency} blocks`);
            if (version === Enums_1.SwapVersion.Taproot) {
                return [
                    Math.ceil((lightningCltv *
                        TimeoutDeltaProvider.blockTimes.get(lightningCurrency)) /
                        TimeoutDeltaProvider.blockTimes.get(chainCurrency)),
                    true,
                ];
            }
            // Add some buffer to make sure we have enough limit when the transaction confirms
            const routeDeltaMinutes = Math.ceil(routeDeltaRelative *
                TimeoutDeltaProvider.blockTimes.get(lightningCurrency));
            const routingOffset = this.routingOffsets.getOffset(pair, decodedInvoice.value, lightningCurrency, [decodedInvoice.destination].concat(decodedInvoice.routingHints.map((hints) => hints[0].nodeId)));
            const finalExpiry = routeDeltaMinutes + routingOffset;
            const minTimeout = Math.ceil(finalExpiry / TimeoutDeltaProvider.blockTimes.get(chainCurrency));
            if (minTimeout > chainTimeout.swapMaximal) {
                throw Errors_1.default.MIN_EXPIRY_TOO_BIG(Math.ceil(chainTimeout.swapMaximal *
                    TimeoutDeltaProvider.blockTimes.get(chainCurrency)), routeDeltaMinutes, routingOffset);
            }
            const cltv = Math.max(chainTimeout.swapMinimal, minTimeout);
            this.logger.debug(`Using timeout of: ${cltv} ${chainCurrency} blocks`);
            return [cltv, true];
        };
        this.minutesToBlocks = (pair, newDeltas) => {
            const calculateBlocks = (symbol, minutes) => {
                const minutesPerBlock = TimeoutDeltaProvider.blockTimes.get(symbol);
                const blocks = minutes / minutesPerBlock;
                // Sanity checks to make sure no impossible deltas are set
                if (blocks % 1 !== 0 || blocks < 1) {
                    throw Errors_1.default.INVALID_TIMEOUT_BLOCK_DELTA();
                }
                return blocks;
            };
            const convertToBlocks = (symbol) => {
                return {
                    reverse: calculateBlocks(symbol, newDeltas.reverse),
                    swapMinimal: calculateBlocks(symbol, newDeltas.swapMinimal),
                    swapMaximal: calculateBlocks(symbol, newDeltas.swapMaximal),
                    swapTaproot: calculateBlocks(symbol, newDeltas.swapTaproot),
                };
            };
            const { base, quote } = (0, Utils_1.splitPairId)(pair);
            return {
                base: convertToBlocks(base),
                quote: convertToBlocks(quote),
            };
        };
        this.routingOffsets = new RoutingOffsets_1.default(this.logger, config);
    }
}
TimeoutDeltaProvider.noRoutes = -1;
// A map of the symbols of currencies and their block times in minutes
TimeoutDeltaProvider.blockTimes = new Map([
    ['BTC', 10],
    ['LTC', 2.5],
    [EvmNetworks_1.Rsk.symbol, 0.5],
    [EvmNetworks_1.Ethereum.symbol, 0.2],
    [ElementsClient_1.default.symbol, 1],
]);
TimeoutDeltaProvider.convertBlocks = (fromSymbol, toSymbol, blocks) => {
    const minutes = blocks * TimeoutDeltaProvider.blockTimes.get(fromSymbol);
    // In the context this function is used, we calculate the timeout of the first leg of a
    // reverse swap which has to be longer than the second one
    return Math.ceil(minutes / TimeoutDeltaProvider.blockTimes.get(toSymbol));
};
exports.default = TimeoutDeltaProvider;
//# sourceMappingURL=TimeoutDeltaProvider.js.map