"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const async_lock_1 = __importDefault(require("async-lock"));
const boltz_core_1 = require("boltz-core");
const node_schedule_1 = require("node-schedule");
const Core_1 = require("../../Core");
const Utils_1 = require("../../Utils");
const Enums_1 = require("../../consts/Enums");
const TypedEventEmitter_1 = __importDefault(require("../../consts/TypedEventEmitter"));
const ChannelCreationRepository_1 = __importDefault(require("../../db/repositories/ChannelCreationRepository"));
const SwapRepository_1 = __importDefault(require("../../db/repositories/SwapRepository"));
const Errors_1 = __importDefault(require("../Errors"));
const TimeoutDeltaProvider_1 = __importDefault(require("../TimeoutDeltaProvider"));
class DeferredClaimer extends TypedEventEmitter_1.default {
    constructor(logger, currencies, walletManager, swapOutputType, config) {
        super();
        this.logger = logger;
        this.currencies = currencies;
        this.walletManager = walletManager;
        this.swapOutputType = swapOutputType;
        this.config = config;
        this.lock = new async_lock_1.default();
        this.swapsToClaim = new Map();
        this.init = async () => {
            this.logger.verbose(`Using deferred claims for: ${this.config.deferredClaimSymbols.join(', ')}`);
            this.logger.verbose(`Batch claim interval: ${this.config.batchClaimInterval} with expiry tolerance of ${this.config.expiryTolerance} minutes`);
            await this.batchClaimLeftovers();
            this.batchClaimSchedule = (0, node_schedule_1.scheduleJob)(this.config.batchClaimInterval, async () => {
                await this.sweep();
            });
        };
        this.close = () => {
            this.batchClaimSchedule?.cancel();
            this.batchClaimSchedule = undefined;
        };
        this.pendingSweeps = () => new Map(Array.from(this.swapsToClaim.entries()).map(([currency, swaps]) => [
            currency,
            Array.from(swaps.keys()),
        ]));
        this.sweep = async () => {
            const claimed = new Map();
            for (const symbol of this.config.deferredClaimSymbols) {
                const ids = await this.sweepSymbol(symbol);
                if (ids.length > 0) {
                    claimed.set(symbol, ids);
                }
            }
            return claimed;
        };
        this.sweepSymbol = async (symbol) => {
            let claimedSwaps = [];
            await this.lock.acquire(DeferredClaimer.batchClaimLock, async () => {
                claimedSwaps = await this.batchClaim(symbol);
            });
            return claimedSwaps;
        };
        this.deferClaim = async (swap, preimage) => {
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
            if (!this.shouldBeDeferred(chainCurrency, swap)) {
                return false;
            }
            this.logger.verbose(`Deferring claim of ${swap.id} to next ${chainCurrency} batch`);
            swap = await SwapRepository_1.default.setSwapStatus(swap, Enums_1.SwapUpdateEvent.TransactionClaimPending);
            await this.lock.acquire(DeferredClaimer.swapsToClaimLock, async () => {
                this.swapsToClaim.get(chainCurrency).set(swap.id, {
                    swap,
                    preimage,
                });
            });
            if (await this.expiryTooSoon(chainCurrency, swap)) {
                await this.sweepSymbol(chainCurrency);
            }
            return true;
        };
        this.getCooperativeDetails = async (swap) => {
            const { toClaim, chainCurrency } = await this.getToClaimDetails(swap);
            if (toClaim === undefined) {
                throw Errors_1.default.NOT_ELIGIBLE_FOR_COOPERATIVE_CLAIM();
            }
            const wallet = this.walletManager.wallets.get(chainCurrency.symbol);
            const address = toClaim.cooperative?.sweepAddress || (await wallet.getAddress());
            toClaim.cooperative = {
                sweepAddress: address,
                musig: (0, Core_1.createMusig)(wallet.getKeysByIndex(swap.keyIndex), (0, Utils_1.getHexBuffer)(swap.refundPublicKey)),
                transaction: (0, Core_1.constructClaimTransaction)(wallet, [
                    await this.constructClaimDetails(chainCurrency.chainClient, wallet, toClaim, true),
                ], address, await chainCurrency.chainClient.estimateFee()),
            };
            return {
                preimage: toClaim.preimage,
                publicKey: wallet.getKeysByIndex(swap.keyIndex).publicKey,
                pubNonce: Buffer.from(toClaim.cooperative.musig.getPublicNonce()),
                transactionHash: await (0, Core_1.hashForWitnessV1)(chainCurrency, toClaim.cooperative.transaction, 0),
            };
        };
        this.broadcastCooperative = async (swap, theirPubNonce, theirPartialSignature) => {
            const { toClaim, chainCurrency } = await this.getToClaimDetails(swap);
            if (toClaim === undefined || toClaim.cooperative === undefined) {
                throw Errors_1.default.NOT_ELIGIBLE_FOR_COOPERATIVE_CLAIM_BROADCAST();
            }
            const { musig, transaction } = toClaim.cooperative;
            (0, Core_1.tweakMusig)(chainCurrency.type, musig, boltz_core_1.SwapTreeSerializer.deserializeSwapTree(swap.redeemScript));
            const theirPublicKey = (0, Utils_1.getHexBuffer)(swap.refundPublicKey);
            musig.aggregateNonces([[theirPublicKey, theirPubNonce]]);
            musig.initializeSession(await (0, Core_1.hashForWitnessV1)(chainCurrency, toClaim.cooperative.transaction, 0));
            if (!musig.verifyPartial(theirPublicKey, theirPartialSignature)) {
                throw Errors_1.default.INVALID_PARTIAL_SIGNATURE();
            }
            musig.addPartial(theirPublicKey, theirPartialSignature);
            musig.signPartial();
            transaction.ins[0].witness = [musig.aggregatePartials()];
            this.logger.info(`Broadcasting cooperative ${chainCurrency.symbol} claim of Swap ${swap.id} in: ${transaction.getId()}`);
            await chainCurrency.chainClient.sendRawTransaction(transaction.toHex(), true);
            await this.lock.acquire(DeferredClaimer.swapsToClaimLock, async () => {
                this.swapsToClaim.get(chainCurrency.symbol)?.delete(swap.id);
            });
            this.emit('claim', {
                swap: await SwapRepository_1.default.setMinerFee(toClaim.swap, await (0, Core_1.calculateTransactionFee)(chainCurrency.chainClient, transaction)),
                channelCreation: (await ChannelCreationRepository_1.default.getChannelCreation({
                    swapId: toClaim.swap.id,
                })) || undefined,
            });
        };
        this.batchClaim = async (symbol) => {
            let swapsToClaim = [];
            await this.lock.acquire(DeferredClaimer.swapsToClaimLock, async () => {
                const swaps = this.swapsToClaim.get(symbol);
                if (swaps === undefined) {
                    return;
                }
                swapsToClaim = Array.from(swaps.values());
                swaps.clear();
            });
            try {
                if (swapsToClaim.length === 0) {
                    this.logger.silly(`Not batch claiming swaps for currency ${symbol}: no swaps to claim`);
                    return [];
                }
                this.logger.verbose(`Batch claiming swaps for currency: ${symbol}`);
                return await this.broadcastClaim(symbol, swapsToClaim);
            }
            catch (e) {
                this.logger.warn(`Batch claim for currency ${symbol} failed: ${(0, Utils_1.formatError)(e)}`);
                await this.lock.acquire(DeferredClaimer.swapsToClaimLock, async () => {
                    const map = this.swapsToClaim.get(symbol);
                    for (const toClaim of swapsToClaim) {
                        map.set(toClaim.swap.id, toClaim);
                    }
                });
                throw e;
            }
        };
        this.broadcastClaim = async (currency, swaps) => {
            const chainClient = this.currencies.get(currency).chainClient;
            const wallet = this.walletManager.wallets.get(currency);
            const claimDetails = (await Promise.all(swaps.map((swap) => this.constructClaimDetails(chainClient, wallet, swap))));
            const claimTransaction = (0, Core_1.constructClaimTransaction)(wallet, claimDetails, await wallet.getAddress(), await chainClient.estimateFee());
            const transactionFeePerSwap = Math.ceil((await (0, Core_1.calculateTransactionFee)(chainClient, claimTransaction)) /
                swaps.length);
            await chainClient.sendRawTransaction(claimTransaction.toHex(), true);
            this.logger.info(`Claimed ${wallet.symbol} of Swaps ${swaps
                .map((toClaim) => toClaim.swap.id)
                .join(', ')} in: ${claimTransaction.getId()}`);
            for (const toClaim of swaps) {
                this.emit('claim', {
                    swap: await SwapRepository_1.default.setMinerFee(toClaim.swap, transactionFeePerSwap),
                    channelCreation: (await ChannelCreationRepository_1.default.getChannelCreation({
                        swapId: toClaim.swap.id,
                    })) || undefined,
                });
            }
            return swaps.map((toClaim) => toClaim.swap.id);
        };
        this.constructClaimDetails = async (chainClient, wallet, toClaim, cooperative = false) => {
            const { swap, preimage } = toClaim;
            const tx = (0, Core_1.parseTransaction)(wallet.type, await chainClient.getRawTransaction(swap.lockupTransactionId));
            return (0, Core_1.constructClaimDetails)(this.swapOutputType, wallet, swap, tx, preimage, cooperative);
        };
        this.shouldBeDeferred = (chainCurrency, swap) => {
            if (!this.config.deferredClaimSymbols.includes(chainCurrency)) {
                this.logNotDeferringReason(swap, `${chainCurrency} transactions should not be deferred`);
                return false;
            }
            if (swap.version !== Enums_1.SwapVersion.Taproot) {
                this.logNotDeferringReason(swap, 'version is legacy');
                return false;
            }
            return true;
        };
        this.expiryTooSoon = async (chainCurrency, swap) => {
            const chainClient = this.currencies.get(chainCurrency).chainClient;
            const { blocks } = await chainClient.getBlockchainInfo();
            const minutesLeft = TimeoutDeltaProvider_1.default.blockTimes.get(chainCurrency) *
                (swap.timeoutBlockHeight - blocks);
            return minutesLeft <= this.config.expiryTolerance;
        };
        this.logNotDeferringReason = (swap, reason) => {
            this.logger.debug(`Not deferring claim of Swap ${swap.id}: ${reason}`);
        };
        this.batchClaimLeftovers = async () => {
            const swapsToClaim = await SwapRepository_1.default.getSwapsClaimable();
            for (const swap of swapsToClaim) {
                const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
                const { lndClient, clnClient } = this.currencies.get((0, Utils_1.getLightningCurrency)(base, quote, swap.orderSide, false));
                const paymentRes = (await Promise.allSettled([
                    lndClient
                        ?.trackPayment((0, Utils_1.getHexBuffer)(swap.preimageHash))
                        .then((res) => (0, Utils_1.getHexBuffer)(res.paymentPreimage)),
                    clnClient?.checkPayStatus(swap.invoice).then((res) => res?.preimage),
                ]))
                    .filter((res) => res.status === 'fulfilled')
                    .map((res) => res.value)
                    .filter((res) => res !== undefined);
                if (paymentRes.length === 0) {
                    this.logger.warn(`Could not prepare claim of Swap ${swap.id}: no lightning client has preimage`);
                    continue;
                }
                await this.lock.acquire(DeferredClaimer.swapsToClaimLock, async () => {
                    this.swapsToClaim
                        .get((0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false))
                        .set(swap.id, {
                        swap,
                        preimage: paymentRes[0],
                    });
                });
            }
            await this.sweep();
        };
        this.getToClaimDetails = async (swap) => {
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const chainCurrency = this.currencies.get((0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false));
            let toClaim;
            await this.lock.acquire(DeferredClaimer.swapsToClaimLock, async () => {
                toClaim = this.swapsToClaim.get(chainCurrency.symbol)?.get(swap.id);
            });
            return {
                toClaim,
                chainCurrency,
            };
        };
        for (const symbol of config.deferredClaimSymbols) {
            this.swapsToClaim.set(symbol, new Map());
        }
    }
    pendingSweepsValues() {
        return new Map(Array.from(this.swapsToClaim.entries()).map(([currency, swaps]) => [
            currency,
            Array.from(swaps.values()),
        ]));
    }
}
DeferredClaimer.batchClaimLock = 'batchClaim';
DeferredClaimer.swapsToClaimLock = 'swapsToClaim';
exports.default = DeferredClaimer;
//# sourceMappingURL=DeferredClaimer.js.map