"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../../Utils");
const Consts_1 = require("../../consts/Consts");
const SwapRepository_1 = __importDefault(require("../../db/repositories/SwapRepository"));
const Errors_1 = __importDefault(require("../Errors"));
const MusigSigner_1 = __importDefault(require("./MusigSigner"));
class EipSigner {
    constructor(logger, currencies, walletManager) {
        this.logger = logger;
        this.currencies = currencies;
        this.walletManager = walletManager;
        this.signSwapRefund = async (swapId) => {
            const swap = await SwapRepository_1.default.getSwap({ id: swapId });
            if (!swap) {
                throw Errors_1.default.SWAP_NOT_FOUND(swapId);
            }
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const chainSymbol = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
            const manager = this.walletManager.ethereumManagers.find((man) => man.hasSymbol(chainSymbol));
            if (manager === undefined) {
                throw 'chain currency is not EVM based';
            }
            if (!(await MusigSigner_1.default.isEligibleForRefund(swap, this.currencies.get((0, Utils_1.getLightningCurrency)(base, quote, swap.orderSide, false))))) {
                this.logger.verbose(`Not creating EIP-712 signature for refund of Swap ${swap.id}: it is not eligible`);
                throw Errors_1.default.NOT_ELIGIBLE_FOR_COOPERATIVE_REFUND();
            }
            this.logger.debug(`Creating EIP-712 signature for refund of Swap ${swap.id}`);
            const { domain, types, value } = await this.getSigningData(manager, chainSymbol, swap);
            return manager.signer.signTypedData(domain, types, value);
        };
        this.getSigningData = async (manager, chainSymbol, swap) => {
            const isEtherSwap = manager.networkDetails.symbol === chainSymbol;
            const contract = isEtherSwap ? manager.etherSwap : manager.erc20Swap;
            const value = {
                claimAddress: manager.address,
                timeout: swap.timeoutBlockHeight,
                preimageHash: (0, Utils_1.getHexBuffer)(swap.preimageHash),
                amount: isEtherSwap
                    ? BigInt(swap.onchainAmount) * Consts_1.etherDecimals
                    : this.walletManager.wallets.get(chainSymbol)
                        .walletProvider.formatTokenAmount(swap.onchainAmount),
            };
            if (!isEtherSwap) {
                value.tokenAddress = manager.tokenAddresses.get(chainSymbol);
            }
            return {
                value,
                domain: {
                    name: isEtherSwap ? 'EtherSwap' : 'ERC20Swap',
                    version: (await contract.version()).toString(),
                    chainId: manager.network.chainId,
                    verifyingContract: await contract.getAddress(),
                },
                types: {
                    Refund: isEtherSwap
                        ? [
                            {
                                type: 'bytes32',
                                name: 'preimageHash',
                            },
                            {
                                type: 'uint256',
                                name: 'amount',
                            },
                            {
                                type: 'address',
                                name: 'claimAddress',
                            },
                            {
                                type: 'uint256',
                                name: 'timeout',
                            },
                        ]
                        : [
                            {
                                type: 'bytes32',
                                name: 'preimageHash',
                            },
                            {
                                type: 'uint256',
                                name: 'amount',
                            },
                            {
                                type: 'address',
                                name: 'tokenAddress',
                            },
                            {
                                type: 'address',
                                name: 'claimAddress',
                            },
                            {
                                type: 'uint256',
                                name: 'timeout',
                            },
                        ],
                },
            };
        };
    }
}
exports.default = EipSigner;
//# sourceMappingURL=EipSigner.js.map