"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const boltz_core_1 = require("boltz-core");
const Core_1 = require("../../Core");
const Utils_1 = require("../../Utils");
const Enums_1 = require("../../consts/Enums");
const ReverseSwapRepository_1 = __importDefault(require("../../db/repositories/ReverseSwapRepository"));
const SwapRepository_1 = __importDefault(require("../../db/repositories/SwapRepository"));
const rpc_pb_1 = require("../../proto/lnd/rpc_pb");
const SwapNursery_1 = __importDefault(require("../../swap/SwapNursery"));
const Errors_1 = __importDefault(require("../Errors"));
// TODO: Should we verify what we are signing? And if so, how strict should we be?
class MusigSigner {
    constructor(logger, currencies, walletManager, nursery) {
        this.logger = logger;
        this.currencies = currencies;
        this.walletManager = walletManager;
        this.nursery = nursery;
        this.signSwapRefund = async (swapId, theirNonce, rawTransaction, index) => {
            const swap = await SwapRepository_1.default.getSwap({ id: swapId });
            if (!swap) {
                throw Errors_1.default.SWAP_NOT_FOUND(swapId);
            }
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const currency = this.currencies.get((0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false));
            if (currency.chainClient === undefined) {
                throw 'chain currency is not UTXO based';
            }
            if (swap.version !== Enums_1.SwapVersion.Taproot ||
                !(await _a.isEligibleForRefund(swap, this.currencies.get((0, Utils_1.getLightningCurrency)(base, quote, swap.orderSide, false))))) {
                this.logger.verbose(`Not creating partial signature for refund of Swap ${swap.id}: it is not eligible`);
                throw Errors_1.default.NOT_ELIGIBLE_FOR_COOPERATIVE_REFUND();
            }
            this.logger.debug(`Creating partial signature for refund of Swap ${swap.id}`);
            const swapTree = boltz_core_1.SwapTreeSerializer.deserializeSwapTree(swap.redeemScript);
            return this.createPartialSignature(currency, swapTree, swap.keyIndex, (0, Utils_1.getHexBuffer)(swap.refundPublicKey), theirNonce, rawTransaction, index);
        };
        this.signReverseSwapClaim = async (swapId, preimage, theirNonce, rawTransaction, index) => {
            const swap = await ReverseSwapRepository_1.default.getReverseSwap({ id: swapId });
            if (!swap) {
                throw Errors_1.default.SWAP_NOT_FOUND(swapId);
            }
            if (swap.version !== Enums_1.SwapVersion.Taproot ||
                ![
                    Enums_1.SwapUpdateEvent.TransactionMempool,
                    Enums_1.SwapUpdateEvent.TransactionConfirmed,
                    Enums_1.SwapUpdateEvent.InvoiceSettled,
                ].includes(swap.status)) {
                this.logger.verbose(`Not creating partial signature for claim of Reverse Swap ${swap.id}: it is not eligible`);
                throw Errors_1.default.NOT_ELIGIBLE_FOR_COOPERATIVE_CLAIM();
            }
            if (preimage.length !== 32 ||
                (0, Utils_1.getHexString)(bitcoinjs_lib_1.crypto.sha256(preimage)) !== swap.preimageHash) {
                this.logger.verbose(`Not creating partial signature for claim of Reverse Swap ${swap.id}: preimage is incorrect`);
                throw Errors_1.default.INCORRECT_PREIMAGE();
            }
            this.logger.debug(`Got preimage for Reverse Swap ${swap.id}: ${(0, Utils_1.getHexString)(preimage)}`);
            await ReverseSwapRepository_1.default.setPreimage(swap, (0, Utils_1.getHexString)(preimage));
            return this.nursery.lock.acquire(SwapNursery_1.default.reverseSwapLock, async () => {
                if (swap.status !== Enums_1.SwapUpdateEvent.InvoiceSettled) {
                    await this.nursery.settleReverseSwapInvoice(swap, preimage);
                }
                this.logger.debug(`Creating partial signature for claim of Reverse Swap ${swap.id}`);
                const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
                const swapTree = boltz_core_1.SwapTreeSerializer.deserializeSwapTree(swap.redeemScript);
                return this.createPartialSignature(this.currencies.get((0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, true)), swapTree, swap.keyIndex, (0, Utils_1.getHexBuffer)(swap.claimPublicKey), theirNonce, rawTransaction, index);
            });
        };
        this.createPartialSignature = async (currency, swapTree, keyIndex, theirPublicKey, theirNonce, rawTransaction, vin) => {
            const tx = (0, Core_1.parseTransaction)(currency.type, rawTransaction);
            if (vin < 0 || tx.ins.length <= vin) {
                throw Errors_1.default.INVALID_VIN();
            }
            const wallet = this.walletManager.wallets.get(currency.symbol);
            const ourKeys = wallet.getKeysByIndex(keyIndex);
            const musig = (0, Core_1.createMusig)(ourKeys, theirPublicKey);
            (0, Core_1.tweakMusig)(currency.type, musig, swapTree);
            musig.aggregateNonces([[theirPublicKey, theirNonce]]);
            const hash = await (0, Core_1.hashForWitnessV1)(currency, tx, vin);
            musig.initializeSession(hash);
            return {
                signature: Buffer.from(musig.signPartial()),
                pubNonce: Buffer.from(musig.getPublicNonce()),
            };
        };
    }
}
_a = MusigSigner;
MusigSigner.isEligibleForRefund = async (swap, lightningCurrency) => Enums_1.FailedSwapUpdateEvents.includes(swap.status) &&
    !(await _a.hasNonFailedLightningPayment(lightningCurrency, swap));
MusigSigner.hasNonFailedLightningPayment = async (currency, swap) => {
    try {
        if (currency.lndClient) {
            const pendingPayment = await currency.lndClient.trackPayment((0, Utils_1.getHexBuffer)(swap.preimageHash));
            if (pendingPayment.status !== rpc_pb_1.Payment.PaymentStatus.FAILED) {
                return true;
            }
        }
    }
    catch (e) {
        /* empty */
    }
    try {
        if (currency.clnClient && swap.invoice) {
            const payment = await currency.clnClient.checkPayStatus(swap.invoice);
            if (payment !== undefined) {
                return true;
            }
        }
    }
    catch (e) {
        return true;
    }
    return false;
};
exports.default = MusigSigner;
//# sourceMappingURL=MusigSigner.js.map