"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bolt11_1 = __importDefault(require("@boltz/bolt11"));
const async_lock_1 = __importDefault(require("async-lock"));
const sequelize_1 = require("sequelize");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const TypedEventEmitter_1 = __importDefault(require("../consts/TypedEventEmitter"));
const ChannelCreationRepository_1 = __importDefault(require("../db/repositories/ChannelCreationRepository"));
const SwapRepository_1 = __importDefault(require("../db/repositories/SwapRepository"));
const ConnectionHelper_1 = __importDefault(require("../lightning/ConnectionHelper"));
// TODO: cln compatibility
class ChannelNursery extends TypedEventEmitter_1.default {
    constructor(logger, settleSwap, overrideAllow = false) {
        super();
        this.logger = logger;
        this.settleSwap = settleSwap;
        this.overrideAllow = overrideAllow;
        this.lock = new async_lock_1.default();
        this.currencies = new Map();
        // Map between Channel Creation Swap ids and the number of settling retries
        //
        // This map is needed because when the node on the other side of a Channel Creation Swap
        // is c-lightning, there is a bug in LND that causes the "channel.active" event to fire
        // although the channel is still marked as inactive in "listchannels" and cannot be used yet.
        // Therefore, we need to retry after a couple seconds in order to settle the Swap.
        this.settleRetries = new Map();
        this.init = async (currencies) => {
            if (!this.overrideAllow) {
                return;
            }
            currencies.forEach((currency) => {
                this.currencies.set(currency.symbol, currency);
                if (!currency.lndClient) {
                    return;
                }
                currency.lndClient.on('peer.online', async (nodePublicKey) => {
                    await this.lock.acquire(ChannelNursery.channelCreationLock, async () => {
                        const channelCreations = await ChannelCreationRepository_1.default.getChannelCreations({
                            nodePublicKey,
                            status: Enums_1.ChannelCreationStatus.Attempted,
                        });
                        for (const channelCreation of channelCreations) {
                            const swap = await SwapRepository_1.default.getSwap({
                                id: channelCreation.swapId,
                                status: {
                                    [sequelize_1.Op.notIn]: [
                                        Enums_1.SwapUpdateEvent.SwapExpired,
                                        Enums_1.SwapUpdateEvent.InvoicePaid,
                                        Enums_1.SwapUpdateEvent.TransactionClaimed,
                                    ],
                                },
                            });
                            if (!swap || !this.eligibleForChannel(swap)) {
                                continue;
                            }
                            const lightningCurrency = this.getCurrency(swap, true);
                            await this.openChannel(this.currencies.get(lightningCurrency), swap, channelCreation);
                        }
                    });
                });
                currency.lndClient.on('channel.active', async (channelPoint) => {
                    const fundingTransactionId = this.parseFundingTransactionId(channelPoint.fundingTxidBytes);
                    const channelCreation = await ChannelCreationRepository_1.default.getChannelCreation({
                        fundingTransactionId,
                        status: Enums_1.ChannelCreationStatus.Created,
                        fundingTransactionVout: channelPoint.outputIndex,
                    });
                    if (!channelCreation) {
                        return;
                    }
                    const swap = await SwapRepository_1.default.getSwap({
                        id: channelCreation.swapId,
                        status: {
                            [sequelize_1.Op.not]: Enums_1.SwapUpdateEvent.TransactionClaimed,
                        },
                    });
                    if (swap.status === Enums_1.SwapUpdateEvent.SwapExpired) {
                        await ChannelCreationRepository_1.default.setAbandoned(channelCreation);
                        return;
                    }
                    await this.settleChannelWrapper(swap, channelCreation);
                });
            });
            await Promise.all([
                this.retryOpeningChannels(),
                this.settleCreatedChannels(),
            ]);
        };
        // TODO: show and reject less than min channel size
        // TODO: handle errors that say that the max number of (pending) channels exceeded
        this.openChannel = async (lightningCurrency, swap, channelCreation) => {
            if (!this.overrideAllow) {
                return;
            }
            const { satoshis, payeeNodeKey } = bolt11_1.default.decode(swap.invoice, lightningCurrency.network);
            this.logger.verbose(`Opening channel for Swap ${swap.id} to ${payeeNodeKey}`);
            if (channelCreation.status !== Enums_1.ChannelCreationStatus.Attempted) {
                await ChannelCreationRepository_1.default.setAttempted(channelCreation);
            }
            const channelCapacity = Math.ceil(satoshis / (1 - channelCreation.inboundLiquidity / 100));
            const feePerVbyte = await lightningCurrency.chainClient.estimateFee();
            // TODO: handle custom errors (c-lightning plugin)?
            try {
                const { fundingTxidBytes, outputIndex } = await lightningCurrency.lndClient.openChannel(payeeNodeKey, channelCapacity, channelCreation.private, feePerVbyte);
                const fundingTransactionId = this.parseFundingTransactionId(fundingTxidBytes);
                this.logger.info(`Opened channel for Swap ${swap.id} to ${payeeNodeKey}: ${fundingTransactionId}:${outputIndex}`);
                await SwapRepository_1.default.setSwapStatus(swap, Enums_1.SwapUpdateEvent.ChannelCreated);
                this.emit('channel.created', {
                    swap,
                    channelCreation: await ChannelCreationRepository_1.default.setFundingTransaction(channelCreation, fundingTransactionId, outputIndex),
                });
            }
            catch (error) {
                // TODO: emit event?
                const formattedError = (0, Utils_1.formatError)(error);
                const retryChannelOpeningAfterTimeout = async () => {
                    this.logger.debug(`Retrying in ${ChannelNursery.lndNotSyncedTimeout}ms`);
                    // Let's just wait for a second and try again
                    await new Promise((resolve) => {
                        setTimeout(resolve, ChannelNursery.lndNotSyncedTimeout);
                    });
                    await this.openChannel(lightningCurrency, swap, channelCreation);
                };
                // The actual error looks like this: "2 UNKNOWN: received funding error from <remote_pukbey>: chan_id=<prposed_channel_id>, err=Synchronizing blockchain"
                // But since we don't know what the "chan_id" would have been, we cannot catch this error in the switch statement
                if (formattedError.endsWith('err=Synchronizing blockchain')) {
                    this.logger.warn(`Could not open channel for Swap ${swap.id}: remote LND is not fully synced`);
                    await retryChannelOpeningAfterTimeout();
                    return;
                }
                switch (formattedError) {
                    // This error is thrown when a block is mined with a lockup transaction that triggers a Channel Creation
                    // in it. In case Boltz processes the block faster than LND does, it will try to open the channel while
                    // LND is still syncing the freshly mined block
                    case '2 UNKNOWN: channels cannot be created before the wallet is fully synced':
                        this.logger.warn(`Could not open channel for Swap ${swap.id}: our LND is not fully synced`);
                        await retryChannelOpeningAfterTimeout();
                        return;
                    // In case the lightning client to which a channel should be opened is not online or not connected to us,
                    // we can try to connect to them
                    case `2 UNKNOWN: peer ${payeeNodeKey} is not online`:
                        try {
                            await this.connectionHelper.connectByPublicKey(lightningCurrency.lndClient, payeeNodeKey);
                            // The channel opening should *not* be retried here since the "peer.online" subscription of the LND client does handle it already
                            return;
                        }
                        catch (error) {
                            this.logger.warn(`Could not connect to ${lightningCurrency.lndClient.symbol} LND node ${payeeNodeKey}: ${(0, Utils_1.formatError)(error)}`);
                        }
                        break;
                }
                this.logger.verbose(`Could not open channel for Swap ${swap.id} to ${payeeNodeKey}: ${formattedError}`);
            }
        };
        this.retryOpeningChannels = async () => {
            await this.lock.acquire(ChannelNursery.channelCreationLock, async () => {
                const channelsToOpen = await ChannelCreationRepository_1.default.getChannelCreations({
                    status: Enums_1.ChannelCreationStatus.Attempted,
                });
                for (const channelToOpen of channelsToOpen) {
                    const swap = await SwapRepository_1.default.getSwap({
                        id: channelToOpen.swapId,
                        status: Enums_1.SwapUpdateEvent.InvoicePending,
                    });
                    if (!swap || !this.eligibleForChannel(swap)) {
                        continue;
                    }
                    const lightningCurrency = this.getCurrency(swap, true);
                    const currency = this.currencies.get(lightningCurrency);
                    const peers = await currency.lndClient.listPeers();
                    // Only try to open a channel if other side is connected to us
                    for (const peer of peers.peersList) {
                        if (peer.pubKey === channelToOpen.nodePublicKey) {
                            await this.openChannel(currency, swap, channelToOpen);
                            break;
                        }
                    }
                }
            });
        };
        this.settleChannel = async (swap, channelCreation) => {
            const chainCurrency = this.currencies.get(this.getCurrency(swap, false));
            const lightningCurrency = this.currencies.get(this.getCurrency(swap, true));
            const activeChannels = await lightningCurrency.lndClient.listChannels(true);
            for (const channel of activeChannels) {
                if (channel.fundingTransactionId === channelCreation.fundingTransactionId &&
                    channel.fundingTransactionVout ===
                        channelCreation.fundingTransactionVout) {
                    this.logger.verbose(`Attempting to settle Channel Creation Swap: ${swap.id}`);
                    try {
                        await this.settleSwap(chainCurrency, swap, channel.chanId);
                        await ChannelCreationRepository_1.default.setSettled(channelCreation);
                        return true;
                    }
                    catch (error) {
                        const formattedError = (0, Utils_1.formatError)(error);
                        if (formattedError === 'could not pay invoice: invoice is already paid') {
                            this.logger.verbose(`Channel Creation Swap ${swap.id} already settled`);
                            await ChannelCreationRepository_1.default.setSettled(channelCreation);
                            return true;
                        }
                        else {
                            this.logger.warn(`Could not settle Channel Creation Swap ${swap.id}: ${formattedError}`);
                        }
                    }
                }
            }
            this.logger.verbose(`Could not settle Channel Creation Swap ${swap.id}: channel is not active`);
            return false;
        };
        this.settleChannelWrapper = async (swap, channelCreation, timeoutAmplifier = 1000) => {
            const settleSuccessful = await this.settleChannel(swap, channelCreation);
            if (settleSuccessful) {
                this.settleRetries.delete(swap.id);
            }
            else {
                let settleTimeout = this.settleRetries.get(swap.id);
                if (!settleTimeout) {
                    settleTimeout = 1;
                }
                else {
                    settleTimeout = settleTimeout * 2;
                }
                if (settleTimeout < 8) {
                    this.settleRetries.set(swap.id, settleTimeout);
                    this.logger.info(`Retrying to settle Channel Creation Swap ${swap.id} in ${settleTimeout} seconds`);
                    setTimeout(async () => {
                        await this.lock.acquire(ChannelNursery.channelSettleLock, async () => {
                            await this.settleChannelWrapper(swap, channelCreation, timeoutAmplifier);
                        });
                    }, settleTimeout * timeoutAmplifier);
                }
                else {
                    this.logger.warn(`Giving up to retry loop to settle Channel Creation Swap ${swap.id}`);
                    this.settleRetries.delete(swap.id);
                }
            }
        };
        this.settleCreatedChannels = async () => {
            const unsettledChannels = await ChannelCreationRepository_1.default.getChannelCreations({
                status: Enums_1.ChannelCreationStatus.Created,
            });
            for (const unsettledChannel of unsettledChannels) {
                const swap = await SwapRepository_1.default.getSwap({
                    id: unsettledChannel.swapId,
                    status: {
                        [sequelize_1.Op.not]: Enums_1.SwapUpdateEvent.TransactionClaimed,
                    },
                });
                await this.settleChannel(swap, unsettledChannel);
            }
        };
        this.eligibleForChannel = (swap) => {
            return (swap.lockupTransactionId !== null &&
                swap.onchainAmount >= swap.expectedAmount);
        };
        this.parseFundingTransactionId = (fundingTransactionIdBytes) => {
            return (0, Utils_1.getHexString)((0, Utils_1.reverseBuffer)(Buffer.from(fundingTransactionIdBytes, 'base64')));
        };
        this.getCurrency = (swap, lightning) => {
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            return lightning
                ? (0, Utils_1.getLightningCurrency)(base, quote, swap.orderSide, false)
                : (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
        };
        this.connectionHelper = new ConnectionHelper_1.default(this.logger);
    }
}
ChannelNursery.channelSettleLock = 'channelSettle';
ChannelNursery.channelCreationLock = 'channelCreation';
ChannelNursery.lndNotSyncedTimeout = 1000;
exports.default = ChannelNursery;
//# sourceMappingURL=ChannelNursery.js.map