"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sequelize_1 = require("sequelize");
const Utils_1 = require("../Utils");
const Consts_1 = require("../consts/Consts");
const Enums_1 = require("../consts/Enums");
const TypedEventEmitter_1 = __importDefault(require("../consts/TypedEventEmitter"));
const ReverseSwapRepository_1 = __importDefault(require("../db/repositories/ReverseSwapRepository"));
const SwapRepository_1 = __importDefault(require("../db/repositories/SwapRepository"));
const Errors_1 = __importDefault(require("./Errors"));
class EthereumNursery extends TypedEventEmitter_1.default {
    constructor(logger, walletManager, ethereumManager, blocks) {
        super();
        this.logger = logger;
        this.walletManager = walletManager;
        this.ethereumManager = ethereumManager;
        this.blocks = blocks;
        this.init = async () => {
            // Fetch all Reverse Swaps with a pending lockup transaction
            const mempoolReverseSwaps = await ReverseSwapRepository_1.default.getReverseSwaps({
                status: Enums_1.SwapUpdateEvent.TransactionMempool,
            });
            for (const mempoolReverseSwap of mempoolReverseSwaps) {
                const { base, quote } = (0, Utils_1.splitPairId)(mempoolReverseSwap.pair);
                const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, mempoolReverseSwap.orderSide, true);
                // Skip all Reverse Swaps that didn't send coins on the Ethereum chain
                if (this.getEthereumWallet(chainCurrency) === undefined) {
                    continue;
                }
                try {
                    const transaction = await this.ethereumManager.provider.getTransaction(mempoolReverseSwap.transactionId);
                    this.logger.debug(`Found pending ${this.ethereumManager.networkDetails.name} lockup transaction of Reverse Swap ${mempoolReverseSwap.id}: ${mempoolReverseSwap.transactionId}`);
                    this.listenContractTransaction(mempoolReverseSwap, transaction);
                }
                catch (error) {
                    // TODO: retry finding that transaction
                    // If the provider can't find the transaction, it is not on the Ethereum chain
                }
            }
        };
        this.listenContractTransaction = (reverseSwap, transaction) => {
            transaction
                .wait(1)
                .then(async () => {
                this.emit('lockup.confirmed', {
                    reverseSwap: await ReverseSwapRepository_1.default.setReverseSwapStatus(reverseSwap, Enums_1.SwapUpdateEvent.TransactionConfirmed),
                    transactionHash: transaction.hash,
                });
            })
                .catch(async (reason) => {
                this.emit('lockup.failedToSend', {
                    reason,
                    reverseSwap: await ReverseSwapRepository_1.default.setReverseSwapStatus(reverseSwap, Enums_1.SwapUpdateEvent.TransactionFailed),
                });
            });
        };
        this.listenEtherSwap = () => {
            this.ethereumManager.contractEventHandler.on('eth.lockup', async ({ transaction, etherSwapValues }) => {
                let swap = await SwapRepository_1.default.getSwap({
                    preimageHash: (0, Utils_1.getHexString)(etherSwapValues.preimageHash),
                    status: {
                        [sequelize_1.Op.or]: [Enums_1.SwapUpdateEvent.SwapCreated, Enums_1.SwapUpdateEvent.InvoiceSet],
                    },
                });
                if (!swap) {
                    return;
                }
                const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
                const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
                if (chainCurrency !== this.ethereumManager.networkDetails.symbol) {
                    return;
                }
                this.logger.debug(`Found lockup in ${this.ethereumManager.networkDetails.name} EtherSwap contract for Swap ${swap.id}: ${transaction.hash}`);
                swap = await SwapRepository_1.default.setLockupTransaction(swap, transaction.hash, Number(etherSwapValues.amount / Consts_1.etherDecimals), true);
                if (etherSwapValues.claimAddress !== this.ethereumManager.address) {
                    this.emit('lockup.failed', {
                        swap,
                        reason: Errors_1.default.INVALID_CLAIM_ADDRESS(etherSwapValues.claimAddress, this.ethereumManager.address).message,
                    });
                    return;
                }
                if (etherSwapValues.timelock !== swap.timeoutBlockHeight) {
                    this.emit('lockup.failed', {
                        swap,
                        reason: Errors_1.default.INVALID_TIMELOCK(etherSwapValues.timelock, swap.timeoutBlockHeight).message,
                    });
                    return;
                }
                if (swap.expectedAmount) {
                    const expectedAmount = BigInt(swap.expectedAmount) *
                        this.ethereumManager.networkDetails.decimals;
                    if (expectedAmount > etherSwapValues.amount) {
                        this.emit('lockup.failed', {
                            swap,
                            reason: Errors_1.default.INSUFFICIENT_AMOUNT(Number(etherSwapValues.amount / Consts_1.etherDecimals), swap.expectedAmount).message,
                        });
                        return;
                    }
                }
                if (this.blocks.isBlocked(transaction.from)) {
                    this.emit('lockup.failed', {
                        swap,
                        reason: Errors_1.default.BLOCKED_ADDRESS().message,
                    });
                    return;
                }
                this.emit('eth.lockup', {
                    swap,
                    etherSwapValues,
                    transactionHash: transaction.hash,
                });
            });
            this.ethereumManager.contractEventHandler.on('eth.claim', async ({ transactionHash, preimageHash, preimage }) => {
                const reverseSwap = await ReverseSwapRepository_1.default.getReverseSwap({
                    preimageHash: (0, Utils_1.getHexString)(preimageHash),
                    status: {
                        [sequelize_1.Op.not]: Enums_1.SwapUpdateEvent.InvoiceSettled,
                    },
                });
                if (!reverseSwap) {
                    return;
                }
                this.logger.debug(`Found claim in ${this.ethereumManager.networkDetails.name} EtherSwap contract for Reverse Swap ${reverseSwap.id}: ${transactionHash}`);
                this.emit('claim', { reverseSwap, preimage });
            });
        };
        this.listenERC20Swap = () => {
            this.ethereumManager.contractEventHandler.on('erc20.lockup', async ({ transaction, erc20SwapValues }) => {
                let swap = await SwapRepository_1.default.getSwap({
                    preimageHash: (0, Utils_1.getHexString)(erc20SwapValues.preimageHash),
                    status: {
                        [sequelize_1.Op.or]: [Enums_1.SwapUpdateEvent.SwapCreated, Enums_1.SwapUpdateEvent.InvoiceSet],
                    },
                });
                if (!swap) {
                    return;
                }
                const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
                const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
                const wallet = this.walletManager.wallets.get(chainCurrency);
                if (wallet === undefined || wallet.type !== Enums_1.CurrencyType.ERC20) {
                    return;
                }
                const erc20Wallet = wallet.walletProvider;
                this.logger.debug(`Found lockup in ${this.ethereumManager.networkDetails.name} ERC20Swap contract for Swap ${swap.id}: ${transaction.hash}`);
                swap = await SwapRepository_1.default.setLockupTransaction(swap, transaction.hash, erc20Wallet.normalizeTokenAmount(erc20SwapValues.amount), true);
                if (erc20SwapValues.claimAddress !== this.ethereumManager.address) {
                    this.emit('lockup.failed', {
                        swap,
                        reason: Errors_1.default.INVALID_CLAIM_ADDRESS(erc20SwapValues.claimAddress, this.ethereumManager.address).message,
                    });
                    return;
                }
                if (erc20SwapValues.tokenAddress !== erc20Wallet.getTokenAddress()) {
                    this.emit('lockup.failed', {
                        swap,
                        reason: Errors_1.default.INVALID_TOKEN_LOCKED(erc20SwapValues.tokenAddress, this.ethereumManager.address).message,
                    });
                    return;
                }
                if (erc20SwapValues.timelock !== swap.timeoutBlockHeight) {
                    this.emit('lockup.failed', {
                        swap,
                        reason: Errors_1.default.INVALID_TIMELOCK(erc20SwapValues.timelock, swap.timeoutBlockHeight).message,
                    });
                    return;
                }
                if (swap.expectedAmount) {
                    if (erc20Wallet.formatTokenAmount(swap.expectedAmount) >
                        erc20SwapValues.amount) {
                        this.emit('lockup.failed', {
                            swap,
                            reason: Errors_1.default.INSUFFICIENT_AMOUNT(erc20Wallet.normalizeTokenAmount(erc20SwapValues.amount), swap.expectedAmount).message,
                        });
                        return;
                    }
                }
                if (this.blocks.isBlocked(transaction.from)) {
                    this.emit('lockup.failed', {
                        swap,
                        reason: Errors_1.default.BLOCKED_ADDRESS().message,
                    });
                    return;
                }
                this.emit('erc20.lockup', {
                    swap,
                    erc20SwapValues,
                    transactionHash: transaction.hash,
                });
            });
            this.ethereumManager.contractEventHandler.on('erc20.claim', async ({ transactionHash, preimageHash, preimage }) => {
                const reverseSwap = await ReverseSwapRepository_1.default.getReverseSwap({
                    preimageHash: (0, Utils_1.getHexString)(preimageHash),
                    status: {
                        [sequelize_1.Op.not]: Enums_1.SwapUpdateEvent.InvoiceSettled,
                    },
                });
                if (!reverseSwap) {
                    return;
                }
                this.logger.debug(`Found claim in ${this.ethereumManager.networkDetails.name} ERC20Swap contract for Reverse Swap ${reverseSwap.id}: ${transactionHash}`);
                this.emit('claim', { reverseSwap, preimage });
            });
        };
        this.listenBlocks = () => {
            this.ethereumManager.provider
                .on('block', async (height) => {
                await Promise.all([
                    this.checkExpiredSwaps(height),
                    this.checkExpiredReverseSwaps(height),
                ]);
            })
                .catch((err) => {
                this.logger.error(`Could not subscribe to ${this.ethereumManager.networkDetails.name} blocks: ${(0, Utils_1.formatError)(err)}`);
            });
        };
        this.checkExpiredSwaps = async (height) => {
            const expirableSwaps = await SwapRepository_1.default.getSwapsExpirable(height);
            for (const expirableSwap of expirableSwaps) {
                const { base, quote } = (0, Utils_1.splitPairId)(expirableSwap.pair);
                const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, expirableSwap.orderSide, false);
                const wallet = this.getEthereumWallet(chainCurrency);
                if (wallet) {
                    this.emit('swap.expired', {
                        swap: expirableSwap,
                        isEtherSwap: wallet.symbol === this.ethereumManager.networkDetails.symbol,
                    });
                }
            }
        };
        this.checkExpiredReverseSwaps = async (height) => {
            const expirableReverseSwaps = await ReverseSwapRepository_1.default.getReverseSwapsExpirable(height);
            for (const expirableReverseSwap of expirableReverseSwaps) {
                const { base, quote } = (0, Utils_1.splitPairId)(expirableReverseSwap.pair);
                const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, expirableReverseSwap.orderSide, true);
                const wallet = this.getEthereumWallet(chainCurrency);
                if (wallet) {
                    this.emit('reverseSwap.expired', {
                        reverseSwap: expirableReverseSwap,
                        isEtherSwap: wallet.symbol === this.ethereumManager.networkDetails.symbol,
                    });
                }
            }
        };
        /**
         * Returns a wallet in case there is one with the symbol, and it is an Ethereum one
         */
        this.getEthereumWallet = (symbol) => {
            const wallet = this.walletManager.wallets.get(symbol);
            if (!wallet) {
                return;
            }
            if (wallet.type === Enums_1.CurrencyType.Ether ||
                wallet.type === Enums_1.CurrencyType.ERC20) {
                return wallet;
            }
            return;
        };
        this.listenBlocks();
        this.listenEtherSwap();
        this.listenERC20Swap();
    }
}
exports.default = EthereumNursery;
//# sourceMappingURL=EthereumNursery.js.map