"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sequelize_1 = require("sequelize");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const TypedEventEmitter_1 = __importDefault(require("../consts/TypedEventEmitter"));
const ReverseSwapRepository_1 = __importDefault(require("../db/repositories/ReverseSwapRepository"));
const InvoiceExpiryHelper_1 = __importDefault(require("../service/InvoiceExpiryHelper"));
/**
 * InvoiceNursery takes care of cancelling pending HTLCs of Reverse Swaps with prepay miner fee
 * in case only one of the two invoices is paid and the other expired and cannot be paid anymore
 */
class InvoiceNursery extends TypedEventEmitter_1.default {
    constructor(logger) {
        super();
        this.logger = logger;
        this.init = async () => {
            this.logger.debug(`Checking for expired invoices every ${InvoiceNursery.checkInterval} seconds`);
            this.interval = setInterval(this.checkExpiredInvoices, InvoiceNursery.checkInterval * 1000);
            await this.checkExpiredInvoices();
        };
        this.destroy = () => {
            if (this.interval) {
                clearInterval(this.interval);
                this.interval = undefined;
            }
        };
        this.checkExpiredInvoices = async () => {
            const pendingSwaps = await ReverseSwapRepository_1.default.getReverseSwaps({
                status: {
                    [sequelize_1.Op.or]: [Enums_1.SwapUpdateEvent.SwapCreated, Enums_1.SwapUpdateEvent.MinerFeePaid],
                },
            });
            if (pendingSwaps.length === 0) {
                return;
            }
            this.logger.silly(`Checking ${pendingSwaps.length} Reverse Swaps for expired invoices`);
            const currentTime = (0, Utils_1.getUnixTime)();
            for (const reverseSwap of pendingSwaps) {
                const { timestamp, timeExpireDate } = (0, Utils_1.decodeInvoice)(reverseSwap.invoice);
                const invoiceExpiry = InvoiceExpiryHelper_1.default.getInvoiceExpiry(timestamp, timeExpireDate);
                if (currentTime > invoiceExpiry) {
                    this.emit('invoice.expired', reverseSwap);
                }
            }
        };
    }
}
// Interval for checking for expired invoices in seconds
InvoiceNursery.checkInterval = 60;
exports.default = InvoiceNursery;
//# sourceMappingURL=InvoiceNursery.js.map