"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const async_lock_1 = __importDefault(require("async-lock"));
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const sequelize_1 = require("sequelize");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const TypedEventEmitter_1 = __importDefault(require("../consts/TypedEventEmitter"));
const ReverseSwapRepository_1 = __importDefault(require("../db/repositories/ReverseSwapRepository"));
const LightningClient_1 = require("../lightning/LightningClient");
class LightningNursery extends TypedEventEmitter_1.default {
    constructor(logger) {
        super();
        this.logger = logger;
        this.lock = new async_lock_1.default();
        this.bindCurrencies = (currencies) => {
            currencies.forEach((currency) => {
                [currency.lndClient, currency.clnClient]
                    .filter((client) => client !== undefined)
                    .map(this.listenInvoices);
            });
        };
        this.listenInvoices = (lightningClient) => {
            lightningClient.on('htlc.accepted', async (invoice) => {
                await this.lock.acquire(_a.invoiceLock, async () => {
                    try {
                        await lightningClient.raceCall(this.handleAcceptedInvoice(lightningClient, invoice), (reject) => reject('invoice acceptance handler timeout out'), _a.lightningClientCallTimeout);
                    }
                    catch (e) {
                        this.logger.warn(`Could not handle accepted invoice of ${lightningClient.serviceName()}: ${(0, Utils_1.formatError)(e)}`);
                    }
                });
            });
        };
        this.handleAcceptedInvoice = async (lightningClient, invoice) => {
            let reverseSwap = await ReverseSwapRepository_1.default.getReverseSwap({
                [sequelize_1.Op.or]: [
                    {
                        invoice,
                    },
                    {
                        minerFeeInvoice: invoice,
                    },
                ],
            });
            if (!reverseSwap) {
                return;
            }
            if (reverseSwap.invoice === invoice) {
                this.logger.verbose(`Hold invoice of Reverse Swap ${reverseSwap.id} was accepted`);
                if (reverseSwap.minerFeeInvoicePreimage === null ||
                    reverseSwap.status === Enums_1.SwapUpdateEvent.MinerFeePaid) {
                    if (reverseSwap.minerFeeInvoicePreimage) {
                        await lightningClient.settleHoldInvoice((0, Utils_1.getHexBuffer)(reverseSwap.minerFeeInvoicePreimage));
                    }
                    this.emit('invoice.paid', reverseSwap);
                }
                else {
                    this.logger.debug(`Did not send onchain coins for Reverse Swap ${reverseSwap.id} because miner fee invoice was not paid yet`);
                }
            }
            else {
                this.logger.debug(`Minerfee prepayment of Reverse Swap ${reverseSwap.id} was accepted`);
                reverseSwap = await ReverseSwapRepository_1.default.setReverseSwapStatus(reverseSwap, Enums_1.SwapUpdateEvent.MinerFeePaid);
                this.emit('minerfee.invoice.paid', reverseSwap);
                // Settle the prepay invoice and emit the "invoice.paid" event in case the hold invoice was paid first
                const holdInvoice = await lightningClient.lookupHoldInvoice((0, Utils_1.getHexBuffer)((0, Utils_1.decodeInvoice)(reverseSwap.invoice).paymentHash));
                if (holdInvoice.state === LightningClient_1.InvoiceState.Accepted) {
                    await lightningClient.settleHoldInvoice((0, Utils_1.getHexBuffer)(reverseSwap.minerFeeInvoicePreimage));
                    this.emit('invoice.paid', reverseSwap);
                }
            }
        };
    }
}
_a = LightningNursery;
LightningNursery.lightningClientCallTimeout = 15000;
LightningNursery.invoiceLock = 'invoice';
LightningNursery.errIsInvoicePaid = (error) => {
    return (error !== undefined &&
        error !== null &&
        error.details === 'invoice is already paid');
};
LightningNursery.errIsPaymentInTransition = (error) => {
    return (error !== undefined &&
        error !== null &&
        error.details === 'payment is in transition');
};
LightningNursery.errIsCltvLimitExceeded = (error) => {
    if (error === undefined || error === null) {
        return false;
    }
    return /^(cltv limit )(\d{1,3}) (should be greater than )(\d{1,3})$/gm.test(error.details);
};
LightningNursery.errIsInvoiceExpired = (error) => {
    return error.toLowerCase().includes('invoice expired');
};
LightningNursery.cancelReverseInvoices = async (lightningClient, reverseSwap, alsoMinerFeeInvoice) => {
    await lightningClient.raceCall(async () => {
        await lightningClient.cancelHoldInvoice((0, Utils_1.getHexBuffer)(reverseSwap.preimageHash));
        if (alsoMinerFeeInvoice && reverseSwap.minerFeeInvoicePreimage) {
            await lightningClient.cancelHoldInvoice(bitcoinjs_lib_1.crypto.sha256((0, Utils_1.getHexBuffer)(reverseSwap.minerFeeInvoicePreimage)));
        }
    }, (reject) => reject('cancelling reverse swap invoices timed out'), _a.lightningClientCallTimeout);
};
exports.default = LightningNursery;
//# sourceMappingURL=LightningNursery.js.map