"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ReverseSwap_1 = require("../db/models/ReverseSwap");
const Errors_1 = __importDefault(require("./Errors"));
class NodeSwitch {
    constructor(logger, cfg) {
        this.logger = logger;
        this.referralIds = new Map();
        this.getSwapNode = (currency, swap) => {
            const client = NodeSwitch.fallback(currency, this.swapNode !== undefined
                ? NodeSwitch.switchOnNodeType(currency, this.swapNode)
                : this.switch(currency, swap.invoiceAmount, swap.referral));
            if (swap.id !== undefined) {
                this.logger.debug(`Using node ${client.serviceName()} for Swap ${swap.id}`);
            }
            return client;
        };
        this.getNodeForReverseSwap = (id, currency, holdInvoiceAmount, referralId) => {
            const client = NodeSwitch.fallback(currency, this.switch(currency, holdInvoiceAmount, referralId));
            this.logger.debug(`Using node ${client.serviceName()} for Reverse Swap Swap ${id}`);
            return {
                lightningClient: client,
                nodeType: client === currency.lndClient ? ReverseSwap_1.NodeType.LND : ReverseSwap_1.NodeType.CLN,
            };
        };
        this.switch = (currency, amount, referralId) => {
            if (referralId && this.referralIds.has(referralId)) {
                return NodeSwitch.fallback(currency, NodeSwitch.switchOnNodeType(currency, this.referralIds.get(referralId)));
            }
            return NodeSwitch.fallback(currency, (amount || 0) > this.clnAmountThreshold
                ? currency.lndClient
                : currency.clnClient);
        };
        this.parseNodeType = (nodeType, valueContext) => {
            const nt = ReverseSwap_1.NodeType[nodeType];
            if (nt === undefined || typeof nodeType !== 'string') {
                this.logger.warn(`Invalid node type for ${valueContext}: "${nodeType}"; available options are: [${Object.values(ReverseSwap_1.NodeType)
                    .filter((val) => typeof val === 'string')
                    .join(', ')}]`);
                return;
            }
            return nt;
        };
        this.clnAmountThreshold =
            cfg?.clnAmountThreshold || NodeSwitch.defaultClnAmountThreshold;
        const swapNode = cfg?.swapNode !== undefined
            ? this.parseNodeType(cfg.swapNode, 'swap node')
            : undefined;
        if (swapNode !== undefined) {
            this.logger.info(`Using ${cfg?.swapNode} for paying invoices of Swaps`);
            this.swapNode = swapNode;
        }
        for (const [referralId, nodeType] of Object.entries(cfg?.referralsIds || {})) {
            const nt = this.parseNodeType(nodeType, `referral id ${referralId}`);
            if (nt === undefined) {
                continue;
            }
            this.referralIds.set(referralId, nt);
        }
    }
}
NodeSwitch.defaultClnAmountThreshold = 1000000;
NodeSwitch.getReverseSwapNode = (currency, reverseSwap) => {
    return NodeSwitch.fallback(currency, NodeSwitch.switchOnNodeType(currency, reverseSwap.node));
};
NodeSwitch.hasClient = (currency) => {
    return [currency.lndClient, currency.clnClient].some((client) => client !== undefined);
};
NodeSwitch.fallback = (currency, client) => {
    const clients = [client, currency.lndClient, currency.clnClient]
        .filter((client) => client !== undefined)
        .filter((client) => client.isConnected());
    if (clients.length === 0) {
        throw Errors_1.default.NO_AVAILABLE_LIGHTNING_CLIENT();
    }
    return clients[0];
};
NodeSwitch.switchOnNodeType = (currency, nodeType) => {
    return nodeType === ReverseSwap_1.NodeType.LND ? currency.lndClient : currency.clnClient;
};
exports.default = NodeSwitch;
//# sourceMappingURL=NodeSwitch.js.map