"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const SwapRepository_1 = __importDefault(require("../db/repositories/SwapRepository"));
const LndClient_1 = __importDefault(require("../lightning/LndClient"));
const ClnClient_1 = __importDefault(require("../lightning/cln/ClnClient"));
const rpc_pb_1 = require("../proto/lnd/rpc_pb");
const Errors_1 = __importDefault(require("./Errors"));
const LightningNursery_1 = __importDefault(require("./LightningNursery"));
class PaymentHandler {
    constructor(logger, nodeSwitch, currencies, channelNursery, timeoutDeltaProvider, emit) {
        this.logger = logger;
        this.nodeSwitch = nodeSwitch;
        this.currencies = currencies;
        this.channelNursery = channelNursery;
        this.timeoutDeltaProvider = timeoutDeltaProvider;
        this.emit = emit;
        this.lastResetMissionControl = undefined;
        this.payInvoice = async (swap, channelCreation, outgoingChannelId) => {
            this.logger.verbose(`Paying invoice of Swap ${swap.id}`);
            if (swap.status !== Enums_1.SwapUpdateEvent.InvoicePending &&
                swap.status !== Enums_1.SwapUpdateEvent.ChannelCreated) {
                this.emit('invoice.pending', await SwapRepository_1.default.setSwapStatus(swap, Enums_1.SwapUpdateEvent.InvoicePending));
            }
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const lightningSymbol = (0, Utils_1.getLightningCurrency)(base, quote, swap.orderSide, false);
            const lightningCurrency = this.currencies.get(lightningSymbol);
            const lightningClient = this.nodeSwitch.getSwapNode(lightningCurrency, swap);
            try {
                return await this.racePayInvoice(swap, outgoingChannelId, lightningClient);
            }
            catch (error) {
                return this.handlePaymentFailure(swap, channelCreation, lightningCurrency, lightningClient, error, outgoingChannelId);
            }
        };
        this.racePayInvoice = async (swap, outgoingChannelId, lightningClient) => {
            const cltvLimit = await this.timeoutDeltaProvider.getCltvLimit(swap);
            if (cltvLimit < 2) {
                throw PaymentHandler.errCltvTooSmall;
            }
            this.logger.debug(`Paying invoice of swap ${swap.id} with CLTV limit: ${cltvLimit}`);
            let timeout = undefined;
            const racePromise = new Promise((resolve) => {
                timeout = setTimeout(() => {
                    resolve(undefined);
                }, PaymentHandler.raceTimeout * 1000);
            });
            try {
                const payResponse = await Promise.race([
                    lightningClient.sendPayment(swap.invoice, cltvLimit, outgoingChannelId),
                    racePromise,
                ]);
                if (timeout !== undefined) {
                    clearTimeout(timeout);
                }
                if (payResponse !== undefined) {
                    return await this.settleInvoice(swap, payResponse);
                }
                this.logger.verbose(`Invoice payment of Swap ${swap.id} is still pending after ${PaymentHandler.raceTimeout} seconds`);
            }
            catch (e) {
                if (timeout !== undefined) {
                    clearTimeout(timeout);
                }
                throw e;
            }
            return undefined;
        };
        this.handlePaymentFailure = async (swap, channelCreation, lightningCurrency, lightningClient, error, outgoingChannelId) => {
            if (lightningClient instanceof LndClient_1.default) {
                return this.handleLndPaymentFailure(swap, channelCreation, lightningCurrency, lightningClient, error, outgoingChannelId);
            }
            else if (lightningClient instanceof ClnClient_1.default) {
                return this.handleClnPaymentFailure(swap, error);
            }
            throw error;
        };
        this.handleLndPaymentFailure = async (swap, channelCreation, lightningCurrency, lndClient, error, outgoingChannelId) => {
            const errorMessage = typeof error === 'number'
                ? LndClient_1.default.formatPaymentFailureReason(error)
                : (0, Utils_1.formatError)(error);
            if (outgoingChannelId !== undefined) {
                throw errorMessage;
            }
            if (errorMessage === PaymentHandler.errCltvTooSmall ||
                LightningNursery_1.default.errIsInvoicePaid(error) ||
                LightningNursery_1.default.errIsCltvLimitExceeded(error) ||
                LightningNursery_1.default.errIsInvoiceExpired(errorMessage)) {
                try {
                    const payment = await lndClient.trackPayment((0, Utils_1.getHexBuffer)(swap.preimageHash));
                    if (payment.status === rpc_pb_1.Payment.PaymentStatus.SUCCEEDED) {
                        this.logger.debug(`Invoice of Swap ${swap.id} is paid already`);
                        return this.settleInvoice(swap, {
                            feeMsat: payment.feeMsat,
                            preimage: (0, Utils_1.getHexBuffer)(payment.paymentPreimage),
                        });
                    }
                    else if (payment.status === rpc_pb_1.Payment.PaymentStatus.IN_FLIGHT) {
                        this.logger.info(`Invoice of Swap ${swap.id} is still pending`);
                        return undefined;
                    }
                }
                catch (e) {
                    /* empty */
                }
            }
            this.logPaymentFailure(swap, errorMessage);
            // If the recipient rejects the payment or the invoice expired, the Swap will be abandoned
            if (error === rpc_pb_1.PaymentFailureReason.FAILURE_REASON_INCORRECT_PAYMENT_DETAILS ||
                LightningNursery_1.default.errIsInvoiceExpired(errorMessage)) {
                await this.abandonSwap(swap, errorMessage);
                return undefined;
            }
            if (errorMessage === PaymentHandler.errCltvTooSmall ||
                LightningNursery_1.default.errIsInvoicePaid(error) ||
                LightningNursery_1.default.errIsPaymentInTransition(error) ||
                LightningNursery_1.default.errIsCltvLimitExceeded(error)) {
                return undefined;
            }
            if (this.lastResetMissionControl === undefined ||
                Date.now() - this.lastResetMissionControl >=
                    PaymentHandler.resetMissionControlInterval) {
                this.logger.debug(`Resetting ${lndClient.symbol} ${LndClient_1.default.serviceName} mission control`);
                this.lastResetMissionControl = Date.now();
                await lndClient.resetMissionControl();
            }
            else {
                this.logger.debug(`Not resetting ${lndClient.symbol} ${LndClient_1.default.serviceName} mission control because last reset was at ${(0, Utils_1.getTsString)(new Date(this.lastResetMissionControl))}`);
            }
            // If the invoice could not be paid but the Swap has a Channel Creation attached to it, a channel will be opened
            if (typeof error === 'number' &&
                channelCreation &&
                channelCreation.status !== Enums_1.ChannelCreationStatus.Created) {
                switch (error) {
                    case rpc_pb_1.PaymentFailureReason.FAILURE_REASON_TIMEOUT:
                    case rpc_pb_1.PaymentFailureReason.FAILURE_REASON_NO_ROUTE:
                    case rpc_pb_1.PaymentFailureReason.FAILURE_REASON_INSUFFICIENT_BALANCE:
                        await this.channelNursery.openChannel(lightningCurrency, swap, channelCreation);
                }
            }
            return undefined;
        };
        this.handleClnPaymentFailure = async (swap, error) => {
            const errorMessage = ClnClient_1.default.isRpcError(error)
                ? ClnClient_1.default.formatPaymentFailureReason(error)
                : (0, Utils_1.formatError)(error);
            this.logPaymentFailure(swap, errorMessage);
            if (ClnClient_1.default.errIsIncorrectPaymentDetails(errorMessage) ||
                LightningNursery_1.default.errIsInvoiceExpired(errorMessage)) {
                await this.abandonSwap(swap, errorMessage);
                return undefined;
            }
            return undefined;
        };
        this.abandonSwap = async (swap, errorMessage) => {
            this.logger.warn(`Abandoning Swap ${swap.id} because: ${errorMessage}`);
            this.emit('invoice.failedToPay', await SwapRepository_1.default.setSwapStatus(swap, Enums_1.SwapUpdateEvent.InvoiceFailedToPay, Errors_1.default.INVOICE_COULD_NOT_BE_PAID().message));
        };
        this.settleInvoice = async (swap, response) => {
            this.logger.verbose(`Paid invoice of Swap ${swap.id}: ${(0, Utils_1.getHexString)(response.preimage)}`);
            this.emit('invoice.paid', await SwapRepository_1.default.setInvoicePaid(swap, response.feeMsat));
            return response.preimage;
        };
        this.logPaymentFailure = (swap, errorMessage) => {
            this.logger.warn(`Could not pay invoice of Swap ${swap.id} because: ${errorMessage}`);
        };
    }
}
PaymentHandler.raceTimeout = 15;
PaymentHandler.resetMissionControlInterval = 10 * 60 * 1000;
PaymentHandler.errCltvTooSmall = 'CLTV limit too small';
exports.default = PaymentHandler;
//# sourceMappingURL=PaymentHandler.js.map