/// <reference types="node" />
import { SwapTreeSerializer } from 'boltz-core';
import Logger from '../Logger';
import { OrderSide, SwapVersion } from '../consts/Enums';
import { PairConfig } from '../consts/Types';
import Swap from '../db/models/Swap';
import LockupTransactionTracker from '../rates/LockupTransactionTracker';
import RateProvider from '../rates/RateProvider';
import Blocks from '../service/Blocks';
import PaymentRequestUtils from '../service/PaymentRequestUtils';
import TimeoutDeltaProvider from '../service/TimeoutDeltaProvider';
import DeferredClaimer, { SwapConfig } from '../service/cooperative/DeferredClaimer';
import WalletManager, { Currency } from '../wallet/WalletManager';
import NodeSwitch from './NodeSwitch';
import SwapNursery from './SwapNursery';
import SwapOutputType from './SwapOutputType';
import RoutingHints from './routing/RoutingHints';
type ChannelCreationInfo = {
    auto: boolean;
    private: boolean;
    inboundLiquidity: number;
};
type SetSwapInvoiceResponse = {
    channelCreationError?: string;
};
type CreatedSwap = {
    id: string;
    timeoutBlockHeight: number;
    address: string;
    redeemScript?: string;
    claimPublicKey?: string;
    swapTree?: SwapTreeSerializer.SerializedTree;
    claimAddress?: string;
    blindingKey?: string;
};
type CreatedReverseSwap = {
    id: string;
    timeoutBlockHeight: number;
    invoice: string;
    minerFeeInvoice: string | undefined;
    redeemScript: string | undefined;
    refundPublicKey?: string;
    swapTree?: SwapTreeSerializer.SerializedTree;
    refundAddress: string | undefined;
    lockupAddress: string;
    blindingKey?: string;
};
declare class SwapManager {
    private readonly logger;
    private readonly walletManager;
    private readonly nodeSwitch;
    private readonly rateProvider;
    private readonly timeoutDeltaProvider;
    private readonly swapOutputType;
    private readonly blocks;
    currencies: Map<string, Currency>;
    nursery: SwapNursery;
    routingHints: RoutingHints;
    readonly deferredClaimer: DeferredClaimer;
    private nodeFallback;
    private invoiceExpiryHelper;
    private readonly reverseRoutingHints;
    constructor(logger: Logger, walletManager: WalletManager, nodeSwitch: NodeSwitch, rateProvider: RateProvider, timeoutDeltaProvider: TimeoutDeltaProvider, paymentRequestUtils: PaymentRequestUtils, swapOutputType: SwapOutputType, retryInterval: number, blocks: Blocks, swapConfig: SwapConfig, lockupTransactionTracker: LockupTransactionTracker);
    init: (currencies: Currency[], pairs: PairConfig[]) => Promise<void>;
    /**
     * Creates a new Submarine Swap from the chain to Lightning with a preimage hash
     */
    createSwap: (args: {
        version: SwapVersion;
        baseCurrency: string;
        quoteCurrency: string;
        orderSide: OrderSide;
        preimageHash: Buffer;
        timeoutBlockDelta: number;
        channel?: ChannelCreationInfo;
        referralId?: string;
        refundPublicKey?: Buffer;
    }) => Promise<CreatedSwap>;
    /**
     * Sets the invoice for a Submarine Swap
     *
     * @param swap database object of the swap
     * @param invoice invoice of the Swap
     * @param invoiceAmount amount of the invoice in satoshis
     * @param expectedAmount amount that is expected onchain
     * @param percentageFee fee Boltz charges for the Swap
     * @param acceptZeroConf whether 0-conf transactions should be accepted
     * @param canBeRouted whether the invoice for the swap
     * @param emitSwapInvoiceSet method to emit an event after the invoice has been set
     */
    setSwapInvoice: (swap: Swap, invoice: string, invoiceAmount: number, expectedAmount: number, percentageFee: number, acceptZeroConf: boolean, canBeRouted: boolean, emitSwapInvoiceSet: (id: string) => void) => Promise<SetSwapInvoiceResponse>;
    /**
     * Creates a new reverse Swap from Lightning to the chain
     */
    createReverseSwap: (args: {
        version: number;
        baseCurrency: string;
        quoteCurrency: string;
        orderSide: OrderSide;
        preimageHash: Buffer;
        holdInvoiceAmount: number;
        onchainAmount: number;
        onchainTimeoutBlockDelta: number;
        lightningTimeoutBlockDelta: number;
        percentageFee: number;
        prepayMinerFeeInvoiceAmount?: number;
        prepayMinerFeeOnchainAmount?: number;
        routingNode?: string;
        referralId?: string;
        claimPublicKey?: Buffer;
        claimAddress?: string;
        userAddress?: string;
        userAddressSignature?: Buffer;
        claimCovenant: boolean;
    }) => Promise<CreatedReverseSwap>;
    private recreateFilters;
    private getCurrencies;
    private getCurrency;
    private getLockupContractAddress;
}
export default SwapManager;
export { ChannelCreationInfo };
