"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const boltz_core_1 = require("boltz-core");
const liquid_1 = require("boltz-core/dist/lib/liquid");
const crypto_1 = require("crypto");
const sequelize_1 = require("sequelize");
const Core_1 = require("../Core");
const Utils_1 = require("../Utils");
const Consts_1 = require("../consts/Consts");
const Enums_1 = require("../consts/Enums");
const ReverseSwap_1 = require("../db/models/ReverseSwap");
const ChannelCreationRepository_1 = __importDefault(require("../db/repositories/ChannelCreationRepository"));
const ReverseRoutingHintRepository_1 = __importDefault(require("../db/repositories/ReverseRoutingHintRepository"));
const ReverseSwapRepository_1 = __importDefault(require("../db/repositories/ReverseSwapRepository"));
const SwapRepository_1 = __importDefault(require("../db/repositories/SwapRepository"));
const InvoiceExpiryHelper_1 = __importDefault(require("../service/InvoiceExpiryHelper"));
const TimeoutDeltaProvider_1 = __importDefault(require("../service/TimeoutDeltaProvider"));
const DeferredClaimer_1 = __importDefault(require("../service/cooperative/DeferredClaimer"));
const Errors_1 = __importDefault(require("./Errors"));
const NodeFallback_1 = __importDefault(require("./NodeFallback"));
const NodeSwitch_1 = __importDefault(require("./NodeSwitch"));
const ReverseRoutingHints_1 = __importDefault(require("./ReverseRoutingHints"));
const SwapNursery_1 = __importDefault(require("./SwapNursery"));
const RoutingHints_1 = __importDefault(require("./routing/RoutingHints"));
class SwapManager {
    constructor(logger, walletManager, nodeSwitch, rateProvider, timeoutDeltaProvider, paymentRequestUtils, swapOutputType, retryInterval, blocks, swapConfig, lockupTransactionTracker) {
        this.logger = logger;
        this.walletManager = walletManager;
        this.nodeSwitch = nodeSwitch;
        this.rateProvider = rateProvider;
        this.timeoutDeltaProvider = timeoutDeltaProvider;
        this.swapOutputType = swapOutputType;
        this.blocks = blocks;
        this.currencies = new Map();
        this.init = async (currencies, pairs) => {
            currencies.forEach((currency) => {
                this.currencies.set(currency.symbol, currency);
            });
            await this.nursery.init(currencies);
            const [pendingSwaps, pendingReverseSwaps] = await Promise.all([
                SwapRepository_1.default.getSwaps({
                    status: {
                        [sequelize_1.Op.notIn]: [
                            Enums_1.SwapUpdateEvent.SwapExpired,
                            Enums_1.SwapUpdateEvent.InvoicePending,
                            Enums_1.SwapUpdateEvent.InvoiceFailedToPay,
                            Enums_1.SwapUpdateEvent.TransactionClaimed,
                        ],
                    },
                }),
                ReverseSwapRepository_1.default.getReverseSwaps({
                    status: {
                        [sequelize_1.Op.notIn]: [
                            Enums_1.SwapUpdateEvent.SwapExpired,
                            Enums_1.SwapUpdateEvent.InvoiceSettled,
                            Enums_1.SwapUpdateEvent.TransactionFailed,
                            Enums_1.SwapUpdateEvent.TransactionRefunded,
                        ],
                    },
                }),
            ]);
            this.recreateFilters(pendingSwaps, false);
            this.recreateFilters(pendingReverseSwaps, true);
            this.logger.info('Recreated input and output filters and invoice subscriptions');
            this.routingHints = new RoutingHints_1.default(this.logger, currencies);
            await this.routingHints.start();
            this.nodeFallback = new NodeFallback_1.default(this.logger, this.nodeSwitch, this.routingHints);
            this.invoiceExpiryHelper = new InvoiceExpiryHelper_1.default(pairs, this.timeoutDeltaProvider);
            await this.deferredClaimer.init();
        };
        /**
         * Creates a new Submarine Swap from the chain to Lightning with a preimage hash
         */
        this.createSwap = async (args) => {
            const { sendingCurrency, receivingCurrency } = this.getCurrencies(args.baseCurrency, args.quoteCurrency, args.orderSide);
            if (!NodeSwitch_1.default.hasClient(sendingCurrency)) {
                throw Errors_1.default.NO_LIGHTNING_SUPPORT(sendingCurrency.symbol);
            }
            const id = (0, Utils_1.generateSwapId)(args.version);
            this.logger.verbose(`Creating new ${(0, Enums_1.swapVersionToString)(args.version)} Swap from ${receivingCurrency.symbol} to ${sendingCurrency.symbol}: ${id}`);
            if (args.referralId) {
                this.logger.silly(`Using referral ID ${args.referralId} for Swap ${id}`);
            }
            const pair = (0, Utils_1.getPairId)({
                base: args.baseCurrency,
                quote: args.quoteCurrency,
            });
            const result = {
                id,
            };
            if (receivingCurrency.type === Enums_1.CurrencyType.BitcoinLike ||
                receivingCurrency.type === Enums_1.CurrencyType.Liquid) {
                const { blocks } = await receivingCurrency.chainClient.getBlockchainInfo();
                result.timeoutBlockHeight = blocks + args.timeoutBlockDelta;
                const { keys, index } = receivingCurrency.wallet.getNewKeys();
                let outputScript;
                let tree;
                switch (args.version) {
                    case Enums_1.SwapVersion.Taproot: {
                        result.claimPublicKey = (0, Utils_1.getHexString)(keys.publicKey);
                        tree = (0, boltz_core_1.swapTree)(receivingCurrency.type === Enums_1.CurrencyType.Liquid, args.preimageHash, keys.publicKey, args.refundPublicKey, result.timeoutBlockHeight);
                        result.swapTree = boltz_core_1.SwapTreeSerializer.serializeSwapTree(tree);
                        const musig = (0, Core_1.createMusig)(keys, args.refundPublicKey);
                        const tweakedKey = (0, Core_1.tweakMusig)(receivingCurrency.type, musig, tree);
                        outputScript = boltz_core_1.Scripts.p2trOutput(tweakedKey);
                        break;
                    }
                    default: {
                        const redeemScript = (0, boltz_core_1.swapScript)(args.preimageHash, keys.publicKey, args.refundPublicKey, result.timeoutBlockHeight);
                        result.redeemScript = (0, Utils_1.getHexString)(redeemScript);
                        const encodeFunction = (0, Utils_1.getScriptHashFunction)(this.swapOutputType.get(receivingCurrency.type));
                        outputScript = encodeFunction(redeemScript);
                        break;
                    }
                }
                result.address = receivingCurrency.wallet.encodeAddress(outputScript);
                receivingCurrency.chainClient.addOutputFilter(outputScript);
                if (receivingCurrency.type === Enums_1.CurrencyType.Liquid) {
                    result.blindingKey = (0, Utils_1.getHexString)(receivingCurrency.wallet.deriveBlindingKeyFromScript(outputScript).privateKey);
                }
                await SwapRepository_1.default.addSwap({
                    id,
                    pair,
                    keyIndex: index,
                    version: args.version,
                    orderSide: args.orderSide,
                    referral: args.referralId,
                    lockupAddress: result.address,
                    status: Enums_1.SwapUpdateEvent.SwapCreated,
                    timeoutBlockHeight: result.timeoutBlockHeight,
                    preimageHash: (0, Utils_1.getHexString)(args.preimageHash),
                    refundPublicKey: (0, Utils_1.getHexString)(args.refundPublicKey),
                    redeemScript: args.version === Enums_1.SwapVersion.Legacy
                        ? result.redeemScript
                        : JSON.stringify(boltz_core_1.SwapTreeSerializer.serializeSwapTree(tree)),
                });
            }
            else {
                result.address = await this.getLockupContractAddress(receivingCurrency.symbol, receivingCurrency.type);
                const blockNumber = await receivingCurrency.provider.getBlockNumber();
                result.timeoutBlockHeight = blockNumber + args.timeoutBlockDelta;
                result.claimAddress = await receivingCurrency.wallet.getAddress();
                await SwapRepository_1.default.addSwap({
                    id,
                    pair,
                    version: args.version,
                    referral: args.referralId,
                    orderSide: args.orderSide,
                    lockupAddress: result.address,
                    status: Enums_1.SwapUpdateEvent.SwapCreated,
                    preimageHash: (0, Utils_1.getHexString)(args.preimageHash),
                    timeoutBlockHeight: result.timeoutBlockHeight,
                });
            }
            if (args.channel !== undefined) {
                this.logger.verbose(`Adding Channel Creation for Swap: ${id}`);
                await ChannelCreationRepository_1.default.addChannelCreation({
                    swapId: id,
                    private: args.channel.private,
                    type: args.channel.auto
                        ? Enums_1.ChannelCreationType.Auto
                        : Enums_1.ChannelCreationType.Create,
                    inboundLiquidity: args.channel.inboundLiquidity,
                });
            }
            return result;
        };
        /**
         * Sets the invoice for a Submarine Swap
         *
         * @param swap database object of the swap
         * @param invoice invoice of the Swap
         * @param invoiceAmount amount of the invoice in satoshis
         * @param expectedAmount amount that is expected onchain
         * @param percentageFee fee Boltz charges for the Swap
         * @param acceptZeroConf whether 0-conf transactions should be accepted
         * @param canBeRouted whether the invoice for the swap
         * @param emitSwapInvoiceSet method to emit an event after the invoice has been set
         */
        this.setSwapInvoice = async (swap, invoice, invoiceAmount, expectedAmount, percentageFee, acceptZeroConf, canBeRouted, emitSwapInvoiceSet) => {
            const response = {};
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const { receivingCurrency } = this.getCurrencies(base, quote, swap.orderSide);
            const decodedInvoice = (0, Utils_1.decodeInvoice)(invoice);
            if (decodedInvoice.paymentHash !== swap.preimageHash) {
                throw Errors_1.default.INVOICE_INVALID_PREIMAGE_HASH(swap.preimageHash);
            }
            const invoiceExpiry = InvoiceExpiryHelper_1.default.getInvoiceExpiry(decodedInvoice.timestamp, decodedInvoice.timeExpireDate);
            if ((0, Utils_1.getUnixTime)() >= invoiceExpiry) {
                throw Errors_1.default.INVOICE_EXPIRED_ALREADY();
            }
            const channelCreation = await ChannelCreationRepository_1.default.getChannelCreation({
                swapId: swap.id,
            });
            if (channelCreation) {
                const getChainInfo = async (currency) => {
                    if (currency.type === Enums_1.CurrencyType.BitcoinLike ||
                        currency.type === Enums_1.CurrencyType.Liquid) {
                        const { blocks } = await currency.chainClient.getBlockchainInfo();
                        return {
                            blocks,
                            blockTime: TimeoutDeltaProvider_1.default.blockTimes.get(currency.symbol),
                        };
                        // All currencies that are not Bitcoin-like are either Ether or an ERC20 token on the Ethereum chain
                    }
                    else {
                        const networkInfo = this.walletManager.ethereumManagers.find((manager) => manager.hasSymbol(currency.symbol)).networkDetails;
                        return {
                            blocks: await currency.provider.getBlockNumber(),
                            blockTime: TimeoutDeltaProvider_1.default.blockTimes.get(networkInfo.symbol),
                        };
                    }
                };
                const { blocks, blockTime } = await getChainInfo(receivingCurrency);
                const blocksUntilExpiry = swap.timeoutBlockHeight - blocks;
                const timeoutTimestamp = (0, Utils_1.getUnixTime)() + blocksUntilExpiry * blockTime * 60;
                if (timeoutTimestamp > invoiceExpiry) {
                    const invoiceError = Errors_1.default.INVOICE_EXPIRES_TOO_EARLY(invoiceExpiry, timeoutTimestamp);
                    // In the auto Channel Creation mode, which is used by the frontend, the invoice check can fail but the Swap should
                    // still be attempted without Channel Creation
                    if (channelCreation.type === Enums_1.ChannelCreationType.Auto) {
                        this.logger.info(`Disabling Channel Creation for Swap ${swap.id}: ${invoiceError.message}`);
                        response.channelCreationError = invoiceError.message;
                        await channelCreation.destroy();
                        if (!canBeRouted) {
                            throw Errors_1.default.NO_ROUTE_FOUND();
                        }
                        // In other modes (only manual right now), a failing invoice Check should result in a failed request
                    }
                    else {
                        throw invoiceError;
                    }
                }
                await ChannelCreationRepository_1.default.setNodePublicKey(channelCreation, decodedInvoice.payeeNodeKey);
            }
            else if (!decodedInvoice.routingInfo ||
                (decodedInvoice.routingInfo && decodedInvoice.routingInfo.length === 0)) {
                if (!canBeRouted) {
                    throw Errors_1.default.NO_ROUTE_FOUND();
                }
            }
            this.logger.debug(`Setting invoice of Swap ${swap.id}: ${invoice}`);
            await this.nursery.lock.acquire(SwapNursery_1.default.swapLock, async () => {
                // Fetch the status again to make sure it is latest from the database
                const previousStatus = (await swap.reload()).status;
                await SwapRepository_1.default.setInvoice(swap, invoice, invoiceAmount, expectedAmount, percentageFee, acceptZeroConf);
                // Fetch the swap
                const updatedSwap = (await SwapRepository_1.default.getSwap({ id: swap.id }));
                // Not the most elegant way to emit this event but the only option
                // to emit it before trying to claim the swap
                emitSwapInvoiceSet(updatedSwap.id);
                // If the onchain coins were sent already and 0-conf can be accepted or
                // the lockup transaction is confirmed the swap should be settled directly
                if (swap.lockupTransactionId &&
                    previousStatus !== Enums_1.SwapUpdateEvent.TransactionZeroConfRejected) {
                    try {
                        await this.nursery.attemptSettleSwap(receivingCurrency, updatedSwap);
                    }
                    catch (error) {
                        this.logger.warn(`Could not settle Swap ${swap.id}: ${(0, Utils_1.formatError)(error)}`);
                    }
                }
            });
            return response;
        };
        /**
         * Creates a new reverse Swap from Lightning to the chain
         */
        this.createReverseSwap = async (args) => {
            const { sendingCurrency, receivingCurrency } = this.getCurrencies(args.baseCurrency, args.quoteCurrency, args.orderSide);
            if (!NodeSwitch_1.default.hasClient(receivingCurrency)) {
                throw Errors_1.default.NO_LIGHTNING_SUPPORT(receivingCurrency.symbol);
            }
            const id = (0, Utils_1.generateSwapId)(args.version);
            this.logger.verbose(`Creating new ${(0, Enums_1.swapVersionToString)(args.version)} Reverse Swap from ${receivingCurrency.symbol} to ${sendingCurrency.symbol}: ${id}`);
            if (args.referralId) {
                this.logger.silly(`Using referral ID ${args.referralId} for Reverse Swap ${id}`);
            }
            const isBitcoinLike = sendingCurrency.type === Enums_1.CurrencyType.BitcoinLike ||
                sendingCurrency.type === Enums_1.CurrencyType.Liquid;
            if (!isBitcoinLike && this.blocks.isBlocked(args.claimAddress)) {
                throw Errors_1.default.BLOCKED_ADDRESS();
            }
            const pair = (0, Utils_1.getPairId)({
                base: args.baseCurrency,
                quote: args.quoteCurrency,
            });
            const hints = this.reverseRoutingHints.getHints(sendingCurrency, args);
            const { nodeType, lightningClient, paymentRequest, routingHints } = await this.nodeFallback.getReverseSwapInvoice(id, args.referralId, args.routingNode, receivingCurrency, args.holdInvoiceAmount, args.preimageHash, args.lightningTimeoutBlockDelta, this.invoiceExpiryHelper.getExpiry(pair), hints.invoiceMemo, hints.routingHint);
            lightningClient.subscribeSingleInvoice(args.preimageHash);
            let minerFeeInvoice = undefined;
            let minerFeeInvoicePreimage = undefined;
            if (args.prepayMinerFeeInvoiceAmount) {
                const preimage = (0, crypto_1.randomBytes)(32);
                minerFeeInvoicePreimage = (0, Utils_1.getHexString)(preimage);
                const minerFeeInvoicePreimageHash = bitcoinjs_lib_1.crypto.sha256(preimage);
                minerFeeInvoice = await lightningClient.addHoldInvoice(args.prepayMinerFeeInvoiceAmount, minerFeeInvoicePreimageHash, undefined, this.invoiceExpiryHelper.getExpiry(pair), (0, Utils_1.getPrepayMinerFeeInvoiceMemo)(sendingCurrency.symbol), routingHints);
                lightningClient.subscribeSingleInvoice(minerFeeInvoicePreimageHash);
                if (args.prepayMinerFeeOnchainAmount) {
                    this.logger.debug(`Sending ${args.prepayMinerFeeOnchainAmount} ${this.walletManager.ethereumManagers.find((manager) => manager.hasSymbol(sendingCurrency.symbol)).networkDetails.name} as prepay miner fee for Reverse Swap: ${id}`);
                }
            }
            const result = {
                id,
                minerFeeInvoice,
                invoice: paymentRequest,
            };
            if (isBitcoinLike) {
                const { keys, index } = sendingCurrency.wallet.getNewKeys();
                const { blocks } = await sendingCurrency.chainClient.getBlockchainInfo();
                result.timeoutBlockHeight = blocks + args.onchainTimeoutBlockDelta;
                let outputScript;
                let tree;
                switch (args.version) {
                    case Enums_1.SwapVersion.Taproot: {
                        result.refundPublicKey = (0, Utils_1.getHexString)(keys.publicKey);
                        if (args.claimCovenant) {
                            if (sendingCurrency.type !== Enums_1.CurrencyType.Liquid) {
                                throw 'claim covenant only supported on Liquid';
                            }
                            if (args.userAddress === undefined) {
                                throw 'userAddress for covenant not specified';
                            }
                            try {
                                sendingCurrency.wallet.decodeAddress(args.userAddress);
                            }
                            catch (e) {
                                throw Errors_1.default.INVALID_ADDRESS();
                            }
                            tree = (0, liquid_1.reverseSwapTree)(args.preimageHash, args.claimPublicKey, keys.publicKey, result.timeoutBlockHeight, [
                                {
                                    expectedAmount: hints.receivedAmount,
                                    type: liquid_1.Feature.ClaimCovenant,
                                    assetHash: this.walletManager.wallets.get(sendingCurrency.symbol)
                                        .network.assetHash,
                                    outputScript: this.walletManager.wallets
                                        .get(sendingCurrency.symbol)
                                        .decodeAddress(args.userAddress),
                                },
                            ]);
                        }
                        else {
                            tree = (0, boltz_core_1.reverseSwapTree)(sendingCurrency.type === Enums_1.CurrencyType.Liquid, args.preimageHash, args.claimPublicKey, keys.publicKey, result.timeoutBlockHeight);
                        }
                        result.swapTree = boltz_core_1.SwapTreeSerializer.serializeSwapTree(tree);
                        const musig = (0, Core_1.createMusig)(keys, args.claimPublicKey);
                        const tweakedKey = (0, Core_1.tweakMusig)(sendingCurrency.type, musig, tree);
                        outputScript = boltz_core_1.Scripts.p2trOutput(tweakedKey);
                        break;
                    }
                    default: {
                        const redeemScript = (0, boltz_core_1.reverseSwapScript)(args.preimageHash, args.claimPublicKey, keys.publicKey, result.timeoutBlockHeight);
                        result.redeemScript = (0, Utils_1.getHexString)(redeemScript);
                        outputScript = (0, Utils_1.getScriptHashFunction)(Consts_1.LegacyReverseSwapOutputType)(redeemScript);
                        break;
                    }
                }
                result.lockupAddress = sendingCurrency.wallet.encodeAddress(outputScript);
                if (sendingCurrency.type === Enums_1.CurrencyType.Liquid) {
                    result.blindingKey = (0, Utils_1.getHexString)(sendingCurrency.wallet.deriveBlindingKeyFromScript(outputScript).privateKey);
                }
                await ReverseSwapRepository_1.default.addReverseSwap({
                    id,
                    pair,
                    minerFeeInvoice,
                    node: nodeType,
                    keyIndex: index,
                    version: args.version,
                    fee: args.percentageFee,
                    invoice: paymentRequest,
                    referral: args.referralId,
                    orderSide: args.orderSide,
                    onchainAmount: args.onchainAmount,
                    lockupAddress: result.lockupAddress,
                    status: Enums_1.SwapUpdateEvent.SwapCreated,
                    invoiceAmount: args.holdInvoiceAmount,
                    timeoutBlockHeight: result.timeoutBlockHeight,
                    preimageHash: (0, Utils_1.getHexString)(args.preimageHash),
                    minerFeeInvoicePreimage: minerFeeInvoicePreimage,
                    claimPublicKey: (0, Utils_1.getHexString)(args.claimPublicKey),
                    minerFeeOnchainAmount: args.prepayMinerFeeOnchainAmount,
                    redeemScript: args.version === Enums_1.SwapVersion.Legacy
                        ? result.redeemScript
                        : JSON.stringify(boltz_core_1.SwapTreeSerializer.serializeSwapTree(tree)),
                });
                if (hints.routingHint && hints.bip21 && args.userAddressSignature) {
                    await ReverseRoutingHintRepository_1.default.addHint({
                        swapId: id,
                        bip21: hints.bip21,
                        signature: (0, Utils_1.getHexString)(args.userAddressSignature),
                    });
                }
            }
            else {
                const blockNumber = await sendingCurrency.provider.getBlockNumber();
                result.timeoutBlockHeight = blockNumber + args.onchainTimeoutBlockDelta;
                result.lockupAddress = await this.getLockupContractAddress(sendingCurrency.symbol, sendingCurrency.type);
                result.refundAddress = await this.walletManager.wallets
                    .get(sendingCurrency.symbol)
                    .getAddress();
                await ReverseSwapRepository_1.default.addReverseSwap({
                    id,
                    pair,
                    minerFeeInvoice,
                    node: nodeType,
                    fee: args.percentageFee,
                    invoice: paymentRequest,
                    orderSide: args.orderSide,
                    referral: args.referralId,
                    version: Enums_1.SwapVersion.Legacy,
                    claimAddress: args.claimAddress,
                    lockupAddress: result.lockupAddress,
                    onchainAmount: args.onchainAmount,
                    status: Enums_1.SwapUpdateEvent.SwapCreated,
                    invoiceAmount: args.holdInvoiceAmount,
                    timeoutBlockHeight: result.timeoutBlockHeight,
                    preimageHash: (0, Utils_1.getHexString)(args.preimageHash),
                    minerFeeInvoicePreimage: minerFeeInvoicePreimage,
                    minerFeeOnchainAmount: args.prepayMinerFeeOnchainAmount,
                });
            }
            return result;
        };
        // TODO: check current status of invoices or do the streams handle that already?
        this.recreateFilters = (swaps, isReverse) => {
            swaps.forEach((swap) => {
                const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
                const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, isReverse);
                const lightningCurrency = (0, Utils_1.getLightningCurrency)(base, quote, swap.orderSide, isReverse);
                if ((swap.status === Enums_1.SwapUpdateEvent.SwapCreated ||
                    swap.status === Enums_1.SwapUpdateEvent.MinerFeePaid) &&
                    isReverse) {
                    const reverseSwap = swap;
                    const { lndClient } = this.currencies.get(lightningCurrency);
                    if (reverseSwap.node === ReverseSwap_1.NodeType.LND &&
                        reverseSwap.minerFeeInvoice &&
                        swap.status !== Enums_1.SwapUpdateEvent.MinerFeePaid) {
                        lndClient?.subscribeSingleInvoice((0, Utils_1.getHexBuffer)((0, Utils_1.decodeInvoice)(reverseSwap.minerFeeInvoice).paymentHash));
                    }
                    lndClient?.subscribeSingleInvoice((0, Utils_1.getHexBuffer)((0, Utils_1.decodeInvoice)(reverseSwap.invoice).paymentHash));
                }
                else if ((swap.status === Enums_1.SwapUpdateEvent.TransactionMempool ||
                    swap.status === Enums_1.SwapUpdateEvent.TransactionConfirmed) &&
                    isReverse) {
                    const { chainClient } = this.currencies.get(chainCurrency);
                    if (chainClient) {
                        const transactionId = (0, Utils_1.reverseBuffer)((0, Utils_1.getHexBuffer)(swap.transactionId));
                        chainClient.addInputFilter(transactionId);
                        // To detect when the transaction confirms
                        if (swap.status === Enums_1.SwapUpdateEvent.TransactionMempool) {
                            const wallet = this.walletManager.wallets.get(chainCurrency);
                            chainClient.addOutputFilter(wallet.decodeAddress(swap.lockupAddress));
                        }
                    }
                }
                else {
                    const { chainClient } = this.currencies.get(chainCurrency);
                    if (chainClient) {
                        const wallet = this.walletManager.wallets.get(chainCurrency);
                        const outputScript = wallet.decodeAddress(swap.lockupAddress);
                        chainClient.addOutputFilter(outputScript);
                    }
                }
            });
        };
        this.getCurrencies = (baseCurrency, quoteCurrency, orderSide) => {
            const { sending, receiving } = (0, Utils_1.getSendingReceivingCurrency)(baseCurrency, quoteCurrency, orderSide);
            return {
                sendingCurrency: {
                    ...this.getCurrency(sending),
                    wallet: this.walletManager.wallets.get(sending),
                },
                receivingCurrency: {
                    ...this.getCurrency(receiving),
                    wallet: this.walletManager.wallets.get(receiving),
                },
            };
        };
        this.getCurrency = (currencySymbol) => {
            const currency = this.currencies.get(currencySymbol);
            if (!currency) {
                throw Errors_1.default.CURRENCY_NOT_FOUND(currencySymbol).message;
            }
            return currency;
        };
        this.getLockupContractAddress = (symbol, type) => {
            const ethereumManager = this.walletManager.ethereumManagers.find((manager) => manager.hasSymbol(symbol));
            return type === Enums_1.CurrencyType.Ether
                ? ethereumManager.etherSwap.getAddress()
                : ethereumManager.erc20Swap.getAddress();
        };
        this.deferredClaimer = new DeferredClaimer_1.default(this.logger, this.currencies, this.walletManager, this.swapOutputType, swapConfig);
        this.nursery = new SwapNursery_1.default(this.logger, this.nodeSwitch, rateProvider, timeoutDeltaProvider, this.walletManager, this.swapOutputType, retryInterval, this.blocks, this.deferredClaimer, lockupTransactionTracker);
        this.reverseRoutingHints = new ReverseRoutingHints_1.default(this.walletManager, this.rateProvider, paymentRequestUtils);
    }
}
exports.default = SwapManager;
//# sourceMappingURL=SwapManager.js.map