"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const async_lock_1 = __importDefault(require("async-lock"));
const boltz_core_1 = require("boltz-core");
const sequelize_1 = require("sequelize");
const Core_1 = require("../Core");
const Utils_1 = require("../Utils");
const Consts_1 = require("../consts/Consts");
const Enums_1 = require("../consts/Enums");
const TypedEventEmitter_1 = __importDefault(require("../consts/TypedEventEmitter"));
const ChannelCreationRepository_1 = __importDefault(require("../db/repositories/ChannelCreationRepository"));
const ReverseSwapRepository_1 = __importDefault(require("../db/repositories/ReverseSwapRepository"));
const SwapRepository_1 = __importDefault(require("../db/repositories/SwapRepository"));
const LightningClient_1 = require("../lightning/LightningClient");
const FeeProvider_1 = __importDefault(require("../rates/FeeProvider"));
const ContractUtils_1 = require("../wallet/ethereum/ContractUtils");
const ChannelNursery_1 = __importDefault(require("./ChannelNursery"));
const Errors_1 = __importDefault(require("./Errors"));
const EthereumNursery_1 = __importDefault(require("./EthereumNursery"));
const InvoiceNursery_1 = __importDefault(require("./InvoiceNursery"));
const LightningNursery_1 = __importDefault(require("./LightningNursery"));
const NodeSwitch_1 = __importDefault(require("./NodeSwitch"));
const PaymentHandler_1 = __importDefault(require("./PaymentHandler"));
const UtxoNursery_1 = __importDefault(require("./UtxoNursery"));
class SwapNursery extends TypedEventEmitter_1.default {
    constructor(logger, nodeSwitch, rateProvider, timeoutDeltaProvider, walletManager, swapOutputType, retryInterval, blocks, claimer, lockupTransactionTracker) {
        super();
        this.logger = logger;
        this.nodeSwitch = nodeSwitch;
        this.rateProvider = rateProvider;
        this.walletManager = walletManager;
        this.swapOutputType = swapOutputType;
        this.retryInterval = retryInterval;
        this.claimer = claimer;
        // Maps
        this.currencies = new Map();
        // Locks
        this.lock = new async_lock_1.default();
        this.init = async (currencies) => {
            currencies.forEach((currency) => {
                this.currencies.set(currency.symbol, currency);
            });
            await Promise.all(this.ethereumNurseries.map((nursery) => this.listenEthereumNursery(nursery)));
            // Swap events
            this.utxoNursery.on('swap.expired', async (swap) => {
                await this.lock.acquire(SwapNursery.swapLock, async () => {
                    await this.expireSwap(swap);
                });
            });
            this.utxoNursery.on('swap.lockup.failed', async ({ swap, reason }) => {
                await this.lock.acquire(SwapNursery.swapLock, async () => {
                    await this.lockupFailed(swap, reason);
                });
            });
            this.utxoNursery.on('swap.lockup.zeroconf.rejected', async ({ swap, transaction, reason }) => {
                await this.lock.acquire(SwapNursery.swapLock, async () => {
                    this.logger.warn(`Rejected 0-conf lockup transaction (${transaction.getId()}:${swap.lockupTransactionVout}) of ${swap.id}: ${reason}`);
                    if (!swap.invoice) {
                        await this.setSwapRate(swap);
                    }
                    this.emit('zeroconf.rejected', await SwapRepository_1.default.setSwapStatus(swap, Enums_1.SwapUpdateEvent.TransactionZeroConfRejected));
                });
            });
            this.utxoNursery.on('swap.lockup', async ({ swap, transaction, confirmed }) => {
                await this.lock.acquire(SwapNursery.swapLock, async () => {
                    this.emit('transaction', {
                        swap,
                        confirmed,
                        transaction,
                        isReverse: false,
                    });
                    if (swap.invoice) {
                        const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
                        const chainSymbol = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
                        const { chainClient } = this.currencies.get(chainSymbol);
                        const wallet = this.walletManager.wallets.get(chainSymbol);
                        await this.claimUtxo(chainClient, wallet, swap, transaction);
                    }
                    else {
                        await this.setSwapRate(swap);
                    }
                });
            });
            // Reverse Swap events
            this.utxoNursery.on('reverseSwap.expired', async (reverseSwap) => {
                await this.lock.acquire(SwapNursery.reverseSwapLock, async () => {
                    await this.expireReverseSwap(reverseSwap);
                });
            });
            this.utxoNursery.on('reverseSwap.lockup.confirmed', async ({ reverseSwap, transaction }) => {
                await this.lock.acquire(SwapNursery.reverseSwapLock, async () => {
                    this.emit('transaction', {
                        transaction,
                        confirmed: true,
                        isReverse: true,
                        swap: reverseSwap,
                    });
                });
            });
            this.lightningNursery.on('minerfee.invoice.paid', async (reverseSwap) => {
                await this.lock.acquire(SwapNursery.reverseSwapLock, async () => {
                    this.emit('minerfee.paid', reverseSwap);
                });
            });
            this.lightningNursery.on('invoice.paid', async (reverseSwap) => {
                await this.lock.acquire(SwapNursery.reverseSwapLock, async () => {
                    const { base, quote } = (0, Utils_1.splitPairId)(reverseSwap.pair);
                    const chainSymbol = (0, Utils_1.getChainCurrency)(base, quote, reverseSwap.orderSide, true);
                    const lightningSymbol = (0, Utils_1.getLightningCurrency)(base, quote, reverseSwap.orderSide, true);
                    const chainCurrency = this.currencies.get(chainSymbol);
                    const lightningClient = NodeSwitch_1.default.getReverseSwapNode(this.currencies.get(lightningSymbol), reverseSwap);
                    const wallet = this.walletManager.wallets.get(chainSymbol);
                    switch (chainCurrency.type) {
                        case Enums_1.CurrencyType.BitcoinLike:
                        case Enums_1.CurrencyType.Liquid:
                            await this.lockupUtxo(chainCurrency.chainClient, this.walletManager.wallets.get(chainSymbol), lightningClient, reverseSwap);
                            break;
                        case Enums_1.CurrencyType.Ether:
                            await this.lockupEther(wallet, lightningClient, reverseSwap);
                            break;
                        case Enums_1.CurrencyType.ERC20:
                            await this.lockupERC20(wallet, lightningClient, reverseSwap);
                            break;
                    }
                });
            });
            this.utxoNursery.on('reverseSwap.claimed', async ({ reverseSwap, preimage }) => {
                await this.lock.acquire(SwapNursery.reverseSwapLock, async () => {
                    await this.settleReverseSwapInvoice(reverseSwap, preimage);
                });
            });
            this.invoiceNursery.on('invoice.expired', async (reverseSwap) => {
                await this.lock.acquire(SwapNursery.reverseSwapLock, async () => {
                    const { base, quote } = (0, Utils_1.splitPairId)(reverseSwap.pair);
                    const receiveCurrency = (0, Utils_1.getLightningCurrency)(base, quote, reverseSwap.orderSide, true);
                    const lightningClient = NodeSwitch_1.default.getReverseSwapNode(this.currencies.get(receiveCurrency), reverseSwap);
                    const plural = reverseSwap.minerFeeInvoicePreimage === null ? '' : 's';
                    try {
                        // Check if the hold invoice has pending HTLCs before actually cancelling
                        const { htlcs, state } = await lightningClient.raceCall(lightningClient.lookupHoldInvoice((0, Utils_1.getHexBuffer)(reverseSwap.preimageHash)), (reject) => reject('lookup for HTLCs of hold invoice timed out'), LightningNursery_1.default.lightningClientCallTimeout);
                        if (state === LightningClient_1.InvoiceState.Cancelled) {
                            this.logger.debug(`Invoice${plural} of Reverse Swap ${reverseSwap.id} already cancelled`);
                        }
                        else {
                            if (htlcs.length !== 0 &&
                                htlcs.some((htlc) => htlc.state !== LightningClient_1.HtlcState.Cancelled)) {
                                this.logger.info(`Not cancelling expired hold invoice${plural} of Reverse Swap ${reverseSwap.id} because it has pending HTLCs`);
                                return;
                            }
                            this.logger.debug(`Cancelling expired hold invoice${plural} of Reverse Swap ${reverseSwap.id}`);
                            await LightningNursery_1.default.cancelReverseInvoices(lightningClient, reverseSwap, true);
                        }
                    }
                    catch (error) {
                        // In case the LND client could not find the invoice(s) of the Reverse Swap, we just ignore the error and mark them as cancelled regardless
                        // This happens quite a lot on regtest environments where the LND client is reset without the database being deleted
                        if (typeof error !== 'object' ||
                            (error.details !== 'unable to locate invoice' &&
                                error.details !== 'there are no existing invoices' &&
                                error.message !== 'hold invoice not found')) {
                            this.logger.error(`Could not cancel invoice${plural} of Reverse Swap ${reverseSwap.id}: ${(0, Utils_1.formatError)(error)}`);
                            return;
                        }
                        else {
                            this.logger.silly(`Cancelling invoice${plural} of Reverse Swap ${reverseSwap.id} failed because they could not be found: ${(0, Utils_1.formatError)(error)}`);
                        }
                    }
                    this.emit('invoice.expired', await ReverseSwapRepository_1.default.setReverseSwapStatus(reverseSwap, Enums_1.SwapUpdateEvent.InvoiceExpired));
                });
            });
            this.utxoNursery.bindCurrency(currencies);
            this.lightningNursery.bindCurrencies(currencies);
            await this.invoiceNursery.init();
            await this.channelNursery.init(currencies);
            if (this.retryInterval !== 0) {
                setInterval(async () => {
                    // Skip this iteration if the last one is still running
                    if (this.lock.isBusy(SwapNursery.retryLock)) {
                        return;
                    }
                    this.logger.silly('Retrying settling Swaps with pending invoices');
                    await this.lock.acquire(SwapNursery.retryLock, async () => {
                        await this.lock.acquire(SwapNursery.swapLock, async () => {
                            const pendingInvoiceSwaps = await SwapRepository_1.default.getSwaps({
                                status: {
                                    [sequelize_1.Op.in]: [
                                        Enums_1.SwapUpdateEvent.InvoicePending,
                                        Enums_1.SwapUpdateEvent.InvoicePaid,
                                    ],
                                },
                            });
                            for (const pendingInvoiceSwap of pendingInvoiceSwaps) {
                                const { base, quote } = (0, Utils_1.splitPairId)(pendingInvoiceSwap.pair);
                                const chainCurrency = this.currencies.get((0, Utils_1.getChainCurrency)(base, quote, pendingInvoiceSwap.orderSide, false));
                                await this.attemptSettleSwap(chainCurrency, pendingInvoiceSwap);
                            }
                        });
                    });
                }, this.retryInterval * 1000);
            }
        };
        this.attemptSettleSwap = async (currency, swap, outgoingChannelId) => {
            switch (currency.type) {
                case Enums_1.CurrencyType.BitcoinLike:
                case Enums_1.CurrencyType.Liquid: {
                    const lockupTransactionHex = await currency.chainClient.getRawTransaction(swap.lockupTransactionId);
                    await this.claimUtxo(currency.chainClient, this.walletManager.wallets.get(currency.symbol), swap, (0, Core_1.parseTransaction)(currency.type, lockupTransactionHex), outgoingChannelId);
                    break;
                }
                case Enums_1.CurrencyType.Ether: {
                    const manager = this.findEthereumNursery(currency.symbol).ethereumManager;
                    await this.claimEther(manager, swap, await (0, ContractUtils_1.queryEtherSwapValuesFromLock)(manager.provider, manager.etherSwap, swap.lockupTransactionId), outgoingChannelId);
                    break;
                }
                case Enums_1.CurrencyType.ERC20: {
                    const manager = this.findEthereumNursery(currency.symbol).ethereumManager;
                    await this.claimERC20(manager.contractHandler, swap, await (0, ContractUtils_1.queryERC20SwapValuesFromLock)(manager.provider, manager.erc20Swap, swap.lockupTransactionId), outgoingChannelId);
                    break;
                }
            }
        };
        this.settleReverseSwapInvoice = async (reverseSwap, preimage) => {
            const { base, quote } = (0, Utils_1.splitPairId)(reverseSwap.pair);
            const lightningCurrency = (0, Utils_1.getLightningCurrency)(base, quote, reverseSwap.orderSide, true);
            const lightningClient = NodeSwitch_1.default.getReverseSwapNode(this.currencies.get(lightningCurrency), reverseSwap);
            try {
                await lightningClient.raceCall(lightningClient.settleHoldInvoice(preimage), (reject) => {
                    reject('invoice settlement timed out');
                }, LightningNursery_1.default.lightningClientCallTimeout);
                this.logger.info(`Settled Reverse Swap ${reverseSwap.id}`);
                this.emit('invoice.settled', await ReverseSwapRepository_1.default.setInvoiceSettled(reverseSwap, (0, Utils_1.getHexString)(preimage)));
            }
            catch (e) {
                this.logger.error(`Could not settle invoice: ${(0, Utils_1.formatError)(e)}`);
            }
        };
        this.listenEthereumNursery = async (ethereumNursery) => {
            const contractHandler = ethereumNursery.ethereumManager.contractHandler;
            // Swap events
            ethereumNursery.on('swap.expired', async ({ swap }) => {
                await this.lock.acquire(SwapNursery.swapLock, async () => {
                    await this.expireSwap(swap);
                });
            });
            ethereumNursery.on('lockup.failed', async ({ swap, reason }) => {
                await this.lock.acquire(SwapNursery.swapLock, async () => {
                    await this.lockupFailed(swap, reason);
                });
            });
            ethereumNursery.on('eth.lockup', async ({ swap, transactionHash, etherSwapValues }) => {
                await this.lock.acquire(SwapNursery.swapLock, async () => {
                    this.emit('transaction', {
                        swap,
                        confirmed: true,
                        isReverse: false,
                        transaction: transactionHash,
                    });
                    if (swap.invoice) {
                        await this.claimEther(ethereumNursery.ethereumManager, swap, etherSwapValues);
                    }
                    else {
                        await this.setSwapRate(swap);
                    }
                });
            });
            ethereumNursery.on('erc20.lockup', async ({ swap, transactionHash, erc20SwapValues }) => {
                await this.lock.acquire(SwapNursery.swapLock, async () => {
                    this.emit('transaction', {
                        swap,
                        confirmed: true,
                        isReverse: false,
                        transaction: transactionHash,
                    });
                    if (swap.invoice) {
                        await this.claimERC20(contractHandler, swap, erc20SwapValues);
                    }
                    else {
                        await this.setSwapRate(swap);
                    }
                });
            });
            // Reverse Swap events
            ethereumNursery.on('reverseSwap.expired', async ({ reverseSwap }) => {
                await this.lock.acquire(SwapNursery.reverseSwapLock, async () => {
                    await this.expireReverseSwap(reverseSwap);
                });
            });
            ethereumNursery.on('lockup.failedToSend', async ({ reverseSwap, reason }) => {
                await this.lock.acquire(SwapNursery.reverseSwapLock, async () => {
                    const { base, quote } = (0, Utils_1.splitPairId)(reverseSwap.pair);
                    const chainSymbol = (0, Utils_1.getChainCurrency)(base, quote, reverseSwap.orderSide, true);
                    const lightningSymbol = (0, Utils_1.getLightningCurrency)(base, quote, reverseSwap.orderSide, true);
                    await this.handleReverseSwapSendFailed(reverseSwap, chainSymbol, NodeSwitch_1.default.getReverseSwapNode(this.currencies.get(lightningSymbol), reverseSwap), reason);
                });
            });
            ethereumNursery.on('lockup.confirmed', async ({ reverseSwap, transactionHash }) => {
                await this.lock.acquire(SwapNursery.reverseSwapLock, async () => {
                    this.emit('transaction', {
                        confirmed: true,
                        isReverse: true,
                        swap: reverseSwap,
                        transaction: transactionHash,
                    });
                });
            });
            ethereumNursery.on('claim', async ({ reverseSwap, preimage }) => {
                await this.lock.acquire(SwapNursery.reverseSwapLock, async () => {
                    await this.settleReverseSwapInvoice(reverseSwap, preimage);
                });
            });
            await ethereumNursery.init();
        };
        /**
         * Sets the rate for a Swap that doesn't have an invoice yet
         */
        this.setSwapRate = async (swap) => {
            if (!swap.rate) {
                const rate = (0, Utils_1.getRate)(this.rateProvider.providers[Enums_1.SwapVersion.Legacy].pairs.get(swap.pair)
                    .rate, swap.orderSide, false);
                await SwapRepository_1.default.setRate(swap, rate);
            }
        };
        this.lockupUtxo = async (chainClient, wallet, lightningClient, reverseSwap) => {
            try {
                let feePerVbyte;
                if (reverseSwap.minerFeeInvoice) {
                    // TODO: how does this behave cross chain
                    feePerVbyte = Math.round((0, Utils_1.decodeInvoice)(reverseSwap.minerFeeInvoice).satoshis /
                        FeeProvider_1.default.transactionSizes[Enums_1.CurrencyType.BitcoinLike][Enums_1.SwapVersion.Legacy].reverseLockup);
                    this.logger.debug(`Using prepay minerfee for lockup of Reverse Swap ${reverseSwap.id}: ${feePerVbyte} sat/vbyte`);
                }
                else {
                    feePerVbyte = await chainClient.estimateFee(SwapNursery.reverseSwapMempoolEta);
                }
                const { transaction, transactionId, vout, fee } = await wallet.sendToAddress(reverseSwap.lockupAddress, reverseSwap.onchainAmount, feePerVbyte);
                this.logger.verbose(`Locked up ${reverseSwap.onchainAmount} ${wallet.symbol} for Reverse Swap ${reverseSwap.id}: ${transactionId}:${vout}`);
                chainClient.addInputFilter(transaction.getHash());
                // For the "transaction.confirmed" event of the lockup transaction
                chainClient.addOutputFilter(wallet.decodeAddress(reverseSwap.lockupAddress));
                this.emit('coins.sent', {
                    transaction: transaction,
                    reverseSwap: await ReverseSwapRepository_1.default.setLockupTransaction(reverseSwap, transactionId, fee, vout),
                });
            }
            catch (error) {
                await this.handleReverseSwapSendFailed(reverseSwap, wallet.symbol, lightningClient, error);
            }
        };
        this.lockupEther = async (wallet, lightningClient, reverseSwap) => {
            try {
                const nursery = this.findEthereumNursery(wallet.symbol);
                let contractTransaction;
                if (reverseSwap.minerFeeOnchainAmount) {
                    contractTransaction =
                        await nursery.ethereumManager.contractHandler.lockupEtherPrepayMinerfee((0, Utils_1.getHexBuffer)(reverseSwap.preimageHash), BigInt(reverseSwap.onchainAmount) * Consts_1.etherDecimals, BigInt(reverseSwap.minerFeeOnchainAmount) * Consts_1.etherDecimals, reverseSwap.claimAddress, reverseSwap.timeoutBlockHeight);
                }
                else {
                    contractTransaction =
                        await nursery.ethereumManager.contractHandler.lockupEther((0, Utils_1.getHexBuffer)(reverseSwap.preimageHash), BigInt(reverseSwap.onchainAmount) * Consts_1.etherDecimals, reverseSwap.claimAddress, reverseSwap.timeoutBlockHeight);
                }
                nursery.listenContractTransaction(reverseSwap, contractTransaction);
                this.logger.verbose(`Locked up ${reverseSwap.onchainAmount} ${wallet.symbol} for Reverse Swap ${reverseSwap.id}: ${contractTransaction.hash}`);
                this.emit('coins.sent', {
                    transaction: contractTransaction.hash,
                    reverseSwap: await ReverseSwapRepository_1.default.setLockupTransaction(reverseSwap, contractTransaction.hash, (0, Utils_1.calculateEthereumTransactionFee)(contractTransaction)),
                });
            }
            catch (error) {
                await this.handleReverseSwapSendFailed(reverseSwap, wallet.symbol, lightningClient, error);
            }
        };
        this.lockupERC20 = async (wallet, lightningClient, reverseSwap) => {
            try {
                const nursery = this.findEthereumNursery(wallet.symbol);
                const walletProvider = wallet.walletProvider;
                let contractTransaction;
                if (reverseSwap.minerFeeOnchainAmount) {
                    contractTransaction =
                        await nursery.ethereumManager.contractHandler.lockupTokenPrepayMinerfee(walletProvider, (0, Utils_1.getHexBuffer)(reverseSwap.preimageHash), walletProvider.formatTokenAmount(reverseSwap.onchainAmount), BigInt(reverseSwap.minerFeeOnchainAmount) * Consts_1.etherDecimals, reverseSwap.claimAddress, reverseSwap.timeoutBlockHeight);
                }
                else {
                    contractTransaction =
                        await nursery.ethereumManager.contractHandler.lockupToken(walletProvider, (0, Utils_1.getHexBuffer)(reverseSwap.preimageHash), walletProvider.formatTokenAmount(reverseSwap.onchainAmount), reverseSwap.claimAddress, reverseSwap.timeoutBlockHeight);
                }
                nursery.listenContractTransaction(reverseSwap, contractTransaction);
                this.logger.verbose(`Locked up ${reverseSwap.onchainAmount} ${wallet.symbol} for Reverse Swap ${reverseSwap.id}: ${contractTransaction.hash}`);
                this.emit('coins.sent', {
                    transaction: contractTransaction.hash,
                    reverseSwap: await ReverseSwapRepository_1.default.setLockupTransaction(reverseSwap, contractTransaction.hash, (0, Utils_1.calculateEthereumTransactionFee)(contractTransaction)),
                });
            }
            catch (error) {
                await this.handleReverseSwapSendFailed(reverseSwap, wallet.symbol, lightningClient, error);
            }
        };
        this.claimUtxo = async (chainClient, wallet, swap, transaction, outgoingChannelId) => {
            const channelCreation = await ChannelCreationRepository_1.default.getChannelCreation({
                swapId: swap.id,
            });
            const preimage = await this.paymentHandler.payInvoice(swap, channelCreation, outgoingChannelId);
            if (!preimage) {
                return;
            }
            if (await this.claimer.deferClaim(swap, preimage)) {
                this.emit('claim.pending', swap);
                return;
            }
            const claimTransaction = (0, Core_1.constructClaimTransaction)(wallet, [
                (0, Core_1.constructClaimDetails)(this.swapOutputType, wallet, swap, transaction, preimage),
            ], await wallet.getAddress(), await chainClient.estimateFee());
            const claimTransactionFee = await (0, Core_1.calculateTransactionFee)(chainClient, claimTransaction);
            await chainClient.sendRawTransaction(claimTransaction.toHex(), true);
            this.logger.info(`Claimed ${wallet.symbol} of Swap ${swap.id} in: ${claimTransaction.getId()}`);
            this.emit('claim', {
                channelCreation: channelCreation || undefined,
                swap: await SwapRepository_1.default.setMinerFee(swap, claimTransactionFee),
            });
        };
        this.claimEther = async (manager, swap, etherSwapValues, outgoingChannelId) => {
            const channelCreation = await ChannelCreationRepository_1.default.getChannelCreation({
                swapId: swap.id,
            });
            const preimage = await this.paymentHandler.payInvoice(swap, channelCreation, outgoingChannelId);
            if (!preimage) {
                return;
            }
            const contractTransaction = await manager.contractHandler.claimEther(preimage, etherSwapValues.amount, etherSwapValues.refundAddress, etherSwapValues.timelock);
            this.logger.info(`Claimed ${manager.networkDetails.name} of Swap ${swap.id} in: ${contractTransaction.hash}`);
            this.emit('claim', {
                channelCreation: channelCreation || undefined,
                swap: await SwapRepository_1.default.setMinerFee(swap, (0, Utils_1.calculateEthereumTransactionFee)(contractTransaction)),
            });
        };
        this.claimERC20 = async (contractHandler, swap, erc20SwapValues, outgoingChannelId) => {
            const channelCreation = await ChannelCreationRepository_1.default.getChannelCreation({
                swapId: swap.id,
            });
            const preimage = await this.paymentHandler.payInvoice(swap, channelCreation, outgoingChannelId);
            if (!preimage) {
                return;
            }
            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
            const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
            const wallet = this.walletManager.wallets.get(chainCurrency);
            const contractTransaction = await contractHandler.claimToken(wallet.walletProvider, preimage, erc20SwapValues.amount, erc20SwapValues.refundAddress, erc20SwapValues.timelock);
            this.logger.info(`Claimed ${chainCurrency} of Swap ${swap.id} in: ${contractTransaction.hash}`);
            this.emit('claim', {
                channelCreation: channelCreation || undefined,
                swap: await SwapRepository_1.default.setMinerFee(swap, (0, Utils_1.calculateEthereumTransactionFee)(contractTransaction)),
            });
        };
        this.handleReverseSwapSendFailed = async (reverseSwap, chainSymbol, lightningClient, error) => {
            await LightningNursery_1.default.cancelReverseInvoices(lightningClient, reverseSwap, false);
            this.logger.warn(`Failed to lockup ${reverseSwap.onchainAmount} ${chainSymbol} for Reverse Swap ${reverseSwap.id}: ${(0, Utils_1.formatError)(error)}`);
            this.emit('coins.failedToSend', await ReverseSwapRepository_1.default.setReverseSwapStatus(reverseSwap, Enums_1.SwapUpdateEvent.TransactionFailed, Errors_1.default.COINS_COULD_NOT_BE_SENT().message));
        };
        this.lockupFailed = async (swap, reason) => {
            this.logger.warn(`Lockup of Swap ${swap.id} failed: ${reason}`);
            this.emit('lockup.failed', await SwapRepository_1.default.setSwapStatus(swap, Enums_1.SwapUpdateEvent.TransactionLockupFailed, reason));
        };
        this.expireSwap = async (swap) => {
            // Check "expireReverseSwap" for reason
            const queriedSwap = await SwapRepository_1.default.getSwap({
                id: swap.id,
            });
            if (queriedSwap.status === Enums_1.SwapUpdateEvent.SwapExpired) {
                return;
            }
            this.emit('expiration', {
                isReverse: false,
                swap: await SwapRepository_1.default.setSwapStatus(swap, Enums_1.SwapUpdateEvent.SwapExpired, Errors_1.default.ONCHAIN_HTLC_TIMED_OUT().message),
            });
        };
        this.expireReverseSwap = async (reverseSwap) => {
            // Sometimes, when blocks are mined quickly (realistically just regtest), it can happen that the
            // nurseries, which are not in the async lock, send the expiration event of a Swap multiple times.
            // To handle this scenario, the Swap is queried again to ensure that it should actually be expired or refunded
            const queriedReverseSwap = await ReverseSwapRepository_1.default.getReverseSwap({
                id: reverseSwap.id,
            });
            if (queriedReverseSwap.status === Enums_1.SwapUpdateEvent.SwapExpired ||
                queriedReverseSwap.status === Enums_1.SwapUpdateEvent.TransactionRefunded) {
                return;
            }
            const { base, quote } = (0, Utils_1.splitPairId)(reverseSwap.pair);
            const chainSymbol = (0, Utils_1.getChainCurrency)(base, quote, reverseSwap.orderSide, true);
            const lightningSymbol = (0, Utils_1.getLightningCurrency)(base, quote, reverseSwap.orderSide, true);
            const chainCurrency = this.currencies.get(chainSymbol);
            const lightningCurrency = this.currencies.get(lightningSymbol);
            if (reverseSwap.transactionId) {
                switch (chainCurrency.type) {
                    case Enums_1.CurrencyType.BitcoinLike:
                    case Enums_1.CurrencyType.Liquid:
                        await this.refundUtxo(reverseSwap, chainSymbol);
                        break;
                    case Enums_1.CurrencyType.Ether:
                        await this.refundEther(reverseSwap, chainSymbol);
                        break;
                    case Enums_1.CurrencyType.ERC20:
                        await this.refundERC20(reverseSwap, chainSymbol);
                        break;
                }
            }
            else {
                this.emit('expiration', {
                    isReverse: true,
                    swap: await ReverseSwapRepository_1.default.setReverseSwapStatus(reverseSwap, Enums_1.SwapUpdateEvent.SwapExpired, Errors_1.default.ONCHAIN_HTLC_TIMED_OUT().message),
                });
            }
            const lightningClient = NodeSwitch_1.default.getReverseSwapNode(lightningCurrency, reverseSwap);
            try {
                await LightningNursery_1.default.cancelReverseInvoices(lightningClient, reverseSwap, true);
            }
            catch (e) {
                this.logger.debug(`Could not cancel invoices of Reverse Swap ${reverseSwap.id} because: ${(0, Utils_1.formatError)(e)}`);
            }
        };
        this.refundUtxo = async (reverseSwap, chainSymbol) => {
            const chainCurrency = this.currencies.get(chainSymbol);
            const chainClient = chainCurrency.chainClient;
            const wallet = this.walletManager.wallets.get(chainSymbol);
            const rawLockupTransaction = await chainCurrency.chainClient.getRawTransaction(reverseSwap.transactionId);
            const lockupTransaction = (0, Core_1.parseTransaction)(chainClient.currencyType, rawLockupTransaction);
            const lockupOutput = lockupTransaction.outs[reverseSwap.transactionVout];
            const refundDetails = {
                ...lockupOutput,
                vout: reverseSwap.transactionVout,
                txHash: lockupTransaction.getHash(),
                keys: wallet.getKeysByIndex(reverseSwap.keyIndex),
            };
            switch (reverseSwap.version) {
                case Enums_1.SwapVersion.Taproot: {
                    refundDetails.type = boltz_core_1.OutputType.Taproot;
                    refundDetails.cooperative = false;
                    refundDetails.swapTree = boltz_core_1.SwapTreeSerializer.deserializeSwapTree(reverseSwap.redeemScript);
                    refundDetails.internalKey = (0, Core_1.createMusig)(refundDetails.keys, (0, Utils_1.getHexBuffer)(reverseSwap.claimPublicKey)).getAggregatedPublicKey();
                    break;
                }
                default: {
                    refundDetails.type = Consts_1.LegacyReverseSwapOutputType;
                    refundDetails.redeemScript = (0, Utils_1.getHexBuffer)(reverseSwap.redeemScript);
                    break;
                }
            }
            const refundTransaction = (0, Core_1.constructRefundTransaction)(wallet, [refundDetails], await wallet.getAddress(), reverseSwap.timeoutBlockHeight, await chainCurrency.chainClient.estimateFee());
            const minerFee = await (0, Core_1.calculateTransactionFee)(chainCurrency.chainClient, refundTransaction);
            await chainClient.sendRawTransaction(refundTransaction.toHex(), true);
            this.logger.info(`Refunded ${chainSymbol} of Reverse Swap ${reverseSwap.id} in: ${refundTransaction.getId()}`);
            this.emit('refund', {
                refundTransaction: refundTransaction.getId(),
                reverseSwap: await ReverseSwapRepository_1.default.setTransactionRefunded(reverseSwap, minerFee, Errors_1.default.REFUNDED_COINS(reverseSwap.transactionId).message),
            });
        };
        this.refundEther = async (reverseSwap, chainSymbol) => {
            const nursery = this.findEthereumNursery(chainSymbol);
            const etherSwapValues = await (0, ContractUtils_1.queryEtherSwapValuesFromLock)(nursery.ethereumManager.provider, nursery.ethereumManager.etherSwap, reverseSwap.transactionId);
            const contractTransaction = await nursery.ethereumManager.contractHandler.refundEther((0, Utils_1.getHexBuffer)(reverseSwap.preimageHash), etherSwapValues.amount, etherSwapValues.claimAddress, etherSwapValues.timelock);
            this.logger.info(`Refunded ${nursery.ethereumManager.networkDetails.name} of Reverse Swap ${reverseSwap.id} in: ${contractTransaction.hash}`);
            this.emit('refund', {
                refundTransaction: contractTransaction.hash,
                reverseSwap: await ReverseSwapRepository_1.default.setTransactionRefunded(reverseSwap, (0, Utils_1.calculateEthereumTransactionFee)(contractTransaction), Errors_1.default.REFUNDED_COINS(reverseSwap.transactionId).message),
            });
        };
        this.refundERC20 = async (reverseSwap, chainSymbol) => {
            const nursery = this.findEthereumNursery(chainSymbol);
            const walletProvider = this.walletManager.wallets.get(chainSymbol)
                .walletProvider;
            const erc20SwapValues = await (0, ContractUtils_1.queryERC20SwapValuesFromLock)(nursery.ethereumManager.provider, nursery.ethereumManager.erc20Swap, reverseSwap.transactionId);
            const contractTransaction = await nursery.ethereumManager.contractHandler.refundToken(walletProvider, (0, Utils_1.getHexBuffer)(reverseSwap.preimageHash), erc20SwapValues.amount, erc20SwapValues.claimAddress, erc20SwapValues.timelock);
            this.logger.info(`Refunded ${chainSymbol} of Reverse Swap ${reverseSwap.id} in: ${contractTransaction.hash}`);
            this.emit('refund', {
                refundTransaction: contractTransaction.hash,
                reverseSwap: await ReverseSwapRepository_1.default.setTransactionRefunded(reverseSwap, (0, Utils_1.calculateEthereumTransactionFee)(contractTransaction), Errors_1.default.REFUNDED_COINS(reverseSwap.transactionId).message),
            });
        };
        this.findEthereumNursery = (symbol) => {
            for (const nursery of this.ethereumNurseries) {
                if (nursery.ethereumManager.hasSymbol(symbol)) {
                    return nursery;
                }
            }
            return undefined;
        };
        this.logger.info(`Setting Swap retry interval to ${retryInterval} seconds`);
        this.utxoNursery = new UtxoNursery_1.default(this.logger, this.walletManager, blocks, lockupTransactionTracker);
        this.lightningNursery = new LightningNursery_1.default(this.logger);
        this.invoiceNursery = new InvoiceNursery_1.default(this.logger);
        this.channelNursery = new ChannelNursery_1.default(this.logger, this.attemptSettleSwap);
        this.ethereumNurseries = this.walletManager.ethereumManagers.map((manager) => new EthereumNursery_1.default(this.logger, this.walletManager, manager, blocks));
        this.paymentHandler = new PaymentHandler_1.default(this.logger, this.nodeSwitch, this.currencies, this.channelNursery, timeoutDeltaProvider, (eventName, arg) => {
            this.emit(eventName, arg);
        });
        this.claimer.on('claim', ({ swap, channelCreation }) => {
            this.emit('claim', { swap, channelCreation });
        });
    }
}
// Constants
SwapNursery.reverseSwapMempoolEta = 2;
SwapNursery.swapLock = 'swap';
SwapNursery.reverseSwapLock = 'reverseSwap';
SwapNursery.retryLock = 'retry';
exports.default = SwapNursery;
//# sourceMappingURL=SwapNursery.js.map