"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const async_lock_1 = __importDefault(require("async-lock"));
const boltz_core_1 = require("boltz-core");
const sequelize_1 = require("sequelize");
const Core_1 = require("../Core");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const TypedEventEmitter_1 = __importDefault(require("../consts/TypedEventEmitter"));
const ReverseSwapRepository_1 = __importDefault(require("../db/repositories/ReverseSwapRepository"));
const SwapRepository_1 = __importDefault(require("../db/repositories/SwapRepository"));
const Errors_1 = __importDefault(require("./Errors"));
class UtxoNursery extends TypedEventEmitter_1.default {
    constructor(logger, walletManager, blocks, lockupTransactionTracker) {
        super();
        this.logger = logger;
        this.walletManager = walletManager;
        this.blocks = blocks;
        this.lockupTransactionTracker = lockupTransactionTracker;
        // Locks
        this.lock = new async_lock_1.default();
        this.bindCurrency = (currencies) => {
            currencies.forEach((currency) => {
                if (currency.chainClient) {
                    const wallet = this.walletManager.wallets.get(currency.symbol);
                    this.listenBlocks(currency.chainClient, wallet);
                    this.listenTransactions(currency.chainClient, wallet);
                }
            });
        };
        this.listenTransactions = (chainClient, wallet) => {
            chainClient.on('transaction', async ({ transaction, confirmed }) => {
                await Promise.all([
                    this.checkSwapOutputs(chainClient, wallet, transaction, confirmed),
                    this.checkReverseSwapClaims(chainClient, transaction),
                    this.checkReverseSwapLockupsConfirmed(chainClient, wallet, transaction, confirmed),
                ]);
            });
        };
        this.checkSwapOutputs = async (chainClient, wallet, transaction, confirmed) => {
            await this.lock.acquire(UtxoNursery.swapLockupLock, async () => {
                for (let vout = 0; vout < transaction.outs.length; vout += 1) {
                    const output = transaction.outs[vout];
                    const swap = await SwapRepository_1.default.getSwap({
                        status: {
                            [sequelize_1.Op.or]: [
                                Enums_1.SwapUpdateEvent.SwapCreated,
                                Enums_1.SwapUpdateEvent.InvoiceSet,
                                Enums_1.SwapUpdateEvent.TransactionMempool,
                                Enums_1.SwapUpdateEvent.TransactionZeroConfRejected,
                            ],
                        },
                        lockupAddress: wallet.encodeAddress(output.script),
                    });
                    if (!swap) {
                        continue;
                    }
                    await this.checkSwapTransaction(swap, chainClient, wallet, transaction, confirmed);
                }
            });
        };
        this.checkReverseSwapClaims = async (chainClient, transaction) => {
            for (let vin = 0; vin < transaction.ins.length; vin += 1) {
                const input = transaction.ins[vin];
                const reverseSwap = await ReverseSwapRepository_1.default.getReverseSwap({
                    status: {
                        [sequelize_1.Op.or]: [
                            Enums_1.SwapUpdateEvent.TransactionMempool,
                            Enums_1.SwapUpdateEvent.TransactionConfirmed,
                        ],
                    },
                    transactionId: (0, Utils_1.transactionHashToId)(input.hash),
                    transactionVout: input.index,
                });
                if (!reverseSwap) {
                    continue;
                }
                this.logger.verbose(`Found claim transaction of Reverse Swap ${reverseSwap.id}: ${transaction.getId()}`);
                chainClient.removeInputFilter(input.hash);
                this.emit('reverseSwap.claimed', {
                    reverseSwap,
                    preimage: (0, boltz_core_1.detectPreimage)(vin, transaction),
                });
            }
        };
        this.checkReverseSwapLockupsConfirmed = async (chainClient, wallet, transaction, confirmed) => {
            await this.lock.acquire(UtxoNursery.reverseSwapLockupConfirmationLock, async () => {
                if (!confirmed) {
                    return;
                }
                const reverseSwap = await ReverseSwapRepository_1.default.getReverseSwap({
                    status: Enums_1.SwapUpdateEvent.TransactionMempool,
                    transactionId: transaction.getId(),
                });
                if (reverseSwap) {
                    await this.reverseSwapLockupConfirmed(chainClient, wallet, reverseSwap, transaction);
                }
            });
        };
        this.listenBlocks = (chainClient, wallet) => {
            chainClient.on('block', async (height) => {
                await Promise.all([
                    this.checkSwapMempoolTransactions(chainClient, wallet),
                    this.checkReverseSwapMempoolTransactions(chainClient, wallet),
                    this.checkExpiredSwaps(chainClient, height),
                    this.checkExpiredReverseSwaps(chainClient, height),
                ]);
            });
        };
        this.checkSwapMempoolTransactions = async (chainClient, wallet) => {
            await this.lock.acquire(UtxoNursery.swapLockupLock, async () => {
                const mempoolSwaps = await SwapRepository_1.default.getSwaps({
                    status: {
                        [sequelize_1.Op.or]: [
                            Enums_1.SwapUpdateEvent.TransactionMempool,
                            Enums_1.SwapUpdateEvent.TransactionZeroConfRejected,
                        ],
                    },
                });
                for (const swap of mempoolSwaps) {
                    const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
                    const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
                    if (chainCurrency !== chainClient.symbol) {
                        continue;
                    }
                    try {
                        const lockupTransaction = await chainClient.getRawTransactionVerbose(swap.lockupTransactionId);
                        if (lockupTransaction.confirmations &&
                            lockupTransaction.confirmations !== 0) {
                            await this.checkSwapTransaction(swap, chainClient, wallet, (0, Core_1.parseTransaction)(wallet.type, lockupTransaction.hex), true);
                        }
                    }
                    catch (e) {
                        this.logger.silly(`Could not find lockup transaction of swap ${swap.id}: ${swap.lockupTransactionId}`);
                    }
                }
            });
        };
        // This method is a fallback for "checkReverseSwapLockupsConfirmed" because that method sometimes misses transactions on mainnet for an unknown reason
        this.checkReverseSwapMempoolTransactions = async (chainClient, wallet) => {
            await this.lock.acquire(UtxoNursery.reverseSwapLockupConfirmationLock, async () => {
                const mempoolReverseSwaps = await ReverseSwapRepository_1.default.getReverseSwaps({
                    status: Enums_1.SwapUpdateEvent.TransactionMempool,
                });
                for (const reverseSwap of mempoolReverseSwaps) {
                    const { base, quote } = (0, Utils_1.splitPairId)(reverseSwap.pair);
                    const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, reverseSwap.orderSide, true);
                    if (chainCurrency !== chainClient.symbol) {
                        continue;
                    }
                    const transaction = await chainClient.getRawTransactionVerbose(reverseSwap.transactionId);
                    if (transaction.confirmations && transaction.confirmations !== 0) {
                        await this.reverseSwapLockupConfirmed(chainClient, wallet, reverseSwap, (0, Core_1.parseTransaction)(wallet.type, transaction.hex));
                    }
                }
            });
        };
        this.checkExpiredSwaps = async (chainClient, height) => {
            const expirableSwaps = await SwapRepository_1.default.getSwapsExpirable(height);
            for (const expirableSwap of expirableSwaps) {
                const { base, quote } = (0, Utils_1.splitPairId)(expirableSwap.pair);
                const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, expirableSwap.orderSide, false);
                if (chainCurrency === chainClient.symbol) {
                    const wallet = this.walletManager.wallets.get(chainCurrency);
                    chainClient.removeOutputFilter(wallet.decodeAddress(expirableSwap.lockupAddress));
                    this.emit('swap.expired', expirableSwap);
                }
            }
        };
        this.checkExpiredReverseSwaps = async (chainClient, height) => {
            const expirableReverseSwaps = await ReverseSwapRepository_1.default.getReverseSwapsExpirable(height);
            for (const expirableReverseSwap of expirableReverseSwaps) {
                const { base, quote } = (0, Utils_1.splitPairId)(expirableReverseSwap.pair);
                const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, expirableReverseSwap.orderSide, true);
                if (chainCurrency === chainClient.symbol) {
                    const wallet = this.walletManager.wallets.get(chainCurrency);
                    chainClient.removeOutputFilter(wallet.decodeAddress(expirableReverseSwap.lockupAddress));
                    if (expirableReverseSwap.transactionId) {
                        chainClient.removeInputFilter((0, Utils_1.reverseBuffer)((0, Utils_1.getHexBuffer)(expirableReverseSwap.transactionId)));
                    }
                    this.emit('reverseSwap.expired', expirableReverseSwap);
                }
            }
        };
        this.reverseSwapLockupConfirmed = async (chainClient, wallet, reverseSwap, transaction) => {
            this.logger.debug(`Lockup transaction of Reverse Swap ${reverseSwap.id} confirmed: ${transaction.getId()}`);
            chainClient.removeOutputFilter(wallet.decodeAddress(reverseSwap.lockupAddress));
            this.emit('reverseSwap.lockup.confirmed', {
                transaction,
                reverseSwap: await ReverseSwapRepository_1.default.setReverseSwapStatus(reverseSwap, Enums_1.SwapUpdateEvent.TransactionConfirmed),
            });
        };
        this.checkSwapTransaction = async (swap, chainClient, wallet, transaction, confirmed) => {
            let redeemScriptOrTweakedKey;
            switch (swap.version) {
                case Enums_1.SwapVersion.Taproot: {
                    const tree = boltz_core_1.SwapTreeSerializer.deserializeSwapTree(swap.redeemScript);
                    const musig = (0, Core_1.createMusig)(wallet.getKeysByIndex(swap.keyIndex), (0, Utils_1.getHexBuffer)(swap.refundPublicKey));
                    redeemScriptOrTweakedKey = (0, Core_1.tweakMusig)(wallet.type, musig, tree);
                    break;
                }
                default: {
                    redeemScriptOrTweakedKey = (0, Utils_1.getHexBuffer)(swap.redeemScript);
                    break;
                }
            }
            const swapOutput = (0, boltz_core_1.detectSwap)(redeemScriptOrTweakedKey, transaction);
            this.logger.verbose(`Found ${confirmed ? '' : 'un'}confirmed ${wallet.symbol} lockup transaction for Swap ${swap.id}: ${transaction.getId()}:${swapOutput.vout}`);
            const outputValue = (0, Core_1.getOutputValue)(wallet, swapOutput);
            const updatedSwap = await SwapRepository_1.default.setLockupTransaction(swap, transaction.getId(), outputValue, confirmed, swapOutput.vout);
            if (updatedSwap.expectedAmount) {
                if (updatedSwap.expectedAmount > outputValue) {
                    chainClient.removeOutputFilter(swapOutput.script);
                    this.emit('swap.lockup.failed', {
                        swap: updatedSwap,
                        reason: Errors_1.default.INSUFFICIENT_AMOUNT(outputValue, updatedSwap.expectedAmount).message,
                    });
                    return;
                }
            }
            const prevAddreses = await this.getPreviousAddresses(transaction, chainClient, wallet);
            if (prevAddreses.some(this.blocks.isBlocked)) {
                this.emit('swap.lockup.failed', {
                    swap: updatedSwap,
                    reason: Errors_1.default.BLOCKED_ADDRESS().message,
                });
                return;
            }
            // Confirmed transactions do not have to be checked for 0-conf criteria
            if (!confirmed) {
                if (updatedSwap.acceptZeroConf !== true ||
                    !this.lockupTransactionTracker.zeroConfAccepted(chainClient.symbol)) {
                    this.emit('swap.lockup.zeroconf.rejected', {
                        transaction,
                        swap: updatedSwap,
                        reason: Errors_1.default.SWAP_DOES_NOT_ACCEPT_ZERO_CONF().message,
                    });
                    return;
                }
                const signalsRbf = await this.transactionSignalsRbf(chainClient, transaction);
                if (signalsRbf) {
                    this.emit('swap.lockup.zeroconf.rejected', {
                        transaction,
                        swap: updatedSwap,
                        reason: Errors_1.default.LOCKUP_TRANSACTION_SIGNALS_RBF().message,
                    });
                    return;
                }
                // Check if the transaction has a fee high enough to be confirmed in a timely manner
                const [feeEstimation, absoluteTransactionFee] = await Promise.all([
                    chainClient.estimateFee(),
                    (0, Core_1.calculateTransactionFee)(chainClient, transaction),
                ]);
                const transactionFeePerVbyte = absoluteTransactionFee / transaction.virtualSize();
                // If the transaction fee is less than 80% of the estimation, Boltz will wait for a confirmation
                //
                // Special case: if the fee estimation is the lowest possible of 2 sat/vbyte,
                // every fee paid by the transaction will be accepted
                if (transactionFeePerVbyte / feeEstimation < 0.8 && feeEstimation !== 2) {
                    this.emit('swap.lockup.zeroconf.rejected', {
                        transaction,
                        swap: updatedSwap,
                        reason: Errors_1.default.LOCKUP_TRANSACTION_FEE_TOO_LOW().message,
                    });
                    return;
                }
                this.logger.debug(`Accepted 0-conf lockup transaction for Swap ${updatedSwap.id}: ${transaction.getId()}:${swapOutput.vout}`);
                await this.lockupTransactionTracker.addPendingTransactionToTrack(swap);
            }
            chainClient.removeOutputFilter(swapOutput.script);
            this.emit('swap.lockup', { transaction, confirmed, swap: updatedSwap });
        };
        /**
         * Detects whether the transaction signals RBF explicitly or inherently
         */
        this.transactionSignalsRbf = async (chainClient, transaction) => {
            // Check for explicit signalling
            if ((0, Utils_1.transactionSignalsRbfExplicitly)(transaction)) {
                return true;
            }
            // Check for inherited signalling from unconfirmed inputs
            for (const input of transaction.ins) {
                const inputId = (0, Utils_1.transactionHashToId)(input.hash);
                const inputTransaction = await chainClient.getRawTransactionVerbose(inputId);
                if (!inputTransaction.confirmations) {
                    const inputSignalsRbf = await this.transactionSignalsRbf(chainClient, (0, Core_1.parseTransaction)(chainClient.currencyType, inputTransaction.hex));
                    if (inputSignalsRbf) {
                        return true;
                    }
                }
            }
            return false;
        };
        this.getPreviousAddresses = async (transaction, chainClient, wallet) => {
            const inputTxsIds = this.chunkArray(Array.from(new Set(transaction.ins.map((input) => (0, Utils_1.getHexString)((0, Utils_1.reverseBuffer)(input.hash)))).values()), UtxoNursery.maxParallelRequests);
            const inputTxs = (await Promise.all(inputTxsIds.map((ids) => this.getTransactions(chainClient, ids))))
                .flat()
                .map((txHex) => (0, Core_1.parseTransaction)(wallet.type, txHex));
            return inputTxs
                .map((tx) => tx.outs)
                .flat()
                .map((output) => wallet.type === Enums_1.CurrencyType.Liquid
                ? wallet.encodeAddress(output.script, false)
                : wallet.encodeAddress(output.script));
        };
        this.getTransactions = async (chainClient, ids) => {
            const txs = [];
            for (const id of ids) {
                txs.push(await chainClient.getRawTransaction(id));
            }
            return txs;
        };
        this.chunkArray = (array, size) => {
            const chunks = Array.from({ length: size }, () => []);
            for (let i = 0; i < array.length; i++) {
                chunks[i % size].push(array[i]);
            }
            return chunks.filter((chunk) => chunk.length !== 0);
        };
    }
}
UtxoNursery.maxParallelRequests = 6;
UtxoNursery.swapLockupLock = 'swapLockupLock';
UtxoNursery.reverseSwapLockupConfirmationLock = 'reverseSwapLockupConfirmation';
exports.default = UtxoNursery;
//# sourceMappingURL=UtxoNursery.js.map