"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Core_1 = require("../Core");
const KeyRepository_1 = __importDefault(require("../db/repositories/KeyRepository"));
const Errors_1 = __importDefault(require("./Errors"));
class Wallet {
    /**
     * Wallet is a hierarchical deterministic wallet for a currency
     */
    constructor(logger, type, walletProvider) {
        this.logger = logger;
        this.type = type;
        this.walletProvider = walletProvider;
        this.serviceName = () => {
            return this.walletProvider.serviceName();
        };
        /**
         * In case the Wallet should also provide keys and de/encode addresses
         * This is only required (and possible) for UTXO based chains like Bitcoin
         */
        this.initKeyProvider = (network, derivationPath, highestUsedIndex, masterNode) => {
            this.network = network;
            this.derivationPath = derivationPath;
            this.highestUsedIndex = highestUsedIndex;
            this.masterNode = masterNode;
        };
        /**
         * Gets a specific pair of keys
         *
         * @param index index of the keys to get
         */
        this.getKeysByIndex = (index) => {
            if (this.masterNode === undefined || this.derivationPath === undefined) {
                throw Errors_1.default.NOT_SUPPORTED_BY_WALLET(this.symbol, 'getKeysByIndex');
            }
            return this.masterNode.derivePath(`${this.derivationPath}/${index}`);
        };
        /**
         * Gets a new pair of keys
         */
        this.getNewKeys = () => {
            if (this.highestUsedIndex === undefined) {
                throw Errors_1.default.NOT_SUPPORTED_BY_WALLET(this.symbol, 'getNewKeys');
            }
            this.highestUsedIndex += 1;
            KeyRepository_1.default.setHighestUsedIndex(this.symbol, this.highestUsedIndex).then();
            return {
                keys: this.getKeysByIndex(this.highestUsedIndex),
                index: this.highestUsedIndex,
            };
        };
        /**
         * Encodes an address
         *
         * @param outputScript the output script to encode
         */
        this.encodeAddress = (outputScript) => {
            if (this.network === undefined) {
                throw Errors_1.default.NOT_SUPPORTED_BY_WALLET(this.symbol, 'encodeAddress');
            }
            try {
                return (0, Core_1.fromOutputScript)(this.type, outputScript, this.network);
            }
            catch (error) {
                // Ignore invalid addresses
                return '';
            }
        };
        /**
         * Decodes an address
         */
        this.decodeAddress = (toDecode) => {
            if (this.network === undefined) {
                throw Errors_1.default.NOT_SUPPORTED_BY_WALLET(this.symbol, 'decodeAddress');
            }
            return (0, Core_1.toOutputScript)(this.type, toDecode, this.network);
        };
        this.getAddress = () => {
            return this.walletProvider.getAddress();
        };
        this.getBalance = () => {
            return this.walletProvider.getBalance();
        };
        this.sendToAddress = (address, amount, satPerVbyte) => {
            this.logger.info(`Sending ${amount} ${this.symbol} to ${address}`);
            return this.walletProvider.sendToAddress(address, amount, satPerVbyte);
        };
        this.sweepWallet = (address, satPerVbyte) => {
            this.logger.warn(`Sweeping ${this.symbol} wallet to ${address}`);
            return this.walletProvider.sweepWallet(address, satPerVbyte);
        };
        this.symbol = this.walletProvider.symbol;
    }
}
exports.default = Wallet;
//# sourceMappingURL=Wallet.js.map