"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bip32_1 = __importDefault(require("bip32"));
const bip39_1 = require("bip39");
const fs_1 = __importDefault(require("fs"));
const slip77_1 = require("slip77");
const ecc = __importStar(require("tiny-secp256k1"));
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const KeyRepository_1 = __importDefault(require("../db/repositories/KeyRepository"));
const Errors_1 = __importDefault(require("./Errors"));
const Wallet_1 = __importDefault(require("./Wallet"));
const WalletLiquid_1 = __importDefault(require("./WalletLiquid"));
const CoreWalletProvider_1 = __importDefault(require("./providers/CoreWalletProvider"));
const ElementsWalletProvider_1 = __importDefault(require("./providers/ElementsWalletProvider"));
const LndWalletProvider_1 = __importDefault(require("./providers/LndWalletProvider"));
const bip32 = (0, bip32_1.default)(ecc);
const slip77 = (0, slip77_1.SLIP77Factory)(ecc);
/**
 * WalletManager creates wallets instances that generate keys derived from the seed and
 * interact with the wallet of LND to send and receive onchain coins
 */
class WalletManager {
    constructor(logger, mnemonicPath, currencies, ethereumManagers) {
        this.logger = logger;
        this.currencies = currencies;
        this.ethereumManagers = ethereumManagers;
        this.wallets = new Map();
        this.derivationPath = 'm/0';
        this.init = async (configCurrencies) => {
            const keyProviderMap = await this.getKeyProviderMap();
            for (const currency of this.currencies) {
                if (currency.type !== Enums_1.CurrencyType.BitcoinLike &&
                    currency.type !== Enums_1.CurrencyType.Liquid) {
                    continue;
                }
                let walletProvider = undefined;
                // The LND client is also used as onchain wallet for UTXO based chains if available
                if (configCurrencies.find((config) => config.symbol === currency.symbol)
                    ?.preferredWallet !== 'core' &&
                    currency.lndClient) {
                    walletProvider = new LndWalletProvider_1.default(this.logger, currency.lndClient, currency.chainClient);
                }
                else {
                    // Else the Bitcoin Core wallet is used
                    if (currency.type === Enums_1.CurrencyType.BitcoinLike) {
                        walletProvider = new CoreWalletProvider_1.default(this.logger, currency.chainClient);
                    }
                    else {
                        walletProvider = new ElementsWalletProvider_1.default(this.logger, currency.chainClient);
                    }
                    // Sanity check that wallet support is compiled in
                    try {
                        await walletProvider.getBalance();
                    }
                    catch (error) {
                        // No wallet support is compiled in
                        if (error.message === 'Method not found') {
                            throw Errors_1.default.NO_WALLET_SUPPORT(currency.symbol);
                        }
                        else {
                            throw error;
                        }
                    }
                }
                let keyProviderInfo = keyProviderMap.get(currency.symbol);
                // Generate a new KeyProvider if that currency does not have one yet
                if (!keyProviderInfo) {
                    keyProviderInfo = {
                        highestUsedIndex: 0,
                        symbol: currency.symbol,
                        derivationPath: `${this.derivationPath}/${this.getHighestDepthIndex(keyProviderMap, 2) + 1}`,
                    };
                    keyProviderMap.set(currency.symbol, keyProviderInfo);
                    await KeyRepository_1.default.addKeyProvider({
                        ...keyProviderInfo,
                        symbol: currency.symbol,
                    });
                }
                const wallet = currency.type !== Enums_1.CurrencyType.Liquid
                    ? new Wallet_1.default(this.logger, currency.type, walletProvider)
                    : new WalletLiquid_1.default(this.logger, walletProvider, this.slip77);
                wallet.initKeyProvider(currency.network, keyProviderInfo.derivationPath, keyProviderInfo.highestUsedIndex, this.masterNode);
                this.wallets.set(currency.symbol, wallet);
            }
            for (const manager of this.ethereumManagers) {
                const ethereumWallets = await manager.init(this.mnemonic);
                for (const [symbol, ethereumWallet] of ethereumWallets) {
                    this.wallets.set(symbol, ethereumWallet);
                }
            }
        };
        this.loadMnemonic = (filename) => {
            if (!fs_1.default.existsSync(filename)) {
                throw Errors_1.default.NOT_INITIALIZED();
            }
            const mnemonic = fs_1.default.readFileSync(filename, 'utf-8').trim();
            if (!(0, bip39_1.validateMnemonic)(mnemonic)) {
                throw Errors_1.default.INVALID_MNEMONIC(mnemonic);
            }
            return mnemonic;
        };
        this.getKeyProviderMap = async () => {
            const map = new Map();
            const keyProviders = await KeyRepository_1.default.getKeyProviders();
            keyProviders.forEach((keyProvider) => {
                map.set(keyProvider.symbol, {
                    symbol: keyProvider.symbol,
                    derivationPath: keyProvider.derivationPath,
                    highestUsedIndex: keyProvider.highestUsedIndex,
                });
            });
            return map;
        };
        this.getHighestDepthIndex = (map, depth) => {
            if (depth === 0) {
                throw Errors_1.default.INVALID_DEPTH_INDEX(depth);
            }
            let highestIndex = -1;
            map.forEach((info) => {
                const split = (0, Utils_1.splitDerivationPath)(info.derivationPath);
                const index = split.sub[depth - 1];
                if (index > highestIndex) {
                    highestIndex = index;
                }
            });
            return highestIndex;
        };
        if (!fs_1.default.existsSync(mnemonicPath)) {
            this.logger.info('Generated new mnemonic');
            fs_1.default.writeFileSync(mnemonicPath, (0, bip39_1.generateMnemonic)());
        }
        this.mnemonic = this.loadMnemonic(mnemonicPath);
        this.masterNode = bip32.fromSeed((0, bip39_1.mnemonicToSeedSync)(this.mnemonic));
        this.slip77 = slip77.fromSeed(this.mnemonic);
    }
}
exports.default = WalletManager;
//# sourceMappingURL=WalletManager.js.map