"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const TypedEventEmitter_1 = __importDefault(require("../../consts/TypedEventEmitter"));
const ContractUtils_1 = require("./ContractUtils");
const EthereumUtils_1 = require("./EthereumUtils");
class ContractEventHandler extends TypedEventEmitter_1.default {
    constructor(logger) {
        super();
        this.logger = logger;
        this.init = async (networkDetails, etherSwap, erc20Swap) => {
            this.etherSwap = etherSwap;
            this.erc20Swap = erc20Swap;
            this.logger.verbose(`Starting ${networkDetails.name} contract event subscriptions`);
            await this.subscribeContractEvents();
        };
        this.rescan = async (startHeight) => {
            const [etherLockups, etherClaims, etherRefunds] = await Promise.all([
                this.etherSwap.queryFilter(this.etherSwap.filters.Lockup(), startHeight),
                this.etherSwap.queryFilter(this.etherSwap.filters.Claim(), startHeight),
                this.etherSwap.queryFilter(this.etherSwap.filters.Refund(), startHeight),
            ]);
            for (const event of etherLockups) {
                this.emit('eth.lockup', {
                    transaction: await event.getTransaction(),
                    etherSwapValues: (0, ContractUtils_1.formatEtherSwapValues)(event.args),
                });
            }
            for (const event of etherClaims) {
                this.emit('eth.claim', {
                    transactionHash: event.transactionHash,
                    preimageHash: (0, EthereumUtils_1.parseBuffer)(event.topics[1]),
                    preimage: (0, EthereumUtils_1.parseBuffer)(event.args.preimage),
                });
            }
            for (const event of etherRefunds) {
                this.emit('eth.refund', {
                    transactionHash: event.transactionHash,
                    preimageHash: (0, EthereumUtils_1.parseBuffer)(event.topics[1]),
                });
            }
            const [erc20Lockups, erc20Claims, erc20Refunds] = await Promise.all([
                this.erc20Swap.queryFilter(this.erc20Swap.filters.Lockup(), startHeight),
                this.erc20Swap.queryFilter(this.erc20Swap.filters.Claim(), startHeight),
                this.erc20Swap.queryFilter(this.erc20Swap.filters.Refund(), startHeight),
            ]);
            for (const event of erc20Lockups) {
                this.emit('erc20.lockup', {
                    transaction: await event.getTransaction(),
                    erc20SwapValues: (0, ContractUtils_1.formatERC20SwapValues)(event.args),
                });
            }
            for (const event of erc20Claims) {
                this.emit('erc20.claim', {
                    transactionHash: event.transactionHash,
                    preimageHash: (0, EthereumUtils_1.parseBuffer)(event.topics[1]),
                    preimage: (0, EthereumUtils_1.parseBuffer)(event.args.preimage),
                });
            }
            for (const event of erc20Refunds) {
                this.emit('erc20.refund', {
                    transactionHash: event.transactionHash,
                    preimageHash: (0, EthereumUtils_1.parseBuffer)(event.topics[1]),
                });
            }
        };
        this.subscribeContractEvents = async () => {
            await this.etherSwap.on('Lockup', async (preimageHash, amount, claimAddress, refundAddress, timelock, event) => {
                this.emit('eth.lockup', {
                    transaction: await event.log.getTransaction(),
                    etherSwapValues: {
                        amount,
                        claimAddress,
                        refundAddress,
                        timelock: Number(timelock),
                        preimageHash: (0, EthereumUtils_1.parseBuffer)(preimageHash),
                    },
                });
            });
            await this.etherSwap.on('Claim', (preimageHash, preimage, event) => {
                this.emit('eth.claim', {
                    transactionHash: event.log.transactionHash,
                    preimageHash: (0, EthereumUtils_1.parseBuffer)(preimageHash),
                    preimage: (0, EthereumUtils_1.parseBuffer)(preimage),
                });
            });
            await this.etherSwap.on('Refund', (preimageHash, event) => {
                this.emit('eth.refund', {
                    transactionHash: event.log.transactionHash,
                    preimageHash: (0, EthereumUtils_1.parseBuffer)(preimageHash),
                });
            });
            await this.erc20Swap.on('Lockup', async (preimageHash, amount, tokenAddress, claimAddress, refundAddress, timelock, event) => {
                this.emit('erc20.lockup', {
                    transaction: await event.log.getTransaction(),
                    erc20SwapValues: {
                        amount,
                        tokenAddress,
                        claimAddress,
                        refundAddress,
                        timelock: Number(timelock),
                        preimageHash: (0, EthereumUtils_1.parseBuffer)(preimageHash),
                    },
                });
            });
            await this.erc20Swap.on('Claim', (preimageHash, preimage, event) => {
                this.emit('erc20.claim', {
                    transactionHash: event.log.transactionHash,
                    preimageHash: (0, EthereumUtils_1.parseBuffer)(preimageHash),
                    preimage: (0, EthereumUtils_1.parseBuffer)(preimage),
                });
            });
            await this.erc20Swap.on('Refund', (preimageHash, event) => {
                this.emit('erc20.refund', {
                    transactionHash: event.log.transactionHash,
                    preimageHash: (0, EthereumUtils_1.parseBuffer)(preimageHash),
                });
            });
        };
    }
}
exports.default = ContractEventHandler;
//# sourceMappingURL=ContractEventHandler.js.map