"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const boltz_core_1 = require("boltz-core");
const ethers_1 = require("ethers");
const Utils_1 = require("../../Utils");
const Enums_1 = require("../../consts/Enums");
const ChainTipRepository_1 = __importDefault(require("../../db/repositories/ChainTipRepository"));
const Errors_1 = __importDefault(require("../Errors"));
const Wallet_1 = __importDefault(require("../Wallet"));
const ERC20WalletProvider_1 = __importDefault(require("../providers/ERC20WalletProvider"));
const EtherWalletProvider_1 = __importDefault(require("../providers/EtherWalletProvider"));
const ContractEventHandler_1 = __importDefault(require("./ContractEventHandler"));
const ContractHandler_1 = __importDefault(require("./ContractHandler"));
const EthereumTransactionTracker_1 = __importDefault(require("./EthereumTransactionTracker"));
const EvmNetworks_1 = require("./EvmNetworks");
const InjectedProvider_1 = __importDefault(require("./InjectedProvider"));
class EthereumManager {
    constructor(logger, isRsk, config) {
        this.logger = logger;
        this.tokenAddresses = new Map();
        this.init = async (mnemonic) => {
            await this.provider.init();
            this.logger.info(`Initialized ${this.networkDetails.name} RPC providers`);
            const network = await this.provider.getNetwork();
            this.network = {
                chainId: network.chainId,
                name: network.name !== 'unknown' ? network.name : undefined,
            };
            this.signer = ethers_1.Wallet.fromPhrase(mnemonic).connect(this.provider);
            this.address = await this.signer.getAddress();
            this.etherSwap = this.etherSwap.connect(this.signer);
            this.erc20Swap = this.erc20Swap.connect(this.signer);
            await Promise.all([
                this.checkContractVersion(`${this.networkDetails.name} EtherSwap`, this.etherSwap, BigInt(EthereumManager.supportedContractVersions.EtherSwap)),
                this.checkContractVersion(`${this.networkDetails.name} ERC20Swap`, this.erc20Swap, BigInt(EthereumManager.supportedContractVersions.ERC20Swap)),
            ]);
            this.logger.verbose(`Using ${this.networkDetails.name} signer: ${this.address}`);
            const currentBlock = await this.signer.provider.getBlockNumber();
            const chainTip = await ChainTipRepository_1.default.findOrCreateTip(this.networkDetails.symbol, currentBlock);
            this.contractHandler.init(this.provider, this.etherSwap, this.erc20Swap);
            await this.contractEventHandler.init(this.networkDetails, this.etherSwap, this.erc20Swap);
            this.logger.verbose(`${this.networkDetails.name} chain status: ${(0, Utils_1.stringify)({
                blockNumber: currentBlock,
                chainId: Number(this.network.chainId),
            })}`);
            const transactionTracker = new EthereumTransactionTracker_1.default(this.logger, this.networkDetails, this.provider, this.signer);
            await transactionTracker.init();
            await this.provider.on('block', async (blockNumber) => {
                this.logger.silly(`Got new ${this.networkDetails.name} block: ${blockNumber}`);
                await Promise.all([
                    ChainTipRepository_1.default.updateTip(chainTip, blockNumber),
                    transactionTracker.scanPendingTransactions(),
                ]);
            });
            const wallets = new Map();
            for (const token of this.config.tokens) {
                if (token.contractAddress) {
                    if (token.decimals) {
                        if (!wallets.has(token.symbol)) {
                            // Wrap the address in "getAddress" to make sure it is a checksum one
                            const checksumAddress = (0, ethers_1.getAddress)(token.contractAddress);
                            this.tokenAddresses.set(token.symbol, checksumAddress);
                            const provider = new ERC20WalletProvider_1.default(this.logger, this.signer, {
                                symbol: token.symbol,
                                decimals: token.decimals,
                                address: checksumAddress,
                                contract: new ethers_1.Contract(token.contractAddress, boltz_core_1.ContractABIs.ERC20, this.signer),
                            });
                            wallets.set(token.symbol, new Wallet_1.default(this.logger, Enums_1.CurrencyType.ERC20, provider));
                            await this.checkERC20Allowance(provider);
                        }
                        else {
                            throw Errors_1.default.INVALID_ETHEREUM_CONFIGURATION(`duplicate ${token.symbol} token config`);
                        }
                    }
                    else {
                        throw Errors_1.default.INVALID_ETHEREUM_CONFIGURATION(`missing decimals configuration for token: ${token.symbol}`);
                    }
                }
                else {
                    if (token.symbol === this.networkDetails.symbol) {
                        if (!wallets.has(this.networkDetails.symbol)) {
                            wallets.set(this.networkDetails.symbol, new Wallet_1.default(this.logger, Enums_1.CurrencyType.Ether, new EtherWalletProvider_1.default(this.logger, this.signer, this.networkDetails)));
                        }
                        else {
                            throw Errors_1.default.INVALID_ETHEREUM_CONFIGURATION('duplicate Ether token config');
                        }
                    }
                    else {
                        throw Errors_1.default.INVALID_ETHEREUM_CONFIGURATION(`missing token contract address for: ${(0, Utils_1.stringify)(token)}`);
                    }
                }
            }
            return wallets;
        };
        this.destroy = async () => {
            await this.provider.removeAllListeners();
        };
        this.getContractDetails = async () => {
            return {
                network: {
                    chainId: Number(this.network.chainId),
                    name: this.network.name,
                },
                tokens: this.tokenAddresses,
                swapContracts: new Map([
                    ['EtherSwap', await this.etherSwap.getAddress()],
                    ['ERC20Swap', await this.erc20Swap.getAddress()],
                ]),
            };
        };
        this.hasSymbol = (symbol) => this.networkDetails.symbol === symbol || this.tokenAddresses.has(symbol);
        this.checkERC20Allowance = async (erc20Wallet) => {
            const allowance = await erc20Wallet.getAllowance(this.config.erc20SwapAddress);
            this.logger.debug(`Allowance of ${erc20Wallet.symbol} is ${allowance.toString()}`);
            if (allowance == BigInt(0)) {
                this.logger.verbose(`Setting allowance of ${erc20Wallet.symbol}`);
                const { transactionId } = await erc20Wallet.approve(this.config.erc20SwapAddress, ethers_1.MaxUint256);
                this.logger.info(`Set allowance of token ${erc20Wallet.symbol}: ${transactionId}`);
            }
        };
        this.checkContractVersion = async (name, contract, supportedVersion) => {
            const contractVersion = await contract.version();
            if (contractVersion !== supportedVersion) {
                throw Errors_1.default.UNSUPPORTED_CONTRACT_VERSION(name, await contract.getAddress(), contractVersion, supportedVersion);
            }
        };
        if (config === null ||
            config === undefined ||
            [config.etherSwapAddress, config.erc20SwapAddress].some((value) => value === undefined || value === '')) {
            throw Errors_1.default.MISSING_SWAP_CONTRACTS();
        }
        this.networkDetails = isRsk ? EvmNetworks_1.Rsk : EvmNetworks_1.Ethereum;
        this.config = config;
        this.provider = new InjectedProvider_1.default(this.logger, this.networkDetails, this.config);
        this.logger.debug(`Using ${this.networkDetails.name} EtherSwap contract: ${this.config.etherSwapAddress}`);
        this.logger.debug(`Using ${this.networkDetails.name} ERC20Swap contract: ${this.config.erc20SwapAddress}`);
        this.etherSwap = new ethers_1.Contract(config.etherSwapAddress, boltz_core_1.ContractABIs.EtherSwap);
        this.erc20Swap = new ethers_1.Contract(config.erc20SwapAddress, boltz_core_1.ContractABIs.ERC20Swap);
        this.contractHandler = new ContractHandler_1.default(this.logger, this.networkDetails);
        this.contractEventHandler = new ContractEventHandler_1.default(this.logger);
    }
}
EthereumManager.supportedContractVersions = {
    EtherSwap: 3,
    ERC20Swap: 3,
};
exports.default = EthereumManager;
//# sourceMappingURL=EthereumManager.js.map