"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EthProviderService = void 0;
const ethers_1 = require("ethers");
const Utils_1 = require("../../Utils");
const PendingEthereumTransactionRepository_1 = __importDefault(require("../../db/repositories/PendingEthereumTransactionRepository"));
const Errors_1 = __importDefault(require("./Errors"));
var EthProviderService;
(function (EthProviderService) {
    EthProviderService["Node"] = "Node";
    EthProviderService["Infura"] = "Infura";
    EthProviderService["Alchemy"] = "Alchemy";
})(EthProviderService || (exports.EthProviderService = EthProviderService = {}));
/**
 * This provider is a wrapper for the JsonRpcProvider of ethers, but it writes sent transactions to the database
 * and, depending on the configuration, falls back to Alchemy and Infura as Web3 provider
 */
class InjectedProvider {
    constructor(logger, networkDetails, config) {
        this.logger = logger;
        this.networkDetails = networkDetails;
        this.providers = new Map();
        this.init = async () => {
            this.logger.verbose(`Trying to connect to ${this.providers.size} ${this.networkDetails.name} RPC providers:\n - ${Array.from(this.providers.keys()).join('\n - ')}`);
            const networks = [];
            for (const [providerName, provider] of this.providers) {
                try {
                    const network = await provider.getNetwork();
                    this.logConnectedProvider(providerName, network);
                    networks.push(network);
                }
                catch (error) {
                    this.logDisabledProvider(providerName, `could not connect: ${(0, Utils_1.formatError)(error)}`);
                    this.providers.delete(providerName);
                }
            }
            const networksAreSame = networks.every((network) => network.chainId === networks[0].chainId);
            if (!networksAreSame) {
                throw Errors_1.default.UNEQUAL_PROVIDER_NETWORKS(networks);
            }
            this.network = networks[0];
            this.logger.info(`Connected to ${this.providers.size} ${this.networkDetails.name} RPC providers:\n - ${Array.from(this.providers.keys()).join('\n - ')}`);
        };
        this.addEthProvider = (name, providerConfig) => {
            if (providerConfig === undefined || providerConfig.apiKey === undefined) {
                this.logDisabledProvider(name, 'no api key was set');
                return;
            }
            if (providerConfig.network === undefined) {
                this.logDisabledProvider(name, 'no network was specified');
                return;
            }
            switch (name) {
                case EthProviderService.Infura:
                    this.providers.set(name, new ethers_1.InfuraProvider(providerConfig.network, providerConfig.apiKey));
                    break;
                case EthProviderService.Alchemy:
                    this.providers.set(name, new ethers_1.AlchemyProvider(providerConfig.network, providerConfig.apiKey));
                    break;
                default:
                    this.logDisabledProvider(name, 'provider not supported');
                    return;
            }
            this.logAddedProvider(name, providerConfig);
        };
        /*
         * Method calls
         */
        this.destroy = () => {
            this.providers.forEach((provider) => provider.destroy());
        };
        this.call = (transaction, blockTag) => {
            return this.forwardMethod('call', transaction, blockTag);
        };
        this.estimateGas = (transaction) => {
            return this.forwardMethod('estimateGas', transaction);
        };
        this.getBalance = (addressOrName, blockTag) => {
            return this.forwardMethod('getBalance', addressOrName, blockTag);
        };
        this.getBlock = (blockHashOrBlockTag) => {
            return this.forwardMethod('getBlock', blockHashOrBlockTag);
        };
        this.getBlockNumber = () => {
            return this.forwardMethod('getBlockNumber');
        };
        this.getCode = (addressOrName, blockTag) => {
            return this.forwardMethod('getCode', addressOrName, blockTag);
        };
        this.getStorage = (address, position, blockTag) => {
            return this.forwardMethod('getStorage', address, position, blockTag);
        };
        this.getFeeData = () => {
            return this.forwardMethod('getFeeData');
        };
        this.getLogs = (filter) => {
            return this.forwardMethod('getLogs', filter);
        };
        this.getNetwork = async () => {
            return this.network;
        };
        this.getStorageAt = (addressOrName, position, blockTag) => {
            return this.forwardMethod('getStorageAt', addressOrName, position, blockTag);
        };
        this.getTransaction = (transactionHash) => {
            return this.forwardMethodNullable('getTransaction', transactionHash);
        };
        this.getTransactionCount = (addressOrName, blockTag) => {
            return this.forwardMethod('getTransactionCount', addressOrName, blockTag);
        };
        this.getTransactionReceipt = (transactionHash) => {
            return this.forwardMethodNullable('getTransactionReceipt', transactionHash);
        };
        this.getTransactionResult = (transactionHash) => {
            return this.forwardMethodNullable('getTransactionResult', transactionHash);
        };
        this.lookupAddress = (address) => {
            return this.forwardMethod('lookupAddress', address);
        };
        this.resolveName = (name) => {
            return this.forwardMethod('resolveName', name);
        };
        this.broadcastTransaction = async (signedTransaction) => {
            const transaction = ethers_1.Transaction.from(signedTransaction);
            await this.addToTransactionDatabase(transaction.hash, transaction.nonce);
            // When sending a transaction, you want it to propagate on the network as quickly as possible
            // Therefore, we send it to all available providers
            const promises = Array.from(this.providers.values()).map((provider) => provider.broadcastTransaction(signedTransaction));
            const settled = await Promise.allSettled(promises);
            const results = settled
                .filter((res) => res.status === 'fulfilled')
                .map((res) => res.value);
            if (results.length > 0) {
                return results[0];
            }
            throw settled[0].reason;
        };
        this.sendTransaction = async (tx) => {
            const res = await this.forwardMethod('sendTransaction', tx);
            await this.addToTransactionDatabase(res.hash, res.nonce);
            return res;
        };
        this.waitForTransaction = (transactionHash, confirmations, timeout) => {
            return this.forwardMethod('waitForTransaction', transactionHash, confirmations, timeout);
        };
        this.waitForBlock = (blockTag) => {
            return this.forwardMethod('waitForBlock', blockTag);
        };
        /*
         * Listeners
         */
        this.emit = async (eventName, ...args) => {
            for (const [, provider] of this.providers) {
                await provider.emit(eventName, args);
            }
            return true;
        };
        this.addListener = async (eventName, listener) => {
            return this.on(eventName, listener);
        };
        this.listenerCount = async (eventName) => {
            return Array.from(this.providers.values())[0].listenerCount(eventName);
        };
        this.listeners = async (eventName) => {
            return Array.from(this.providers.values())[0].listeners(eventName);
        };
        this.off = async (eventName, listener) => {
            for (const [, provider] of this.providers) {
                await provider.off(eventName, listener);
            }
            return this;
        };
        this.on = async (eventName, listener) => {
            const providerDeltas = new Map();
            const injectedListener = (...args) => {
                if (this.providers.size === 1) {
                    listener(...args);
                    return;
                }
                const hashCode = this.hashCode(args.map((entry) => JSON.stringify(entry)).join());
                const currentDelta = providerDeltas.get(hashCode) || 0;
                if (currentDelta === this.providers.size - 1) {
                    providerDeltas.delete(hashCode);
                }
                else {
                    providerDeltas.set(hashCode, currentDelta + 1);
                }
                if (currentDelta === 0) {
                    listener(...args);
                }
            };
            for (const provider of this.providers.values()) {
                await provider.on(eventName, injectedListener);
            }
            return this;
        };
        this.once = async (eventName, listener) => {
            let emittedEvent = false;
            const injectedListener = (...args) => {
                if (!emittedEvent) {
                    emittedEvent = true;
                    listener(...args);
                }
            };
            for (const provider of this.providers.values()) {
                await provider.once(eventName, injectedListener);
            }
            return this;
        };
        this.removeAllListeners = async (eventName) => {
            for (const [, provider] of this.providers) {
                await provider.removeAllListeners(eventName);
            }
            return this;
        };
        this.removeListener = (eventName, listener) => {
            return this.off(eventName, listener);
        };
        /*
         * Helper utils
         */
        this.forwardMethod = async (method, ...args) => {
            const res = await this.forwardMethodNullable(method, ...args);
            if (res === null) {
                throw Errors_1.default.REQUESTS_TO_PROVIDERS_FAILED(['null returned']);
            }
            return res;
        };
        this.forwardMethodNullable = async (method, ...args) => {
            const errors = [];
            for (const [providerName, provider] of this.providers) {
                try {
                    const result = await this.promiseWithTimeout(provider[method](...args), 'timeout');
                    if (result !== null) {
                        return result;
                    }
                }
                catch (error) {
                    const formattedError = (0, Utils_1.formatError)(error);
                    this.logger.warn(`Request to ${this.networkDetails.name} RPC provider ${providerName} failed: ${method}: ${formattedError}`);
                    errors.push(formattedError);
                }
            }
            if (errors.length > 0) {
                throw Errors_1.default.REQUESTS_TO_PROVIDERS_FAILED(errors);
            }
            return null;
        };
        this.promiseWithTimeout = async (promise, errorMessage) => {
            let timeoutHandle;
            const timeoutPromise = new Promise((_, reject) => {
                timeoutHandle = setTimeout(() => reject(errorMessage), InjectedProvider.requestTimeout);
            });
            return Promise.race([promise, timeoutPromise]).then((result) => {
                clearTimeout(timeoutHandle);
                return result;
            });
        };
        this.addToTransactionDatabase = async (hash, nonce) => {
            this.logger.silly(`Sending ${this.networkDetails.name} transaction: ${hash}`);
            await PendingEthereumTransactionRepository_1.default.addTransaction(hash, nonce);
        };
        this.hashCode = (value) => {
            let hash = 0;
            for (let i = 0; i < value.length; i++) {
                const char = value.charCodeAt(i);
                hash = (hash << 5) - hash + char;
                hash |= 0;
            }
            return hash;
        };
        this.logAddedProvider = (name, config) => {
            this.logger.debug(`Adding ${this.networkDetails.name} RPC provider ${name}: ${(0, Utils_1.stringify)(config)}`);
        };
        this.logConnectedProvider = (name, network) => {
            this.logger.verbose(`Connected to ${this.networkDetails.name} RPC provider ${name} on network: ${network.chainId}`);
        };
        this.logDisabledProvider = (name, reason) => {
            this.logger.warn(`Disabled ${this.networkDetails.name} RPC provider ${name}: ${reason}`);
        };
        this.provider = this;
        if (config.providerEndpoint) {
            this.providers.set(EthProviderService.Node, new ethers_1.JsonRpcProvider(config.providerEndpoint));
            this.logAddedProvider(EthProviderService.Node, {
                endpoint: config.providerEndpoint,
            });
        }
        else {
            this.logDisabledProvider(EthProviderService.Node, 'no endpoint was specified');
        }
        this.addEthProvider(EthProviderService.Infura, config.infura);
        this.addEthProvider(EthProviderService.Alchemy, config.alchemy);
        if (this.providers.size === 0) {
            throw Errors_1.default.NO_PROVIDER_SPECIFIED();
        }
    }
}
InjectedProvider.requestTimeout = 5000;
exports.default = InjectedProvider;
//# sourceMappingURL=InjectedProvider.js.map