"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EthereumUtils_1 = require("../ethereum/EthereumUtils");
class ERC20WalletProvider {
    constructor(logger, signer, token) {
        this.logger = logger;
        this.signer = signer;
        this.token = token;
        this.serviceName = () => {
            return 'Wallet';
        };
        this.getTokenAddress = () => {
            return this.token.address;
        };
        this.getAddress = () => {
            return this.signer.getAddress();
        };
        this.getBalance = async () => {
            const balance = this.normalizeTokenAmount(await this.token.contract.balanceOf(await this.getAddress()));
            return {
                confirmedBalance: balance,
                unconfirmedBalance: 0,
            };
        };
        this.sendToAddress = async (address, amount) => {
            const actualAmount = this.formatTokenAmount(amount);
            const transaction = await this.token.contract.transfer(address, actualAmount, {
                ...(await (0, EthereumUtils_1.getGasPrices)(this.signer.provider)),
            });
            return {
                transactionId: transaction.hash,
            };
        };
        this.sweepWallet = async (address) => {
            const balance = await this.token.contract.balanceOf(await this.getAddress());
            const transaction = await this.token.contract.transfer(address, balance, {
                ...(await (0, EthereumUtils_1.getGasPrices)(this.signer.provider)),
            });
            return {
                transactionId: transaction.hash,
            };
        };
        this.getAllowance = async (spender) => {
            return this.token.contract.allowance(await this.signer.getAddress(), spender);
        };
        this.approve = async (spender, amount) => {
            const transaction = await this.token.contract.approve(spender, amount, {
                ...(await (0, EthereumUtils_1.getGasPrices)(this.signer.provider)),
            });
            return {
                transactionId: transaction.hash,
            };
        };
        /**
         * Formats the token amount to send from 10 ** -8 decimals
         */
        this.formatTokenAmount = (amount) => {
            const amountBn = BigInt(amount);
            if (this.token.decimals === 8) {
                return amountBn;
            }
            else {
                const exponent = BigInt(10) ** BigInt(Math.abs(this.token.decimals - 8));
                if (this.token.decimals > 8) {
                    return amountBn * exponent;
                }
                else {
                    return amountBn / exponent;
                }
            }
        };
        /**
         * Normalizes the token balance to 10 ** -8 decimals
         */
        this.normalizeTokenAmount = (amount) => {
            if (this.token.decimals === 8) {
                return Number(amount);
            }
            else {
                const exponent = BigInt(10) ** BigInt(Math.abs(this.token.decimals - 8));
                if (this.token.decimals > 8) {
                    return Number(amount / exponent);
                }
                else {
                    return Number(amount * exponent);
                }
            }
        };
        this.symbol = token.symbol;
        this.logger.info(`Initialized ${this.symbol} ERC20 wallet with contract: ${this.token.address}`);
    }
}
exports.default = ERC20WalletProvider;
//# sourceMappingURL=ERC20WalletProvider.js.map