"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Core_1 = require("../../Core");
const Utils_1 = require("../../Utils");
const ChainClient_1 = __importDefault(require("../../chain/ChainClient"));
const Enums_1 = require("../../consts/Enums");
class ElementsWalletProvider {
    constructor(logger, chainClient) {
        this.logger = logger;
        this.chainClient = chainClient;
        this.serviceName = () => {
            return 'Elements';
        };
        this.getBalance = async () => {
            const balances = await this.chainClient.getBalances();
            const confirmedBalance = balances.mine.trusted[ElementsWalletProvider.assetLabel];
            const unconfirmedBalance = balances.mine.untrusted_pending[ElementsWalletProvider.assetLabel];
            return {
                confirmedBalance,
                unconfirmedBalance,
            };
        };
        this.getAddress = () => {
            return this.chainClient.getNewAddress();
        };
        this.dumpBlindingKey = async (address) => {
            return (0, Utils_1.getHexBuffer)(await this.chainClient.dumpBlindingKey(address));
        };
        this.sendToAddress = async (address, amount, satPerVbyte) => {
            const transactionId = await this.chainClient.sendToAddress(address, amount, await this.getFeePerVbyte(satPerVbyte));
            return this.handleLiquidTransaction(transactionId, address);
        };
        this.sweepWallet = async (address, satPerVbyte) => {
            const balance = await this.getBalance();
            const transactionId = await this.chainClient.sendToAddress(address, balance.confirmedBalance + balance.unconfirmedBalance, await this.getFeePerVbyte(satPerVbyte), true);
            return this.handleLiquidTransaction(transactionId, address);
        };
        this.handleLiquidTransaction = async (transactionId, address) => {
            const [addressInfo, transactionVerbose] = await Promise.all([
                this.chainClient.getAddressInfo(address),
                this.chainClient.getRawTransactionVerbose(transactionId),
            ]);
            const decodedAddress = addressInfo.unconfidential;
            return {
                transactionId,
                transaction: (0, Core_1.parseTransaction)(Enums_1.CurrencyType.Liquid, transactionVerbose.hex),
                vout: transactionVerbose.vout.find((output) => output.scriptPubKey.address === decodedAddress ||
                    output.scriptPubKey.addresses?.includes(decodedAddress))?.n,
                fee: Math.ceil(transactionVerbose.vout.find((output) => output.scriptPubKey.type === ElementsWalletProvider.feeOutputType).value * ChainClient_1.default.decimals),
            };
        };
        this.getFeePerVbyte = async (satPerVbyte) => {
            return satPerVbyte || (await this.chainClient.estimateFee());
        };
        this.symbol = chainClient.symbol;
        this.logger.info(`Initialized ${this.symbol} Elements wallet`);
    }
}
ElementsWalletProvider.assetLabel = 'bitcoin';
ElementsWalletProvider.feeOutputType = 'fee';
exports.default = ElementsWalletProvider;
//# sourceMappingURL=ElementsWalletProvider.js.map