"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const rpc_pb_1 = require("../../proto/lnd/rpc_pb");
class LndWalletProvider {
    constructor(logger, lndClient, chainClient) {
        this.logger = logger;
        this.lndClient = lndClient;
        this.chainClient = chainClient;
        this.serviceName = () => {
            return this.lndClient.serviceName();
        };
        this.getBalance = () => {
            return this.lndClient.getWalletBalance();
        };
        this.getAddress = async () => {
            const response = await this.lndClient.newAddress(rpc_pb_1.AddressType.WITNESS_PUBKEY_HASH);
            return response.address;
        };
        this.sendToAddress = async (address, amount, satPerVbyte) => {
            // To avoid weird race conditions (insanely unlikely but still), the start height of the LND transaction list call
            // is queried *before* sending the onchain transaction
            const { blockHeight } = await this.lndClient.getInfo();
            const response = await this.lndClient.sendCoins(address, amount, await this.getFeePerVbyte(satPerVbyte));
            return this.handleLndTransaction(response.txid, address, blockHeight);
        };
        this.sweepWallet = async (address, satPerVbyte) => {
            // See "sendToAddress"
            const { blockHeight } = await this.lndClient.getInfo();
            const response = await this.lndClient.sweepWallet(address, await this.getFeePerVbyte(satPerVbyte));
            return this.handleLndTransaction(response.txid, address, blockHeight);
        };
        this.handleLndTransaction = async (transactionId, address, listStartHeight) => {
            const rawTransaction = await this.chainClient.getRawTransactionVerbose(transactionId);
            let vout = 0;
            for (const output of rawTransaction.vout) {
                if (output.scriptPubKey.address === address ||
                    (output.scriptPubKey.addresses &&
                        output.scriptPubKey.addresses.includes(address))) {
                    vout = output.n;
                }
            }
            let fee = 0;
            // To limit the number of onchain transactions LND has to query, the start height is set
            const { transactionsList } = await this.lndClient.getOnchainTransactions(listStartHeight);
            for (let i = 0; i < transactionsList.length; i += 1) {
                const transaction = transactionsList[i];
                if (transaction.txHash === transactionId) {
                    fee = transaction.totalFees;
                    break;
                }
            }
            return {
                fee,
                vout,
                transactionId,
                transaction: bitcoinjs_lib_1.Transaction.fromHex(rawTransaction.hex),
            };
        };
        this.getFeePerVbyte = async (satPerVbyte) => {
            return satPerVbyte || (await this.chainClient.estimateFee());
        };
        this.symbol = chainClient.symbol;
        this.logger.info(`Initialized ${this.symbol} LND wallet`);
    }
}
exports.default = LndWalletProvider;
//# sourceMappingURL=LndWalletProvider.js.map