"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../../Utils");
const Consts_1 = require("../../consts/Consts");
const EthereumUtils_1 = require("./EthereumUtils");
class ContractHandler {
    constructor(logger, networkDetails) {
        this.logger = logger;
        this.networkDetails = networkDetails;
        this.init = (provider, etherSwap, erc20Swap) => {
            this.provider = provider;
            this.etherSwap = etherSwap;
            this.erc20Swap = erc20Swap;
        };
        this.lockupEther = async (preimageHash, amount, claimAddress, timeLock) => {
            this.logger.debug(`Locking ${amount} ${this.networkDetails.symbol} with preimage hash: ${(0, Utils_1.getHexString)(preimageHash)}`);
            return this.etherSwap.lock(preimageHash, claimAddress, timeLock, {
                value: amount,
                ...(await (0, EthereumUtils_1.getGasPrices)(this.provider)),
            });
        };
        this.lockupEtherPrepayMinerfee = async (preimageHash, amount, amountPrepay, claimAddress, timeLock) => {
            const transactionValue = amount + amountPrepay;
            const gasLimitEstimationWithoutPrepay = await this.etherSwap.lock.estimateGas(preimageHash, claimAddress, timeLock, {
                value: transactionValue,
            });
            this.logger.debug(`Locking ${amount} and sending prepay ${amountPrepay} ${this.networkDetails.symbol} with preimage hash: ${(0, Utils_1.getHexString)(preimageHash)}`);
            return this.etherSwap.lockPrepayMinerfee(preimageHash, claimAddress, timeLock, amountPrepay, {
                value: transactionValue,
                // TODO: integration test that tries to exploit the attack vector of using an insane amount of gas in the fallback function of the contract at the claim address
                gasLimit: gasLimitEstimationWithoutPrepay + Consts_1.ethereumPrepayMinerFeeGasLimit,
                ...(await (0, EthereumUtils_1.getGasPrices)(this.provider)),
            });
        };
        this.claimEther = async (preimage, amount, refundAddress, timelock) => {
            this.logger.debug(`Claiming ${this.networkDetails.symbol} with preimage: ${(0, Utils_1.getHexString)(preimage)}`);
            return this.etherSwap['claim(bytes32,uint256,address,uint256)'](preimage, amount, refundAddress, timelock, {
                ...(await (0, EthereumUtils_1.getGasPrices)(this.provider)),
            });
        };
        this.refundEther = async (preimageHash, amount, claimAddress, timelock) => {
            this.logger.debug(`Refunding ${this.networkDetails.symbol} with preimage hash: ${(0, Utils_1.getHexString)(preimageHash)}`);
            return this.etherSwap.refund(preimageHash, amount, claimAddress, timelock, {
                ...(await (0, EthereumUtils_1.getGasPrices)(this.provider)),
            });
        };
        this.lockupToken = async (token, preimageHash, amount, claimAddress, timeLock) => {
            this.logger.debug(`Locking ${amount} ${token.symbol} with preimage hash: ${(0, Utils_1.getHexString)(preimageHash)}`);
            return this.erc20Swap.lock(preimageHash, amount, token.getTokenAddress(), claimAddress, timeLock, {
                ...(await (0, EthereumUtils_1.getGasPrices)(this.provider)),
            });
        };
        this.lockupTokenPrepayMinerfee = async (token, preimageHash, amount, amountPrepay, claimAddress, timeLock) => {
            const gasLimitEstimationWithoutPrepay = await this.erc20Swap.lock.estimateGas(preimageHash, amount, token.getTokenAddress(), claimAddress, timeLock);
            this.logger.debug(`Locking ${amount} ${token.symbol} and sending prepay ${amountPrepay} Ether with preimage hash: ${(0, Utils_1.getHexString)(preimageHash)}`);
            return this.erc20Swap.lockPrepayMinerfee(preimageHash, amount, token.getTokenAddress(), claimAddress, timeLock, {
                value: amountPrepay,
                gasLimit: gasLimitEstimationWithoutPrepay + Consts_1.ethereumPrepayMinerFeeGasLimit,
                ...(await (0, EthereumUtils_1.getGasPrices)(this.provider)),
            });
        };
        this.claimToken = async (token, preimage, amount, refundAddress, timeLock) => {
            this.logger.debug(`Claiming ${token.symbol} with preimage: ${(0, Utils_1.getHexString)(preimage)}`);
            return this.erc20Swap['claim(bytes32,uint256,address,address,uint256)'](preimage, amount, token.getTokenAddress(), refundAddress, timeLock, {
                ...(await (0, EthereumUtils_1.getGasPrices)(this.provider)),
            });
        };
        this.refundToken = async (token, preimageHash, amount, claimAddress, timeLock) => {
            this.logger.debug(`Refunding ${token.symbol} with preimage hash: ${(0, Utils_1.getHexString)(preimageHash)}`);
            return this.erc20Swap.refund(preimageHash, amount, token.getTokenAddress(), claimAddress, timeLock, {
                ...(await (0, EthereumUtils_1.getGasPrices)(this.provider)),
            });
        };
    }
}
exports.default = ContractHandler;
//# sourceMappingURL=ContractHandler.js.map